/***************************************************************************
                          3Dview.cpp
                    -------------------
    begin                : Fri Jun 15 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : http://lx-viewer.sourceforge.ne
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//   this is the implemtation of the 3D view dialog box

#include "resource.h"  // macros used for commands
#include "3Dview.h"

LX_3DViewDialog::LX_3DViewDialog(QWidget* parent,const char* name,WFlags flags):QDialog(parent,name,flags)
{
   const short minslider = -180;  // minimum value -180 degrees
   const short maxslider = 180;   // maximum value +180 degrees
   const short sliderjump = 15;   // slider tick incremen
   const short sliderinit = 0;    // slider initial value
   const short xstart = 300;      // upper left corner of dialog x pixels
   const short ystart = 200;      // upper left corner of dialog y pixels
   const short xsize = 500;       // start width of the dialog
   const short ysize = 300;       // start height of the dialog
   const short blankspace = 3;    // number of pixels around frames

   setGeometry(xstart,ystart,xsize,ysize);

   //   create the main area of the widge
   p2pixarea = new QWidget(this);
   p2pixarea->setBackgroundColor(QColor (0x00,0x00,0x00));   // Black
   XColor = QColor (0xff, 0x00, 0x00);                       // Red
   YColor = QColor (0x00, 0xff, 0x00);                       // Green
   ZColor = QColor (0x00, 0xff, 0xff);                       // Cyan
   pixpainter = new QPainter(p2pixarea);

   XSlider = new QSlider(minslider,maxslider,sliderjump,sliderinit,Horizontal,this);
   YSlider = new QSlider(minslider,maxslider,sliderjump,sliderinit,Horizontal,this);
   ZSlider = new QSlider(minslider,maxslider,sliderjump,sliderinit,Horizontal,this);
   XSlider->setTickmarks(QSlider::Below);
   YSlider->setTickmarks(QSlider::Below);
   ZSlider->setTickmarks(QSlider::Below);

   QLabel* XLabel = new QLabel(tr("X Axis"),this);
   QLabel* YLabel = new QLabel(tr("Y Axis"),this);
   QLabel* ZLabel = new QLabel(tr("Z Axis"),this);
   XLabel->setAlignment(AlignCenter);
   YLabel->setAlignment(AlignCenter);
   ZLabel->setAlignment(AlignCenter);
   XLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
   YLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
   ZLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);

  //   create a custom palette for the coordinate label widge
   QPalette macroB = this->palette();
   macroB.setColor(QColorGroup::Foreground,QColor(0xff,0x00,0x00));      //   Red

   XCoorddisp = new QLabel(this);
   YCoorddisp = new QLabel(this);
   ZCoorddisp = new QLabel(this);
   XCoorddisp->setPalette(macroB);
   YCoorddisp->setPalette(macroB);
   ZCoorddisp->setPalette(macroB);
   XCoorddisp->setNum(sliderinit);
   YCoorddisp->setNum(sliderinit);
   ZCoorddisp->setNum(sliderinit);
   XCoorddisp->setAlignment(AlignCenter);
   YCoorddisp->setAlignment(AlignCenter);
   ZCoorddisp->setAlignment(AlignCenter);
   XCoorddisp->setFont(QFont("helvetica",14,QFont::Bold));
   YCoorddisp->setFont(QFont("helvetica",14,QFont::Bold));
   ZCoorddisp->setFont(QFont("helvetica",14,QFont::Bold));
   QLabel* XCoordlabl = new QLabel(this);
   QLabel* YCoordlabl = new QLabel(this);
   QLabel* ZCoordlabl = new QLabel(this);
   XCoordlabl->setText(tr("X Rotation"));
   YCoordlabl->setText(tr("Y Rotation"));
   ZCoordlabl->setText(tr("Z Rotation"));
   XCoordlabl->setFont(QFont("helvetica",10,QFont::Normal));
   YCoordlabl->setFont(QFont("helvetica",10,QFont::Normal));
   ZCoordlabl->setFont(QFont("helvetica",10,QFont::Normal));

   resetbut = new QPushButton(tr("Initial Values"),this);
   cancelbut=new QPushButton(tr("Cancel"),this);
   okbut    = new QPushButton(tr("Accept"),this);

   QVBoxLayout* vbox1 = new QVBoxLayout (this);
   QVBoxLayout* vbox2 = new QVBoxLayout();
   QHBoxLayout* hbox1 = new QHBoxLayout();
   QHBoxLayout* hbox2 = new QHBoxLayout();
   QHBoxLayout* hbox3 = new QHBoxLayout();
   QHBoxLayout* hbox4 = new QHBoxLayout();
   QHBoxLayout* hbox5 = new QHBoxLayout();
   QHBoxLayout* hbox6 = new QHBoxLayout();
   vbox1->addLayout(hbox1,1);
   vbox1->addLayout(hbox2,0);
   vbox1->addLayout(hbox3,0);
   hbox1->addWidget(p2pixarea,1);
   hbox1->addLayout(vbox2,0);
   vbox2->addLayout(hbox4);
   vbox2->addLayout(hbox5);
   vbox2->addLayout(hbox6);
   vbox2->addStretch(1);
   vbox2->addWidget(resetbut);
   vbox2->addWidget(cancelbut);
   vbox2->addWidget(okbut);
   hbox4->addWidget(XCoorddisp);
   hbox4->addWidget(XCoordlabl);
   hbox5->addWidget(YCoorddisp);
   hbox5->addWidget(YCoordlabl);
   hbox6->addWidget(ZCoorddisp);
   hbox6->addWidget(ZCoordlabl);
   hbox2->addWidget(XLabel);
   hbox2->addWidget(YLabel);
   hbox2->addWidget(ZLabel);
   hbox3->addWidget(XSlider);
   hbox3->addWidget(YSlider);
   hbox3->addWidget(ZSlider);
   hbox2->setMargin(blankspace);
   hbox3->setMargin(blankspace);
   hbox2->insertSpacing(1,blankspace*2);
   hbox2->insertSpacing(3,blankspace*2);
   hbox3->insertSpacing(1,blankspace*2);
   hbox3->insertSpacing(3,blankspace*2);

//   now connect some signals and slots
   connect(XSlider,SIGNAL(valueChanged(int)),XCoorddisp,SLOT(setNum(int)));
   connect(YSlider,SIGNAL(valueChanged(int)),YCoorddisp,SLOT(setNum(int)));
   connect(ZSlider,SIGNAL(valueChanged(int)),ZCoorddisp,SLOT(setNum(int)));
   connect(resetbut,SIGNAL(clicked()),this,SLOT(resetall()));
   connect(cancelbut,SIGNAL(clicked()),this,SLOT(cancel()));
   connect(okbut,SIGNAL(clicked()),this,SLOT(okpressed()));
   connect(XSlider,SIGNAL(valueChanged(int)),this,SLOT(rotateX(int)));
   connect(YSlider,SIGNAL(valueChanged(int)),this,SLOT(rotateY(int)));
   connect(ZSlider,SIGNAL(valueChanged(int)),this,SLOT(rotateZ(int)));
}

//   Event handlers
void LX_3DViewDialog::resizeEvent (QResizeEvent*)
{
   const float percentsize = 0.42f;

   axislen=percentsize*p2pixarea->height();
   if (p2pixarea->width()<p2pixarea->height()) axislen = percentsize*p2pixarea->width();

   center = QPoint(p2pixarea->width()/2,p2pixarea->height()/2);

   pixpainter->fillRect(0,0,p2pixarea->width(),p2pixarea->height(),QColor(0x00,0x00,0x00));
}

void LX_3DViewDialog::showEvent(QShowEvent*)
{
   resetall();
}

void LX_3DViewDialog::paintEvent(QPaintEvent*)
{
   showIt();
}

//   Functions
void LX_3DViewDialog::showIt()
//   For speed these routines (showIt and eraseIt) will draw over last triad in the
//   background color, then draw the new triad with the proper colors.  This way we
//   do not need to redraw the entire widget background at each slider step
{

   const short nib = 2;
   static QPoint pt;

   pixpainter->moveTo(center);
   pixpainter->setPen(QPen(XColor,nib));
   pixpainter->lineTo(QPoint(static_cast<int>(center.x()+axislen*XVect.x),static_cast<int>(center.y()-axislen*XVect.y)));
   pt = pixpainter->pos();
   pixpainter->drawText(pt.x()+5,pt.y()-3,"X");
   pixpainter->moveTo(center);
   pixpainter->setPen(QPen(YColor,nib));
   pixpainter->lineTo(QPoint(static_cast<int>(center.x()+axislen*YVect.x),static_cast<int>(center.y()-axislen*YVect.y)));
   pt = pixpainter->pos();
   pixpainter->drawText(pt.x()+5,pt.y()-3,"Y");
   pixpainter->moveTo(center);
   pixpainter->setPen(QPen(ZColor,nib));
   pixpainter->lineTo(QPoint(static_cast<int>(center.x()+axislen*ZVect.x),static_cast<int>(center.y()-axislen*ZVect.y)));
   pt = pixpainter->pos();
   pixpainter->drawText(pt.x()+5,pt.y()-3,"Z");
}

void LX_3DViewDialog::eraseIt()
{
   const short nib = 2;
   static QPoint pt;

   pixpainter->setRasterOp(CopyROP);
   pixpainter->moveTo(center);
   pixpainter->setPen(QPen(p2pixarea->backgroundColor(),nib));
   pixpainter->lineTo(QPoint(static_cast<int>(center.x()+axislen*XVect.x),static_cast<int>(center.y()-axislen*XVect.y)));
   pt = pixpainter->pos();
   pixpainter->drawText(pt.x()+5,pt.y()-3,"X");
   pixpainter->moveTo(center);
   pixpainter->lineTo(QPoint(static_cast<int>(center.x()+axislen*YVect.x),static_cast<int>(center.y()-axislen*YVect.y)));
   pt = pixpainter->pos();
   pixpainter->drawText(pt.x()+5,pt.y()-3,"Y");
   pixpainter->moveTo(center);
   pixpainter->lineTo(QPoint(static_cast<int>(center.x()+axislen*ZVect.x),static_cast<int>(center.y()-axislen*ZVect.y)));
   pt = pixpainter->pos();
   pixpainter->drawText(pt.x()+5,pt.y()-3,"Z");
   pixpainter->setRasterOp(CopyROP);
}


void LX_3DViewDialog::setInitial3DViewParms()
{
//   This routine will determine the current view state of the drawing
   const XYZ screencamera = {0.0,0.0,1.0};
   const XYZ screenup = {0.0,1.0,0.0};
   XYZ temp;
   double phi = 0.0;    // angle from Z axis to viewdir vector
   double theta = 0.0;  // angle from X axis to viewdir vector
   double alpha = 0.0;  // angle in XY plane between up (as rotated) and Y axis
   double num = 0.0;
   double den = 0.0;

   XVect.x = 1.0;
   XVect.y = 0.0;
   XVect.z = 0.0;

   YVect.x = 0.0;
   YVect.y = 1.0;
   YVect.z = 0.0;

   ZVect.x = 0.0;
   ZVect.y = 0.0;
   ZVect.z = 1.0;

   //   phi and theta are the two angles of a spherical coordinate system
   //   below we are just converting from rectangular coordinates to spherical coords.

   phi = acos(camera.z/sqrt(camera.x*camera.x+camera.y*camera.y+camera.z*camera.z));

   num = camera.y;
   den = camera.x;
   if (num < 0.0) num = -num;
   if (den < 0.0) den = -den;
   if (den != 0.0) theta = atan(num/den);
   if (camera.x > 0.0 && camera.y > 0.0)      theta = theta;
   if (camera.x == 0.0 && camera.y > 0.0)   theta = M_PI/2.0;
   if (camera.x < 0.0 && camera.y > 0.0)      theta = (1.0*M_PI - theta);
   if (camera.x == -1.0 && camera.y == 0.0)theta = M_PI;
   if (camera.x < 0.0 && camera.y < 0.0)      theta = (1.0*M_PI + theta);
   if (camera.x == 0.0 && camera.y < 0.0)   theta = 1.5*M_PI;
   if (camera.x > 0.0 && camera.y < 0.0)      theta = (2.0*M_PI - theta);

   //   now rotate the up vector through theta and phi.  After these rotations the up
   //   vector should lie on the XY plane.  Find out how much we need to rotate i
   //   about the global Z axis to align with the top of our screen.  This angle is alpha.

   temp   = ArbitraryRotate(up,-theta,screencamera);
   temp   = ArbitraryRotate(temp,-phi,screenup);

   num = temp.x;
   den = temp.y;
   if (num < 0.0) num = -temp.x;
   if (den < 0.0) den = -temp.y;
   if (den != 0.0) alpha = atan(num/den);
   if (temp.x < 0.0 && temp.y > 0.0)      alpha = alpha;
   if (temp.x < 0.0 && temp.y == 0.0)   alpha = M_PI/2.0;
   if (temp.x < 0.0 && temp.y < 0.0)      alpha = (1.0*M_PI - alpha);
   if (temp.x == 0.0 && temp.y == -1.0)alpha = M_PI;
   if (temp.x > 0.0 && temp.y < 0.0)      alpha = (1.0*M_PI + alpha);
   if (temp.x > 0.0 && temp.y == 0.0)   alpha = 1.5*M_PI;
   if (temp.x > 0.0 && temp.y > 0.0)      alpha = (2.0*M_PI - alpha);

   //   now rotate reverse the rotations to display the triad
   XVect = ArbitraryRotate(XVect,-theta,screencamera);
   YVect = ArbitraryRotate(YVect,-theta,screencamera);
   ZVect = ArbitraryRotate(ZVect,-theta,screencamera);

   XVect = ArbitraryRotate(XVect,-phi,screenup);
   YVect = ArbitraryRotate(YVect,-phi,screenup);
   ZVect = ArbitraryRotate(ZVect,-phi,screenup);

   XVect = ArbitraryRotate(XVect,-alpha,screencamera);
   YVect = ArbitraryRotate(YVect,-alpha,screencamera);
   ZVect = ArbitraryRotate(ZVect,-alpha,screencamera);
}

XYZ LX_3DViewDialog::ArbitraryRotate(const XYZ& p, const double& theta, const XYZ& r)
//   Rotate a point p by angle theta around an arbitrary axis r. Positive angles are
//   CCW looking down the axis towards the origin.  Discussion is from a paper written
//   by Paul Bourke found at http://astronomy.swin.edu.au/pbourke/geometry/rotate.
//   This C routine is attributed to Ronald Goldman, slightly revised for this program
{
   static XYZ q;
   static double costheta;
   static double sintheta;

   costheta = cos(theta);
   sintheta = sin(theta);
   q.x = q.y = q.z = 0.0;

   q.x +=(costheta + (1 - costheta) * r.x * r.x) * p.x;
   q.x +=((1 - costheta) * r.x * r.y - r.z * sintheta) * p.y;
   q.x +=((1 - costheta) * r.x * r.z + r.y * sintheta) * p.z;

   q.y +=((1 - costheta) * r.x * r.y + r.z * sintheta) * p.x;
   q.y +=(costheta + (1 - costheta) * r.y * r.y) * p.y;
   q.y +=((1 - costheta) * r.y * r.z - r.x * sintheta) * p.z;

   q.z +=((1 - costheta) * r.x * r.z - r.y * sintheta) * p.x;
   q.z +=((1 - costheta) * r.y * r.z + r.x * sintheta) * p.y;
   q.z +=(costheta + (1 - costheta) * r.z * r.z) * p.z;

   return (q);
}

//   normalize a vector to 1
XYZ LX_3DViewDialog::normalize(const XYZ& b)
{
   double l2norm;
   XYZ a;

   a = b;
   l2norm = sqrt(a.x*a.x + a.y*a.y + a.z*a.z);

   a.x /= l2norm;
   a.y /= l2norm;
   a.z /= l2norm;

   return (a);
}

//   Slots
void LX_3DViewDialog::resetall()
{
//   XSlider->setValue(0);
//   YSlider->setValue(0);
//   ZSlider->setValue(0);

   extern AD_VIEW_PARMS adviewparms;

   XSlider->setValue(0);
   YSlider->setValue(0);
   ZSlider->setValue(0);
   oldXRotation=0;
   oldYRotation=0;
   oldZRotation=0;

   camera.x = adviewparms.viewdir[0];
   camera.y = adviewparms.viewdir[1];
   camera.z = adviewparms.viewdir[2];
   up.x = adviewparms.up[0];
   up.y = adviewparms.up[1];
   up.z = adviewparms.up[2];

   eraseIt();
   setInitial3DViewParms();
   showIt();
}

void LX_3DViewDialog::cancel() {return reject();}
void LX_3DViewDialog::okpressed() {return accept();}

void LX_3DViewDialog::rotateX(int angle)
{
   static double radangle;

   static const XYZ Xwcs = {1.0,0.0,0.0};
   static XYZ crossprod;

   radangle = (M_PI*(angle-oldXRotation)/180.0);

   eraseIt();
   XVect = ArbitraryRotate(XVect,radangle,Xwcs);
   YVect = ArbitraryRotate(YVect,radangle,Xwcs);
   ZVect = ArbitraryRotate(ZVect,radangle,Xwcs);

   crossprod.x = up.y*camera.z - up.z*camera.y;
   crossprod.y = up.z*camera.x - up.x*camera.z;
   crossprod.z = up.x*camera.y - up.y*camera.x;

   camera = ArbitraryRotate(camera,-radangle,crossprod);
   up = ArbitraryRotate(up,-radangle,crossprod);

   oldXRotation = angle;
   showIt();
}

void LX_3DViewDialog::rotateY(int angle)
{
   static double radangle;
   static const XYZ Ywcs = {0.0,1.0,0.0};

   radangle = (M_PI*(angle-oldYRotation)/180.0);

   eraseIt();
   XVect = ArbitraryRotate(XVect,radangle,Ywcs);
   YVect = ArbitraryRotate(YVect,radangle,Ywcs);
   ZVect = ArbitraryRotate(ZVect,radangle,Ywcs);

   camera = ArbitraryRotate(camera,-radangle,up);
   up = ArbitraryRotate(up,-radangle,up);

   oldYRotation = angle;
   showIt();
}


void LX_3DViewDialog::rotateZ(int angle)
{
   static double radangle;
   static const XYZ Zwcs = {0.0,0.0,1.0};

   radangle = (M_PI*(angle-oldZRotation)/180.0);

   eraseIt();
   XVect = ArbitraryRotate(XVect,radangle,Zwcs);
   YVect = ArbitraryRotate(YVect,radangle,Zwcs);
   ZVect = ArbitraryRotate(ZVect,radangle,Zwcs);

   camera = ArbitraryRotate(camera,-radangle,camera);
   up = ArbitraryRotate(up,-radangle,camera);

   oldZRotation = angle;
   showIt();
}


