/***************************************************************************
                          3Dview.h
                    -------------------
    begin                : Fri Jun 15 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : http://lx-viewer.sourceforge.ne
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

// This is where we define a class to control 3D viewing
#ifndef THREEDVIEW_H
#define THREEDVIEW_H

/* QT includes */
#include <qdialog.h>      // base class of dialog windows
#include <qslider.h>      // provides a vertical or horizontal slider
#include <qlayout.h>      // base class of geometry managers
#include <qlabel.h>       // QLabel widget provides a text or image display
#include <qpainter.h>     // low-level painting e.g. on widgets
#include <qpoint.h>       // defines a point in the plane
#include <qpalette.h>     // contains color groups for each widget state
#include <qfont.h>        // specifies a font used for drawing tex
#include <qpushbutton.h>  // provides a command button
#include <qcolor.h>       // provides colors based on RGB or HSV values

// C++ includes
#include <math.h>

// OD includes
#include "ad2.h"
#include "ad3.h"

typedef struct{
   double x,y,z;
   } XYZ;

class LX_3DViewDialog : public QDialog
{
    Q_OBJECT

   public:
      LX_3DViewDialog(QWidget* parent=0, const char* name=0, WFlags f = 0);
      void showIt();
      void eraseIt();
      void setInitial3DViewParms();
      XYZ ArbitraryRotate(const XYZ& p, const double& theta, const XYZ& r);
      XYZ normalize (const XYZ&);

    XYZ camera;                        // equiv. to adviewparms.viewdir but used locally
      XYZ up;                          // equiv. to adviewparms.up but used locally
      QPushButton*   okbut;            // public because it is linked to DrawingBoard
      double            oldXRotation;  // last slider values X,Y,Z
      double            oldYRotation;
      double            oldZRotation;

   protected:
      QSlider*         XSlider;
      QSlider*         YSlider;
      QSlider*         ZSlider;
      QWidget*         p2pixarea;
      QLabel*          XCoorddisp;
      QLabel*          YCoorddisp;
      QLabel*          ZCoorddisp;
      QPushButton*     resetbut;
      QPushButton*     cancelbut;
      QPainter*        pixpainter;
      XYZ              XVect;       // X axis vector as rotated
      XYZ              YVect;       // Y axis vector as rotated
      XYZ              ZVect;       // Z axis vector as rotated
      QColor           XColor;      // x axis color
      QColor           YColor;      // y axis color
      QColor           ZColor;      // z axis color
    double             axislen;     // length of the axis in pixels
      QPoint           center;      // center of the drawing widget

   // custom event handlers
      void resizeEvent   (QResizeEvent*);
      void showEvent      (QShowEvent*);
      void paintEvent      (QPaintEvent*);

   public slots:
      void resetall();
      void cancel();
      void okpressed();
      void rotateX(int);
      void rotateY(int);
      void rotateZ(int);
};

#endif

