/***************************************************************************
                          OD_interface.cpp
                             -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : http://lx-viewer.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//   implementation of the interface functions for the OpenDWG ToolKit and ViewKit

//   LX includes
#include "OD_interface.h"
#include "drawingboard.h"
#include "colorfunctions.h"
#include "errorhandlers.h"
#include "utilities.h"
#include "serialport.h"
#include "infomessage.h"

//   Initilize and Environment functions

PAD_DWGHDR adhd;
PAD_ENT_HDR adenhd;
PAD_ENT aden;
PAD_TB adtb;
PAD_XD adxd;

//   other global variables
bool lookforxrefs;
bool lookforfonts;
int hpglport = 0;
svgExport* p2svgexport;

//   function to allocate OD toolkit memory pointers
short   allocateadptrs(void)
{
  if ((adhd=(PAD_DWGHDR)malloc(sizeof(AD_DWGHDR)))!=NULL) {
    if ((adenhd=(PAD_ENT_HDR)malloc(sizeof(AD_ENT_HDR)))!=NULL) {
      if ((aden=(PAD_ENT)malloc(sizeof(AD_ENT)))!=NULL) {
        if ((adtb=(PAD_TB)malloc(sizeof(AD_TB)))!=NULL) {
          if ((adxd=(PAD_XD)malloc(sizeof(AD_XD)))!=NULL) {
             return(1);
          }
          free(adtb);
        }
        free(aden);
      }
      free(adenhd);
    }
    free(adhd);

  }
  return(0);
}


//   function to release OD toolkit memory pointers
void freeadptrs(void)
{
  if (!(adxd==0)) free(adxd);
  if (!(adtb==0)) free(adtb);
  if (!(aden==0))  free(aden);
  if (!(adenhd==0)) free(adenhd);
  if (!(adhd==0)) free(adhd);
}

//   function to initilize the OD tool ki
void initializeODtoolkit()
{
   short initerr=0;

   //   register the critical error function.  This must be done prior
   //   to calling adInitAd2
   adSetAd2CriticalErrorFn(LX_CriticalMessage);


//   construct the path to the executable then look for "adinit.dat" in
//   several logical locations below and then in the directory with the executable
//   try to find the location of help manual
   QFileInfo fi = QFileInfo();
   unsigned int i = 0;
   QString initfile = "/adinit.dat";
   QString searcharray[]= {QString(PathToExec() + "/OpenDWG"),
                                       QString(PathToExec() + "/Opendwg"),
                                       QString(PathToExec() + "/opendwg"),
                                       PathToExec(),
                                       "/usr/local/bin/OpenDWG",
                                       "/usr/local/bin/Opendwg",
                                       "/usr/local/bin/opendwg",
                                       "/usr/local/bin",
                                       "/usr/local"
                                       "./OpenDWG"
                                       };
  do
   {
      fi.setFile(searcharray[i++] + initfile);
   } while (!fi.exists() && i < sizeof searcharray / sizeof searcharray[0]);

   if (!adInitAd2 (fi.filePath().latin1(),&initerr))
      {
         QString msg =   (QWidget::tr("<center>OpenDWG Toolkit Initilization Failed</center>"
                              "<center>Error: <b>%1</b></center>"
                              "<center>Move the file ADINIT.DAT into a directory named OpenDWG.</center>"
                              "The OpenDWG directory should be placed below the directory which contains the executable").arg(adErrorStr(initerr)));
         DisplayMessage(CUSTOM_MESSAGE,CAPTION_INITIALIZE,QMessageBox::Critical,msg);
         freeadptrs();
         exit(1);
      }
}

//   function to initilize the OD view ki
void initializeODviewkit()
{
   if (!adInitAd3())
      {
         DisplayMessage(CANNOT_INITIALIZE_AD3,CAPTION_INITIALIZE,QMessageBox::Critical);
         adCloseAd2();
         freeadptrs();
         exit(1);
     }
}

//   Qt to OD link functions - tie OD capabilities to Qt classes
//   function to pass a C string along to the printMessage routine
//   used to catch the output of the adAudit function

short passmessage(char* c)
{
   extern DrawingBoard* db;
   db->printMessage(QString(c),TWIN);
   return 1;
}

//   function to send a short int to the progress bar during file loads -- Toolki
void setprogbarload(short val)

{
   extern DrawingBoard* db;
   db->LX_ProgBarVal(static_cast<int>(val));
   db->printMessage(QWidget::tr(" Loading....."),SBAR);
}

//   function to send a short int to the progress bar during file loads -- Toolki
void setprogbarsave(short val)
{
   extern DrawingBoard* db;
   db->LX_ProgBarVal(static_cast<int>(val));
   db->printMessage(QWidget::tr(" Saving....."),SBAR);
}

//   function to send a short int to the progress bar during dxf scans -- Toolki
void setprogbarscan(short val)
{
   extern DrawingBoard* db;
   db->LX_ProgBarVal(static_cast<int>(val));
   db->printMessage(QWidget::tr(" Scanning DXF....."),SBAR);
}

//   function to find a font file requested by OD toolkit or viewki
void findfonts(char* font, char* pathtofont, void* userdata)
{
  extern DrawingBoard* db;
  userdata=userdata;

	QChar pathchar=QChar('/');
  QChar dospathchar=QChar('\\');
  short lastslash=0;
  const bool acceptabspath=TRUE;
	const short buffsize(256);	//	Maximum size of "pathtofont" variable according to OD documentation
	short sourcelength(0);			//	initialize at 0, will only be changed if a fi.exists() returns true

  QString infont = QString(font);
  QFileInfo fi = QFileInfo();

//   if font contains a LINUX path character, it is probably not going to
//   reside in the same location here, but give it a shot anyway
  if (infont.contains(pathchar)>0)
  {
  	fi.setFile(infont);
    infont = fi.fileName();
    if (fi.exists())
		{
			sourcelength = strlen(fi.filePath().latin1());
			if (sourcelength < buffsize)
			{
				memcpy(pathtofont,fi.filePath().latin1(),sourcelength);
				pathtofont[sourcelength] = '\0';
				return;
			}
		}	//	if file exists
  }

//   if font contains a DOS path character it is almost certain that the file will no
//   exist in the same place on a LINUX system. Don't even bother too look, jus
//   strip off all the path characters and start with the font itself
   if (infont.contains(dospathchar)>0)
   {
      lastslash=infont.findRev(dospathchar);
      infont = infont.remove(0,lastslash+1);
   }

//   At this point in the sieve "infont" can only be the name of the SHX file (no path characters).
//   Start adding variations of "FONT" to the likely path locations to try to find it.
	QString searchpath[]={QDir::homeDirPath() + LXV_HOME, QDir::homeDirPath(),PathToExec(),"/usr/local/bin","/usr/local"};
  QString searcharray[]={LXV_FONT,"/fonts","/Fonts","/FONTS","/font","/Font","/FONT","/shx","/SHX"};

  for (unsigned int j = 0; j < sizeof searchpath / sizeof searchpath[0] ; j++)
  {
  	for (unsigned int i = 0; i <sizeof searcharray / sizeof searcharray[0] ; i++)
    {
    	fi.setFile(searchpath[j]+searcharray[i]+pathchar+infont);
      if (fi.exists())
			{
				sourcelength = strlen(fi.filePath().latin1());
				if (sourcelength < buffsize)
				{
					memcpy(pathtofont,fi.filePath().latin1(),sourcelength);
					pathtofont[sourcelength] = '\0';
					return;
				}
				else break;
			}	//	if fi.exists()				
		}  //   i for loop
  }   //   j for loop

//   now look for it in the executable directory
	fi.setFile(PathToExec() + pathchar + infont);
	if (fi.exists())
	{
		sourcelength = strlen(fi.filePath().latin1());
		if (sourcelength < buffsize)
		{
			memcpy(pathtofont,fi.filePath().latin1(),sourcelength);
			pathtofont[sourcelength] = '\0';
			return;
		}
	}	//	if fi.exists()			

//   finally look in the directory containing the drawing
  fi.setFile(db->getOpenFileName());
  QString pathtodwg = fi.dirPath(acceptabspath);
  fi.setFile(pathtodwg + pathchar + infont);
  if (fi.exists())
  {
		sourcelength = strlen(fi.filePath().latin1());
		if (sourcelength < buffsize)
		{
			memcpy(pathtofont,fi.filePath().latin1(),sourcelength);
			pathtofont[sourcelength] = '\0';
			return;
		}
	}	//	if fi.exists()			

//	We can't find the font anywhere or font found and is the name is too large
//	so substitute the default for it
  pathtofont[0]='\0';
	if (lookforfonts)
	{
		if (sourcelength > buffsize) db->printMessage(QWidget::tr("Font \"%1\" exceeds the limit of %2 characters").arg(QString(font)).arg(QString::number(buffsize)),TWIN);
  	db->printMessage(QWidget::tr("LX-Viewer default font has been substituted for: %1").arg(QString(font)),TWIN);
	}
}


//   function to move the graphic cursor without drawing a line
//   since Y is measured from the top left, convert here
//   and send the point to be plotted to the LX function -- ViewKi
void moveto (long x, long y, void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;
   db->LX_MoveTo(x,db->XY(1)-1-y);
}

//   function to draw a line from the current point to the new poin
//   since Y is measured from the top left, convert here
//   and send the point to be plotted to the LX function -- ViewKi
void lineto(long x, long y, void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;
   db->LX_LineTo(x,db->XY(1)-1-y);
}

//   function to pass a QColor to QPainter.  This function receives an integer representing
//   an AutoCad color.  This routine just receives the integer from OD, sends it to LX_GetColor
//   and receives a QColor in return.  This function then sends the QColor to the db->painter.
void pencolor(short acadcolor, void* userdata)
{
	extern DrawingBoard* db;
	userdata=userdata;
	db->LX_PenColor(LX_GetColor(acadcolor));
}

//	function to write a series of points from the OD point struct.
//	to a QPointArray.  Since Y is measured from the top left of the
//	screen downwards, reverse the Y value here so that QArray has the
//	correct points.  Then send the points to the paintpoly function
//	to display them
void	paintpoly(polyHeadlong* head,pointlong* pt,void* userdata)
{
	extern DrawingBoard* db;
	userdata=userdata;
	QPointArray a(0);
	for (short i=head->polyBgn; i<=head->polyEnd;i++)
		{
			a.putPoints(i,1,pt[i].XC,db->XY(1)-1-pt[i].YC);
		}

	db->LX_PaintPoly(a);
		
}

//	function to set the progress bar for vectorizing operations -- Viewkit
void	odometer(short lastperc, void* userdata)
{
	extern DrawingBoard* db;
	userdata=userdata;
	db->LX_ProgBarVal(static_cast<int>(lastperc));

	db->printMessage(" Drawing.....",SBAR);
}

//	function to test for a user break (press "ESC") during vectorizing.
void vectorfn(void* userdata)
{
	extern DrawingBoard* db;
	userdata=userdata;
	if (db->getCancelPending()) ad3parm.keepvectorizing = 0;
}

//   function that is called whenever an unresolvable external reference is found
void badxref(char* xreffilename,void* userdata)
{
	extern DrawingBoard* db;
  userdata=userdata;

  if (!lookforxrefs) return;

  db->printMessage(QWidget::tr("Warning: Unable to find External Reference file: %1").arg(xreffilename),TWIN);
  DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,
  	QWidget::tr("<center>Unable to find External Reference file: <b>%1</b></center>").arg(xreffilename));
}

//   function to try to find a requested Xref.  Very similar to the findfonts function above
void examinexref(char* oldxref, char* newxref,void* userdata)
{
	extern DrawingBoard* db;
  userdata=userdata;

  QChar pathchar=QChar('/');
  QChar dospathchar=QChar('\\');
  short lastslash=0;
  const bool acceptabspath=TRUE;
	const short buffsize(256);	//	Maximum size of "newxref" variable according to OD documentation
	short sourcelength(strlen(oldxref));	//	initialize at length of old xref name
  QString inxref = QString(oldxref);
  QString qpath = QString(oldxref);

  if (lookforxrefs) db->printMessage(QWidget::tr("Attempting to Find External Reference file: %1").arg(oldxref),TWIN);

  //   if xref path contains a LINUX path character, it is probably not going to
  //   reside in the same location here, but give it a shot anyway
  if (inxref.contains(pathchar)>0)
  {
		QString inpath = QString(oldxref);
    lastslash=inxref.findRev(pathchar);
    inxref = inxref.remove(0,lastslash+1);
    inpath = inpath.remove(lastslash,inpath.length());
    QDir d(inpath);
    if (d.exists(inxref,acceptabspath))
    {
			qpath = QString(oldxref);     // this line and one below are duplicates and not really
			sourcelength = strlen(oldxref);	// needed, but code is easier to follow if they are here			
			if (sourcelength < buffsize)
			{
				memcpy(newxref,oldxref,sourcelength);
				newxref[sourcelength] = '\0';
				return;
			}
		}	//	if d.exists()	
 	}

   //   if inxref contains a DOS path character it is almost certain that the file will no
   //   exist in the same place on a LINUX system. Don't even bother too look, just
   //   strip off all the path characters and start with the drawing itself

  if (inxref.contains(dospathchar)>0)
  {
  	lastslash=inxref.findRev(dospathchar);
    inxref = inxref.remove(0,lastslash+1);
  }

  //   look in the directory containing the drawing
  QString pathtodwg = db->getOpenFileName();
  lastslash=pathtodwg.findRev(QChar(pathchar));
  pathtodwg=pathtodwg.remove(lastslash,pathtodwg.length());
  QDir d1(pathtodwg);
  pathtodwg=d1.absPath();
  if (d1.exists(inxref,acceptabspath))
  {
		qpath=pathtodwg+pathchar+inxref;
		sourcelength = strlen(qpath.latin1());
		if (sourcelength < buffsize)
		{
			memcpy(newxref,qpath.latin1(),sourcelength);
			newxref[sourcelength] = '\0';
			return;
		}
  }

//  can't find it anywhere or the name is too big (probably because of a drawing located well
//	down in a directory tree).
//	Let function BadXref send a message about it
	if (sourcelength >= buffsize)
	{
		db->printMessage(QWidget::tr("XRef \"%1\" exceeds the limit of %2 characters").arg(qpath).arg(QString::number(buffsize-1)),TWIN);
		memcpy(newxref,qpath.latin1(),buffsize-1);
	  newxref[buffsize-1] = '\0';	//	truncate the xref name at the limit of array
	}
	else
	{
		memcpy(newxref,qpath.latin1(),sourcelength);
		newxref[sourcelength] = '\0';
	}	
}

//   function that will print a message when an xref has been completely drawn
void xrefcomplete(AD_DB_HANDLE xhandle, void* userdata)
{
   (void)xhandle;
   extern DrawingBoard* db;
   userdata=userdata;

   if (lookforxrefs) db->printMessage(QWidget::tr("External reference file successfully read"),TWIN);
}

//   function to notify you that OD is about to open an external reference
void xrefopennotify(char* xrefname,PAD_XREFOPENNOTIFY_PARMS, void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;

   if (lookforxrefs) db->printMessage(QWidget::tr("Attempting to Open External Reference file: %1").arg(xrefname),TWIN);
}

//   function that could report the handle under which the xref was opened.
//   Not really implemented in VIEWER, it is only here for a place holder in
//   case we want it in the future.
void   xrefhandleis(AD_DB_HANDLE xhandle, void* userdata)
{
   (void)xhandle;
   userdata=userdata;
}

//   function to register various functions(defined above and elsewhere) with
//   OD toolkit and OD Viewki
void registerfuns()
{
   //   Tool kit functions
   adSetAd2LoadometerFn(setprogbarload);
   adSetAd2SaveometerFn(setprogbarsave);
   adSetAd2DxfscanometerFn(setprogbarscan);
   adSetAd2ExamineshapefileFn(findfonts);
   adSetAd2WarningFn(LX_WarningMessage);

   //   View kit functions
   adSetAd3MoveToFn(moveto);
   adSetAd3LineToFn(lineto);
   adSetAd3PenColorFn(pencolor);
   adSetAd3PaintPolyFn(paintpoly);
   adSetAd3OdometerFn(odometer);
   adSetAd3VectorFn(vectorfn);
  adSetAd3BadxrefFn(badxref);
   adSetAd3ExaminexrefFn(examinexref);
   adSetAd3XrefcompleteFn(xrefcomplete);
   adSetAd3XrefopennotifyFn(xrefopennotify);
   adSetAd3XrefhandleisFn(xrefhandleis);
}

///////////////////////////////////////////////////////////////////////////
//         Other plotting and saveing interface functions
///////////////////////////////////////////////////////////////////////////
//   function to register special plot functions
//   after plotting redefine these using registerfuns()
//   OD toolkit and OD Viewki
void setplotfuns()
{
   //   View kit functions
   adSetAd3MoveToFn(plotmoveto);
   adSetAd3LineToFn(plotlineto);
   adSetAd3PenColorFn(plotpencolor);
   adSetAd3PaintPolyFn(plotpaintpoly);
}

void sethpglfuns()
{
   //   View kit functions
   adSetAd3MoveToFn(hpglmoveto);
   adSetAd3LineToFn(hpgllineto);
   adSetAd3PenColorFn(hpglpencolor);
   adSetAd3PaintPolyFn(hpglpaintpoly);
}

void setsvgfuns(svgExport* s)
{
   p2svgexport = s;   //   store pointer to the svg class for use by functions below
   //   View kit functions
   adSetAd3MoveToFn(svgmoveto);
   adSetAd3LineToFn(svglineto);
   adSetAd3PenColorFn(svgpencolor);
   adSetAd3PaintPolyFn(svgpaintpoly);
}


void plotmoveto (long x, long y, void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;

   int margin = static_cast<int>(db->getmargin());
   db->LX_PlotMoveTo(x+margin,static_cast<int>(db->getoutputheight())+margin-y);
}

void plotlineto(long x, long y, void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;

  int margin = static_cast<int>(db->getmargin());
   db->LX_PlotLineTo(x+margin,static_cast<int>(db->getoutputheight())+margin-y);
}

void plotpencolor(short acadcolor, void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;
   db->LX_PlotPenColor(acadcolor);
}

void   plotpaintpoly(polyHeadlong* head,pointlong* pt,void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;
   QPointArray a(0);

   int margin = static_cast<int>(db->getmargin());

   for (short i=head->polyBgn; i<=head->polyEnd;i++)
      {
         a.putPoints(i,1,pt[i].XC+margin,static_cast<int>(db->getoutputheight())+margin-pt[i].YC);
      }

   db->LX_PlotPaintPoly(a);

}


//   Functions for interfacing with the plotter device in hpgl
//   basically the same as the functions above for drawing on the screen
//   except these are the final functions
void hpglmoveto (long x, long y, void* userdata)
{
   const int base = 10;
   const QChar comma(',');
   const QChar semicolon(';');
   extern DrawingBoard* db;
   userdata=userdata;

   int margin = static_cast<int>(db->getmargin());

   QString temp("");
   temp = "PA" + QString::number(x+margin,base) + comma + QString::number(y+margin,base) + semicolon;
   writeToPort(hpglport,const_cast<char*>(temp.latin1()));
}

void hpgllineto(long x, long y, void* userdata)
{
   const int base = 10;
   const QChar comma(',');
   const QChar semicolon(';');
   extern DrawingBoard* db;
   userdata=userdata;

  int margin = static_cast<int>(db->getmargin());

   QString temp = "PA" + QString::number(x+margin,base) + comma + QString::number(y+margin,base) + semicolon;
   writeToPort(hpglport,"PD;");                  //   put pen down
   writeToPort(hpglport,const_cast<char*>(temp.latin1()));      //   move the pen to coord
   writeToPort(hpglport,"PU;");                  //   lift up the pen
}

void hpglpencolor(short ac, void* userdata)
{
   const int base = 10;
   const QChar semicolon(';');
   extern DrawingBoard* db;
   userdata=userdata;
   short pennum;
   pennum = db->getpennumber(ac);

   QString temp = QString("SP") + QString::number(pennum,base) + semicolon;
   writeToPort(hpglport,const_cast<char*>(temp.latin1()));
}

void   hpglpaintpoly(polyHeadlong* head,pointlong* pt,void* userdata)
{
   const int base = 10;
   const QChar comma(',');
   const QChar semicolon(';');
   extern DrawingBoard* db;
   userdata=userdata;

   int margin = static_cast<int>(db->getmargin());
   writeToPort(hpglport,"PD;");                  //   pen down
   for (short i=head->polyBgn; i<=head->polyEnd;i++)
      {
         QString temp = "PA" + QString::number(pt[i].XC+margin,base) + comma + QString::number(pt[i].YC+margin,base) + semicolon;
         writeToPort(hpglport,const_cast<char*>(temp.latin1()));
      }
   writeToPort(hpglport,"PU;");                  //   lift up the pen
}


//   Functions for writing the file to svg forma
//   basically the same as the functions as above for drawing on the screen

void svgmoveto (long x, long y, void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;
   p2svgexport->moveTo(x,db->XY(1)-1-y);
}

void svglineto(long x, long y, void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;
   p2svgexport->lineTo(x,db->XY(1)-1-y);
}

void svgpencolor(short acadcolor, void* userdata)
{
   userdata=userdata;
   p2svgexport->setColor(LX_GetColor(acadcolor));
}

void   svgpaintpoly(polyHeadlong* head,pointlong* pt,void* userdata)
{
   extern DrawingBoard* db;
   userdata=userdata;
   QPointArray a(0);

   for (short i=head->polyBgn; i<=head->polyEnd;i++)
         a.putPoints(i,1,pt[i].XC,db->XY(1)-1-pt[i].YC);
   p2svgexport->paintPoly(a);
}

