/***************************************************************************
                          OD_interface.h
                        -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : http://lx-viewer.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

// Functions required to interface with OpenDwg Toolkit and ViewKi
#ifndef OD_INTERFACE_H
#define OD_INTERFACE_H

// C++ includes
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <stdio.h>
#include <fcntl.h>

// OpenDwg includes
#define AD_PROTOTYPES
#include "ad2.h"
#include "ad3.h"

// Qt includes
#include <qstring.h>      // abstraction of unicode text
#include <qmessagebox.h>  // provides a modal dialog with a short message
#include <qpointarray.h>  // provides an array of points
#include <qdir.h>         // provides access to directory strucures
#include <qfileinfo.h>    // system independant file information

//   LX Includes
#include "svgexport.h"

//   Initilize and Environment functions
short  allocateadptrs(void);       // function to allocate OD toolkit memory pointers
void   freeadptrs(void);           // function to release OD toolkit memory pointers
void   initializeODtoolkit(void);  // function that will initilize the OD tool ki
void   initializeODviewkit(void);  // function that will initilize the OD view ki

/* Qt to OD link functions - tie OD capabilities to Qt classes
These are all callback functions.  Examineentity is not implemented in this program.
It will need to be if this is ever extended to be a CAD program.  That is the
function that needs to keep track of screen vectors and link them to drawing entities
vectorfn and xrefhandleis are only implemented as place holders.  xrefopennotify is only
partially implemented (the drawing handles are ignored by us because we don't need them)*/

short  passmessage(char* c);              // function to pass a C string along to the printmessage routine
void   setprogbarload(short);             // function to send a short int to the progress bar
void   setprogbarsave(short);             // function to send a short int to the progress bar
void   setprogbarscan(short);             // function to send a short int to the progress bar
void   findfonts(char*, char*, void*);    // function to find a font requested by OD tool ki
void   moveto(long, long, void*);         // function to move the cursor without drawing a line
void   lineto(long, long, void*);         // function to draw a line from one point to another
void   pencolor(short, void*);            // function to select the pen color
void   paintpoly(polyHeadlong*, pointlong*, void*);   // function to draw a polygon through points
void   odometer(short, void*);            // vectorizing Odometer
void   vectorfn(void*);                   // function to test for a user break during vectorizing
void   badxref(char*, void*);             // called whenever an unresovable external reference is found
void   examinexref(char*, char*, void*);  // function to find a requested Xref
// void   examineentity(void(*adExamineentity)(PAD_ENT_HDR, PAD_ENT,void*));
void   xrefcomplete(AD_DB_HANDLE, void*);
void   xrefopennotify(char*,PAD_XREFOPENNOTIFY_PARMS, void*);
void   xrefhandleis(AD_DB_HANDLE, void*);
void   registerfuns(void);      // function to register various functions with OD
void    setplotfuns(void);      // function to register the special ps plot functions
void    sethpglfuns(void);      // function to register the special hpgl plot functions
void   setsvgfuns(svgExport*);  // function to register the special svg write functions

// Interface functions to use with plotting postscrip
void   plotmoveto(long, long, void*);
void   plotlineto(long, long, void*);
void   plotpencolor(short, void*);
void   plotpaintpoly(polyHeadlong*, pointlong*, void* );

// Interface functions to use with plotting hpgl
void   hpglmoveto(long, long, void*);
void   hpgllineto(long, long, void*);
void   hpglpencolor(short, void*);
void   hpglpaintpoly(polyHeadlong*, pointlong*, void* );

// Interface functions to use for writing an svg file
void   svgmoveto(long, long, void*);
void   svglineto(long, long, void*);
void   svgpencolor(short, void*);
void   svgpaintpoly(polyHeadlong*, pointlong*, void* );

#endif

