/***************************************************************************
                          colorfunctions.cpp
                             -------------------
    begin                : Fri May 4 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : http://lx-viewer.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//	Functions dealing with display colors, color maps, etc are here

#include "colorfunctions.h"

////////////////////////////////////////////////////////////////////////////////
//					custom color button derived from QPushbutton
///////////////////////////////////////////////////////////////////////////////

//	constructor for the custom pushbutton to select a color
//	construct the buttons with no border (setflat) and with a color defined
//	by the short int sent to the constructor as the first argument
ColorButton::ColorButton(short clky,QWidget* parent,const char* name):QPushButton(parent,name),colorkey(clky)
{
	setFlat(TRUE);						
	setPalette(LX_GetColor(colorkey));
	connect(this,SIGNAL(clicked()),this,SLOT(ButtonClicked()));
}

//	redefine the clicked member function
void ColorButton::ButtonClicked()
{
	emit ColorSelected(colorkey);
}

/////////////////////////////////////////////////////////////////
//							Color Dialog box
/////////////////////////////////////////////////////////////////



//	constructor for the color dialog box
//	The return on OK is an integer representing the choosen color
LX_ColorDialog::LX_ColorDialog(int mode,QWidget* parent,const char* name, bool modal, WFlags f):QDialog(parent,name,modal,f)
{
	const int commandbuttonwidth = 80;
	const int commandbuttonheight = 30;
	const int largecolorbutton = 38;
	const int smallcolorbutton = 19;
	const int border = 12;
	const int gap = 3;

	const int initialcolor = 256; // black
	int x = 0;   //	from upperleft positive to the right
	int y = 0;   //	from upperleft positive downward
	int minspin = 0;  //	minimum value for color spin box
	int maxspin = 0;//	maximum value for color spin box

//	decide which of the special color buttons will be used
	switch (mode)
	{
		case COLOR_DIAG_FOR_PLOTTER:
			minspin = 1;
			maxspin = 256;
			ByBlock = new QPushButton(tr("By Block"),this);
			ByLayer = new QPushButton(this);
			ByBlock->setDisabled(TRUE);
			ByLayer->setPalette(LX_GetColor(256));
			connect(ByLayer,SIGNAL(clicked()),this,SLOT(Color256Picked()));
			break;
		case COLOR_DIAG_FOR_LAYER:
			minspin = 1;
			maxspin = 255;
			ByBlock = new QPushButton(tr("By Block"),this);
			ByLayer = new QPushButton(tr("By Layer"),this);
			ByBlock->setDisabled(TRUE);
			ByLayer->setDisabled(TRUE);
			break;
		default:	//(same as COLOR_DIAG_FOR_ENTITY)
			minspin = 0;
			maxspin = 256;
			ByBlock = new QPushButton(tr("By Block"),this);
			ByLayer = new QPushButton(tr("By Layer"),this);
		//		need to create slots here
			break;
	}
	x = border;
	y = border;
	ByBlock->setGeometry(x,y,commandbuttonwidth,commandbuttonheight);
	y += (commandbuttonheight + gap);
	ByLayer->setGeometry(x,y,commandbuttonwidth,commandbuttonheight);
	

//	Define the command buttons
	x = border*2 + smallcolorbutton*24 + gap*23 - commandbuttonwidth*5 - gap*4 - border;
	y = border*3 + largecolorbutton + gap*11 + smallcolorbutton*11;

	colorlabel = new QLabel(this);
	colorlabel->setGeometry(x,y,commandbuttonwidth,commandbuttonheight);
	colorlabel->setFrameStyle(QFrame::Box | QFrame::Plain);
	colorlabel->setLineWidth(1);
	x += (commandbuttonwidth + gap);

	colorspin = new QSpinBox(minspin,maxspin,1,this);
	colorspin->setGeometry(x,y,commandbuttonwidth,commandbuttonheight);
	colorspin->setWrapping(TRUE);
	connect(colorspin,SIGNAL(valueChanged(int)),this,SLOT(ReceiveBoxSpin(int)));	
	x += (commandbuttonwidth + gap);

	OKButton = new QPushButton(tr("OK"),this);
	OKButton->setGeometry(x,y,commandbuttonwidth,commandbuttonheight);
	connect(OKButton,SIGNAL(clicked()),this,SLOT(FoundColor()));
	x += (commandbuttonwidth + gap);

	CancelButton = new QPushButton(tr("Cancel"),this);
	CancelButton->setGeometry(x,y,commandbuttonwidth,commandbuttonheight);
	connect(CancelButton,SIGNAL(clicked()),this,SLOT(ColorNotPicked()));
	x += (commandbuttonwidth + gap);

	HelpButton = new QPushButton(tr("Help"),this);
	HelpButton->setGeometry(x,y,commandbuttonwidth,commandbuttonheight);
	x += (commandbuttonwidth + gap);
	HelpButton->setDisabled(TRUE);


//	define the color buttons
	x = border*2 + smallcolorbutton*24 + gap*23 - largecolorbutton*9 - gap*8 - border;
	y = border;	

	int i;
	for (i = 1; i <= 9; i++)
	{
		cbset[i] = new ColorButton(i,this);
		cbset[i]->setGeometry(x,y,largecolorbutton,largecolorbutton);
		cbset[i]->setFlat(FALSE);
		connect(cbset[i],SIGNAL(ColorSelected(int)),this,SLOT(ReceiveClick(int)));
		x += (largecolorbutton + gap);
	}

  x = border*2 + smallcolorbutton*24 + gap*23 - smallcolorbutton*6 - gap*5 - border;
	y += largecolorbutton + gap*2;

	for (i = 250; i <=255; i++)
	{
		cbset[i] = new ColorButton(i,this);
		cbset[i]->setGeometry(x,y,smallcolorbutton,smallcolorbutton);
		cbset[i]->setFlat(FALSE);
		connect(cbset[i],SIGNAL(ColorSelected(int)),this,SLOT(ReceiveClick(int)));
		x += (smallcolorbutton + gap);
	}

	x = border;
	y += smallcolorbutton + border;

	int j;
	for (j = 19; j >= 11; j-= 2)
	{
		for (int i = j; i <=j+230; i+= 10)
		{
			cbset[i] = new ColorButton(i,this);
			cbset[i]->setGeometry(x,y,smallcolorbutton,smallcolorbutton);
			connect(cbset[i],SIGNAL(ColorSelected(int)),this,SLOT(ReceiveClick(int)));
			x += (smallcolorbutton + gap);
		}
		x = border;
		y += smallcolorbutton + gap;
	}

	for (j = 10; j <= 18; j+= 2)
	{
		for (int i = j; i <=j+230; i+= 10)
		{
			cbset[i] = new ColorButton(i,this);
			cbset[i]->setGeometry(x,y,smallcolorbutton,smallcolorbutton);
			connect(cbset[i],SIGNAL(ColorSelected(int)),this,SLOT(ReceiveClick(int)));
			x += (smallcolorbutton + gap);
		}
		x = border;
		y += smallcolorbutton + gap;
	}
	
//	Finally set some parameters for the dialog widget
	setCaption(tr("Color"));
	x = border*2 + smallcolorbutton*24 + gap*23;
	y = border*4 + largecolorbutton + gap*11 + smallcolorbutton*11 + commandbuttonheight;
	setMinimumSize(x,y);
	setMaximumSize(x,y);
	ReceiveClick(initialcolor);	
}

//	slots to change the spinbox and label based on color picked
//	receiveclick changes the spinbox which then changes the label color
void LX_ColorDialog::ReceiveClick(int colorkey){colorspin->setValue(colorkey);}
void LX_ColorDialog::ReceiveBoxSpin(int colorkey){colorlabel->setPalette(LX_GetColor(colorkey));}
//	slot to return the color picked
void LX_ColorDialog::FoundColor(){done(colorspin->value());}
void LX_ColorDialog::Color256Picked(){ReceiveClick(256);}
void LX_ColorDialog::ColorNotPicked(){done(-1);}


/////////////////////////////////////////////////////////////////
//					Other Color Functions
////////////////////////////////////////////////////////////////


//	function to return a Qcolor from a acadcolor sent as an argument
QColor LX_GetColor(short acadcolor)
{
	
static const QColor colormap[257]=
	{
		QColor (0x00, 0x00, 0x00),     // color 0	Black, also By Block
		QColor (0xff, 0x00, 0x00),     // color 1	Red
		QColor (0xff, 0xff, 0x00),     // color 2	Yellow
		QColor (0x00, 0xff, 0x00),     // color 3	Green
		QColor (0x00, 0xff, 0xff),     // color 4 Cyan
		QColor (0x00, 0x00, 0xff),     // color 5	Blue
		QColor (0xff, 0x00, 0xff),     // color 6 Magenta
		QColor (0xff, 0xff, 0xff),     // color 7	White
		QColor (0x98, 0x98, 0x98),     // color 8	Grey
		QColor (0xC0, 0xC0, 0xC0),     // color 9
		QColor (0xff, 0x00, 0x00),     // color 10
		QColor (0xff, 0x80, 0x80),     // color 11
		QColor (0xa6, 0x00, 0x00),     // color 12
		QColor (0xa6, 0x53, 0x53),     // color 13
		QColor (0x80, 0x00, 0x00),     // color 14
		QColor (0x80, 0x40, 0x40),     // color 15
		QColor (0x4c, 0x00, 0x00),     // color 16
		QColor (0x4c, 0x26, 0x26),     // color 17
		QColor (0x26, 0x00, 0x00),     // color 18
		QColor (0x26, 0x13, 0x13),     // color 19
		QColor (0xff, 0x40, 0x00),     // color 20
		QColor (0xff, 0x9f, 0x80),     // color 21
		QColor (0xa6, 0x29, 0x00),     // color 22
		QColor (0xa6, 0x68, 0x53),     // color 23
		QColor (0x80, 0x20, 0x00),     // color 24
		QColor (0x80, 0x50, 0x40),     // color 25
		QColor (0x4c, 0x13, 0x00),     // color 26
		QColor (0x4c, 0x30, 0x26),     // color 27
		QColor (0x26, 0x0a, 0x00),     // color 28
		QColor (0x26, 0x18, 0x13),     // color 29
		QColor (0xff, 0x80, 0x00),     // color 30
		QColor (0xff, 0xbf, 0x80),     // color 31
		QColor (0xa6, 0x53, 0x00),     // color 32
		QColor (0xa6, 0x7c, 0x53),     // color 33
		QColor (0x80, 0x40, 0x00),     // color 34
		QColor (0x80, 0x60, 0x40),     // color 35
		QColor (0x4c, 0x26, 0x00),     // color 36
		QColor (0x4c, 0x39, 0x26),     // color 37
		QColor (0x26, 0x13, 0x00),     // color 38
		QColor (0x26, 0x1d, 0x13),     // color 39
		QColor (0xff, 0xbf, 0x00),     // color 40
		QColor (0xff, 0xdf, 0x80),     // color 41
		QColor (0xa6, 0x7c, 0x00),     // color 42
		QColor (0xa6, 0x91, 0x53),     // color 43
		QColor (0x80, 0x60, 0x00),     // color 44
		QColor (0x80, 0x70, 0x40),     // color 45
		QColor (0x4c, 0x39, 0x00),     // color 46
		QColor (0x4c, 0x43, 0x26),     // color 47
		QColor (0x26, 0x1d, 0x00),     // color 48
		QColor (0x26, 0x21, 0x13),     // color 49
		QColor (0xff, 0xff, 0x00),     // color 50
		QColor (0xff, 0xff, 0x80),     // color 51
		QColor (0xa6, 0xa6, 0x00),     // color 52
		QColor (0xa6, 0xa6, 0x53),     // color 53
		QColor (0x80, 0x80, 0x00),     // color 54
		QColor (0x80, 0x80, 0x40),     // color 55
		QColor (0x4c, 0x4c, 0x00),     // color 56
		QColor (0x4c, 0x4c, 0x26),     // color 57
		QColor (0x26, 0x26, 0x00),     // color 58
		QColor (0x26, 0x26, 0x13),     // color 59
		QColor (0xbf, 0xff, 0x00),     // color 60
		QColor (0xdf, 0xff, 0x80),     // color 61
		QColor (0x7c, 0xa6, 0x00),     // color 62
		QColor (0x91, 0xa6, 0x53),     // color 63
		QColor (0x60, 0x80, 0x00),     // color 64
		QColor (0x70, 0x80, 0x40),     // color 65
		QColor (0x39, 0x4c, 0x00),     // color 66
		QColor (0x43, 0x4c, 0x26),     // color 67
		QColor (0x1d, 0x26, 0x00),     // color 68
		QColor (0x21, 0x26, 0x13),     // color 69
		QColor (0x80, 0xff, 0x00),     // color 70
		QColor (0xbf, 0xff, 0x80),     // color 71
		QColor (0x53, 0xa6, 0x00),     // color 72
		QColor (0x7c, 0xa6, 0x53),     // color 73
		QColor (0x40, 0x80, 0x00),     // color 74
		QColor (0x60, 0x80, 0x40),     // color 75
		QColor (0x26, 0x4c, 0x00),     // color 76
		QColor (0x39, 0x4c, 0x26),     // color 77
		QColor (0x13, 0x26, 0x00),     // color 78
		QColor (0x1d, 0x26, 0x13),     // color 79
		QColor (0x40, 0xff, 0x00),     // color 80
		QColor (0x9f, 0xff, 0x80),     // color 81
		QColor (0x29, 0xa6, 0x00),     // color 82
		QColor (0x68, 0xa6, 0x53),     // color 83
		QColor (0x20, 0x80, 0x00),     // color 84
		QColor (0x50, 0x80, 0x40),     // color 85
		QColor (0x13, 0x4c, 0x00),     // color 86
		QColor (0x30, 0x4c, 0x26),     // color 87
		QColor (0x0a, 0x26, 0x00),     // color 88
		QColor (0x18, 0x26, 0x13),     // color 89
		QColor (0x00, 0xff, 0x00),     // color 90
		QColor (0x80, 0xff, 0x80),     // color 91
		QColor (0x00, 0xa6, 0x00),     // color 92
		QColor (0x53, 0xa6, 0x53),     // color 93
		QColor (0x00, 0x80, 0x00),     // color 94
		QColor (0x40, 0x80, 0x40),     // color 95
		QColor (0x00, 0x4c, 0x00),     // color 96
		QColor (0x26, 0x4c, 0x26),     // color 97
		QColor (0x00, 0x26, 0x00),     // color 98
		QColor (0x13, 0x26, 0x13),     // color 99
		QColor (0x00, 0xff, 0x40),     // color 100
		QColor (0x80, 0xff, 0x9f),     // color 101
		QColor (0x00, 0xa6, 0x29),     // color 102
		QColor (0x53, 0xa6, 0x68),     // color 103
		QColor (0x00, 0x80, 0x20),     // color 104
		QColor (0x40, 0x80, 0x50),     // color 105
		QColor (0x00, 0x4c, 0x13),     // color 106
		QColor (0x26, 0x4c, 0x30),     // color 107
		QColor (0x00, 0x26, 0x0a),     // color 108
		QColor (0x13, 0x26, 0x18),     // color 109
		QColor (0x00, 0xff, 0x80),     // color 110
		QColor (0x80, 0xff, 0xbf),     // color 111
		QColor (0x00, 0xa6, 0x53),     // color 112
		QColor (0x53, 0xa6, 0x7c),     // color 113
		QColor (0x00, 0x80, 0x40),     // color 114
		QColor (0x40, 0x80, 0x60),     // color 115
		QColor (0x00, 0x4c, 0x26),     // color 116
		QColor (0x26, 0x4c, 0x39),     // color 117
		QColor (0x00, 0x26, 0x13),     // color 118
		QColor (0x13, 0x26, 0x1d),     // color 119
		QColor (0x00, 0xff, 0xbf),     // color 120
		QColor (0x80, 0xff, 0xdf),     // color 121
		QColor (0x00, 0xa6, 0x7c),     // color 122
		QColor (0x53, 0xa6, 0x91),     // color 123
		QColor (0x00, 0x80, 0x60),     // color 124
		QColor (0x40, 0x80, 0x70),     // color 125
		QColor (0x00, 0x4c, 0x39),     // color 126
		QColor (0x26, 0x4c, 0x43),     // color 127
		QColor (0x00, 0x26, 0x1d),     // color 128
		QColor (0x13, 0x26, 0x21),     // color 129
		QColor (0x00, 0xff, 0xff),     // color 130
		QColor (0x80, 0xff, 0xff),     // color 131
		QColor (0x00, 0xa6, 0xa6),     // color 132
		QColor (0x53, 0xa6, 0xa6),     // color 133
		QColor (0x00, 0x80, 0x80),     // color 134
		QColor (0x40, 0x80, 0x80),     // color 135
		QColor (0x00, 0x4c, 0x4c),     // color 136
		QColor (0x26, 0x4c, 0x4c),     // color 137
		QColor (0x00, 0x26, 0x26),     // color 138
		QColor (0x13, 0x26, 0x26),     // color 139
		QColor (0x00, 0xbf, 0xff),     // color 140
		QColor (0x80, 0xdf, 0xff),     // color 141
		QColor (0x00, 0x7c, 0xa6),     // color 142
		QColor (0x53, 0x91, 0xa6),     // color 143
		QColor (0x00, 0x60, 0x80),     // color 144
		QColor (0x40, 0x70, 0x80),     // color 145
		QColor (0x00, 0x39, 0x4c),     // color 146
		QColor (0x26, 0x43, 0x4c),     // color 147
		QColor (0x00, 0x1d, 0x26),     // color 148
		QColor (0x13, 0x21, 0x26),     // color 149
		QColor (0x00, 0x80, 0xff),     // color 150
		QColor (0x80, 0xbf, 0xff),     // color 151
		QColor (0x00, 0x53, 0xa6),     // color 152
		QColor (0x53, 0x7c, 0xa6),     // color 153
		QColor (0x00, 0x40, 0x80),     // color 154
		QColor (0x40, 0x60, 0x80),     // color 155
		QColor (0x00, 0x26, 0x4c),     // color 156
		QColor (0x26, 0x39, 0x4c),     // color 157
		QColor (0x00, 0x13, 0x26),     // color 158
		QColor (0x13, 0x1d, 0x26),     // color 159
		QColor (0x00, 0x40, 0xff),     // color 160
		QColor (0x80, 0x9f, 0xff),     // color 161
		QColor (0x00, 0x29, 0xa6),     // color 162
		QColor (0x53, 0x68, 0xa6),     // color 163
		QColor (0x00, 0x20, 0x80),     // color 164
		QColor (0x40, 0x50, 0x80),     // color 165
		QColor (0x00, 0x13, 0x4c),     // color 166
		QColor (0x26, 0x30, 0x4c),     // color 167
		QColor (0x00, 0x0a, 0x26),     // color 168
		QColor (0x13, 0x18, 0x26),     // color 169
		QColor (0x00, 0x00, 0xff),     // color 170
		QColor (0x80, 0x80, 0xff),     // color 171
		QColor (0x00, 0x00, 0xa6),     // color 172
		QColor (0x53, 0x53, 0xa6),     // color 173
		QColor (0x00, 0x00, 0x80),     // color 174
		QColor (0x40, 0x40, 0x80),     // color 175
		QColor (0x00, 0x00, 0x4c),     // color 176
		QColor (0x26, 0x26, 0x4c),     // color 177
		QColor (0x00, 0x00, 0x26),     // color 178
		QColor (0x13, 0x13, 0x26),     // color 179
		QColor (0x40, 0x00, 0xff),     // color 180
		QColor (0x9f, 0x80, 0xff),     // color 181
		QColor (0x29, 0x00, 0xa6),     // color 182
		QColor (0x68, 0x53, 0xa6),     // color 183
		QColor (0x20, 0x00, 0x80),     // color 184
		QColor (0x50, 0x40, 0x80),     // color 185
		QColor (0x13, 0x00, 0x4c),     // color 186
		QColor (0x30, 0x26, 0x4c),     // color 187
		QColor (0x0a, 0x00, 0x26),     // color 188
		QColor (0x18, 0x13, 0x26),     // color 189
		QColor (0x80, 0x00, 0xff),     // color 190
		QColor (0xbf, 0x80, 0xff),     // color 191
		QColor (0x53, 0x00, 0xa6),     // color 192
		QColor (0x7c, 0x53, 0xa6),     // color 193
		QColor (0x40, 0x00, 0x80),     // color 194
		QColor (0x60, 0x40, 0x80),     // color 195
		QColor (0x26, 0x00, 0x4c),     // color 196
		QColor (0x39, 0x26, 0x4c),     // color 197
		QColor (0x13, 0x00, 0x26),     // color 198
		QColor (0x1d, 0x13, 0x26),     // color 199
		QColor (0xbf, 0x00, 0xff),     // color 200
		QColor (0xdf, 0x80, 0xff),     // color 201
		QColor (0x7c, 0x00, 0xa6),     // color 202
		QColor (0x91, 0x53, 0xa6),     // color 203
		QColor (0x60, 0x00, 0x80),     // color 204
		QColor (0x70, 0x40, 0x80),     // color 205
		QColor (0x39, 0x00, 0x4c),     // color 206
		QColor (0x43, 0x26, 0x4c),     // color 207
		QColor (0x1d, 0x00, 0x26),     // color 208
		QColor (0x21, 0x13, 0x26),     // color 209
		QColor (0xff, 0x00, 0xff),     // color 210
		QColor (0xff, 0x80, 0xff),     // color 211
		QColor (0xa6, 0x00, 0xa6),     // color 212
		QColor (0xa6, 0x53, 0xa6),     // color 213
		QColor (0x80, 0x00, 0x80),     // color 214
		QColor (0x80, 0x40, 0x80),     // color 215
		QColor (0x4c, 0x00, 0x4c),     // color 216
		QColor (0x4c, 0x26, 0x4c),     // color 217
		QColor (0x26, 0x00, 0x26),     // color 218
		QColor (0x26, 0x13, 0x26),     // color 219
		QColor (0xff, 0x00, 0xbf),     // color 220
		QColor (0xff, 0x80, 0xdf),     // color 221
		QColor (0xa6, 0x00, 0x7c),     // color 222
		QColor (0xa6, 0x53, 0x91),     // color 223
		QColor (0x80, 0x00, 0x60),     // color 224
		QColor (0x80, 0x40, 0x70),     // color 225
		QColor (0x4c, 0x00, 0x39),     // color 226
		QColor (0x4c, 0x26, 0x43),     // color 227
		QColor (0x26, 0x00, 0x1d),     // color 228
		QColor (0x26, 0x13, 0x21),     // color 229
		QColor (0xff, 0x00, 0x80),     // color 230
		QColor (0xff, 0x80, 0xbf),     // color 231
		QColor (0xa6, 0x00, 0x53),     // color 232
		QColor (0xa6, 0x53, 0x7c),     // color 233
		QColor (0x80, 0x00, 0x40),     // color 234
		QColor (0x80, 0x40, 0x60),     // color 235
		QColor (0x4c, 0x00, 0x26),     // color 236
		QColor (0x4c, 0x26, 0x39),     // color 237
		QColor (0x26, 0x00, 0x13),     // color 238
		QColor (0x26, 0x13, 0x1d),     // color 239
		QColor (0xff, 0x00, 0x40),     // color 240
		QColor (0xff, 0x80, 0x9f),     // color 241
		QColor (0xa6, 0x00, 0x29),     // color 242
		QColor (0xa6, 0x53, 0x68),     // color 243
		QColor (0x80, 0x00, 0x20),     // color 244
		QColor (0x80, 0x40, 0x50),     // color 245
		QColor (0x4c, 0x00, 0x13),     // color 246
		QColor (0x4c, 0x26, 0x30),     // color 247
		QColor (0x26, 0x00, 0x0a),     // color 248
		QColor (0x26, 0x13, 0x18),     // color 249
		QColor (0x54, 0x54, 0x54),     // color 250
		QColor (0x76, 0x76, 0x76),     // color 251
		QColor (0x98, 0x98, 0x98),     // color 252
		QColor (0xbb, 0xbb, 0xbb),     // color 253
		QColor (0xdd, 0xdd, 0xdd),     // color 254
		QColor (0xff, 0xff, 0xff),		 // color 255
		QColor (0x00, 0x00, 0x00) 		 // color 256 Black, also by By Layer	
	};

	return colormap[acadcolor];
}

