/***************************************************************************
                          colorfunctions.h
                             -------------------
    begin                : Fri May 4 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : http://lx-viewer.sourceforge.net    
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

// Functions dealing with display colors, color maps, etc are here

#ifndef COLORFUNCTIONS_H
#define COLORFUNCTIONS_H

// QT Includes
#include <qcolor.h>       // provides colors based on RGB or HSV
#include <qpushbutton.h>  // provides a command button
#include <qpalette.h>     // color groups for each widget state
#include <qwidget.h>      // base class of all user interface objects
#include <qdialog.h>      // base class of dialog windows
#include <qlabel.h>       // provides a text or image display
#include <qspinbox.h>     // provides a spinbox widget

// LX includes
#include "resource.h"

// Class to define a custom pushbutton for the choose color dialog box
class ColorButton : public QPushButton
{
    Q_OBJECT

   public:
      ColorButton(short colorkey = 0, QWidget* parent=0, const char* name=0);

   public slots:
      void ButtonClicked();

   private:
      const short colorkey;

   signals:
      void ColorSelected(int);

};

// Class for a custom color dialog - to display and pick the 256 colors used for cad work

class LX_ColorDialog : public QDialog
{
    Q_OBJECT

   public:
      LX_ColorDialog(int mode = COLOR_DIAG_FOR_ENTITY,QWidget* parent=0, const char* name=0, bool modal=TRUE, WFlags f=0);

   public slots:
      void ReceiveClick(int);
      void ReceiveBoxSpin(int);
      void FoundColor();
      void Color256Picked();
      void ColorNotPicked();
/*      void Help();   */

   private:
      int mode;
    QPushButton* OKButton;
      QPushButton* CancelButton;
      QPushButton* HelpButton;
      QLabel* colorlabel;
      QSpinBox*   colorspin;
      QPushButton* ByBlock;
      QPushButton* ByLayer;

     ColorButton* cbset[256];
};

// Other functions dealing with color
QColor LX_GetColor(short);      // function to return a QColor based on the ACAD color scheme

#endif

