/***************************************************************************
                          commandproc.cpp  -  description
                             -------------------
    begin                : Fri Sep 13 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : buyer242299@aol.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//	file that will hold the class for the command bar and also
//	will process any command line arguments

#include "commandproc.h"
#include "resource.h"
#include "drawingboard.h"
#include "infomessage.h"
#include "utilities.h"

#include <qstringlist.h>

////////////////////////////////////////////////////////////////////////
//      				constructor for the alias dialog box
/////////////////////////////////////////////////////////////////////////
LX_AliasDialog::LX_AliasDialog(AssocMap cMap, AssocMap pMap, AssocMap aMap, QWidget* parent,const char* name, bool modal, WFlags f):QDialog(parent,name,modal,f)
{
	//	size of graphics primitives
	const int border = 11;
	const int labelheight = 25;
	const int boxwidth = 130;
	const int boxheight = 30;
	
	//	create the list view
	lv1 = new QListView(this,"alias dialog lv1");	
		lv1->setRootIsDecorated(false);
		lv1->addColumn(tr("Command"));
		lv1->addColumn(tr("Alias"));
		lv1->setShowSortIndicator(true);
		lv1->setAllColumnsShowFocus(true);
		lv1->setSelectionMode(QListView::Single);
		lv1->setItemMargin(2);

	//	create the aliasgroupbox
	editalias = new QGroupBox(this,"alias dialog editalias");
		editalias->setTitle(tr("Set Alias"));
			commandlabel = new QLabel(tr("Command"),editalias);
				commandlabel->setGeometry(border,border,boxwidth,labelheight);
				commandlabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
			commandbox = new QLineEdit(editalias);
				commandbox->setGeometry(border,border+labelheight,boxwidth,boxheight);
				commandbox->setReadOnly(true);
			aliaslabel = new QLabel(tr("Alias"),editalias);
				aliaslabel->setGeometry(border,border+labelheight+boxheight,boxwidth,labelheight);
				aliaslabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
			aliasbox = new QLineEdit(editalias);
				aliasbox->setGeometry(border,border+2*labelheight+boxheight,boxwidth,boxheight);

	//	create the button group
		buttonbox = new QVButtonGroup(this,"alias dialog buttonbox");
			fopen = new QPushButton(tr("Import"),buttonbox,"fopen");
			cancelbutton = new QPushButton(tr("Cancel"),buttonbox,"cancel");
			okbutton = new QPushButton(tr("Ok"),buttonbox,"ok button");
			fopen->setDisabled(true);

	//	now create the layouts and insert all the parts into them
		QHBoxLayout* hb1 = new QHBoxLayout(this,border,border,"hb1");
		QVBoxLayout* vb1 = new QVBoxLayout(0,0,2*border,"vb1");
		hb1->addWidget(lv1);
		hb1->addLayout(vb1);
		vb1->addWidget(editalias);
		vb1->addStretch();
		vb1->addWidget(buttonbox);		

	//	now populate the list view
	AssocMap::Iterator it;
	AssocMap invertaMap;
	QListViewItem* lvitem;
	QString scmd;
	QString cmdl;
	QString reserved;

	//	need a maping of [command] = alias, and aMap is of the form [alias] = command
	//	so invert the mapping
	for (it = aMap.begin(); it != aMap.end(); ++it)
		invertaMap[it.data()] = it.key();

	for (it = pMap.begin(); it != pMap.end(); ++it)
	{
	//	extract the actual command from the prompt key, ie: ZOOM out of ZOOM1
		scmd = it.key();
		int len = scmd.length();
		scmd = scmd.left(--len);
		scmd = scmd.upper();
	//	now get the command list from the prompt data
		cmdl = it.data();
		len = cmdl.find('<');
		if (len >= 0) cmdl = cmdl.left(len);
		cmdl = cmdl.upper();
		QStringList sl = QStringList::split("\\",cmdl);
		reserved.append(it.data());
	//	now create all the listview entries of the form: supercommand<space>command (ie: ZOOM EXTENTS)
	//	if the alias exists use it, otherwise map alias == command
		for (QStringList::Iterator slit = sl.begin(); slit != sl.end(); ++slit)
		{
			QString scc = scmd + " " + (*slit);
			if (invertaMap.contains(scc))
				lvitem = new QListViewItem(lv1,scc,invertaMap[scc]);
			else lvitem = new QListViewItem(lv1,scc,scc);
		}
	}
	//	now create all the listview entries of the form: command (ie: OPEN)
	for (it = cMap.begin(); it != cMap.end(); ++it)
		if (!reserved.contains(it.data(),true))
		{
			if (invertaMap.contains(it.data()))
				lvitem = new QListViewItem(lv1,it.data(),invertaMap[it.data()]);
			else lvitem = new QListViewItem(lv1,it.data(),it.data());
		}

	//	now connect signals and slots
	connect(cancelbutton,SIGNAL(clicked()),this,SLOT(cancelpressed()));
	connect(okbutton,SIGNAL(clicked()),this,SLOT(okbutpressed()));
	connect(lv1,SIGNAL(doubleClicked(QListViewItem*)),this,SLOT(lvitemclicked(QListViewItem*)));
	connect(lv1,SIGNAL(returnPressed(QListViewItem*)),this,SLOT(lvitemclicked(QListViewItem*)));
	connect(lv1,SIGNAL(rightButtonPressed(QListViewItem*,const QPoint&,int)),this,SLOT(lvitemclicked(QListViewItem*)));
	connect(aliasbox,SIGNAL(textChanged(const QString&)),this,SLOT(aliaschanged(const QString&)));

	//	finally provide a title to the dialog
	setCaption(tr("Command Alias"));	
}

void LX_AliasDialog::fopenpressed()
{
}
void LX_AliasDialog::cancelpressed(){done(0);}	//	exit without saving changes
void LX_AliasDialog::okbutpressed()  //	save all alias definitions to a file and re-map all aliases
{
	const QChar comma(',');
	const QChar star('*');
	const QString tab("\t");
	extern DrawingBoard* db;

	QString lxv_pgp = QDir::homeDirPath() + LXV_HOME + "/lxv.pgp";
	QFileInfo fi = QFileInfo(lxv_pgp);
	QFileInfo fd = fi.dirPath(true);
	if ((fi.exists() && !fi.isWritable()) || !fd.isWritable())
	{
		DisplayMessage(NO_WRITE_PERMISSION);
		db->printMessage(tr("\nWarning: %1 was not saved to disk - write permission not granted").arg(lxv_pgp));
		done(-1);
	}

	QFile f(lxv_pgp);
	if (f.open(IO_WriteOnly))
	{
		QTextStream stream( &f );
		stream << tr(";  LX-VIEWER - Command Alias Definitions");
		stream << tr("\n;\n;  Command alias format:");
		stream << tr("\n;\t<Alias>, *<Full command name>");
		stream << tr("\n;\n;  The following are convenient abbreviations for most commands");
		stream << tr("\n;  and selected settings in LX-Viewer.\n;");
		QListViewItemIterator it(lv1);
    for ( ; it.current(); ++it )
			stream << "\n" << it.current()->text(1) << comma << tab << star << it.current()->text(0);		
		f.close();
	}

	done(1);
}
//	slot to place the command and alias into the edit boxes
void LX_AliasDialog::lvitemclicked(QListViewItem* lvi)
{
	commandbox->setText(lvi->text(0));
	aliasbox->setText(lvi->text(1));	
	aliasbox->setFocus();
}
void LX_AliasDialog::aliaschanged(const QString& s)
{
	QListViewItem* current = lv1->selectedItem();
	QString newalias = s;
	newalias = newalias.simplifyWhiteSpace();
	newalias = newalias.upper();
	current->setText(1,newalias);
}

//////////////////////////////////////////////////////////////////////////////////////////////
//	custom zero height text edit
//////////////////////////////////////////////////////////////////////////////////////////////
ZeroHeightTextEdit::ZeroHeightTextEdit(QWidget* parent, const char* name):QTextEdit(parent,name){}
QSize ZeroHeightTextEdit::minimumSizeHint() const {return QSize(QTextEdit::minimumSizeHint().width(),0);}
QSize ZeroHeightTextEdit::sizeHint() const {return QSize(QTextEdit::sizeHint().width(),0);}


//////////////////////////////////////////////////////////////////////////////////////////////
//	command bar
//////////////////////////////////////////////////////////////////////////////////////////////

//	constructor for the command bar
LX_CommandBar::LX_CommandBar(Place p, QWidget* parent, const char* name):QDockWindow(p,parent,name)
{
	promptdelim = (" : ");
	cmdprompt = tr("Command%1").arg(promptdelim);
	const QString promptend = QString(">" + promptdelim);
	
	cMap["HELP"]		= tr("HELP");		//	display list of commands
	cMap["MAN"]			=	tr("MANUAL");		//	call program manual
	cMap["VERSION"]	=	tr("VERSION");//	display program version
	cMap["AUDIT"] 	= tr("AUDIT");	//	call audit
	cMap["QUIT"]		=	tr("QUIT");		//	call quit
	cMap["OPEN"]		=	tr("OPEN");		//	open file
	cMap["PAN"]			= tr("PAN");		//	pan command
	cMap["UP"]			=	tr("Up");					//	pan up
	cMap["RIGHT"]		=	tr("Right");			//	pan right
	cMap["DOWN"]		=	tr("Down");				//	pan down
	cMap["LEFT"]		=	tr("Left");				//	pan left
	cMap["POINT"]		=	tr("base point:");			//	point pan
	cMap["REGEN"]		=	tr("REGEN");	//	regen command
	cMap["ZOOM"]		=	tr("ZOOM");		//	zoom command
	cMap["IN"]			=	tr("In");					//	zoom in
	cMap["OUT"]			= tr("Out");				//	zoom out
	cMap["WINDOW"]	=	tr("Window");			//	zoom window
	cMap["EXTENTS"]	=	tr("Extents");		//	zoom extents
	cMap["PREVIOUS"]=	tr("Previous");		//	zoom previous
	cMap["VIEWMAX"]	=	tr("Viewmax");		//	zoom out to maximum
	cMap["CENTER"]	= tr("Center");			//	zoom center
	cMap["REPORT"]	=	tr("REPORT");			//	report feature
	cMap["PLOT"]		=	tr("PLOT");			//	plot the drawing
	cMap["PRINT"]		=	tr("PRINT");		//	plot the drawing (same as plot)
	cMap["QSAVE"]		=	tr("QSAVE");		//	a quick save
	cMap["SAVE"]		=	tr("SAVE");			//	opens the file save dialog
	cMap["CLOSE"]		=	tr("CLOSE");		//	close the drawing
	cMap["BCONV"]		=	tr("BCONVERT");	//	batch convert
	cMap["BPLOT"]		=	tr("BPLOT");		//	batch plot

//	prompts below do not need to be translated since they will be translated above
	pMap["ZOOM1"] = cMap["IN"] + '\\' + cMap["OUT"] + '\\' + cMap["WINDOW"] + '\\' +
									cMap["EXTENTS"] + '\\' + cMap["PREVIOUS"] + '\\' + cMap["VIEWMAX"] +
									'\\' + cMap["CENTER"] + '\\' + '<' + cMap["WINDOW"] + promptend;
	pMap["PAN1"]	=	cMap["UP"] + '\\' + cMap["RIGHT"] + '\\' + cMap["DOWN"] + '\\' +
									cMap["LEFT"] + '\\' + '<' + cMap["POINT"] + promptend;


//	initialize the keyboard stack
	cmdhistory.clear();

//	try to locate and initialize the alias file
	initializeCmdBar();

//	create the widgets to populate the command bar with
	le = new QLineEdit(this,"Command Bar text line");
	le->setFrame(false);
	le->show();

	pe = new QLabel(this,"Command Bar prompt");
	pe->setFrameStyle(QFrame::NoFrame);
	pe->setBackgroundMode(Qt::PaletteBase);
	pe->setPaletteForegroundColor(le->paletteForegroundColor());
	pe->setText(cmdprompt);
	pe->show();

	setOrientation(Qt::Horizontal);
	setBackgroundMode(Qt::PaletteBase);

	QVBoxLayout* vb1 = new QVBoxLayout(0,0,-1,"command bar vb1");
	QHBoxLayout* hb1 = new QHBoxLayout(0,0,0,"command bar hb1");
	te = new ZeroHeightTextEdit(this,"command bar text edit");
	te->setReadOnly(true);
	
	boxLayout()->addLayout(vb1);
	vb1->addWidget(te,1);
	vb1->addLayout(hb1);
	hb1->addWidget(pe,0);
	hb1->addWidget(le,1);

	setResizeEnabled(true);
			
// connect signals to slots
	connect(le,SIGNAL(returnPressed()),this,SLOT(translateAlias()));
}

//	slot to process commands typed to the command bar
void LX_CommandBar::processCommand()
{
	extern DrawingBoard* db;
  const bool cs(false);		//string matches are case insensitive	

	QString cmd(le->text());
	QString pmt(pe->text());
	QString scmd = QString();
	QString astyped = cmd;

//	code below to parse commands such as "zoom extents"
	cmd = cmd.simplifyWhiteSpace();
	cmd = cmd.upper();
	if (cmd.contains(" "))
	{
		int space = cmd.find(" ");
		scmd = cmd.left(space);
		cmd = cmd.remove(0,++space);
	}

	pe->setText(cmdprompt);
	le->clear();

	if (cmd.contains(cMap["HELP"],cs)) {db->displayTWINmessage(CALL_HELP); addCommand("HELP"); return;}
	if (cmd.contains(cMap["MAN"],cs))  {db->togglemanual(); return;}
	if (cmd.contains(cMap["VERSION"],cs)) {db->displayTWINmessage(CALL_VERSION); addCommand("VERSION"); return;}
	if (cmd.contains(cMap["AUDIT"],cs)) {db->auditdrawing(); return;}
	if (cmd.contains(cMap["QUIT"],cs))  {db->quit(); return;}
	if (cmd.contains(cMap["OPEN"],cs))	{db->openfiledia(); return;}
	if (cmd.contains(cMap["PAN"],cs))		{pe->setText(pMap["PAN1"]); db->switchview(ID_VIEW_PAN); return;}
	if (pmt.contains(pMap["PAN1"],cs) || (scmd.contains(cMap["PAN"],cs) && !cmd.isEmpty()))
	{
		if (cmd.isEmpty()) {db->switchview(ID_VIEW_PAN_POINT); return;}
		if (cmd.left(1) == cMap["LEFT"].left(1)) {db->switchview(ID_VIEW_PAN_LEFT); return;}
		if (cmd.left(1) == cMap["UP"].left(1)) {db->switchview(ID_VIEW_PAN_UP); return;}
		if (cmd.left(1) == cMap["RIGHT"].left(1)) {db->switchview(ID_VIEW_PAN_RIGHT); return;}
		if (cmd.left(1) == cMap["DOWN"].left(1)) {db->switchview(ID_VIEW_PAN_DOWN); return;}
	}
	if (cmd.contains(cMap["REGEN"],cs))	{db->switchview(ID_VIEW_REGEN); return;}
	if (cmd.contains(cMap["ZOOM"],cs))	{pe->setText(pMap["ZOOM1"]); addCommand("ZOOM"); return;}
	if (pmt.contains(pMap["ZOOM1"],cs) || (scmd.contains(cMap["ZOOM"],cs) && !cmd.isEmpty()))
	{
		if (cmd.isEmpty()) {db->switchview(ID_VIEW_ZOOM_WINDOW); return;}
		if (cmd.left(1) == cMap["IN"].left(1)) {db->switchview(ID_VIEW_ZOOM_IN); return;}	
		if (cmd.left(1) == cMap["OUT"].left(1)) {db->switchview(ID_VIEW_ZOOM_OUT); return;}	
		if (cmd.left(1) == cMap["WINDOW"].left(1)) {db->switchview(ID_VIEW_ZOOM_WINDOW); return;}	
		if (cmd.left(1) == cMap["EXTENTS"].left(1)) {db->switchview(ID_VIEW_ZOOM_EXTENTS); return;}	
		if (cmd.left(1) == cMap["PREVIOUS"].left(1)) {db->switchview(ID_VIEW_ZOOM_PREVIOUS); return;}	
		if (cmd.left(1) == cMap["VIEWMAX"].left(1)) {db->switchview(ID_VIEW_ZOOM_VIEWMAX); return;}	
		if (cmd.left(1) == cMap["CENTER"].left(1)) {db->switchview(ID_VIEW_ZOOM_CENTER); return;}	
	}						
	if (cmd.contains(cMap["REPORT"],cs))	{db->showreportdia();return;}
	if (cmd.contains(cMap["BPLOT"],cs))	{db->batchPlot(); return;}
	if (cmd.contains(cMap["PLOT"],cs))  {db->callplotter(); return;}
	if (cmd.contains(cMap["PRINT"],cs))  {db->callplotter(); return;}
	if (cmd.contains(cMap["QSAVE"],cs))	{db->quicksave(); return;}
	if (cmd.contains(cMap["SAVE"],cs))	{db->saveas(); return;}
	if (cmd.contains(cMap["CLOSE"],cs))	{db->closefile(); return;}
	if (cmd.contains(cMap["BCONV"],cs)) {db->batchConvert(); return;}
	
	if (cmd.isEmpty() && !cmdhistory.isEmpty()){le->setText(cmdhistory[0]); processCommand();return;}	//	repeat last command

	//	no command recognized
	te->setText(te->text() + tr("  <== Unrecognized Command"));
	db->printMessage(tr("\nUnrecognized Command: %1").arg(astyped));
	return;
}

//	slot to initialize the command bar including locating and reading the alias file
void LX_CommandBar::initializeCmdBar()
{
	QFile lxv_pgp(QDir::homeDirPath() + LXV_HOME + "/lxv.pgp");
  if(!lxv_pgp.exists()) return;		//	there is no alias file so return

  if(!lxv_pgp.open (IO_ReadOnly))	//	alias file exists but for some reason we can't open it	
  {	
		DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,
                   tr("<center>An error occured opening"
											"<center>The command alias definition file: %1"
                       "<center>Please check that you have permission to open it.").arg(lxv_pgp.name()));
    return;
  }
			
//	now read each line of the file and place data into a clean aMap
	aMap.clear();
	QTextStream rs (&lxv_pgp);
	const QChar comma(',');
	const QChar star('*');
	const QChar semi(';');
	const bool cs(false);	//	searchs are case insensitive
	QString cmd;
	QString ali;

	while (!rs.eof())
	{
		cmd = rs.readLine();
		cmd = cmd.simplifyWhiteSpace();
		ali = cmd;
		if (cmd[0] != semi && cmd.contains(comma,cs) && cmd.contains(star,cs))		//	the line read is not a comment and appears properly formatted
		{
			int split = ali.find(comma,0,cs);
	    ali = ali.left(split);
			split = cmd.find(star,0,cs);
			cmd = cmd.remove(0,++split);
			aMap[ali.upper()] = cmd.upper();
    }
	}	//	while rs is not at eof()		
	
	lxv_pgp.close();
  return;
}

// slot to clear the command and prompt areas of pending commands which will
//	effectively cancel a command and then reset the bar
void LX_CommandBar::clearCmdBar()
{
		le->clear();
		pe->setText(cmdprompt);
}


//	slot to translate an alias into a proper command
//	this function looks at what was typed into the command bar to see if it
//	matches an alias.  If it does this function will replace the alias in
//	the command bar with the proper command
void LX_CommandBar::translateAlias()
{
	QString s(pe->text() + le->text());
	te->append(s);	//	record what was typed into history

	s = le->text();
	s = s.simplifyWhiteSpace();
	s = s.upper();
	AssocMap::Iterator it;
	for (it = aMap.begin(); it != aMap.end(); ++it)
		if (s == it.key()) le->setText(it.data());
		
	processCommand();
}


//	slot to create and show the edit alias dialog
void LX_CommandBar::showEditAliasDialog()
{
	initializeCmdBar();	//	read the alias file and assign to aMap
	LX_AliasDialog* ad = new LX_AliasDialog(cMap,pMap,aMap,this);
	int rtnval = ad->exec();
	if (rtnval == 1) initializeCmdBar();	// a new lxv.pgp was written so load new alias def's

	delete ad;
}


//	Function to process command line arguments.  If this function is called we
//	already know that there is at least one command line argument supplied
//	format of the command line arguments is the file name to open is provided
//	without a leading dash.  All commands to be processed have a leading dash.
//	batch processes have an = sign in them
void LX_CommandBar::processCommandLineArgs()
{
	extern DrawingBoard* db;

	const QChar dash = QChar('-');
	const QChar eq = QChar('=');
	const bool cs(false);		//string matches are case insensitive	
	QString cmd = QString();
	QString fn = QString();
	QString s = QString();

//	look at all arguments and assign them accordingly
	for (int i = 1; i < qApp->argc(); i++)
	{
		s = QString(qApp->argv()[i]);
		s = s.stripWhiteSpace();
		if (s.left(1) == dash) cmd.prepend(s);
		else
		{
			if (s.contains(eq)) cmd.append(s);
			else fn = s;
		}
	}
	
//	if it appears we have a file name try to open the file.  There is no error
//	checking in this routine for valid file names, or even if the file exists.
//	If the openfile routine of db can't open the file the drawinghandle will be
//	set to null by that routine.
	if (!fn.isNull()) db->openfile(fn);

	//	now process any command line arguments
	if (cmd.contains(cMap["MAN"],cs))	{db->togglemanual();return;}
	if (cmd.contains(cMap["AUDIT"],cs))
	{
		if(db->auditdrawing() == -1) db->printMessage(GetResourceString(NO_DRAWING_LOADED_CMD_LINE),TWIN);		 	
		return;
	}
	if (cmd.contains(cMap["REPORT"],cs)){db->showreport(cmd);	return;}
	if (cmd.contains(cMap["BCONV"],cs)) {db->batchConvertSetOptions(cmd);return;}
	if (cmd.contains(cMap["BPLOT"],cs)) {db->batchPlotSetOptions(cmd);return;}
}	


//	function to push another command onto the stack.  Called from DrawingBoard
//	and other places to put commands from mouse and menu picks into the stack
void LX_CommandBar::addCommand(const QString& key)
{
	QString cmd = key;
	QString scmd = QString();

//	if c contains supercommand and command (such as "ZOOM EXTENTS") then
//	break the pieces up to add appropriately.  If "zoom" was typed previously from the
//	keyboard than only place "extents" into the command history, if "zoom extents"
//	was picked from the menu or typed that way then put "zoom extents" into the history.
	cmd = cmd.simplifyWhiteSpace();
	cmd = cmd.upper();
	if (cmd.contains(" "))
	{
		int space = cmd.find(" ");
		scmd = cmd.left(space);
		cmd = cmd.remove(0,++space);
		if (cMap.contains(scmd) && cMap.contains(cmd))
		{
			if (cmdhistory[0] == cMap[scmd]) cmdhistory.prepend(cMap[cmd]);
			else cmdhistory.prepend(cMap[scmd] + " " + cMap[cmd]);
		}
		return;
	}

//	only a command was typed, see if it exists and add to the stack
	else if (cMap.contains(cmd)) cmdhistory.prepend(cMap[cmd]);

}

//			private functions
void LX_CommandBar::keyPressEvent(QKeyEvent* k)
{
	static int curpos = -1;

	if (cmdhistory.empty()) {k->ignore();return;}

	if (k->key() == Qt::Key_Up)
	{
		if (++curpos >= static_cast<int>(cmdhistory.count())) curpos = 0;
		le->setText(cmdhistory[curpos]);
		return;
	}

	if (k->key() == Qt::Key_Down)
	{
		if (--curpos < 0) curpos = cmdhistory.count() - 1;
		le->setText(cmdhistory[curpos]);
		return;
	}

	//	k->key() == Qt::Key_Escape is grabbed automatically by DrawingBoard's event filter
	k->ignore();
	return;
}
