/***************************************************************************
                          commandproc.h  -  description
                             -------------------
    begin                : Fri Sep 13 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : buyer242299@aol.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/


//	file that will hold the class for the command bar and also
//	will process any command line arguments


#ifndef COMMANDPROC_H
#define COMMANDPROC_H

#include <qlineedit.h>
#include <qstring.h>
#include <qapplication.h>
#include <qdockwindow.h>
#include <qmap.h>
#include <qlabel.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdialog.h>
#include <qlistview.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qvbuttongroup.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qtextedit.h>

typedef QMap<QString,QString>AssocMap;

//	class for the dialog used to modify command alias' and to read and save them to file
class LX_AliasDialog : public QDialog
{
	Q_OBJECT
	
	public:
		LX_AliasDialog(AssocMap cMap, AssocMap pMap, AssocMap aMap, QWidget* parent = 0, const char* name = 0, bool modal = TRUE, WFlags = 0);

	public slots:
		void fopenpressed();
		void cancelpressed();
		void okbutpressed();
		void lvitemclicked(QListViewItem*);
		void aliaschanged(const QString&);

	private:
	//	listview members
		QListView* lv1;
		QListViewItem* lv1item;
//	editalias groupbox and members
  	QGroupBox* editalias;
			QLineEdit* commandbox;
			QLineEdit* aliasbox;
			QLabel* commandlabel;
			QLabel* aliaslabel;
	//	buttonbox and members
		QVButtonGroup* buttonbox;
			QPushButton* fopen;
			QPushButton* cancelbutton;
			QPushButton* okbutton;	
};

//	class for custom text edit window (really need one that can shrink to 0 pixels height)
class ZeroHeightTextEdit : public QTextEdit
{
	public:
		ZeroHeightTextEdit (QWidget* parent = 0, const char* name = 0);	//	constructor
  	virtual QSize minimumSizeHint() const;
		virtual QSize sizeHint() const;
};

//	class for the command bar - just a lineedit inside a QDockWindow with a couple
//	of data members and member functions
//	added a zeroheighttextedit (normally hidden) do display a command history
class LX_CommandBar : public QDockWindow
{
  Q_OBJECT
	
	public:
		LX_CommandBar (Place p = InDock, QWidget* parent = 0, const char* name = 0);	//	constructor
		void processCommandLineArgs();				//	function to process command line arguments		
		void addCommand(const QString&);			//	function to add a command to the history stack
		
	public slots:
		void processCommand();						//	slot to process commands with	
		void initializeCmdBar();					//	slot to initialize the command bar
		void clearCmdBar();								//	slot to cancel pending commands and reset the bar	
		void translateAlias();						//	slot to an alias into a proper command	
		void showEditAliasDialog();				//	slot to create and show the dialog	

	private:
		QLineEdit* le;            //	line edit widget inserted into this
		QLabel* pe;								//	QLabel to hold the command prompt
		ZeroHeightTextEdit* te;		//	text edit to hold command history
		QString cmdprompt;				//	standard command prompt presented to the user
		QString promptdelim;			//	command prompt delimiter
		AssocMap cMap;						//	associaton mapping for commands
		AssocMap pMap;						//	associaton mapping for command bar prompts
		AssocMap aMap;						//	association mapping for command alias'
		QStringList cmdhistory;		//	stack contains everything typed into the command bar
		
		void keyPressEvent(QKeyEvent*);	
};


#endif




