/***************************************************************************
                          drawingboard.cpp
                             -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : http://lx-viewer.sourceforge.ne
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

// This is the implementation of the drawingboard window

#include "drawingboard.h"

// XPM includes
#include "xpm/fileopen.xpm"
#include "xpm/fileopen_s.xpm"
#include "xpm/filesave.xpm"
#include "xpm/filesave_s.xpm"
#include "xpm/print.xpm"
#include "xpm/audit.xpm"
#include "xpm/audit_s.xpm"
#include "xpm/exitprog.xpm"
#include "xpm/wcsicon.xpm"
#include "xpm/zoomIN.xpm"
#include "xpm/zoomWINDOW.xpm"
#include "xpm/zoomEXTENTS.xpm"
#include "xpm/zoomPREVIOUS.xpm"
#include "xpm/zoomCENTER.xpm"
#include "xpm/zoomOUT.xpm"
#include "xpm/panbutton.xpm"
#include "xpm/triad.xpm"
#include "xpm/help.xpm"
#include "xpm/help_s.xpm"
#include "xpm/layerprop.xpm"
#include "xpm/lxv.xpm"
#include "xpm/troll_logo.xpm"
#include "xpm/open_dwg_logo.xpm"
#include "xpm/panup.xpm"
#include "xpm/panright.xpm"
#include "xpm/pandown.xpm"
#include "xpm/panleft.xpm"
#include "xpm/redraw.xpm"    //inside file the pixmap is called "redrawbutton"

#include <qtooltip.h>        // provides tooltips for any widget

DrawingBoard::DrawingBoard(QWidget* parent, const char* name) : QMainWindow(parent, name)
{
	//	Variables and constants
	const short coordboxminwidth = 150;
	const short coordboxmaxwidth = 150;

	//	Define pixmaps for pulldown menus and for toolbars
	QIconSet b11pm,b12pm,b13pm,b14pm,b15pm;
	b11pm = QIconSet(QPixmap(fileopen_s),QPixmap(fileopen));
	b12pm = QIconSet(QPixmap(filesave_s),QPixmap(filesave));
	b13pm = QIconSet(QPixmap(print));
	b14pm = QIconSet(QPixmap(audit_s),QPixmap(audit));
	b15pm = QIconSet(QPixmap(exitprog));
	
	QIconSet b21pm;
	b21pm = QIconSet(QPixmap(redrawbutton));
	QIconSet b23pm;
	b23pm = QIconSet(QPixmap(panbutton));
	QIconSet b231pm, b232pm, b233pm, b234pm;
	b231pm = QIconSet(QPixmap(panleft));
	b232pm = QIconSet(QPixmap(panup));
	b233pm = QIconSet(QPixmap(panright));
	b234pm = QIconSet(QPixmap(pandown));
	QIconSet b241pm, b242pm, b243pm, b244pm, b245pm, b247pm;
	b241pm = QIconSet(QPixmap(zoomIN));
	b242pm = QIconSet(QPixmap(zoomOUT));
	b243pm = QIconSet(QPixmap(zoomWINDOW));
	b244pm = QIconSet(QPixmap(zoomEXTENTS));
	b245pm = QIconSet(QPixmap(zoomPREVIOUS));
	b247pm = QIconSet(QPixmap(zoomCENTER));
	QIconSet b25pm, b27pm;
	b25pm = QIconSet(QPixmap(triad));
	b27pm = QIconSet(QPixmap(wcsicon));
	QIconSet b31pm;
	b31pm = QIconSet(QPixmap(layerprop));
	QIconSet b51pm, b52pm, b53pm,b54pm;
	b51pm = QIconSet(QPixmap(help_s),QPixmap(help));
	b52pm = QIconSet(QPixmap(lxv));
	b53pm = QIconSet(QPixmap(troll_logo));
	b54pm = QIconSet(QPixmap(open_dwg_logo));

	//	set the main drawing area
	p2dwgarea=new DrawingArea(this,"Drawing Area");
	setCentralWidget(p2dwgarea);
	whitebackground = false;
	
	//	now create the painters to draw on the graphics screen and the snapshot pixmap
	p2painter = new QPainter(p2dwgarea);
	snapshotpainter = new QPainter;

	//	now construct the popupmenus
	p2filemenu=new QPopupMenu(this,"File Menu");
	p2filemenu->insertItem(b11pm,tr("&Open"),this,SLOT(openfiledia()),QKeySequence(tr("CTRL+O","Open File Accelerator")),ID_FILE_OPEN);
	p2filemenu->insertItem(tr("&Close"),this,SLOT(closefile()),0,ID_FILE_CLOSE);
	p2filemenu->insertItem(b12pm,tr("&Save"),this,SLOT(quicksave()),QKeySequence(tr("CTRL+S","Save File Accelerator")),ID_FILE_SAVE);
	p2filemenu->insertItem(tr("Save &As"),this,SLOT(saveas()),0,ID_FILE_SAVE_AS);
	p2filemenu->insertSeparator();
	p2filemenu->insertItem(b13pm,tr("&Print"),this,SLOT(callplotter()),QKeySequence(tr("CTRL+P","Plot Drawing Accelerator")),ID_FILE_PRINT);
	p2filemenu->insertSeparator();
	p2filemenu->insertItem(b14pm,tr("A&udit Drawing"),this,SLOT(auditdrawing()),0,ID_FILE_AUDIT);
	p2filemenu->insertItem(tr("&Report"),this,SLOT(showreportdia()),QKeySequence(tr("CTRL+R","Report Accelerator")),ID_FILE_REPORT);
  p2filemenu->insertSeparator();
	p2filemenu->insertItem(tr("Batch Con&vert"),this,SLOT(batchConvert()),0,ID_FILE_BATCH_CONVERT);
	p2filemenu->insertItem(tr("Batch P&lot"),this,SLOT(batchPlot()),0,ID_FILE_BATCH_PLOT);
	p2filemenu->insertSeparator();
	p2filemenu->insertItem(b15pm,tr("&Quit"),this,SLOT(quit()),QKeySequence(tr("CTRL+Q","Quit Application Accelerator")),ID_FILE_QUIT);

	p2viewmenu=new QPopupMenu(this,"View Menu");
	p2viewmenu->insertItem(b21pm,tr("&Redraw"),this,SLOT(switchview(int)),0,ID_VIEW_REDRAW);
	p2viewmenu->insertItem(tr("Re&gen"),this,SLOT(switchview(int)),0,ID_VIEW_REGEN);
	p2viewmenu->insertSeparator();
	p2pan=new QPopupMenu(p2viewmenu,"Pan menu");
		p2pan->insertItem(b23pm,tr("&Point"),this,SLOT(switchview(int)),0,ID_VIEW_PAN_POINT);
		p2pan->insertSeparator();
		p2pan->insertItem(b231pm,tr("&Left"),this,SLOT(switchview(int)),0,ID_VIEW_PAN_LEFT);
		p2pan->insertItem(b232pm,tr("&Up"),this,SLOT(switchview(int)),0,ID_VIEW_PAN_UP);
		p2pan->insertItem(b233pm,tr("&Right"),this,SLOT(switchview(int)),0,ID_VIEW_PAN_RIGHT);
		p2pan->insertItem(b234pm,tr("&Down"),this,SLOT(switchview(int)),0,ID_VIEW_PAN_DOWN);	
	p2viewmenu->insertItem(tr("&Pan"),p2pan);
	p2zoom=new QPopupMenu(p2viewmenu,"Zoom menu");
		p2zoom->insertItem(b241pm,tr("&In"),this,SLOT(switchview(int)),ZOOM_IN,ID_VIEW_ZOOM_IN);
		p2zoom->insertItem(b242pm,tr("&Out"),this,SLOT(switchview(int)),ZOOM_OUT,ID_VIEW_ZOOM_OUT);
		p2zoom->insertSeparator();
		p2zoom->insertItem(b243pm,tr("&Window"),this,SLOT(switchview(int)),0,ID_VIEW_ZOOM_WINDOW);
		p2zoom->insertItem(b244pm,tr("&Extents"),this,SLOT(switchview(int)),0,ID_VIEW_ZOOM_EXTENTS);
		p2zoom->insertSeparator();
		p2zoom->insertItem(b245pm,tr("&Previous"),this,SLOT(switchview(int)),ZOOM_PREVIOUS,ID_VIEW_ZOOM_PREVIOUS);
		p2zoom->insertItem(tr("&ViewMax"),this,SLOT(switchview(int)),0,ID_VIEW_ZOOM_VIEWMAX);
		p2zoom->insertSeparator();
		p2zoom->insertItem(b247pm,tr("&Center"),this,SLOT(switchview(int)),0,ID_VIEW_ZOOM_CENTER);
	p2viewmenu->insertItem(tr("&Zoom"),p2zoom);
	p2viewmenu->insertSeparator();
	p2viewmenu->insertItem(b25pm,tr("&3D View"),this,SLOT(toggle3Dview()),VIEWDIALOG,ID_VIEW_3DVIEW);
	p2predef=new QPopupMenu(p2viewmenu,"Predefined menu");
		p2predef->insertItem(tr("&Top View"),this,SLOT(switchview(int)),TOP_VIEW,ID_VIEW_TOP);
		p2predef->insertItem(tr("&Front View"),this,SLOT(switchview(int)),FRONT_VIEW,ID_VIEW_FRONT);
		p2predef->insertItem(tr("&Right Side View"),this,SLOT(switchview(int)),RIGHT_SIDE,ID_VIEW_RIGHT_SIDE);
		p2predef->insertItem(tr("&Left Side View"),this,SLOT(switchview(int)),LEFT_SIDE,ID_VIEW_LEFT_SIDE);
		p2predef->insertItem(tr("Bac&k Side View"),this,SLOT(switchview(int)),BACK_VIEW,ID_VIEW_BACK);
		p2predef->insertItem(tr("Botto&m View"),this,SLOT(switchview(int)),0,ID_VIEW_BOTTOM);
		p2predef->insertSeparator();
		p2predef->insertItem(tr("Aerial Right Front"),this,SLOT(switchview(int)),AERIAL_RFRONT,ID_VIEW_AERIAL_RFRONT);
		p2predef->insertItem(tr("Aerial Left Front"),this,SLOT(switchview(int)),AERIAL_LFRONT,ID_VIEW_AERIAL_LFRONT);
		p2predef->insertItem(tr("Aerial Right Rear"),this,SLOT(switchview(int)),AERIAL_RREAR,ID_VIEW_AERIAL_RREAR);
		p2predef->insertItem(tr("Aerial Left Rear"),this,SLOT(switchview(int)),AERIAL_LREAR,ID_VIEW_AERIAL_LREAR);
		p2predef->insertSeparator();
		p2predef->insertItem(tr("Below Right Front"),this,SLOT(switchview(int)),0,ID_VIEW_BELOW_RFRONT);
		p2predef->insertItem(tr("Below Left Front"),this,SLOT(switchview(int)),0,ID_VIEW_BELOW_LFRONT);
		p2predef->insertItem(tr("Below Right Rear"),this,SLOT(switchview(int)),0,ID_VIEW_BELOW_RREAR);
		p2predef->insertItem(tr("Below Left Rear"),this,SLOT(switchview(int)),0,ID_VIEW_BELOW_LREAR);
	p2viewmenu->insertItem(tr("Pre&defined"),p2predef);
	p2viewmenu->insertItem(b27pm,tr("Pla&n View"),this,SLOT(wcsplanview()),TOP_VIEW,ID_VIEW_WCSPLANVIEW);
	p2viewmenu->setItemParameter(ID_VIEW_REDRAW,ID_VIEW_REDRAW);
	p2viewmenu->setItemParameter(ID_VIEW_REGEN,ID_VIEW_REGEN);
	p2viewmenu->setItemParameter(ID_VIEW_PAN_LEFT,ID_VIEW_PAN_LEFT);
	p2viewmenu->setItemParameter(ID_VIEW_PAN_UP,ID_VIEW_PAN_UP);
	p2viewmenu->setItemParameter(ID_VIEW_PAN_RIGHT,ID_VIEW_PAN_RIGHT);
	p2viewmenu->setItemParameter(ID_VIEW_PAN_DOWN,ID_VIEW_PAN_DOWN);
	p2viewmenu->setItemParameter(ID_VIEW_ZOOM_IN,ID_VIEW_ZOOM_IN);
	p2viewmenu->setItemParameter(ID_VIEW_ZOOM_OUT,ID_VIEW_ZOOM_OUT);
	p2viewmenu->setItemParameter(ID_VIEW_ZOOM_WINDOW,ID_VIEW_ZOOM_WINDOW);
	p2viewmenu->setItemParameter(ID_VIEW_ZOOM_EXTENTS,ID_VIEW_ZOOM_EXTENTS);
	p2viewmenu->setItemParameter(ID_VIEW_ZOOM_PREVIOUS,ID_VIEW_ZOOM_PREVIOUS);
	p2viewmenu->setItemParameter(ID_VIEW_ZOOM_VIEWMAX,ID_VIEW_ZOOM_VIEWMAX);
	p2viewmenu->setItemParameter(ID_VIEW_ZOOM_CENTER,ID_VIEW_ZOOM_CENTER);
	p2viewmenu->setItemParameter(ID_VIEW_3DVIEW,ID_VIEW_3DVIEW);
	
	p2toolmenu=new QPopupMenu(this,"Tool Menu");
	p2toolmenu->setCheckable(true);
	p2toolmenu->insertItem(tr("&Text Window"),this,SLOT(toggletextwin()),TEXTWINDOW,ID_TOOL_TEXTWINDOW);
  p2toolmenu->insertItem(tr("&Command Bar"),this,SLOT(togglecmdbar()),CMDBAR,ID_TOOL_CMDBAR);
	p2toolmenu->insertItem(tr("&Status Bar"),this,SLOT(togglesbar()),STATUSBAR,ID_TOOL_STATUSBAR);	
	p2toolmenu->setItemChecked(ID_TOOL_STATUSBAR,true);
	p2toolmenu->setItemChecked(ID_TOOL_CMDBAR,true);
	p2toolmenu->insertSeparator();
	p2toolmenu->insertItem(tr("&File Tool Bar"),this,SLOT(toggletoolbar(int)),0,ID_TOOL_TOOLBAR_FILE);
	p2toolmenu->insertItem(tr("&View Tool Bar"),this,SLOT(toggletoolbar(int)),0,ID_TOOL_TOOLBAR_VIEW);
	p2toolmenu->insertItem(tr("&Layer Tool Bar"),this,SLOT(toggletoolbar(int)),0,ID_TOOL_TOOLBAR_LAYER);
	p2toolmenu->insertItem(tr("&Help Tool Bar"),this,SLOT(toggletoolbar(int)),0,ID_TOOL_TOOLBAR_HELP);
	p2toolmenu->setItemChecked(ID_TOOL_TOOLBAR_FILE,true);
	p2toolmenu->setItemChecked(ID_TOOL_TOOLBAR_VIEW,true);
	p2toolmenu->setItemChecked(ID_TOOL_TOOLBAR_LAYER,true);
	p2toolmenu->setItemChecked(ID_TOOL_TOOLBAR_HELP,true);
	p2toolmenu->insertSeparator();
	p2toolmenu->insertItem(tr("Large &Buttons"),this,SLOT(togglebuttonsize(int)),0,ID_TOOL_TOOLBUTTON_SIZE);
	p2toolmenu->insertItem(tr("Tool Ti&ps"),this,SLOT(toggletooltips()),0,ID_TOOL_TOOLTIP_ENABLE);
	p2toolmenu->insertItem(tr("Blac&k Background"),this,SLOT(togglebackground()),0,ID_TOOL_BACKGROUND_COLOR);
	p2toolmenu->setItemChecked(ID_TOOL_TOOLTIP_ENABLE,true);
	p2toolmenu->setItemChecked(ID_TOOL_BACKGROUND_COLOR,true);
	p2toolmenu->setItemParameter(ID_TOOL_TOOLBAR_FILE,ID_TOOL_TOOLBAR_FILE);
	p2toolmenu->setItemParameter(ID_TOOL_TOOLBAR_VIEW,ID_TOOL_TOOLBAR_VIEW);
	p2toolmenu->setItemParameter(ID_TOOL_TOOLBAR_LAYER,ID_TOOL_TOOLBAR_LAYER);
	p2toolmenu->setItemParameter(ID_TOOL_TOOLBAR_HELP,ID_TOOL_TOOLBAR_HELP);

	p2settingsmenu=new QPopupMenu(this,"Settings Menu");

	p2settingsmenu->insertItem(b31pm,tr("&Layers"),this,SLOT(calllayerdialog()),0,ID_SETTINGS_LAYERDIAG);
	p2settingsmenu->setItemEnabled(ID_SETTINGS_LAYERDIAG,FALSE);
	p2settingsmenu->insertItem(tr("&Command Alias"),this,SLOT(callaliasdialog()),0,ID_SETTINGS_ALIASDIAG);

	p2helpmenu=new QPopupMenu(this,"Help Menu");
	p2helpmenu->insertItem(b51pm,tr("&Help"),this,SLOT(togglemanual()),HELP,ID_HELP_MAN);
	p2helpmenu->insertSeparator();
	p2helpmenu->insertItem(b52pm,tr("About LX-Viewer"),this,SLOT(displayaboutmessage(int)),0,ID_HELP_ABOUT_LX);
	p2helpmenu->insertItem(b53pm,tr("About Qt"),this,SLOT(displayaboutmessage(int)),0,ID_HELP_ABOUT_QT);
	p2helpmenu->insertItem(b54pm,tr("About OpenDWG"),this,SLOT(displayaboutmessage(int)),0,ID_HELP_ABOUT_OD);
	p2helpmenu->setItemParameter(ID_HELP_ABOUT_LX,ID_HELP_ABOUT_LX);
	p2helpmenu->setItemParameter(ID_HELP_ABOUT_QT,ID_HELP_ABOUT_QT);
	p2helpmenu->setItemParameter(ID_HELP_ABOUT_OD,ID_HELP_ABOUT_OD);

	//	now insert the popup menus into the main menu bar
	menuBar()->insertItem(tr(" &File") ,p2filemenu);
	menuBar()->insertItem(tr(" &View") ,p2viewmenu);
	menuBar()->insertItem(tr(" &Tools"),p2toolmenu);
	menuBar()->insertItem(tr(" &Settings"),p2settingsmenu);
	menuBar()->insertItem(tr(" &Help") ,p2helpmenu);
	
	//	create the tool bars for the application
	p2filetoolbar=new QToolBar(this, "File menu tool bar");
		QToolButton* b11 = new QToolButton (b11pm,tr("Open an Existing File"),0,this,SLOT(openfiledia()),p2filetoolbar);
		QToolButton* b12 = new QToolButton (b12pm,tr("Save the File to Disk"),0,this,SLOT(quicksave()),p2filetoolbar,"b12");
		QToolButton* b13 = new QToolButton (b13pm,tr("Plot the Drawing"),0,this,SLOT(callplotter()),p2filetoolbar,"b13");
		QToolButton* b14 = new QToolButton (b14pm,tr("Audit the Drawing"),0,this,SLOT(auditdrawing()),p2filetoolbar,"b14");
		QToolButton* b15 = new QToolButton (b15pm,tr("Quit LX_Viewer"),0,this,SLOT(quit()),p2filetoolbar);
	p2viewtoolbar=new QToolBar(this, "View menu tool bar");
		QToolButton* b21 = new QToolButton (b21pm,tr("Redraw"),0,this,SLOT(redraw()),p2viewtoolbar);
		p2viewtoolbar->addSeparator();
		QToolButton* b25 = new QToolButton (b25pm,tr("3D View Dialog"),0,this,SLOT(toggle3Dview()),p2viewtoolbar);
		QToolButton* b27 = new QToolButton (b27pm,tr("WCS Plan View"), 0,this,SLOT(wcsplanview()),p2viewtoolbar);	
		p2viewtoolbar->addSeparator();
		QToolButton* b23 = new QToolButton (b23pm,tr("Pan"),0,this,SLOT(pan()),p2viewtoolbar);	
		QToolButton* b231 = new QToolButton (b231pm,tr("Pan Left"),0,this,SLOT(panleftslot()),p2viewtoolbar);
		QToolButton* b232 = new QToolButton (b232pm,tr("Pan Up"),0,this,SLOT(panupslot()),p2viewtoolbar);	
		QToolButton* b233 = new QToolButton (b233pm,tr("Pan Right"),0,this,SLOT(panrightslot()),p2viewtoolbar);
		QToolButton* b234 = new QToolButton (b234pm,tr("Pan Down"),0,this,SLOT(pandownslot()),p2viewtoolbar);		
		p2viewtoolbar->addSeparator();
		QToolButton* b241 = new QToolButton (b241pm,tr("Zoom In"),0,this,SLOT(zoomin()),p2viewtoolbar);
		QToolButton* b242 = new QToolButton (b242pm,tr("Zoom Out"),0,this,SLOT(zoomout()),p2viewtoolbar);
		QToolButton* b243 = new QToolButton (b243pm,tr("Zoom Window"),0,this,SLOT(zoomwindow()),p2viewtoolbar);
		QToolButton* b244 = new QToolButton (b244pm,tr("Zoom Extents"),0,this,SLOT(zoomextents()),p2viewtoolbar);
		QToolButton* b245 = new QToolButton (b245pm,tr("Zoom Previous"),0,this,SLOT(zoomprevious()),p2viewtoolbar);
		QToolButton* b247 = new QToolButton (b247pm,tr("Zoom Center"),0,this,SLOT(zoomcenter()),p2viewtoolbar);
	p2layertoolbar=new QToolBar(this, "Layer settings tool bar");
		p2layerbar = new LX_LayerBar(p2layertoolbar);
	p2helptoolbar=new QToolBar(this, "Help menu tool bar");
		QWhatsThis::whatsThisButton(p2helptoolbar);
		QToolButton* b51 = new QToolButton (b51pm,tr("Help"),0,this,SLOT(togglemanual()),p2helptoolbar);

	
	//	now add a status bar at the bottom, give it a progress bar child
	//	and coordinate box
	p2sbar = statusBar();

	p2progbar=new QProgressBar(100,p2sbar,"Progress Bar");
	p2progbar->reset();
	p2progbar->hide();
	
	p2setbar=new LX_SettingsBar(p2sbar,"Settings Bar");

	p2coords=new QLabel(p2sbar,"Coordinate Display");
	p2coords->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	p2coords->setMinimumWidth(coordboxminwidth);
	p2coords->setMaximumWidth(coordboxmaxwidth);

	p2sbar->addWidget(p2progbar, 0, TRUE);
	p2sbar->addWidget(p2setbar,0,TRUE);
	p2sbar->addWidget(p2coords,0,TRUE);

 	//	now add a text window, normally hidden, but can be activated by F2
	//	I want to be able to move this window to another desktop so it must

	//	be a top level window (not a child of db).  We will need to duplicate
	//	the F2 accelerator key and also intercept the window close event so that
	//	we change the check box in the menu (closes are really hides anyway).
  p2textwin=new TextWindow(0,"Text Window");
	QAccel* accel_F2 = new QAccel (p2textwin);
	accel_F2->connectItem(accel_F2->insertItem(TEXTWINDOW),this,SLOT(toggletextwin()));
	p2textwin->setCaption(tr("LX-Viewer Text Window"));
	p2textwin->setReadOnly(true);
	p2textwin->hide();


//	now create custom dialog box for 3D view (activated by F5 defined above)	
	threedview = new LX_3DViewDialog(this,"3D dialog");

	//	now create the display manual class.  Register the F1 key to this class
	//	because its parent is 0 (therefore F1 defined above would not be active
	//	when the manual is displayed
	manual = new HelpWindow(0, "help viewer");
	QAccel* accel_F1 = new QAccel (manual);
	accel_F1->connectItem(accel_F1->insertItem(HELP),this,SLOT(togglemanual()));


	//	now create the a plotter from the LX_Plotter class
	plotter = new LX_PlotDialog(this,"Plot Dialog");

	//	now create the commandbar window (activated by F3 defined above)
	cmdbar = new LX_CommandBar(QDockWindow::InDock,this,"Command Bar");
	moveDockWindow(cmdbar,Qt::DockBottom);
	cmdbar->show();
	installEventFilter(cmdbar);
	
	//	create a report dialog
	report = new LX_ReportDialog(this,"Report Dialog",false);
	
	//	now connect custom signals and slots together
  connect (p2textwin,SIGNAL(textWinClosed()),this,SLOT(toggletextwin()));
	connect (p2dwgarea,SIGNAL(mouseCoordinate(const QString&)),p2coords,SLOT(setText(const QString&)));
	connect (p2setbar,SIGNAL(commandString(const QString&)),this,SLOT(printMessage(const QString&)));

	//	disactivate all items that are useless until a drawing is open
	greyout(true);

	//	set the application icon for this window
	QPixmap iconpm = QPixmap(PathToExec() + QString("/graphics/LX_Icon.png"));
	setIcon(iconpm);
	
	//	finally implement the What's This feature
	QWhatsThis::add(p2dwgarea,tr("Main window where the drawing will be displayed"));
  QWhatsThis::add(p2sbar,tr("Status bar used to display short messages and progress bars."
											"This bar can be raised and lowered by using the F10 key\n\n"
											"Use the text window (F2 key) for the complete message history"));
	QWhatsThis::add(cmdbar,tr("Command bar where you may type commands to be processed. "
											"This bar can be raised and lowered by using the F3 key.  "
											"This is a dock window so it can be moved around the desktop and "
											"docked on other edges.  You can also stretch the top of the bar to "
											"reveal a history of previous commands\n\n"
											"Use the text window (F2 key) for a complete message history"));
	QWhatsThis::add(p2coords,tr("Coordinate display shows the current mouse postion once a drawing is loaded."));
	QWhatsThis::add(b11,tr("Click this button to open a new file.\n\n"
  	               		"You can also select the Open command from the File menu."));
 	QWhatsThis::add(b12,tr("Click this button to quick save the file you are "
                  		"viewing.\n\nYou can also select the Save command from the File menu."));
	QWhatsThis::add(b13,tr("Use this button to call up the plot dialog to print"));
	QWhatsThis::add(b14,tr("Use this command to audit the drawing database to look for errors."));
	QWhatsThis::add(b15,tr("Exit the program - make sure you have saved your work"));
	QWhatsThis::add(b231,tr("Use this button to Pan Left"));
  QWhatsThis::add(b232,tr("Use this button to Pan Up"));
	QWhatsThis::add(b233,tr("Use this button to Pan Right"));
	QWhatsThis::add(b234,tr("Use this button to Pan Down"));
	QWhatsThis::add(b241,tr("Use this button to Zoom In"));
	QWhatsThis::add(b242,tr("Use this button to Zoom Out"));
	QWhatsThis::add(b243,tr("Use this button for a Zoom Window"));
	QWhatsThis::add(b244,tr("Use this button to Zoom Extents"));
	QWhatsThis::add(b245,tr("Use this button to show the Previous view"));
	QWhatsThis::add(b247,tr("Use this button to Zoom Center"));
	QWhatsThis::add(b21,tr("Use this button to redraw the screen without recalculating the vectors"));
	QWhatsThis::add(b23,tr("Use this button to Pan across the drawing"));
	QWhatsThis::add(b25,tr("Display the 3D view dialog box to dynamically change your view direction"));
	QWhatsThis::add(b27,tr("Restore the WCS plan view of the drawing"));
	QWhatsThis::add(b51,tr("Display the Help Manual"));
	QWhatsThis::add(p2layerbar,tr("Layer Manager Bar"));
	QWhatsThis::add(p2setbar,tr("Settings Bar - Use the buttons on this bar to change drawing settings"));

}

//	private functions
//	function to disable or enable various menu and tool buttons
//	if oo is true grey out (or disable) the item
//	if oo is false activate the item
void DrawingBoard::greyout(bool oo)
{
//	toggle the menu items that can be greyed out
	p2filemenu->setItemEnabled(ID_FILE_SAVE,!oo);
	p2filemenu->setItemEnabled(ID_FILE_SAVE_AS,!oo);
	p2filemenu->setItemEnabled(ID_FILE_PRINT,!oo);
	p2filemenu->setItemEnabled(ID_FILE_CLOSE,!oo);
	p2filemenu->setItemEnabled(ID_FILE_AUDIT,!oo);

	p2viewmenu->setItemEnabled(ID_VIEW_REDRAW,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_REGEN,!oo);								
	p2viewmenu->setItemEnabled(ID_VIEW_PAN,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_PAN_POINT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_PAN_LEFT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_PAN_UP,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_PAN_RIGHT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_PAN_DOWN,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_ZOOM,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_ZOOM_IN,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_ZOOM_OUT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_ZOOM_WINDOW,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_ZOOM_EXTENTS,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_ZOOM_PREVIOUS,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_ZOOM_VIEWMAX,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_ZOOM_CENTER,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_3DVIEW,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_PREDEFINED_VIEW,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_TOP,!oo);
  p2viewmenu->setItemEnabled(ID_VIEW_FRONT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_RIGHT_SIDE,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_LEFT_SIDE,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_BACK,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_BOTTOM,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_AERIAL_RFRONT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_AERIAL_LFRONT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_AERIAL_RREAR,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_AERIAL_LREAR,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_BELOW_RFRONT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_BELOW_LFRONT,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_BELOW_RREAR,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_BELOW_LREAR,!oo);
	p2viewmenu->setItemEnabled(ID_VIEW_WCSPLANVIEW,!oo);

	p2settingsmenu->setItemEnabled(ID_SETTINGS_LAYERDIAG,!oo);

//	now toggle the tool bars that can be greyed out
//	note that help is always on
	p2viewtoolbar->setDisabled(oo);
	p2layerbar->setDisabled(oo);

//	only these buttons on the filetoolbar should change state
	if (QToolButton* p2tb = dynamic_cast<QToolButton*>(p2filetoolbar->child("b12"))) p2tb->setDisabled(oo);
	if (QToolButton* p2tb = dynamic_cast<QToolButton*>(p2filetoolbar->child("b13"))) p2tb->setDisabled(oo);
	if (QToolButton* p2tb = dynamic_cast<QToolButton*>(p2filetoolbar->child("b14"))) p2tb->setDisabled(oo);
}

///////////////////////////////////////////////////////////////////////////////////////////
//	custom EVENT HANDLERS
////////////////////////////////////////////////////////////////////////////////////////////

//	resize event handler - redraw the screen when a resize occurs
void DrawingBoard::resizeEvent(QResizeEvent*)
{
   AD_OBJHANDLE psBlockHandle;

   if (!filedata.dwghandle) return;

   const short regenredraw = 1;      //    force a regeneration to the new window size
   p2setbar->getLayoutHandle(&psBlockHandle);
   resizewindow(filedata.dwghandle,psBlockHandle,regenredraw);
}

//   show event handler - redraw the screen when we return from iconization
void DrawingBoard::showEvent(QShowEvent*)
{
   AD_OBJHANDLE psBlockHandle;

   if (!filedata.dwghandle) return;

   const short regenredraw = 0;      //   allow a redraw
   p2setbar->getLayoutHandle(&psBlockHandle);
   resizewindow(filedata.dwghandle,psBlockHandle,regenredraw);
}

//	event filter - process ESC pressed, send everything else along provided
//	Drawingboard does not own keyboard focus
bool DrawingBoard::eventFilter(QObject* o, QEvent* e)
{
	if (e->type() == QEvent::KeyPress)
	{
		keypressed = true;
		QKeyEvent* k = (QKeyEvent*)e;
		if (k->key() == Qt::Key_Escape)
		{
			printMessage(GetResourceString(LX_CANCEL));
			cmdbar->clearCmdBar();
			setCancelPending(true);
			return true;	//	we processed this keystroke so no reason to send it along
		}

		if (hasFocus())	return true;	//	we own the keyboard so eat anything else
	}
	return false;	//	we don't want the event so send it along            	
}

//	Public SLOTS
//	slot that calls the opendrawing file routine.  Argument s is null by default
//	s is checked in fileio and if null opens the file dialog box.  If s is defined
//	then the program assumes it is a file name and tries to open it.
void DrawingBoard::openfile(const QString& s)
{
	extern bool lookforxrefs;
	extern bool lookforfonts;
	extern bool extentscalculated;
	AD_DB_HANDLE olddwghandle;
	
	cmdbar->addCommand("OPEN");
	printMessage(GetResourceString(ID_FILE_OPEN));
	olddwghandle = NULL;
	setCancelPending(false);
	if (filedata.dwghandle) olddwghandle = filedata.dwghandle;
	
	short rtnval = filedata.openfile(s);
	if (rtnval <  0) return;				//	no new file selected, file could not be read, or does not exist
	if (rtnval == 0)								//	file read, but returned an error
	{
		LX_ClearScreen();
		if (filedata.dwghandle)
		{
		  adTerminateVectorize(filedata.dwghandle);
			adCloseFile(filedata.dwghandle);	
		}	
		p2coords->clear();
		extentscalculated = false;
		p2layerbar->clearcombos();
		filedata.openname = QString();
		filedata.savename = QString();
		filedata.dwghandle = NULL;
		setCaption(tr("LX-Drawing Viewer:"));
		greyout(true);
		return;
	}

	threedview->setInitial3DViewParms();
	p2layerbar->initializebar(filedata.dwghandle);
	greyout(false);

	setcontrolvariables(); //	function in vectorize.cpp (for OD-viewkit)
	initializegraphics();  //	function in vectorize.cpp (to set graphics area)
	LX_ClearScreen();
	p2setbar->initializebar(filedata.dwghandle);	//	This will also vectorize the drawing
	lookforxrefs = FALSE;	// we have notified the user of all unresolved xrefs and fonts during
	lookforfonts = FALSE;	// the file load so don't do it during regens, pans etc.
	setCaption(tr("LX-Drawing Viewer: [ %1 ]").arg(filedata.openname));
	if (olddwghandle){adTerminateVectorize(olddwghandle); adCloseFile(olddwghandle);}
}

//   slot to close a file (may not be real useful but it dosen't hurt to include it)
void DrawingBoard::closefile()
{
   extern bool extentscalculated;

	if (!filedata.dwghandle) return;
	printMessage(GetResourceString(ID_FILE_CLOSE));	
	printMessage(filedata.openname);
	LX_ClearScreen();
	adTerminateVectorize(filedata.dwghandle);
	adCloseFile(filedata.dwghandle);	
	p2coords->clear();
	extentscalculated = false;
	p2layerbar->clearcombos();
	filedata.openname = QString();
	filedata.savename = QString();
	filedata.dwghandle = NULL;
	setCaption(tr("LX-Drawing Viewer:"));
	greyout(true);
	cmdbar->addCommand("CLOSE");
}

//	slot that calls adAudit and prints the results to the text window
//	passmessage is defined in OD_interface.
//	Return -1 if no drawing loaded, 0 if errors found, and 1 if no errors
short DrawingBoard::auditdrawing()
{
   const short auditmode = AD_SAFEMODE;
   const short auditverb = AD_VERBOSITY_QUIET;

   if (!filedata.dwghandle) return -1;
   if (!p2textwin->isVisible()) toggletextwin();
   printMessage(GetResourceString(ID_FILE_AUDIT) + filedata.openname);
   short rtnval = adAudit(filedata.dwghandle,auditmode,auditverb,&passmessage);
   switch(rtnval)
   {
      case 1:
         printMessage(tr("No errors found in the audit "));
         break;
      case 0:
         printMessage(tr("The drawing audit found at least one error - please review the output above. "));
         break;
   }
   cmdbar->addCommand("AUDIT");
   return rtnval;
}

//   slot to display the list of available commands
void DrawingBoard::displayTWINmessage(int key)
{
   if (!p2textwin->isVisible()) toggletextwin();

   switch (key)
   {
      case CALL_HELP:
         printMessage(GetResourceString(CALL_HELP));
         printMessage(GetResourceString(COMMAND_LIST),TWIN);
      break;
      case CALL_VERSION:
         printMessage(GetResourceString(CALL_VERSION));
         printMessage(tr("LX-Viewer Version: %1").arg(LX_VERSION));
      break;
      default:
      printMessage(GetResourceString(COMMAND_LIST),TWIN);
   }
}

//   slot to display the report dialog
short DrawingBoard::showreport(const QString& cmd)
{
   cmdbar->addCommand("REPORT");

   if (!filedata.dwghandle && cmd.isNull())
   {
      report->initializeListView();
      report->show();
      return -1;
   }

   report->initializeListView();
   report->setOptions(cmd);    // initialize the report listview

   report->show();
   return 1;
}

//   slot that calls the quicksave file routine
void DrawingBoard::quicksave()
{
   if (!filedata.dwghandle) return;
   printMessage(GetResourceString(ID_FILE_SAVE) + filedata.openname);
   filedata.quicksave();
   cmdbar->addCommand("QSAVE");
}

//   slot that calls the file save as routine
void DrawingBoard::saveas()
{
	if (!filedata.dwghandle) return;
	printMessage(GetResourceString(ID_FILE_SAVE_AS));
	setCancelPending(false);
	filedata.saveas();
	cmdbar->addCommand("SAVE");
}

//   slot that cleans up when quit is called
void DrawingBoard::quit()
{
   if (filedata.dwghandle) adTerminateVectorize(filedata.dwghandle);
   if (filedata.dwghandle) adCloseFile(filedata.dwghandle);
   if (p2painter->isActive()) p2painter->end();
   if (snapshotpainter->isActive()) snapshotpainter->end();
   if (miscpainter.isActive()) miscpainter.end();
   qApp->closeAllWindows();
}

//   slot that toggles between a white and black (default) background
void DrawingBoard::togglebackground()
{
   if (whitebackground)
   {
      p2dwgarea->backgroundcolor = QColor(0x00,0x00,0x00);
      p2dwgarea->setBackgroundColor(p2dwgarea->backgroundcolor);
    whitebackground = false;
      p2toolmenu->setItemChecked(ID_TOOL_BACKGROUND_COLOR,true);
   }
   else
   {
     p2dwgarea->backgroundcolor = QColor(0xff,0xff,0xff);
      p2dwgarea->setBackgroundColor(p2dwgarea->backgroundcolor);
    whitebackground = true;
      p2toolmenu->setItemChecked(ID_TOOL_BACKGROUND_COLOR,false);
   }
   if (!filedata.dwghandle) LX_ClearScreen();
   else switchview(ID_VIEW_REDRAW);
}

//   slot that toggles the status bar on and off
void DrawingBoard::togglesbar()
{
   if (p2sbar->isVisible())
      {
         p2sbar->hide();
         p2toolmenu->setItemChecked(ID_TOOL_STATUSBAR,false);
      }
   else
      {
         p2sbar->show();
         p2toolmenu->setItemChecked(ID_TOOL_STATUSBAR,true);
      }
}

//   slot that toggles the text window up and down (on and off)
void DrawingBoard::toggletextwin()
{
   if (p2textwin->isVisible())
      {
         p2textwin->hide();
         p2toolmenu->setItemChecked(ID_TOOL_TEXTWINDOW,false);
      }
   else
      {
         p2textwin->show();
         p2toolmenu->setItemChecked(ID_TOOL_TEXTWINDOW,true);
      }
}

//   slot that toggles the command bar on and off
void DrawingBoard::togglecmdbar()
{
   if (cmdbar->isVisible())
      {
         cmdbar->hide();
         p2toolmenu->setItemChecked(ID_TOOL_CMDBAR,false);
      }
   else
      {
         cmdbar->show();
         p2toolmenu->setItemChecked(ID_TOOL_CMDBAR,true);
      }
}

//   slot that will toggle on and off a tool bar
void DrawingBoard::toggletoolbar(int keynumber)
{
   QToolBar* tempptr = 0;

   switch (keynumber)
   {
      case ID_TOOL_TOOLBAR_FILE:
         tempptr = p2filetoolbar;
         break;
      case ID_TOOL_TOOLBAR_VIEW:
         tempptr = p2viewtoolbar;
         break;
      case ID_TOOL_TOOLBAR_LAYER:
         tempptr = p2layertoolbar;
         break;
      case ID_TOOL_TOOLBAR_HELP:
         tempptr = p2helptoolbar;
         break;
   }

   if (tempptr->isVisible())
      {
         tempptr->hide();
         p2toolmenu->setItemChecked(keynumber,false);
      }
   else
      {
         tempptr->show();
         p2toolmenu->setItemChecked(keynumber,true);
      }
}

//   slot that will toggle the 3D dynamic view dialog up and down
void DrawingBoard::toggle3Dview()
{
   if (!filedata.dwghandle)
      {
         DisplayMessage(NO_DRAWING_LOADED_3D);
         return;
      }
   else
      {
         printMessage(GetResourceString(ID_VIEW_3DVIEW));
         int rtnval = threedview->exec();
         if (rtnval == QDialog::Accepted) call3Dtransform();
      }
}

//   slot that will toggle (display) the user manual
void DrawingBoard::togglemanual()
{
   if (manual->isVisible())
      {
          manual->hide();
      }
   else
      {
         manual->show();
         printMessage(GetResourceString(ID_HELP_MAN));
         cmdbar->addCommand("MAN");
      }
}

//   slot that will change the size of the tool buttons
void DrawingBoard::togglebuttonsize(int item_number)
{
   bool tf = false;

   p2toolmenu->isItemChecked(item_number)? tf = false: tf = true;
   p2toolmenu->setItemChecked(item_number,tf);
   this->setUsesBigPixmaps(tf);
}

//   slot to toggle the tool tips on and off
void DrawingBoard::toggletooltips()
{
   bool setting = QToolTip::isGloballyEnabled();

   QToolTip::setGloballyEnabled(setting ? false : true);
   p2toolmenu->setItemChecked(ID_TOOL_TOOLTIP_ENABLE,(setting ? false : true));
}

//   slot that will display a help or about message box on the screen
void DrawingBoard::displayaboutmessage(int keynumber){DisplayMessage(keynumber,CAPTION_ABOUT,QMessageBox::NoIcon);}

//   slot that selects the proper function to call depending on the VIEW MENU selected
void DrawingBoard::switchview(int keynumber)
{
	//	grab the keyboard and mouse to keep people from flooding the input buffer by
	//	holding down keys, clear the screen since we are changing the picture
	if (!filedata.dwghandle) return;
	printMessage(GetResourceString(keynumber));
	grabKeyboard();
	QPoint anchorpoint;
	
	const float zoomfactor = 0.8f;	//	zoom in or out by this factor
	const float panfactor = 0.2f;   //	pan across the screen by this factor
	setCancelPending(false);

	switch (keynumber)
	{
		case ID_VIEW_REDRAW:
			LX_Redraw(filedata.dwghandle);
			break;
		case ID_VIEW_REGEN:
			cmdbar->addCommand("REGEN");
			LX_Regen(filedata.dwghandle);
			break;
		case ID_VIEW_PAN:
			cmdbar->addCommand("PAN");
			keypressed = false;	
			LX_InitializePoints();
			releaseKeyboard();
			do
			{
				anchorpoint=getselectedpoint();
				qApp->processOneEvent();
			} while (anchorpoint.x() < 0 && !keypressed);
			if (anchorpoint.x() >= 0){grabKeyboard();	LX_Pan(filedata.dwghandle,getselectedpoint());}
			break;
		case ID_VIEW_PAN_POINT:
			cmdbar->addCommand("PAN");
			LX_Pan(filedata.dwghandle);
			break;
		case ID_VIEW_PAN_UP:
			cmdbar->addCommand("PAN UP");
			LX_PanOrtho(filedata.dwghandle,0.0,panfactor);
			break;	
		case ID_VIEW_PAN_RIGHT:		
			cmdbar->addCommand("PAN RIGHT");
			LX_PanOrtho(filedata.dwghandle,panfactor,0.0);
			break;		
		case ID_VIEW_PAN_DOWN:
			cmdbar->addCommand("PAN DOWN");
			LX_PanOrtho(filedata.dwghandle,0.0,panfactor * (-1.0) );
			break;		
		case ID_VIEW_PAN_LEFT:
			cmdbar->addCommand("PAN LEFT");
			LX_PanOrtho(filedata.dwghandle,panfactor * (-1.0),0.0);
			break;				
		case ID_VIEW_ZOOM_IN:
			cmdbar->addCommand("ZOOM IN");
			LX_ZoomIn(filedata.dwghandle,zoomfactor);
			break;
		case ID_VIEW_ZOOM_OUT:
			cmdbar->addCommand("ZOOM OUT");
			LX_ZoomOut(filedata.dwghandle,zoomfactor);
			break;
		case ID_VIEW_ZOOM_WINDOW:
			cmdbar->addCommand("ZOOM WINDOW");
			LX_ZoomWindow(filedata.dwghandle);
			break;
		case ID_VIEW_ZOOM_EXTENTS:
			cmdbar->addCommand("ZOOM EXTENTS");
			LX_ZoomExtents(filedata.dwghandle);
			break;
		case ID_VIEW_ZOOM_PREVIOUS:
			cmdbar->addCommand("ZOOM PREVIOUS");
			LX_ZoomPrevious(filedata.dwghandle);
			break;
		case ID_VIEW_ZOOM_VIEWMAX:
			cmdbar->addCommand("ZOOM VIEWMAX");
			LX_ZoomViewmax(filedata.dwghandle);
			break;
		case ID_VIEW_ZOOM_CENTER:
			cmdbar->addCommand("ZOOM CENTER");
			LX_ZoomCenter(filedata.dwghandle);
			break;
		case ID_VIEW_AERIAL_LFRONT:
			LX_PredefinedView(filedata.dwghandle,1);
			break;
		case ID_VIEW_FRONT:
			LX_PredefinedView(filedata.dwghandle,2);
			break;
		case ID_VIEW_AERIAL_RFRONT:
			LX_PredefinedView(filedata.dwghandle,3);
			break;
		case ID_VIEW_LEFT_SIDE:
			LX_PredefinedView(filedata.dwghandle,4);
			break;
		case ID_VIEW_TOP:
			LX_PredefinedView(filedata.dwghandle,5);
			break;
		case ID_VIEW_WCSPLANVIEW:
			LX_PredefinedView(filedata.dwghandle,5);
			break;	
		case ID_VIEW_RIGHT_SIDE:
			LX_PredefinedView(filedata.dwghandle,6);
			break;
		case ID_VIEW_AERIAL_LREAR:
			LX_PredefinedView(filedata.dwghandle,7);
			break;
		case ID_VIEW_BACK:
			LX_PredefinedView(filedata.dwghandle,8);
			break;
		case ID_VIEW_AERIAL_RREAR:
			LX_PredefinedView(filedata.dwghandle,9);
			break;
		case ID_VIEW_BOTTOM:
			LX_PredefinedView(filedata.dwghandle,11);
			break;
		case ID_VIEW_BELOW_RFRONT:
			LX_PredefinedView(filedata.dwghandle,12);
			break;
		case ID_VIEW_BELOW_LFRONT:
			LX_PredefinedView(filedata.dwghandle,13);
			break;
		case ID_VIEW_BELOW_RREAR:
			LX_PredefinedView(filedata.dwghandle,14);
			break;
		case ID_VIEW_BELOW_LREAR:
			LX_PredefinedView(filedata.dwghandle,15);
			break;
	}
	releaseKeyboard();
}

//   slot that calls the 3D view routine in vectorize
//   this is connected to the OK button in 3DViewDialog
void DrawingBoard::call3Dtransform()
{
   extern AD_VIEW_PARMS adviewparms;

   threedview->camera = threedview->normalize(threedview->camera);
   threedview->up = threedview->normalize(threedview->up);

   adviewparms.viewdir[0] = threedview->camera.x;
   adviewparms.viewdir[1] = threedview->camera.y;
   adviewparms.viewdir[2] = threedview->camera.z;
   adviewparms.up[0] = threedview->up.x;
   adviewparms.up[1] = threedview->up.y;
   adviewparms.up[2] = threedview->up.z;

   LX_3DView(filedata.dwghandle);
}

//   slot to display the plotter dialog
void DrawingBoard::callplotter()
{
	AD_OBJHANDLE psBlockHandle;
	p2setbar->getLayoutHandle(&psBlockHandle);
	cmdbar->addCommand("PLOT");
	printMessage(GetResourceString(ID_FILE_PRINT));
	setCancelPending(false);

	int r = -99;
	do
	{
	 	r = plotter->exec();
	
		if (r == 1)   						//	plot postscript
		{
			if (!filedata.dwghandle){DisplayMessage(NO_DRAWING_LOADED_PRINT);return;}
			miscpainter.begin(plotter->plotter);
			plotps(filedata.dwghandle,psBlockHandle);
			miscpainter.end();
			break;
		}	//	if r == 1 - plot ps

		if (r == 2)		            //	plot HPGL
		{
			if (!filedata.dwghandle){DisplayMessage(NO_DRAWING_LOADED_PRINT);return;}
			plothpgl(filedata.dwghandle,psBlockHandle);
			break;
		}	//	if r == 2 - plot hpgl

		if (r == 0) break;	//	cancel pressed

		if (r == -1)		// user needs help
		{
			if (!manual->getPath().isNull())
			{
				QString source = manual->getPath() + "/index-5.html";
				QFileInfo fi = QFileInfo(source);
				if (fi.exists()) manual->setSource(source);
			}
			manual->show();
			break;
		}	//	if r = -1  need to display a help page
		
		if (r == -2)		//	closed the dialog to get a plot window
		{
			extern int rubberbandtype;
			QString coordstring = tr("\n\tFrom: ");
			QString scratchtext;
			QPoint anchorpoint;
			QPoint endpoint;

			LX_SetCursor(QCursor(Qt::CrossCursor));
			printMessage(tr("Window base point"));
			LX_InitializePoints();
			do
			{
				anchorpoint=getselectedpoint();
				qApp->processOneEvent();
				scratchtext = p2coords->text();
			} while (anchorpoint.x()<0);
 			coordstring.append(scratchtext);

			rubberbandtype=RUBBERBOX;
			LX_InitializePoints();
			printMessage(tr("Other Corner"));
			do
			{
				endpoint=getselectedpoint();
				qApp->processEvents();
				scratchtext =  p2coords->text();
			} while (endpoint.x()<0);	
 	
			rubberbandtype=NORUBBERBAND;
			LX_SetCursor(QCursor(Qt::ArrowCursor));
			coordstring.append(tr("\n\tTo: "));
			coordstring.append(scratchtext);	
			plotter->setPlotCoordsLabel(anchorpoint,endpoint,coordstring);
		}	//	r == -2 - needed to select a plot window
	} while (r < 0);
}

//   slot to call the layer dialog
void DrawingBoard::calllayerdialog(){p2layerbar->showdialog();}

//   slot to call the alias dialog
void DrawingBoard::callaliasdialog(){cmdbar->showEditAliasDialog();}

//   slot to catch signals, and to print program notes and updates
//   basic version prints to text window and status bar,
//   overloaded version specifies where to prin

// show messages for 2 seconds
const int messageDisplayTime = 2000;

void DrawingBoard::printMessage(const QString& s)
{
   QString t = s.simplifyWhiteSpace();

   p2textwin->append(s);
   p2sbar->message(t, messageDisplayTime);
}

void DrawingBoard::printMessage(const QString& s, unsigned char printto)
{
   QString t = s.simplifyWhiteSpace();

   if (isBitSet(printto,1)) p2sbar->message(t, messageDisplayTime);   //   print to status bar
   if (isBitSet(printto,2)) p2textwin->append(s);   //   print to text window
///   if (isBitSet(printto,4)) function here         ;   //   print to command bar if ever needed
}

//	batching slots
void DrawingBoard::batchConvert(){cmdbar->addCommand("BCONV");filedata.batchProcess();}
void DrawingBoard::batchPlot(){cmdbar->addCommand("BPLOT");plotter->batchProcess();}

//	command slots, link tool buttons which don't emit integers and other signals
//	with incompatable arguments to these
void DrawingBoard::openfiledia(){openfile(QString());}
void DrawingBoard::showreportdia(){showreport(QString());}
void DrawingBoard::redraw() {switchview (ID_VIEW_REDRAW);}
void DrawingBoard::pan() {switchview(ID_VIEW_PAN_POINT);}
void DrawingBoard::panleftslot() {switchview(ID_VIEW_PAN_LEFT);}
void DrawingBoard::panupslot() {switchview(ID_VIEW_PAN_UP);}
void DrawingBoard::panrightslot() {switchview(ID_VIEW_PAN_RIGHT);}
void DrawingBoard::pandownslot() {switchview(ID_VIEW_PAN_DOWN);}
void DrawingBoard::wcsplanview() {switchview(ID_VIEW_WCSPLANVIEW);}
void DrawingBoard::zoomin() {switchview(ID_VIEW_ZOOM_IN);}
void DrawingBoard::zoomout() {switchview(ID_VIEW_ZOOM_OUT);}
void DrawingBoard::zoomwindow() {switchview(ID_VIEW_ZOOM_WINDOW);}
void DrawingBoard::zoomextents() {switchview(ID_VIEW_ZOOM_EXTENTS);}
void DrawingBoard::zoomprevious() {switchview(ID_VIEW_ZOOM_PREVIOUS);}
void DrawingBoard::zoomcenter() {switchview(ID_VIEW_ZOOM_CENTER);}

//   Public Functions:

//   Function that receives an integer representing the percent complete for
//   file loads, saves and scans.  This function receives the short integer as an argumen
//   and broadcasts it as a signal.

void DrawingBoard::LX_ProgBarVal(const int& pct)
{
   if (pct == 100)
   {
      p2progbar->reset();
      p2progbar->hide();
   }
   else
   {
      if (p2progbar->progress() != pct)
      {
         if (!p2progbar->isVisible()) p2progbar->show();
         p2progbar->setProgress(pct);
         qApp->processEvents();
       }
   }
}

//   Function to return the max x (width) and y (height) of the drawing area
long DrawingBoard::XY(short idx)
{
   QPaintDeviceMetrics pdm(p2dwgarea);
   if (idx == 0) return static_cast<long>(pdm.width());
   if (idx == 1) return static_cast<long>(pdm.height());


   return 0L;
}

//   Function to move the graphic cursor to (x,y) without drawing a line
void DrawingBoard::LX_MoveTo(const long& x, const long& y)
{
   p2painter->moveTo(x,y);
   snapshotpainter->moveTo(x,y);
}

//   Function to draw a line from the current point to a new poin
void DrawingBoard::LX_LineTo(const long& x, const long& y)
{
   p2painter->lineTo(x,y);
   snapshotpainter->lineTo(x,y);
}

//   Function to select the pen color
//   colors are based upon a black background (ACAD prior ot R12) so only need to adjus
//   the colors if a white background is selected, and then I only need to convert a returned
//   color white to black
void DrawingBoard::LX_PenColor(const QColor& acolor)
{
   QColor c = acolor;
   if (whitebackground && c == QColor(0xff,0xff,0xff)) c = QColor(0x00,0x00,0x00);
   p2painter->setPen(c);
   p2painter->setBrush(QBrush(c,SolidPattern));
   snapshotpainter->setPen(c);
   snapshotpainter->setBrush(QBrush(c,SolidPattern));
}

//   function to paint a polygon passed a QArray
void DrawingBoard::LX_PaintPoly(const QPointArray& a)
{
  int index = 0;
   bool winding = FALSE;
   int npoints = -1;

   p2painter->drawPolygon(a,winding,index,npoints);
   snapshotpainter->drawPolygon(a,winding,index,npoints);
}

//   Plotter versions of the functions above
void DrawingBoard::LX_PlotMoveTo(const long& x, const long& y)
{
   miscpainter.moveTo(x,y);
}

void DrawingBoard::LX_PlotLineTo(const long& x, const long& y)
{
   miscpainter.lineTo(x,y);
}

void DrawingBoard::LX_PlotPenColor(const int& ac)
{
   QColor outcolor;
   int nib;

   outcolor = LX_GetColor(plotter->getpencolor(ac));
   nib = plotter->getoutpenwidth(ac);

   miscpainter.setPen(QPen(outcolor,nib,Qt::SolidLine,Qt::SquareCap,Qt::BevelJoin));
   miscpainter.setBrush(QBrush(outcolor,Qt::SolidPattern));
}

//   function to paint a polygon passed a QArray
void DrawingBoard::LX_PlotPaintPoly(const QPointArray& a)
{
  int index = 0;
   bool winding = FALSE;
   int npoints = -1;

   miscpainter.drawPolygon(a,winding,index,npoints);
}

//   function to clear the graphics screen and reset the snapshot pixmap
void DrawingBoard::LX_ClearScreen(void)
{
   p2painter->fillRect(0,0,XY(0),XY(1),p2dwgarea->backgroundcolor);

   if (snapshotpainter->isActive()) snapshotpainter->end();
   p2dwgarea->snapshot.resize(XY(0),XY(1));
   p2dwgarea->snapshot.fill(p2dwgarea->backgroundcolor);
   snapshotpainter->begin(&p2dwgarea->snapshot);
}

//   function to set the Cursor in the paint area
void DrawingBoard::LX_SetCursor(const QCursor& curs)
{
   p2dwgarea->setCursor(curs);
}

//   function to set the current state of RasterOp
void DrawingBoard::LX_SetRasterOp(const short& rop)
{
   switch(rop)
   {
      case XOR:
         p2painter->setPen(p2dwgarea->rubberbandcolor);
         p2painter->setRasterOp(XorROP);
         break;
      case COPY:
         p2painter->setPen((p2painter->brush()).color());
         p2painter->setRasterOp(CopyROP);
         break;
   }
}

//   function to paint a polyline
void DrawingBoard::LX_PaintPline(const QPointArray& a){p2painter->drawPolyline(a);}

//   function to initilize the mouse selection points
void DrawingBoard::LX_InitializePoints(){p2dwgarea->presspoint=QPoint(-1,-1);}

//   function to return the mouse poin
QPoint DrawingBoard::getselectedpoint(void){return p2dwgarea->presspoint;}

//   function to return the name of the currently open file
QString DrawingBoard::getOpenFileName(void){return  filedata.openname;}

//   function to return the AD_DB_HANDLE of the currently open file
AD_DB_HANDLE DrawingBoard::getDwgHandle(void){return filedata.dwghandle;}

//   function to look for the support files and directories in the users home
//   directory.  If not found ask the user to create them, and finally create
//   them if permission is granted
void DrawingBoard::lookForSpptFiles()
{
	QDir lxhome = QDir::homeDirPath() + LXV_HOME;
	QDir lxfont = QDir::homeDirPath() + LXV_HOME + LXV_FONT;
	QDir lxpcp = QDir::homeDirPath() + LXV_HOME + LXV_PCP;
	
	if (!lxhome.exists() || !lxfont.exists() || !lxpcp.exists())
	{
		QMessageBox* mb = new QMessageBox (tr("Create Support Files"),
					tr("<center>LX-Viewer needs to create several support files and directories"
						" in your HOME directory.  "
						"<center>Will you allow LX-Viewer to create these files? "),
					QMessageBox::Information,
					QMessageBox::Yes | QMessageBox::Default,
					QMessageBox::No  | QMessageBox::Escape,
					0,this);
		int rtnval = mb->exec();
		delete mb;
		if (rtnval == QMessageBox::No) return; 	//	permission was not granted to make these files
		
		printMessage(tr("\nCreating Support Files and Directories"));
	  if (!lxhome.exists())
			if (lxhome.mkdir(lxhome.canonicalPath())) printMessage(tr("Directory: %1 has been created").arg(lxhome.canonicalPath()),TWIN);
		if (!lxfont.exists())
		{
			if (lxfont.mkdir(lxfont.canonicalPath())) printMessage(tr("Directory: %1 has been created").arg(lxfont.canonicalPath()),TWIN);
			QFile f(lxfont.canonicalPath() + "/readme_font.txt");
			if (f.open(IO_WriteOnly))
			{
				QTextStream ts(&f);
				ts << tr("Place your third party font files (*.shx) files in this directory");
				f.close();
				printMessage(tr("File: %1 has been created").arg(f.name()),TWIN);
			}
		}	//	font
		if (!lxpcp.exists())
		{
			if (lxpcp.mkdir(lxpcp.canonicalPath())) printMessage(tr("Directory: %1 has been created").arg(lxpcp.canonicalPath()),TWIN);
		  QFile f(lxpcp.canonicalPath() + "/readme_pcp.txt");
			if (f.open(IO_WriteOnly))
			{
				QTextStream ts(&f);
				ts << tr("Place your printer control ( *.pcp) files in this directory");
				f.close();
				printMessage(tr("File: %1 has been created").arg(f.name()),TWIN);
			}
		}	//	pcp
	}
}

//   getxxx functions for plotter subclass
double DrawingBoard::getplotareawidth() {return plotter->getplotareawidth();}
double DrawingBoard::getplotareaheight(){return plotter->getplotareaheight();}
double DrawingBoard::getoutputwidth()   {return plotter->getoutputwidth();}
double DrawingBoard::getoutputheight()  {return plotter->getoutputheight();}
double DrawingBoard::getmargin()            {return plotter->getmargin();}
QString DrawingBoard::gethpgloutdevice(){return plotter->gethpgloutdevice();}
short DrawingBoard::getpennumber(int pn){return plotter->getpencolor(pn);}
void DrawingBoard::setCancelPending(bool oo) {cancelpending = (oo ? true : false);}
bool DrawingBoard::getCancelPending() {return cancelpending;}
void DrawingBoard::batchConvertSetOptions(const QString& cmd){filedata.setOptions(cmd);}	
void DrawingBoard::batchPlotSetOptions(const QString& cmd){plotter->setOptions(cmd);}	
void DrawingBoard::setPlotParms(QPoint p1,QPoint p2)
{
	extern AD_VIEW_PARMS adviewparms;
	extern AD_VIEW_PARMS plotviewparms;

	switch (plotter->getPrintAreaSetting())
	{
	case 0:	//	plot the current view	
		for (short i = 0; i < 4; i++)
		{
			plotviewparms.target[i] = adviewparms.target[i];
			plotviewparms.viewdir[i] = adviewparms.viewdir[i];
			plotviewparms.up[i] = adviewparms.up[i];
		}
		break;
	case 2:	//	plot extents
		adZoomExtents(getDwgHandle(),&plotviewparms);
		break;
	case 4:	//	plot window
		double deltax = 0.0;
		double deltay = 0.0;
		QPoint fp = plotter->getWindowCorner(0);
		QPoint tp = plotter->getWindowCorner(1);

		deltax=((fp.x()+tp.x())/2.0-(XY(0)-1)/2.0)*adviewparms.xsize/(XY(0)-1);
		deltay=((fp.y()+tp.y())/2.0-(XY(1)-1)/2.0)*adviewparms.ysize/(XY(1)-1);
		deltay*=(-1.0);	// y pixels are positive downwards so we need to multiply by -1
		
		for (short i = 0; i < 4; i++)
		{
			plotviewparms.target[i] = adviewparms.target[i];
			plotviewparms.viewdir[i] = adviewparms.viewdir[i];
			plotviewparms.up[i] = adviewparms.up[i];
		}
		adPanTarget(&plotviewparms,deltax,deltay);
		plotviewparms.xsize = abs(fp.x() - tp.x()) * adviewparms.xsize/(XY(0)-1);
		plotviewparms.ysize = abs(fp.y() - tp.y()) * adviewparms.ysize/(XY(1)-1);
		break;
	}
	plotviewparms.xsize = getplotareawidth();
	plotviewparms.ysize = getplotareaheight();
}
//**********************************************************************************************
//********************************** TEXT WINDOW ***********************************************
//**********************************************************************************************
//   Text window constructor
TextWindow::TextWindow(QWidget* parent, const char* name) : QTextEdit(parent, name)
{
   setGeometry(50,150,700,450);
}

//   Text window close even
void TextWindow::closeEvent(QCloseEvent* e)
{
   e->accept();
   emit textWinClosed();
}

//**********************************************************************************************
//********************************** DRAWING AREA***********************************************
//**********************************************************************************************
//   Drawing area constructor
DrawingArea::DrawingArea(QWidget* parent, const char* name)
	:QWidget(parent,name,WNorthWestGravity),pline(2),mousepressed(FALSE)
{
   backgroundcolor = QColor(0x00,0x00,0x00);
   rubberbandcolor = QColor(0xff,0xff,0xff);

	setBackgroundColor(backgroundcolor);	
	setMouseTracking(TRUE);

	QPixmap snapshot;
}

//   Drawing Area mouse press even
void DrawingArea::mousePressEvent(QMouseEvent* e)
{
   mousepressed=TRUE;
   presspoint=e->pos();
   pline[0]=pline[1]=e->pos();
}

//   Drawing Area mouse release even
void DrawingArea::mouseReleaseEvent(QMouseEvent*)
{
   mousepressed=FALSE;
}

//   Drawing Area mouse move even
void DrawingArea::mouseMoveEvent(QMouseEvent* e)
{
	extern DrawingBoard* db;
	extern int rubberbandtype;
	extern bool extentscalculated;

	static double pixpt[2];
	static double coord[3];
	static const QString comma(" , ");
	static const int decimalplaces = 3;

	static QString coordstring;
	
	if (!extentscalculated) return;

	currentpoint=e->pos();
	pixpt[0]=static_cast<double>(currentpoint.x());
	pixpt[1]=static_cast<double>(db->XY(1)-currentpoint.y());
	if (db->getDwgHandle())
	{
		adCurrentPixelToWorld(db->getDwgHandle(),pixpt,coord);
		coordstring=QString::number(coord[0],'f',decimalplaces)+comma+
							QString::number(coord[1],'f',decimalplaces)+comma+
							QString::number(coord[2],'f',decimalplaces);
		emit mouseCoordinate(coordstring);
	}

	switch (rubberbandtype)
		{
			case NORUBBERBAND:
				db->LX_SetRasterOp(COPY);
				break;
			case RUBBERLINE:
				db->LX_SetRasterOp(XOR);
				db->LX_PaintPline(pline);
				pline[0]=currentpoint;	
				db->LX_PaintPline(pline);
				db->LX_SetRasterOp(COPY);
				break;
			case RUBBERBOX:
				db->LX_SetRasterOp(XOR);
				db->LX_PaintPline(QPointArray(QRect(pline[1],pline[0]),TRUE));
				pline[0]=currentpoint;	
				db->LX_PaintPline(QPointArray(QRect(pline[1],pline[0]),TRUE));
			  db->LX_SetRasterOp(COPY);
				break;
			default:
				db->LX_SetRasterOp(COPY);
				break;
		}
}

//   paint event for drawing area.  Basically just repaint the screen
//   with the saved snapsho
void DrawingArea::paintEvent(QPaintEvent*)
{
   bitBlt(this,0,0,&snapshot,0,0,-1,-1,Qt::CopyROP);
}
