/***************************************************************************
                          drawingboard.h
                             -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/


/*
	This is the header file to define the main work area or the drawingboard
	It is based upon the QMainWindow widget
*/


#ifndef DRAWINGBOARD_H
#define DRAWINGBOARD_H

//	QT includes
#include <qapplication.h>
#include <qcolor.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qwidget.h>
#include <qstatusbar.h>
#include <qprogressbar.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qtextedit.h>
#include <qaccel.h>
#include <qevent.h>
#include <qpointarray.h>
#include <qpoint.h>
#include <qstring.h>
#include <qrect.h>
#include <qlabel.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpixmap.h>
#include <qwhatsthis.h>
#include <qiconset.h>
#include <qdir.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qtextstream.h>


//	LX includes
#include "resource.h"
#include "OD_interface.h"
#include "fileio.h"
#include "vectorize.h"
#include "helpabout.h"
#include "infomessage.h"
#include "3Dview.h"
#include "plotter.h"
#include "utilities.h"
#include "layermanager.h"
#include "drawingenvironment.h"
#include "commandproc.h"
#include "report.h"

//	c++ includes
#include <math.h>

//	class to define the text window, really just a QTextEdit object with a
//	custom close event to link window closes (hides) with the check boxes on the menus
class TextWindow : public QTextEdit
{
	Q_OBJECT
	public:
		TextWindow(QWidget* parent=0,const char*name = 0);
	protected:
		void closeEvent(QCloseEvent*);
  signals:
		void textWinClosed(void);
};
//	class to define the drawing area, really just a QWidget object with
//	various data members and event handlers
class DrawingArea : public QWidget
{
	Q_OBJECT
	public:
		DrawingArea(QWidget* parent=0,const char* name = 0);
	
		//	public data members
		QPoint presspoint;
		QPoint currentpoint;
		QPointArray pline;
		QColor backgroundcolor;
		QColor rubberbandcolor;
		QPixmap snapshot;					//	pixmap to hold a snapshot of the graphics screen
		
	protected:
		//	data members
		bool mousepressed;
		
		//	event handlers
		void mousePressEvent(QMouseEvent*);
		void mouseReleaseEvent(QMouseEvent*);
		void mouseMoveEvent(QMouseEvent*);
		void paintEvent(QPaintEvent*);

	signals:
		void mouseCoordinate(const QString&);
};

//	Main display class -- this is the desktop you see when the program starts
class DrawingBoard : public QMainWindow
{
	Q_OBJECT

	public:
		DrawingBoard(QWidget* parent = 0, const char* name = 0);
		
		void LX_ProgBarVal(const int&);
		long XY(short);
		void LX_MoveTo(const long& x, const long& y);
		void LX_LineTo(const long& x, const long& y);
		void LX_PenColor(const QColor& acolor);
		void LX_PaintPoly(const QPointArray& a);	
		void LX_PlotMoveTo(const long& x, const long& y);
		void LX_PlotLineTo(const long& x, const long& y);
		void LX_PlotPenColor(const int& acolor);
		void LX_PlotPaintPoly(const QPointArray& a);	
		void LX_ClearScreen(void);
		void LX_SetCursor(const QCursor&);
		void LX_SetRasterOp(const short&);
		void LX_PaintPline(const QPointArray& a);
		void LX_InitializePoints();
		QPoint getselectedpoint(void);
		QString getOpenFileName(void);
		AD_DB_HANDLE getDwgHandle(void);
		void lookForSpptFiles();						//	function to look for the support files and then create them
		double getplotareawidth();	        //	function to return the width of the plot area
		double getplotareaheight();					//	function to return the height of the plot area
		double getoutputwidth();						//	function to return the width of the output area
		double getoutputheight();						//	function to return the height of the output area	
		double getmargin();									//	function to return the paper margin in pixels	
		QString gethpgloutdevice();					//	function to return the selected hpgl output device
		short getpennumber(int);						//	function to return the pen number (same as output color
		void setCancelPending(bool);				//	function to set the cancel pending flag
		bool getCancelPending();						//	function to return the value of the cancel pending flag
		void batchConvertSetOptions(const QString&);	//	function to read the convert BCF
		void batchPlotSetOptions(const QString&);			//	function to read the plot BCF
		void setPlotParms(QPoint p1 = QPoint(), QPoint p2 = QPoint());	//	function to set the plot paramaters
		
		//	data members
		QPainter miscpainter;								//	painter available for plotting etc.
		DrawingArea*	p2dwgarea;						//	drawing area widget
		LX_CommandBar* cmdbar;							//	command bar
	
	private:
//	define the  painter for for the main drawing area and the snapshot		
		QPainter* p2painter;
		QPainter* snapshotpainter;
		
//	now define the textwindow
		TextWindow* p2textwin;

//	now define the popupmenus	and menu buttons
		QPopupMenu*	p2filemenu;
		QPopupMenu*	p2viewmenu;
			QPopupMenu* p2pan;
			QPopupMenu*	p2zoom;
			QPopupMenu* p2predef;
		QPopupMenu* p2toolmenu;
		QPopupMenu* p2settingsmenu;
		QPopupMenu*	p2helpmenu;

//	and some toolbars also
		QToolBar* p2filetoolbar;
		QToolBar* p2viewtoolbar;
		QToolBar* p2layertoolbar;
			LX_LayerBar* p2layerbar;
		QToolBar* p2helptoolbar;
	
//	now define the status bar and its children
		QStatusBar*		p2sbar;
		QProgressBar* p2progbar;
		QLabel*				p2coords;
		LX_SettingsBar* p2setbar;
		
//	now declare the sub-class members
		fileio filedata;
		LX_3DViewDialog*	threedview;
		HelpWindow*	manual;
		LX_PlotDialog* plotter;
		LX_ReportDialog* report;
		
//	data members
		bool whitebackground;
		bool cancelpending;
		bool keypressed;
	
//	finally declare custom event handlers and event filter
		void resizeEvent (QResizeEvent*);	
		void showEvent (QShowEvent*);
		bool eventFilter(QObject*,QEvent*);
			
	public slots:		
		void openfile(const QString& s = QString());	//	default is a null string
		void closefile();
		short auditdrawing();
		void displayTWINmessage(int);
		short showreport(const QString& s = QString());//	default is a null string	
		void quicksave();
		void saveas();
		void quit();
		void togglebackground();
		void togglesbar();
		void toggletextwin();
		void togglecmdbar();
		void toggletoolbar(int);
		void toggle3Dview();
		void togglemanual();
		void togglebuttonsize(int);
		void toggletooltips();
		void displayaboutmessage(int);
		void switchview(int);
		void call3Dtransform();
		void callplotter();
		void calllayerdialog();
		void callaliasdialog();
		void printMessage(const QString&);
		void printMessage(const QString&, unsigned char);
		void batchConvert();
		void batchPlot();
		
	//	command slots (link tool buttons that don't emit integers to these)
		void openfiledia();
		void showreportdia();
		void redraw();
		void wcsplanview();
		void zoomin();
		void zoomout();
		void zoomwindow();
		void zoomextents();
		void zoomprevious();
		void zoomcenter();
		void pan();
		void panleftslot();
		void panupslot();
		void panrightslot();
		void pandownslot();
		

	private:
		void greyout(bool);		//	function to greyout and display menus
};

#endif

