/***************************************************************************
                          drawingenvironment.h  -  description
                             -------------------
    begin                : Sun Apr 14 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//	Functions to control the drawing environment such as
//	ortho
//	snap
//	modelspace
//	paperspace

#ifndef DRAWING_ENVIRONMENT_H
#define DRAWING_ENVIRONMENT_H

//	C++ includes

#include "string.h"

//	Qt includes
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qcombobox.h>

//	OpenDwg includes
#define AD_PROTOTYPES
#include "ad2.h"
#include "ad3.h"

//	class for a lighted toggle button
class LX_LightedButton : public QPushButton
{
	Q_OBJECT
	
	public:
		LX_LightedButton(QWidget* parent = 0, const char* name = 0);		//	constructor
	
	private slots:		
		void buttonDown(bool);
		void displayon();
		void displayoff();		

	private:
		QPixmap bon;
		QPixmap boff;
};


//	settings bar which is inserted in the status bar at the bottom
class LX_SettingsBar : public QWidget
{
 		Q_OBJECT
	
	public:
		LX_SettingsBar(QWidget* parent=0, const char* name=0, WFlags f=0);	//	constructor

	private:				
	// settings bar members
	LX_LightedButton* snapbutton;
	LX_LightedButton* gridbutton;
	LX_LightedButton* orthobutton;
	QPushButton* isnapbutton;
	LX_LightedButton* tilebutton;
	QComboBox* layoutbox;
	
	//	other data members
	AD_DB_HANDLE filehandle;
	const QString snap_string;
	const QString grid_string;
	const QString ortho_string;
	const QString isnap_string;
	const QString model_string;
	const QString paper_string;
	const QString tile_string;
	short lastpslayout;
	
	//	functions
	bool validPSBlock(AD_OBJHANDLE);
	void findlayouts();

	//	public functions
	public:
	void initializebar(AD_DB_HANDLE);
	void snapMode(short);
	void gridMode(short);
	void orthoMode(short);
	void getLayoutHandle(AD_OBJHANDLE*);
	
	public slots:
	void snapclicked(bool);
	void gridclicked(bool);
	void orthoclicked(bool);
	void isnapclicked();
	void tileclicked();
	void layoutpicked(int);

	signals:
	void commandString(const QString&);
};



#endif

