/***************************************************************************
                          errorhandlers.cpp
                             -------------------
    begin                : Fri May 4 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/
//	error handlers for OD_toolkit and OD_viewkit


#include "errorhandlers.h"

//	OD toolkit warnings.  Pop up a Messagebox and echo the output to the textwindow
void LX_WarningMessage(short errorno)
{
	extern DrawingBoard* db;
	
	db->printMessage(QT_TR_NOOP("\nWarning: OpenDWG Error - ") + (QString(adErrorStr(errorno))));

	QMessageBox* mbox = new QMessageBox (
			QString(QT_TR_NOOP("OpenDrawing Warning")),
			QString(QT_TR_NOOP("<center>OpenDWG warning error: \n") + (QString(adErrorStr(errorno)))),
			QMessageBox::Warning,0,0,0,db);
			mbox->exec();

	return;
}

//	OD toolkit Critical errors.  OD allows certain of these to be non-fatal.
//	I am not so generous.  If one of these trips, shut down the program
short LX_CriticalMessage(short num)
{
	extern DrawingBoard* db;

/* file read errors can be retried - but don't*/
  if (num==AD_CRITERR_FILEREADERROR)
	{
		QMessageBox* mbox = new QMessageBox (
			QString(QT_TR_NOOP("Critical Error")),
			QString(QT_TR_NOOP("<center>File Read Error\nTERMINATING")),
			QMessageBox::Critical,0,0,0,db);
		mbox->exec();
    exit(1);
	}

/*	disk full can be retried  - but don't*/
/*	Returning AD_DISKFULL_RETRY on a disk error means retry.  Returning
		AD_DISKFULL_CALLWRITEABORT means close the file and generate critical
		error AD_WRITE_ABORT (to allow a graceful exit).
		Returning AD_DISKFULL_STOPWRITING means return errors back through
		the writing process to stop the file write. */
	if(num==AD_CRITERR_DISKFULL)
	{
		QMessageBox* mbox = new QMessageBox (
			QString(QT_TR_NOOP("Critical Error")),
			QString(QT_TR_NOOP("<center>Disk Full -- TERMINATING")),
			QMessageBox::Critical,0,0,0,db);
		mbox->exec();
    return (AD_DISKFULL_CALLWRITEABORT);
	}

/* write abort means disk full was reached and the program requested */
/* termination.  This is a fatal error. */
/* You should delete the partial file here.  OD DOES NOT do so. */
	if (num==AD_CRITERR_WRITEABORT)
	{
		QMessageBox* mbox = new QMessageBox (
			QString(QT_TR_NOOP("Critical Error")),
			QString(QT_TR_NOOP("<center>Write Abort -- TERMINATING")),
			QMessageBox::Critical,0,0,0,db);
		mbox->exec();
    exit(1);
	}

/* malloc errors are fatal */
  if (num==AD_CRITERR_MALLOCERROR)
	{
		QMessageBox* mbox = new QMessageBox (
			QString(QT_TR_NOOP("Critical Error")),
			QString(QT_TR_NOOP("<center>Critical Malloc Error:") +
                                (QString::number(adError())) +
                                (QString(adErrorStr(adError())))),
			QMessageBox::Critical,0,0,0,db);
		mbox->exec();
    exit(1);
	}

/* as are buffer overruns */
  if (num==AD_CRITERR_BUFFEROVERRUN)
	{
		QMessageBox* mbox = new QMessageBox (
			QString(QT_TR_NOOP("Critical Error")),
			QString(QT_TR_NOOP("<center>Buffer Overrun -- TERMINATING")),
			QMessageBox::Critical,0,0,0,db);
		mbox->exec();
    exit(1);
	}
/*	ditto for bad data */
  if (num==AD_CRITERR_BADDATA)
	{
		QMessageBox* mbox = new QMessageBox (
			QString(QT_TR_NOOP("Critical Error")),
			QString(QT_TR_NOOP("<center>Bad Data Detected -- TERMINATING")),
			QMessageBox::Critical,0,0,0,db);
		mbox->exec();
    exit(1);
	}
	
	return (0);
}
