/***************************************************************************
                          fileio.cpp
                             -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

/*
	This file implements the fileio class
	used for reading and writing DWG and DXF files
*/

#include "fileio.h"
#include "drawingboard.h"
#include "svgexport.h"

//	constructor
fileio::fileio(): dwg_string(QWidget::tr("Standard Drawing Files (*.dwg *.dwG *.dWg *.dWG *.Dwg *.DwG *.DWg *.DWG)")),
									dxf_string(QWidget::tr("Drawing Exchange Format Files  (*.dxf *.dxF *.dXf *.dXF *.Dxf *.DxF *.DXf *.DXF)")),
									bmp_string(QWidget::tr("Bitmap Files (*.bmp *.bmP *.bMp *.bMP *.Bmp *.BmP *.BMp *.BMP)")),
									png_string(QWidget::tr("Portable Network Graphics (*.png *.pnG *.pNg *.pNG *.Png *.PnG *.PNg *.PNG)")),
									svg_string(QWidget::tr("Scalable Vector Graphics (*.svg *.svG *.sVg *.sVG *.Svg *.SvG *.SVg *.SVG)")),
									all_string(QWidget::tr("All Files (*.*)")),
									savedxfnegz(1),savedxfwritezero(1)
{
	extern DrawingBoard* db;

	dwghandle = NULL;
	openname = QString();
	savename = QString();

	p2fsd = new LX_FileSaveDialog(db,"File Save Dialog",true);
	p2fsd->dwg_string = dwg_string;
	p2fsd->dxf_string = dxf_string;
	p2fsd->bmp_string = bmp_string;
	p2fsd->png_string = png_string;
	p2fsd->svg_string = svg_string;
	p2fsd->setViewMode(QFileDialog::List);

	p2fod = new QFileDialog(db,"Open File Dialog",true);		
	pictPreview* pp = new pictPreview(p2fod);
	infoPreview* ip = new infoPreview(p2fod);
	p2fod->setMode(QFileDialog::ExistingFile);
	p2fod->setContentsPreviewEnabled(true);
	p2fod->setContentsPreview(pp,pp);
	p2fod->setInfoPreviewEnabled(true);
	p2fod->setInfoPreview(ip,ip);
	p2fod->setDir(QDir::homeDirPath());
	p2fod->setFilter(dwg_string);
	p2fod->addFilter(dxf_string);
	p2fod->addFilter(all_string);	//	I want the *.* choice last
	p2fod->setSelectedFilter(dwg_string);
	p2fod->setCaption(QFileDialog::tr("Open File"));
	p2fod->setViewMode(QFileDialog::List);
	p2fod->setPreviewMode(QFileDialog::Contents);

}

//	picture preview class constructor
//	note: minimum width and height are set in infopreview
pictPreview::pictPreview(QWidget* parent) : QLabel(parent)
{
	setAlignment(QLabel::AlignCenter);
	setFrameStyle(QFrame::Panel | QFrame::Sunken);
}

//	picture Preview class member function (reimplementation)
void pictPreview::previewUrl( const QUrl &u )
{
	unsigned char* imagebuffer;
	long imagesize = 0;

	adSetupDwgRead();

	if (!u.isLocalFile()) return;



	QString fpath = u.path();
	QFileInfo fi(fpath);	
	this->setText("");

	if (fi.isFile())
	{
		imagesize = adBmpSizeFromFile(const_cast<char*>(fpath.latin1()));
		if (imagesize)
		{
			imagebuffer = new unsigned char[imagesize];
			adExtractBmpFromFileToBuffer(const_cast<char*>(fpath.latin1()),imagebuffer,&imagesize);
			QPixmap pix;
			pix.loadFromData(imagebuffer,static_cast<uint>(imagesize));
			setPixmap(pix);
			delete [] imagebuffer;
		}
		else{this->setText(tr("Preview Unavailable"));}
	}
};

//	information preview class constructor
infoPreview::infoPreview(QWidget* parent) : QLabel(parent)
{
	const short previewheight = 200;
	const short previewwidth = static_cast<short>((4.0/3.0)*previewheight);
	const short previewindent = 10;

	setAlignment(QLabel::AlignTop | QLabel::AlignLeft);
	setFrameStyle(QFrame::Panel | QFrame::Sunken);
	setIndent(previewindent);
	setMinimumWidth(previewwidth);
	setMinimumHeight(previewheight);
}

//	information Preview class member function (reimplementation)
void infoPreview::previewUrl( const QUrl &u )
{
	if (!u.isLocalFile()) return;

	QString fpath = u.path();
	QFileInfo fi(fpath);	
	this->setText("");

	if (fi.isFile())
	{
		QString g	 = QFileDialog::tr("granted.");
		QString ng = QFileDialog::tr("NOT granted.");

		this->setText(QFileDialog::tr("File Name: %0"
									"\nDirectory: %1"
									"\nOwner: %2"
									"\nGroup: %3"
									"\nRead permission is %4"
									"\nWrite permission is %5"
									"\n__________________________________"
									"\nSize: %6"
									"\nCreated: %7"
									"\nLast modified: %8"
									"\nLast read: %9"
									)
									.arg(fi.fileName())
									.arg(fi.dirPath(true))
									.arg(fi.owner())
									.arg(fi.group())
									.arg(fi.isReadable() ? g : ng)
									.arg(fi.isWritable() ? g : ng)
									.arg(QString::number(fi.size()))
									.arg(fi.created().toString(Qt::TextDate))
									.arg(fi.lastModified().toString(Qt::TextDate))
									.arg(fi.lastRead().toString(Qt::TextDate))
									);
	}
};

//	function to open a file
//	return -3 for file does not exist
//	return -2 for no file selected
//	return -1 for file selected but we don't have read permission for it
//	return 0 for error opening file
//	return 1 on success
short fileio::openfile(const QString& s)
{
	extern DrawingBoard* db;
	extern bool lookforxrefs;
	extern bool lookforfonts;
	QChar dot('.');
	short lastdot=0;
	QString dtype;
	short rtnval = 0;

//	if no file was provided, open a file dialog to look for one
	if (s.isNull())
	{
		rtnval = p2fod->exec();
		openname = p2fod->selectedFile();

//	if no file was selected display a message and return
		if (rtnval == QDialog::Rejected)
		{
			db->printMessage(QFileDialog::tr("No file selected"));
			return -2;
		}
	}
	else openname = s;

//	a file was selected, so set up to read it
  db->printMessage(openname,TWIN);
	adSetupDwgRead();
	adSetupDxfRead();

//	First check to make sure file exists
	QFileInfo fi = QFileInfo(openname);
	if (!fi.exists())
	{
		DisplayMessage(FILE_DOES_NOT_EXIST);
		db->printMessage(QFileDialog::tr("Warning: %1 does not exist").arg(openname));
		return -3;
	}

//	Check to make sure we have permission to read the file
	if (!fi.isReadable())
	{
		DisplayMessage(NO_READ_PERMISSION);
		db->printMessage(QFileDialog::tr("Warning: %1 was not read - read permission not granted").arg(openname));
		return -1;
	}

//	now we have a drawing name, lets sniff it out before we open it
//	only run sniffer if the file appears to be a drawing however
	if (openname.contains(".DWG",false))	//	false = search is case insensitive	
		switch (adDwgSmellsBad(const_cast<char*>(openname.latin1())))
		{
			case -1:	//	file is pre-R11
				db->printMessage(QFileDialog::tr("Drawing Sniffer: Not run - this file appears to be prior to release 11"),TWIN);
				break;
			case 0:		//	file comes up clean
				db->printMessage(QFileDialog::tr("Drawing Sniffer: Passed"),TWIN);
				break;
			case 1:	//	file is suspect
				db->printMessage(QFileDialog::tr("Drawing Sniffer: Found the following errors: %1").arg(adErrorStr(adError())), TWIN);
				DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,
					QFileDialog::tr("<center>Drawing Sniffer has reported the following error: %1<center>If the drawing loads we recommend that you run Audit to try to determine the cause").arg(adErrorStr(adError())));
				break;
		}	
	
//	we are all setup and we also have permission to read - lets give it a shot
	if (!(dwghandle=adLoadFile(const_cast<char*>(openname.latin1()),AD_PRELOAD_ALL,1)))
		{
			db->printMessage(QFileDialog::tr("Error opening file %1 : %2")
				.arg(openname)
				.arg(adErrorStr(adError()))
				);
      DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,QFileDialog::tr("<center>Error opening file.  %1").arg(adErrorStr(adError())));
			return 0;
		}

  db->printMessage(QFileDialog::tr("%1 successfully opened.").arg(openname),TWIN);
	lookforxrefs = TRUE;
	lookforfonts = TRUE;	
//	now we have read the file into memory, so set some default save data members
//	File save version is set equal to the ACAD version of the file just read
//	Open file type is determined (DXF or DWG) and the Save file type is set the same
	p2fsd->setDir(fi.dirPath(true));
	p2fsd->saveversion = adFileVersionLoaded(dwghandle);
	p2fsd->saver12dxfbvls = 1;	//	no way to tell automatically R11 from R12, so assume R12
//	now lets sync the selectfver buttons with the dwg version actually read
	if (p2fsd->saveversion == AD_ACAD2000)	p2fsd->fver->setButton(29);
		else if (p2fsd->saveversion == AD_ACAD14)	p2fsd->fver->setButton(28);
			else if (p2fsd->saveversion == AD_ACAD13)	p2fsd->fver->setButton(27);
				else if (p2fsd->saveversion == AD_ACAD11)	p2fsd->fver->setButton(26);	
					else if (p2fsd->saveversion == AD_ACAD10)	p2fsd->fver->setButton(24);
						else if (p2fsd->saveversion == AD_ACAD9)	p2fsd->fver->setButton(23);
							else if (p2fsd->saveversion == AD_ACAD26)	p2fsd->fver->setButton(22);
								else if (p2fsd->saveversion == AD_ACAD25)	p2fsd->fver->setButton(21);
									else p2fsd->fver->setButton(29);
	dtype=openname;
	lastdot=dtype.findRev(dot);
	dtype=dtype.remove(0,lastdot+1);	
	if (dtype.lower()=="dxf")
		p2fsd->savefiletype = AD_DXF;

	else p2fsd->savefiletype = AD_DWG;
	return 1;
}

//	function that will save the drawing without any input from the user (a quick save)
//	not really useful in a viewer program, but may need it in the future
void fileio::quicksave()
{
	extern DrawingBoard* db;
	
	QFileInfo fi = QFileInfo(openname);
	if (!fi.isWritable())
	{
		DisplayMessage(NO_WRITE_PERMISSION);
		db->printMessage(QFileDialog::tr("Warning: %1 was not saved to disk - write permission not granted").arg(openname));
		return;
	}
	
	addBmpToFile();
  clearRoad(openname);		//	make a backup
	
	if ((adSaveFile(
				dwghandle,
				const_cast<char*>(openname.latin1()),
				p2fsd->savefiletype,
				p2fsd->saveversion,
				savedxfnegz,
				p2fsd->getdxfprec(),
				savedxfwritezero,
				p2fsd->saver12dxfbvls))
			)
    	{
				db->printMessage(QFileDialog::tr("%1 saved to disk").arg(openname));
			}
}

//	function to "save as" the file.  This function is the useful one
void fileio::saveas(const QString& file2save)
{
	extern DrawingBoard* db;	
	QChar dot('.');
	short lastdot=0;

//	no file name passed so open a dialog to interact with the user
	if (file2save.isEmpty())
	{
		//	set up the file filters and call dialog
		p2fsd->setCaption(QFileDialog::tr("Save File"));
		p2fsd->setMode(QFileDialog::AnyFile);
		p2fsd->setFilters(dwg_string + ";;" + dxf_string + ";;" + bmp_string + ";;" + png_string + ";;" + svg_string);
		p2fsd->setSelectedFilter(dwg_string);
		p2fsd->setftypefromdialog();
		int rtnval = p2fsd->exec();
		savename = p2fsd->selectedFile();
		if (rtnval == QDialog::Rejected || savename.isNull())	
		{
			db->printMessage(QFileDialog::tr("No file selected"));
			return;
		}		
	}
	else savename = file2save;

	lastdot=savename.findRev(dot);
	if (lastdot<0) 		//	no dots, so add a file type
	{
		if (p2fsd->savefiletype == AD_DWG) savename+=(".dwg");
		if (p2fsd->savefiletype == AD_DXF) savename+=(".dxf");
		if (p2fsd->savefiletype == QT_BMP) savename+=(".bmp");
		if (p2fsd->savefiletype == QT_PNG) savename+=(".png");
		if (p2fsd->savefiletype == LX_SVG) savename+=(".svg");
	}

	QFileInfo fi = QFileInfo(savename);
	QFileInfo fd = fi.dirPath(true);
	if ((fi.exists() && !fi.isWritable()) || !fd.isWritable())
	{
		DisplayMessage(NO_WRITE_PERMISSION);
		db->printMessage(QFileDialog::tr("Warning: %1 was not saved to disk - write permission not granted").arg(openname));
		return;
	}

	if (p2fsd->savefiletype == AD_DWG || p2fsd->savefiletype == AD_DXF)
	{
		if (p2fsd->savefiletype == AD_DWG) addBmpToFile();
		openname=savename;
		if ((adSaveFile(dwghandle,const_cast<char*>(openname.latin1()),p2fsd->savefiletype,p2fsd->saveversion,savedxfnegz,p2fsd->getdxfprec(),savedxfwritezero,p2fsd->saver12dxfbvls)))
			db->printMessage(QFileDialog::tr("%1 saved to disk").arg(openname));
	} 	//	close savefiletype>=0 is true if statement
	else if (p2fsd->savefiletype < 0)  // the savefiletype is some image we use QT to convert
		{
			QPixmap p;
			p = db->p2dwgarea->snapshot;
			db->printMessage(QFileDialog::tr("Please wait while the format is converted...."),SBAR);
			switch (p2fsd->savefiletype)
			{
				case QT_BMP:
					if (p.save(savename,"BMP"))db->printMessage(QFileDialog::tr("%1 saved to disk").arg(savename));
					break;
				case QT_PNG:
					if (p.save(savename,"PNG"))db->printMessage(QFileDialog::tr("%1 saved to disk").arg(savename));
					break;
			} //	close the switch
		}	//	close the else if
  else if (p2fsd->savefiletype == LX_SVG)
	{
		/* I have set it up so the svg pixel space is identical to the drawing screen
		pixel space, so we only need to redefine the OD interface functions and do an
		addrawdrawing.  Everything else will be set properly */

		svgExport svgout(savename);
		svgout.setColor(db->p2dwgarea->backgroundcolor);
		svgout.createRoot(db->XY(0),db->XY(1));
		if (svgout.startWrite())
		{
			setsvgfuns(&svgout);		//	global function from OD_Interface.cpp
			adDrawDrawing(dwghandle);			//	redraw the drawing
			registerfuns();					//	reset the vectorize to screen fuctions (in OD_Interface.cpp)

  	  if (svgout.endWrite())
				db->printMessage(QFileDialog::tr("%1 saved to disk").arg(savename));
			else db->printMessage(QFileDialog::tr("error writing svg file"));
		}
	}
}

//	function to add the current bitmap image into the drawing file
void fileio::addBmpToFile()
{
	extern DrawingBoard* db;	
	const short imageheight = 175;
	QImage qi;
	QBuffer qbuff;
	QImageIO qiio;

	if (adFileHasBmp(dwghandle) == 1)
		if (adDeleteBmp(dwghandle) == 0) return;

	qi = db->p2dwgarea->snapshot;
	double aspect = static_cast<double>(qi.width())/static_cast<double>(qi.height());
	qi = qi.scale(static_cast<int>(aspect*imageheight),imageheight);

//	Undocumentated feature - OD toolkit will only read bitmaps at 8bpp.  I found this
//	out by trying 32,16,1 and then finally 8 (the last choice available!!!!)
	 qi = qi.convertDepth(8);

//	Code below is only because I can't find a way to get QT to write to a generic
//	memory location.  Need QImageIO (qiio) to hold the paramaters for writing, and a
//	QIODevice (qbuff) which is the actual device that receives the data. I then can
//	access the QByteArray by using the ::buffer() function, and from that I can get
//	a pointer to the beginning of the character array (::begin()) and the size of
//	tha array (::size()).  Kind of a long way around, but the alternative is to
//	write the BMP to the hard drive, read it back into the drawing, and then
//	delete the file on the hard drive.
	qiio.setFormat("BMP");
	qiio.setIODevice(&qbuff);
	qiio.setImage(qi);
	qbuff.open(IO_ReadWrite);
	qiio.write();
  adAddBmpFromBuffer(dwghandle,(qbuff.buffer()).begin(),(qbuff.buffer()).size());
	qbuff.close();
	return;
}

//	slot to read a batch control file and then set the dialog options accordingly.
//	at the end jump to the batch process routine
void fileio::setOptions(QString dir2process)
{
	bool terminate(false);
	extern DrawingBoard* db;

//	open the batch control file for reading, and set the options accordingly
	QFile convert_bcf(QDir::homeDirPath() + LXV_HOME + "/convert.bcf");
  if(convert_bcf.exists()) 		// there is a BCF so continue processing
	{
		db->printMessage(QFileDialog::tr("Attempting to open %1").arg(convert_bcf.name()),TWIN);
  	if(!convert_bcf.open (IO_ReadOnly))	//	BCF file exists but for some reason we can't open it	
  	{	
			db->printMessage(QFileDialog::tr("Read error opening %1 -- File not read").arg(convert_bcf.name()),TWIN);
	 	}
    else
		{
			//	now read each line of the file
			QTextStream rs (&convert_bcf);
			const QChar eq('=');
			const QChar semi(';');
			const bool cs(false);
			bool ok;
			QString key;
			QString val;

			while (!rs.eof())
			{
				key = rs.readLine();
				key = key.simplifyWhiteSpace();
				val = key;
				if (key != semi && key.contains(eq,cs))		//	the line read is not a comment and appears properly formatted
				{
					int split = key.find(eq,0,cs);
		 		  key = key.left(split);
					val = val.remove(0,++split);
					key = key.simplifyWhiteSpace();
					val = val.simplifyWhiteSpace();
					if (key == "File Type")
					{
						int b = val.toInt(&ok);
						if (ok) p2fsd->selectftype(b);
					}
						else if (key == "File Version")
						{
							int b = val.toInt(&ok);
							if (ok) p2fsd->selectfver(b);
						}
							else if (key == "DXF Precision")
							{
								int b = val.toInt(&ok);
								if (ok) p2fsd->dxfprec->setValue(b);
							}
							else if (key == "Terminate")
								terminate = ((val == "YES") ? true : false);
				}
			}	//	while rs is not at eof()		
			db->printMessage(QFileDialog::tr("Batch Control File %1 processed").arg(convert_bcf.name()),TWIN);
		}	//	else statement		
		convert_bcf.close();
	}
	else
	{
		db->printMessage(QFileDialog::tr("Batch Control File %1 not found").arg(convert_bcf.name()),TWIN);
		return;
	}

//	commandproc.cpp arranges for the directory to process to be the last item in the
//	options list.  By doing this we only need to remove the text up to and including
//	the = sign to get the directory name to process
	short mark = 0;
	mark = dir2process.find('=');
	if (mark  > 0)
	{
		dir2process.remove(0,++mark);
	}
	batchProcess(dir2process);

//	Can't exit normally at this point because we have not started the main
//	event loop a.exec().  This is a cruddy piece of code, but it does allow a clean
//	exit from the program when started from the command line
	if (terminate)
	{
		if (dwghandle) adTerminateVectorize(dwghandle);
		if (dwghandle) adCloseFile(dwghandle);
		exit(0);
	}
}


//	batch process, either call from setoptions (command line) or call directly from menu
void fileio::batchProcess(QString dir2process)
{
	extern DrawingBoard* db;
	QStringList filebatch;
	QString currentDrawing;
	AD_OBJHANDLE psBlockHandle;
	
	adSetupDwgRead();
	adSetupDxfRead();
	db->setCancelPending(false);	

//	if a drawing is currently open save its name and release all of the memory associated with it
	if (dwghandle)
	{
		if (db->getOpenFileName()) currentDrawing = db->getOpenFileName();
		adTerminateVectorize(dwghandle);
		adCloseFile(dwghandle);
		dwghandle = NULL;
	}	
	
	if (dir2process.isNull())	//	only call the openfile dialog if no directory was provided
	{	
		p2fsd->setMode(QFileDialog::ExistingFiles);
		p2fsd->setDir(QDir::homeDirPath());
		QString dwgdxf_string(QWidget::tr("Drawing or DXF Files (*.dwg *.DWG *.dxf *.DXF)"));
		p2fsd->setFilter(dwgdxf_string);
		p2fsd->addFilter(all_string);
		p2fsd->setSelectedFilter(dwgdxf_string);
		p2fsd->setCaption(QFileDialog::tr("Batch Convert - Multiple File Select"));
		p2fsd->setViewMode(QFileDialog::List);
		int rtnval = p2fsd->exec();
		filebatch = p2fsd->selectedFiles();

		if (rtnval == QDialog::Rejected || filebatch.isEmpty())	
		{
			db->printMessage(QFileDialog::tr("No files selected"),SBAR_TWIN);
			return;
		}
	}

	else		//	directory provided so process it
	{
		QDir d = QDir(dir2process);
		d.setNameFilter("*.dwg *.DWG *.dxf  *.DXF");
		QStringList namelist = d.entryList();
		QStringList::Iterator it = namelist.begin();
		filebatch.clear();
		while (it != namelist.end())
		{	
			filebatch.append(dir2process + '/' + *it);
			++it;
		}
	}
	
	db->LX_ClearScreen();
	QString extension;
	QStringList::Iterator it = filebatch.begin();
	db->printMessage(QFileDialog::tr("\n\nBatch Convert:"));

	//	set the extension on the file type to save
	if (p2fsd->savefiletype == AD_DWG) extension = ".dwg";
		else if (p2fsd->savefiletype == AD_DXF) extension = ".dxf";
			else if (p2fsd->savefiletype == QT_BMP) extension = ".bmp";
				else if (p2fsd->savefiletype == QT_PNG) extension = ".png";
					else if (p2fsd->savefiletype == LX_SVG) extension = ".svg";
	
	while( it != filebatch.end() )
	{
		openname = *it;
		QFileInfo fi = QFileInfo(openname);
		savename = fi.dirPath(true) + '/' + fi.baseName(true) + extension;
		if (fi.isReadable())
		{
			if ((dwghandle=adLoadFile(const_cast<char*>(openname.latin1()),AD_PRELOAD_ALL,1)))
			{
				db->printMessage(QFileDialog::tr("Converting: %1").arg(openname));	

			//	either process any file saves that don't require a vectorization
				if (p2fsd->savefiletype == AD_DWG || p2fsd->savefiletype == AD_DXF)
				{
					clearRoad(savename);
					saveas(savename);
 		 			if (dwghandle) adCloseFile(dwghandle);
				}
			//	or process files that do require vectorization
				else
				{
					setcontrolvariables(); //	function in vectorize.cpp (for OD-viewkit)
					initializegraphics();  //	function in vectorize.cpp (to set graphics area)
					//	we need to vectorize for svg, but it can all be done off screen
					if (p2fsd->savefiletype == LX_SVG)
					{
						extern AD_VIEW_PARMS adviewparms;
						extern rectlong globwincoords;
						extern rectlong dlrect;
						if (!adInitVectorize(dwghandle,&adviewparms,&globwincoords,&dlrect,psBlockHandle))
						{
							DisplayMessage(VECTORIZE_ERROR,CAPTION_INITIALIZE,QMessageBox::Critical);
							freeadptrs();
							exit(1);
						}
						adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
					}				
					//	bmp and png require an on screen vectorization (they are really just screen captures)
					else vectorize(dwghandle,psBlockHandle);

					//	now do a zoom extents - requested by a user of the program
					//	for version 1.0 make this optional selected by a dialog
					LX_ZoomExtents(dwghandle);
          //	end of the code for zoom extents
							
					clearRoad(savename);
					saveas(savename);	
					db->LX_ClearScreen();
					if (dwghandle) {adTerminateVectorize(dwghandle); adCloseFile(dwghandle);}	
					registerfuns();		
				}	//	else for files that need vectorization
			}	//	if we got a drawing handle
			else db->printMessage(QFileDialog::tr("Error opening file %1: %2").arg(openname).arg(QString(adErrorStr(adError()))));			 		
		}	//	if the file was readable
		else db->printMessage(QFileDialog::tr("Warning: %1  was not read - read permission not granted").arg(openname));

		dwghandle = NULL;
		openname = QString();
		savename = QString();
		++it;
	}	//	while loop

	// now reload the saved drawing
	if (!currentDrawing.isEmpty()) db->openfile(currentDrawing);
}


//	Function that will rename an existing file to a backup file (ending in ~).
//	This will clear the road for other functions to save a new version of the file
void fileio::clearRoad(const QString& newfn)
{
	extern DrawingBoard* db;
	QFileInfo fi(newfn);
	QString fn = fi.fileName();
	QDir fd = (fi.dirPath(true));

	if (!fi.exists()) return;	//	we are not going to overwrite an existing file so return
	fi.setFile((fi.absFilePath()).append('~'));
	if (fi.exists())					//	if a backup already exists then delete it
		if (!fd.remove(fi.fileName()))
		{
			db->printMessage(QFileDialog::tr("Cannot delete existing backup file  - backup not made"),TWIN);
			return;
		}
	if (!fd.rename(fn,fi.fileName(),true))
	{
		db->printMessage(QFileDialog::tr("Error writing backup file - backup not made"),TWIN);
		return;
	}
}



//**********************************************************************************************
//**********************************FileSaveDialog**********************************************
//**********************************************************************************************
#define FTYPE_DWG	11
#define FTYPE_DXF 12
#define FTYPE_BMP	13
#define FTYPE_PNG 14
#define FTYPE_SVG 15

//	file save dialog class constructor
LX_FileSaveDialog::LX_FileSaveDialog(QWidget* parent,const char* name, bool modal):QFileDialog(parent,name,modal)
{
	const short horspace = 10;
	const short verspace = 25;
	const short pad = 25;
	collapsedwidth = 600;
	collapsedheight = 350;

	saveversion = AD_ACAD2000;
	savefiletype = AD_DWG;
	
	p2advanced = new QPushButton(this,"Advanced Button");
	p2advanced->setText(tr("Advanced"));

	QLabel* p2advlabel = new QLabel(this,"Advanced Label");
	p2advlabel->setText(tr("Select the Advanced button for different file save options"));

	p2rsidebox = new QWidget(this,"Save dialog right side box");

	ftype = new QButtonGroup(tr("Save Type"),p2rsidebox,"Save type button group");
	QRadioButton* b11 = new QRadioButton (tr("DWG"),ftype);
	QRadioButton* b12 = new QRadioButton (tr("DXF"),ftype);
	QRadioButton* b13 = new QRadioButton (tr("BMP"),ftype);
	QRadioButton* b14	= new QRadioButton (tr("PNG"),ftype);
	QRadioButton* b15 = new QRadioButton (tr("SVG"),ftype);
	b11->move (horspace,verspace*1);
	b12->move (horspace,verspace*2);
	b13->move (horspace,verspace*3);
	b14->move (horspace,verspace*4);
	b15->move (horspace,verspace*5);
	ftype->insert(b11,FTYPE_DWG);
	ftype->insert(b12,FTYPE_DXF);
	ftype->insert(b13,FTYPE_BMP);
	ftype->insert(b14,FTYPE_PNG);
	ftype->insert(b15,FTYPE_SVG);
	b11->setChecked(TRUE);

	writeBCFbut = new QPushButton(p2rsidebox,"writeBCF button");
	writeBCFbut->setText(tr("Write BCF"));

	dxfprec = new QSpinBox(3,16,1,p2rsidebox,"DXF spin box");
	dxfprec->setWrapping(TRUE);
	dxfprec->setPrefix(tr("DXF precision: "));
	dxfprec->setValue(8);		//	default to 8 decimal places

	fver	= new QButtonGroup(tr("DWG Version"),p2rsidebox,"DWG version button group");
	QRadioButton*	b21 = new QRadioButton (tr("Version 2.5"),fver);
	QRadioButton*	b22 = new QRadioButton (tr("Version 2.6"),fver);
	QRadioButton*	b23 = new QRadioButton (tr("Version 9"),fver);
	QRadioButton*	b24 = new QRadioButton (tr("Version 10"),fver);
	QRadioButton*	b25 = new QRadioButton (tr("Version 11"),fver);
	QRadioButton* b26 = new QRadioButton (tr("Version 12"),fver);
	QRadioButton*	b27 = new QRadioButton (tr("Version 13"),fver);
	QRadioButton*	b28 = new QRadioButton (tr("Version 14"),fver);
	QRadioButton*	b29 = new QRadioButton (tr("Version 2000/2002"),fver);
	b21->setMinimumWidth((b29->sizeHint()).width() + pad);
	b22->setMinimumWidth((b29->sizeHint()).width() + pad);
	b23->setMinimumWidth((b29->sizeHint()).width() + pad);
	b24->setMinimumWidth((b29->sizeHint()).width() + pad);
	b25->setMinimumWidth((b29->sizeHint()).width() + pad);
	b26->setMinimumWidth((b29->sizeHint()).width() + pad);
	b27->setMinimumWidth((b29->sizeHint()).width() + pad);
	b28->setMinimumWidth((b29->sizeHint()).width() + pad);
	b29->setMinimumWidth((b29->sizeHint()).width() + pad);
  b21->move (horspace,verspace*1);
  b22->move (horspace,verspace*2);
  b23->move (horspace,verspace*3);
  b24->move (horspace,verspace*4);
  b25->move (horspace,verspace*5);
  b26->move (horspace,verspace*6);
  b27->move (horspace,verspace*7);
  b28->move (horspace,verspace*8);
	b29->move (horspace,verspace*9);
	fver->insert(b21,21);
	fver->insert(b22,22);
	fver->insert(b23,23);
	fver->insert(b24,24);
	fver->insert(b25,25);
	fver->insert(b26,26);
	fver->insert(b27,27);
	fver->insert(b28,28);
	fver->insert(b29,29);
	b29->setChecked(TRUE);

	QHBoxLayout* hbox = new QHBoxLayout(p2rsidebox);
	QVBoxLayout* vbox1 = new QVBoxLayout();
	QVBoxLayout* vbox2 = new QVBoxLayout();
	hbox->addLayout(vbox1);
	hbox->addLayout(vbox2);
	vbox1->addWidget(ftype);
	vbox1->addWidget(dxfprec);
	vbox1->addWidget(writeBCFbut);
	vbox2->addWidget(fver);
	
	p2rsidebox->setMinimumWidth((hbox->sizeHint()).width());
	addWidgets(p2advlabel,0,p2advanced);
	addRightWidget(p2rsidebox);
	dxfprec->hide();
	p2rsidebox->hide();		
	
	//	now connect slots and signals
	connect (p2advanced, SIGNAL(clicked()),this,SLOT(togglerightwin()));
	connect (ftype,SIGNAL(clicked(int)),this,SLOT(selectftype(int)));
	connect (fver ,SIGNAL(clicked(int)),this,SLOT(selectfver (int)));
	connect (this,SIGNAL(filterSelected(const QString&)),this,SLOT(setftypefromdialog()));
	connect	(writeBCFbut,SIGNAL(clicked()),this,SLOT(writeBCF()));

	//	set a default size for the dialog
	resize(collapsedwidth,collapsedheight);

	// give it a title
	setCaption(tr("Save File"));
}

//	Public slots in class LX_FileSaveDialog
void LX_FileSaveDialog::setftypefromdialog()
{
	int i= FTYPE_DWG;

	if (selectedFilter() == dwg_string) i = FTYPE_DWG;
			else if (selectedFilter() == dxf_string) i = FTYPE_DXF;
				else if (selectedFilter() == bmp_string) i = FTYPE_BMP;
					else if (selectedFilter() == png_string) i = FTYPE_PNG;
						else if (selectedFilter() == svg_string) i = FTYPE_SVG;

	ftype->setButton(i);
	selectftype(i);	
	return;
}

void LX_FileSaveDialog::togglerightwin()
{
	if (p2rsidebox->isVisible())
	{
		p2rsidebox->hide();
		setMinimumWidth(collapsedwidth);
		resize(collapsedwidth,collapsedheight);
	}
	else p2rsidebox->show();	
}

void LX_FileSaveDialog::selectftype(int button)
{
	switch (button)
		{
			case FTYPE_DWG:
				fver->show();
				dxfprec->hide();
				setSelectedFilter(dwg_string);
				savefiletype = AD_DWG;
				break;
			case FTYPE_DXF:
				fver->show();
				dxfprec->show();
				setSelectedFilter(dxf_string);
				savefiletype = AD_DXF;
				break;
			case FTYPE_BMP:
				fver->hide();
				dxfprec->hide();
				setSelectedFilter(bmp_string);
				savefiletype = QT_BMP;
				break;
			case FTYPE_PNG:
				fver->hide();
				dxfprec->hide();
				setSelectedFilter(png_string);
				savefiletype = QT_PNG;
				break;
			case FTYPE_SVG:
			  fver->hide();
				dxfprec->hide();
				setSelectedFilter(svg_string);
				savefiletype = LX_SVG;
				break;
			default:
				savefiletype = AD_DWG;			
		}
}

void LX_FileSaveDialog::selectfver(int button)
{
	saver12dxfbvls = 1;
	switch (button)
		{
			case 21:
				saveversion = AD_ACAD25;
				break;
			case 22:
				saveversion = AD_ACAD26;
				break;
			case 23:
				saveversion = AD_ACAD9;
				break;
			case 24:
				saveversion = AD_ACAD10;
				break;
			case 25:
				saveversion = AD_ACAD11;		//	acad 11
				saver12dxfbvls = 0;
				break;
			case 26:
				saveversion = AD_ACAD11;    //	acad 12
				break;				
			case 27:
				saveversion = AD_ACAD13;
				break;
			case 28:
				saveversion = AD_ACAD14;
				break;
			case 29:
				saveversion = AD_ACAD2000;
				break;
			default:
				saveversion = AD_ACAD2000;
		}
}
	
//	slot to store the dxf precision set in the spin box			
short LX_FileSaveDialog::getdxfprec(){return dxfprec->value();}

//	slot to write the batch control file to disk
void LX_FileSaveDialog::writeBCF()
{
	const QString eq ("  =  ");
	extern DrawingBoard* db;

	QString convert_bcf = QDir::homeDirPath() + LXV_HOME + "/convert.bcf";
	QFileInfo fi = QFileInfo(convert_bcf);
	QFileInfo fd = fi.dirPath(true);
	if ((fi.exists() && !fi.isWritable()) || !fd.isWritable())
	{
		DisplayMessage(NO_WRITE_PERMISSION);
		db->printMessage(tr("\nWarning: %1 was not saved to disk - write permission not granted").arg(convert_bcf));
		return;
	}

	QFile f(convert_bcf);
	if (f.open(IO_WriteOnly))
	{
		QTextStream stream( &f );
		stream << (";   LX-VIEWER BATCH CONTROL FILE - convert.bcf");
		stream << ("\n;\n;   This file contains the information needed to batch process file conversions.");
		stream << ("\n;   It is only used when supplying a directory of drawings to the program on the command line");
		stream << ("\n;\n;   We recommend that you do not edit this file manually");
		stream << ("\n;\n;");
		stream << "\nFile Type" << eq << ftype->id(ftype->selected());
		stream << "\nFile Version" << eq << fver->id(fver->selected());	
		stream << "\nDXF Precision" << eq << dxfprec->value();
  	QMessageBox* mb = new QMessageBox (tr("Need User Input"),
					tr("<center>After batch processing do you wish to exit LX-Viewer?"
						"<center>If you select <b>YES</b> then LX-Viewer will exit to a command prompt after "		
						"the batch processing ends.  If you select <b>NO</b> then LX-Viewer will remain open "
						"and running after the batch process ends."),
					QMessageBox::Information,
					QMessageBox::Yes | QMessageBox::Default,
					QMessageBox::No  | QMessageBox::Escape,
					0,this);
		int rtnval = mb->exec();
		delete mb;
		stream << "\nTerminate" << eq << (rtnval == QMessageBox::No ? "NO" : "YES");	
		f.close();
	}
	db->printMessage(tr("\nBatch Control File %1 saved to disk").arg(convert_bcf));
}


