/***************************************************************************
                          fileio.h
                             -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Andrew Bibb
    source               : http://lx-viewer.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/


/*	The header file of the class that manages file input and output tasks.
	Functions in the FILE pulldown menu are here 	*/

#ifndef FILEIO_H
#define FILEIO_H

/* QT includes */
#include <qfiledialog.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qbuffer.h>

/* LX includes */
#include "OD_interface.h"
#include "utilities.h"

/* picture preview class for the file open dialog */
class pictPreview : public QLabel, public QFilePreview
{
		Q_OBJECT

	public:
		pictPreview(QWidget* parent=0);
		void previewUrl(const QUrl &u );
};

/* information preview class for the file open dialog */
class infoPreview : public QLabel, public QFilePreview
{
	public:
		infoPreview(QWidget* parent=0);
		void previewUrl(const QUrl &u );
};

/* class is a used only as a protected member of drawingboard, so most data members can be public */
class LX_FileSaveDialog : public QFileDialog
{
 	Q_OBJECT

	public:
		LX_FileSaveDialog(QWidget* parent=0, const char* name=0, bool modal=TRUE);
		
		QString dwg_string;
		QString dxf_string;
		QString bmp_string;
		QString png_string;	
		QString svg_string;
		short saveversion;
	 	short	savefiletype;
		short	saver12dxfbvls;
		QButtonGroup* fver;
		QButtonGroup* ftype;
		QSpinBox* dxfprec;

	private:
		QPushButton* p2advanced;
		QPushButton* writeBCFbut;
		QWidget* p2rsidebox;
		
	public slots:
		void setftypefromdialog();
		void togglerightwin();	
		void selectftype(int);
		void selectfver(int);
		short getdxfprec();
		void writeBCF();

	private:
		int collapsedwidth;
		int collapsedheight;
};

class fileio
{
  public:
	fileio();                       // constructor for this class
		
	// functions
	short openfile(const QString&);
	void quicksave();
	void saveas(const QString& s = QString());
	void addBmpToFile();
	void setOptions(QString s = QString());		//	read BCF and set the options for command line batching
	void batchProcess(QString files = QString());	//	batch process
	void clearRoad(const QString&);								//	function to make backups if appropriate

	// data members
	AD_DB_HANDLE dwghandle;
	QString openname;
	QString savename;
	const QString dwg_string;
	const QString dxf_string;
	const QString bmp_string;
	const QString png_string;
	const QString svg_string;
	const QString all_string;
	
	private:
	LX_FileSaveDialog* p2fsd;	
	QFileDialog* p2fod;
	
	/* data members (we don't implement any adjustments to these)	*/
	const short	savedxfnegz;
	const short	savedxfwritezero;
	
};

#endif
