/***************************************************************************
                          infomessage.cpp
                        -------------------
    begin                : Mon Jul 23 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : lx-viewer.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//  Messages that need to be displayed in a message box

#include "infomessage.h"

//  function that will display an information/warning dialog box on the screen
//  with one button (OK) to acknowledge it.
//  keys are kept in resource.h
void DisplayMessage(int keynumber,int caption,QMessageBox::Icon icon,QString custommessage)
{
   QMessageBox* mbox = new QMessageBox (
         GetResourceString(caption),
         keynumber >= 0? GetResourceString(keynumber):custommessage,
         icon,0,0,0,0);
         mbox->exec();

   delete mbox;
}

QString GetResourceString(const int& key)
{
   switch(key)
   {
      case ID_FILE_OPEN:
         return QMessageBox::tr("\nOpen: ");
         break;
      case ID_FILE_CLOSE:
         return QMessageBox::tr("\nClose: ");
         break;
      case ID_FILE_SAVE:
         return QMessageBox::tr("\nSave: ");
         break;
      case ID_FILE_SAVE_AS:
         return QMessageBox::tr("\nSave As: ");
         break;
      case ID_FILE_AUDIT:
         return QMessageBox::tr("\nAudit Drawing: ");
         break;
      case ID_FILE_PRINT:
         return QMessageBox::tr("\nPrint: ");
         break;
      case ID_FILE_QUIT:
         return QMessageBox::tr("\nQuit: ");
         break;
      case ID_VIEW_REDRAW:
         return QMessageBox::tr("\nRedraw: ");
         break;
      case ID_VIEW_REGEN:
         return QMessageBox::tr("\nRegen: ");
         break;
      case ID_VIEW_PAN:
         return QMessageBox::tr("\nPan: ");
      	break;
	case ID_VIEW_PAN_POINT:
		return QMessageBox::tr("\nPan:   ");
		break;
	case ID_VIEW_PAN_UP:
		return QMessageBox::tr("\nPan Up:   ");
      	break;
      case ID_VIEW_PAN_RIGHT:
         return QMessageBox::tr("\nPan Right: ");
      break;
      case ID_VIEW_PAN_DOWN:
         return QMessageBox::tr("\nPan Down: ");
      break;
      case ID_VIEW_PAN_LEFT:
         return QMessageBox::tr("\nPan Left: ");
      break;
		case ID_VIEW_ZOOM_IN:
			return QMessageBox::tr("\nZoom:   In");
			break;
		case ID_VIEW_ZOOM_OUT:
			return QMessageBox::tr("\nZoom:   Out");
			break;
		case ID_VIEW_ZOOM_WINDOW:
			return QMessageBox::tr("\nZoom:   Window");
			break;
		case ID_VIEW_ZOOM_EXTENTS:
			return QMessageBox::tr("\nZoom:   Extents");
			break;
		case ID_VIEW_ZOOM_PREVIOUS:
			return QMessageBox::tr("\nZoom:   Previous");
			break;
		case ID_VIEW_ZOOM_VIEWMAX:
			return QMessageBox::tr("\nZoom:   Maximum Out");
			break;
		case ID_VIEW_ZOOM_CENTER:
			return QMessageBox::tr("\nZoom:   Center");
			break;
		case ID_VIEW_3DVIEW:
			return QMessageBox::tr("\n3D Dynamic View Dialog");
			break;
		case ID_VIEW_AERIAL_LFRONT:
			return QMessageBox::tr("\nPredefined View:   Aerial Left Front View");
			break;
		case ID_VIEW_FRONT:
			return QMessageBox::tr("\nPredefined View:   Front View");
			break;
		case ID_VIEW_AERIAL_RFRONT:
			return QMessageBox::tr("\nPredefined View:   Aerial Right Front View");
			break;
		case ID_VIEW_LEFT_SIDE:
			return QMessageBox::tr("\nPredefined View:   Left Side View");
			break;
		case ID_VIEW_TOP:
			return QMessageBox::tr("\nPredefined View:   Top View");
			break;
		case ID_VIEW_WCSPLANVIEW:
			return QMessageBox::tr("\nPredefined View:   WCS Plan View");
			break;	
		case ID_VIEW_RIGHT_SIDE:
			return QMessageBox::tr("\nPredefined View:   Right Side View");
			break;
		case ID_VIEW_AERIAL_LREAR:
			return QMessageBox::tr("\nPredefined View:   Aerial Left Rear View");
			break;
		case ID_VIEW_BACK:
			return QMessageBox::tr("\nPredefined View:   Back View");
			break;
		case ID_VIEW_AERIAL_RREAR:
			return QMessageBox::tr("\nPredefined View:   Aerial Right Rear View");
			break;
		case ID_VIEW_BOTTOM:
			return QMessageBox::tr("\nPredefined View:   Bottom View");
			break;
		case ID_VIEW_BELOW_RFRONT:
			return QMessageBox::tr("\nPredefined View:   Bottom Right Front View");
			break;
		case ID_VIEW_BELOW_LFRONT:
			return QMessageBox::tr("\nPredefined View:   Bottom Left Front View");
			break;
		case ID_VIEW_BELOW_RREAR:
			return QMessageBox::tr("\nPredefined View:   Bottom Right Rear View");
			break;
		case ID_VIEW_BELOW_LREAR:
			return QMessageBox::tr("\nPredefined View:   Bottom Left Rear View");
			break;
		case  ID_HELP_MAN:
			return QMessageBox::tr("\nManual:   ");
			break;	
		case ID_SETTINGS_LAYERDIAG:
			return QMessageBox::tr("\nLayer Dialog:  ");
			break;
		case TILE_MODE:
			return QMessageBox::tr("\nTilemode:   ");
			break;
		case LX_CANCEL:
			return QMessageBox::tr("*CANCEL*");
			break;
		case CAPTION_INFORMATION:
			return QMessageBox::tr("LX_Viewer - Information Message");
			break;
		case CAPTION_WARNING:
			return QMessageBox::tr("LX_Viewer - Warning Message");
			break;
		case CAPTION_ABOUT:
			return QMessageBox::tr("About LX Drawing Viewer");
			break;
		case CAPTION_INITIALIZE:
			return QMessageBox::tr("OpenDWG Initialization Error");
			break;
		case END_APP_QUESTION:
			return QMessageBox::tr("<center>Do you wish to exit LX-Viewer after the batch process finishes?");
			break;
		case VECTORIZE_ERROR:
			return QMessageBox::tr("<center>Error on adInitVectorize");
			break;
		case CANNOT_INITIALIZE_AD3:
			return QMessageBox::tr("<center>Unable to initilize OpenDWG ViewKit - Ad3");
			break;
		case FILE_DOES_NOT_EXIST:
			return QMessageBox::tr("<center>File specified does not exist");
			break;
		case NO_READ_PERMISSION:
			return QMessageBox::tr("<center>You do not have read permission for this file");
			break;
		case NO_WRITE_PERMISSION:
			return QMessageBox::tr("<center>You do not have permission to write this file");
			break;
		case NO_DRAWING_LOADED_3D:
			return QMessageBox::tr("<center>In order to use the 3D View Controls you must open a drawing first.");
			break;
		case NO_DRAWING_LOADED_PRINT:
			return QMessageBox::tr("<center>In order to print you must open a drawing first.");
			break;
		case NO_DRAWING_LOADED_CMD_LINE:
			return QMessageBox::tr("\nA drawing must be opened before execution of your command line argument.");
			break;
		case CANNOT_SET_CURRENT_TO_FROZEN:
			return QMessageBox::tr("<center>You cannot set a frozen layer to be the current layer."
								"<center>Either thaw this layer or select another layer to be current");
			break;
		case ID_HELP_ABOUT_LX:
			extern bool lx_global_fast_read_enabled;
			return QMessageBox::tr("<center>Drawing Viewer Version %1"	
										"<center>Compiled using %2 read routines"				
										"<center>(C) 2001 by: <b>Andrew J. Bibb</b>"
										"<center>Project Site: <a href=\"http://lx-viewer.sourceforge.net\">http://lx-viewer.sourceforge.net</a></center>"
            "<center>Distributed under an ammended GNU Public License v2</center>"
            "<center>_________________________________________________________</center>"
            "<center>Project Team:</center>"
            "<p><b>Andrew Bibb</b> - Main developer and Project Administrator<br>"
            "<b>Paul Robertson</b> - Windows porting and Qt development<br>"
            "<b>Guy Edwards</b> - Web Site Manager, Documentation Editor and Project Administrator<br>"
            "<b>Brad Hubbard</b> - Support and Debugging<br>"
            "<b>Keith Frost</b> - Graphics, Splash Screens and Tool Buttons<br>"
            "<b>Nuno Sucena Almeida</b> - Translation to Portuguese and development of translation integration into the program.<br>"
            "<b>Joao Veiga</b> - Feedback and help with Brazillian Portugese translation<br>"
            "<center><em>Translators:</em></center>"
            "<p><b>Cyro Corte Real</b> - Translation to Brazillian Portugese<br>"
            "<b>Gan Hong Cien</b> - Translation to Indonesian<br>"
            "<b>Jurgen Pohl</b> - Translation to German<br>"
            "<b>Luis Melgratti</b> - Translation to Spanish<br>"
            "<b>Serena Ciriello</b> - Translation to Italian</p>"
            "<b>Michael Montagne</b> - New feature ideas and interface suggestions<br>"

            "<center><em>Testers:</em></center>"
            "<p><b>Alex Kirk</b> - Testing on OpenBSD 3.1 and NetBSD<br>"
            "<b>Miguel A. Mota</b> - Testing on Red Hat distributions<br>"
            "<b>Okeno Palmer</b> - General testing</p>"

            "<center><em>Thanks to:</em></center>"
            "<p><b>Valter Ballantini</b> - Helping with the check of the Italian translations<br>"
            "<b>Jacques Mony</b> - Help with the French translation<br>"
            "<b>Claudio Bonfanti</b> - Submitting graphics<br>"
	    "<b>O. Leroy</b> for publising the project and giving feedback</p>")
		.arg(LX_VERSION).arg(lx_global_fast_read_enabled ? "Fast" : "Generic");
			break;		
		case ID_HELP_ABOUT_QT:
			return QMessageBox::tr("<center>Drawing Viewer is built using the QT libraries.  Version %1 (c) 2002 Troll Tech. Inc. All rights reserved.").arg(qVersion());
			break;
		case ID_HELP_ABOUT_OD:
			return QMessageBox::tr("<center>Drawing Viewer is built using the OpenDWG ToolKit and ViewKit Libraries.  %1").arg(QString(adAdVersion()));
			break;		
		case NO_PS_VIEWPORTS:
			return QMessageBox::tr("<center>There are no valid PaperSpace viewports defined in this drawing."
										"<center> Remaining in ModelSpace.");
			break;
		case CALL_HELP:
			return QMessageBox::tr("\nHelp:   ");
			break;
		case CALL_VERSION:
			return QMessageBox::tr("\nVersion:   ");
			break;
		case COMMAND_LIST: 	//	list of all commands avaliable inside viewer
			return QMessageBox::tr("HELP   	- Displays this command listing"
													"\nVERSION	- Reports on the current version of the program"
													"\nMANUAL 	- Opens the user manual"		
													"\nAUDIT  	- Perform an audit on the drawing"
													"\nCLOSE  	- Will close the drawing and reset the screen elements"
													"\nSAVE   	- Will call up the file save dialog"
													"\nQSAVE  	- Will perform a quick save"	
													"\nQUIT   	- Quits the drawing"
													"\nOPEN   	- Opens a file dialog"
													"\nPAN    	- Starts the pan command"
													"\nREGEN  	- Regenerates and redraws the drawing on the screen"
													"\nZOOM   	- Starts the zoom command."
													"\nREPORT 	- Opens the report dialog"
													"\nPLOT   	- Will open the print dialog box for printing"
													"\nPRINT  	- Will also open the print dialog box (another name for PLOT)"
													"\n");
			break;  	
		case COMMAND_LINE_LIST:		//	list of all commands available on the command line
			return QMessageBox::tr("LX-Viewer Command Line Help"
							"\n\nUsage: viewer [FILE] [COMMAND] [OPTIONS]"
							"\n\nOptions:"
							"\n\t-help     Displays this command listing and exits."
							"\n\t-version  Reports on the current version of the program and exits."
							"\n\t-manual   Viewer loads and opens the user manual."
							"\n\nThe following commands must be called after a drawing name has been provided."
							"\nThe format is \"viewer [filename] [-command]\""
							"\n\t-audit    Will perform an audit on the drawing."
							"\n\t-report   Will open the report dialog for the current drawing."
							"\n\nThe following commands can be run in batch mode, that is"
							"\nthey will operate on an entire directory of drawings."
							"\nThe format is \"viewer [dir= path/to/directory] [-command] [-options]\""
							"\n\tCommand: -report"
							"\n\tOptions: -blocks -attributes"
														"\n\n");        	
			break;
		default:
			return QMessageBox::tr("\nInvalid Key sent to GetInfoMessage");
	}
}


