/***************************************************************************
                          layermanager.cpp  -  description
                             -------------------
    begin                : Tue Feb 26 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//	stuff to manage (on/off freeze/thaw/ lock/unlock layers) plus
//	set layer line types and colors

#include "layermanager.h"
#include "drawingboard.h"

#include "xpm/layeron.xpm"
#include "xpm/layeroff.xpm"
#include "xpm/layerfreeze.xpm"
#include "xpm/layerthaw.xpm"
#include "xpm/layerfreezeallvp.xpm"
#include "xpm/layerthawallvp.xpm"
#include "xpm/layerlock.xpm"
#include "xpm/layerunlock.xpm"
#include "xpm/layerprop.xpm"
#include <qlistbox.h>
#include <qtooltip.h>

extern PAD_TB adtb;
extern PAD_DWGHDR adhd;

/////////////////////////////////////////////////////////////////////////
//      				constructor for the layer dialog box
/////////////////////////////////////////////////////////////////////////
LX_LayerDialog::LX_LayerDialog(AD_DB_HANDLE fh, QWidget* parent,const char* name, bool modal, WFlags f):QDialog(parent,name,modal,f),
	varies_string(tr("Varies")),on_string(tr("  On ")),off_string(tr("  Off ")),thaw_string(tr(" Thawed ")),freeze_string(tr(" Frozen ")),
	lock_string(tr(" Locked ")),unlock_string(tr(" Unlocked ")),blank_string("   "),current_string(tr("Current"))
{
	//	size of graphics primitives
	const int border = 20;
	const int gap = 15;
	const int buttonwidth =  100;
	const int buttonheight = 32;
	const int labelheight = 25;
	const int boxwidth = 150;
	const int sboxwidth = 100;
	const int boxheight = 30;

	//	derived heights
	const int lv1width = 650;
	const int layerpropwidth = 2*border+gap+boxwidth+boxheight;
	const int layerpropheight = 2*border+7*labelheight+7*boxheight;
	const int lv1height = layerpropheight;
	int x = 0;
	int y = 0;

	//	data member and bool value set
	filehandle = fh;
	layerupdaterequired = false;

	//	give it a title
	setCaption(tr("Layer Dialog"));

	//	construct the list view portion of the dialog
	x = border;
	y = border;
 	lv1 = new QListView(this);	
		lv1->setGeometry(x,y,lv1width,lv1height);
		lv1->setRootIsDecorated(false);
		lv1->addColumn(tr("Layer"));
		lv1->addColumn(tr("Current"));
		lv1->addColumn(tr("Color"));
		lv1->addColumn(tr("Linetype"));
		lv1->addColumn(tr("On/Off"));
		lv1->addColumn(tr("Freeze/Thaw"));
		lv1->addColumn(tr("ViewPort"));
		lv1->addColumn(tr("Lock/Unlock"));
		lv1->setShowSortIndicator(true);
		lv1->setAllColumnsShowFocus(true);
		lv1->setSelectionMode(QListView::Extended);
		lv1->setItemMargin(2);

	//	construct the layer property portion of the dialog
	x += (lv1width + border);
	layerprop = new QGroupBox(this);
		layerprop->setGeometry(x,y,layerpropwidth,layerpropheight);
		layerprop->setTitle(tr("Layer Properties"));
			layernamelabel = new QLabel(tr("Layer Name"),layerprop);
				layernamelabel->setGeometry(border,y,boxwidth,labelheight);
				layernamelabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
				y += labelheight;
			layernamebox = new QLineEdit(layerprop);
				layernamebox->setGeometry(border,y,boxwidth+gap+boxheight,boxheight);
		    y += boxheight;
			layercolorlabel = new QLabel(tr("Color"),layerprop);
				layercolorlabel->setGeometry(border,y,boxwidth,labelheight);
				layercolorlabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
				y += labelheight;
			layercolorbox = new QLineEdit(layerprop);
				layercolorbox->setGeometry(border,y,boxwidth,boxheight);
			selectcolorbutton = new QPushButton(layerprop);
				selectcolorbutton->setGeometry(border+boxwidth+gap,y,boxheight,boxheight);
				y += boxheight;
			ltypelabel = new QLabel(tr("Linetype"),layerprop);
				ltypelabel->setGeometry(border,y,boxwidth,labelheight);
				ltypelabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
				y += labelheight;
			ltypebox = new QComboBox(false,layerprop);
				ltypebox->setGeometry(border,y,boxwidth,boxheight);
			selectltypebutton = new QPushButton(layerprop);
				selectltypebutton->setGeometry(border+boxwidth+gap,y,boxheight,boxheight);
				y += boxheight;
			oolabel = new QLabel(tr("On/Off"),layerprop);
				oolabel->setGeometry(border,y,sboxwidth,labelheight);
				oolabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
				y += labelheight;
			oobox = new QComboBox(false,layerprop);
				oobox->setGeometry(border,y,sboxwidth,boxheight);
		    y += boxheight;
			ftlabel = new QLabel(tr("Freeze/Thaw"),layerprop);
				ftlabel->setGeometry(border,y,sboxwidth,labelheight);
				ftlabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
				y += labelheight;
			ftbox = new QComboBox(false,layerprop);
				ftbox->setGeometry(border,y,sboxwidth,boxheight);
		    y += boxheight;
			vplabel = new QLabel(tr("ViewPort"),layerprop);
				vplabel->setGeometry(border,y,sboxwidth,labelheight);
				vplabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
				y += labelheight;
			vpbox = new QComboBox(false,layerprop);
				vpbox->setGeometry(border,y,sboxwidth,boxheight);
		    y += boxheight;
			lulabel = new QLabel(tr("Lock/Unlock"),layerprop);
				lulabel->setGeometry(border,y,sboxwidth,labelheight);
				lulabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
				y += labelheight;
			lubox = new QComboBox(false,layerprop);
				lubox->setGeometry(border,y,sboxwidth,boxheight);

	//	construct the button bars at the bottom
	x = border + lv1width + border + layerpropwidth - 8* buttonwidth - 7*gap;
	y += boxheight + 3*border;
	selectallbutton = new QPushButton(tr("Select All"),this);
		selectallbutton->setGeometry(x,y,buttonwidth,buttonheight);
		x += gap + buttonwidth;
	unselectallbutton = new QPushButton(tr("Unselect All"),this);
		unselectallbutton->setGeometry(x,y,buttonwidth,buttonheight);
		x += gap + buttonwidth;
	onallbutton = new QPushButton(tr("On All"),this);
		onallbutton->setGeometry(x,y,buttonwidth,buttonheight);
		x += gap + buttonwidth;
	offallbutton = new QPushButton(tr("Off All"),this);
		offallbutton->setGeometry(x,y,buttonwidth,buttonheight);
		x += gap + buttonwidth;
	thawallbutton = new QPushButton(tr("Thaw All"),this);
		thawallbutton->setGeometry(x,y,buttonwidth,buttonheight);
		x += gap + buttonwidth;
	freezeallbutton = new QPushButton(tr("Freeze All"),this);
		freezeallbutton->setGeometry(x,y,buttonwidth,buttonheight);
		x += gap + buttonwidth;
	deletelayerbutton = new QPushButton(tr("Delete Layer"),this);
		deletelayerbutton->setGeometry(x,y,buttonwidth,buttonheight);
		deletelayerbutton->setDisabled(true);
		x += gap + buttonwidth;
	newlayerbutton = new QPushButton(tr("New Layer"),this);
		newlayerbutton->setGeometry(x,y,buttonwidth,buttonheight);
	x = border + lv1width + border + layerpropwidth - 3*buttonwidth - 2*gap;
	y += buttonheight + gap;
	helpbutton = new QPushButton(tr("Help"),this);
		helpbutton->setGeometry(x,y,buttonwidth,buttonheight);
		helpbutton->setDisabled(true);
	x += gap + buttonwidth;
	okbutton = new QPushButton(tr("OK"),this);
		okbutton->setGeometry(x,y,buttonwidth,buttonheight);
	x += gap + buttonwidth;
	cancelbutton = new QPushButton(tr("Cancel"),this);
		cancelbutton->setGeometry(x,y,buttonwidth,buttonheight);

//	now connect signals and slots
	connect(lv1,SIGNAL(selectionChanged()),this,SLOT(lvselectionchanged()));
	connect(lv1,SIGNAL(mouseButtonClicked(int,QListViewItem*, const QPoint&, int)),this,SLOT(lvitemclicked(int,QListViewItem*,const QPoint&,int)));
	connect(selectcolorbutton,SIGNAL(clicked()),this,SLOT(selectcolorbuttonpushed()));
	connect(layercolorbox,SIGNAL(textChanged(const QString&)),this,SLOT(colorchanged(const QString&)));
	connect(layernamebox,SIGNAL(textChanged(const QString&)),this,SLOT(layernamechanged(const QString&)));
	connect(ltypebox,SIGNAL(activated(const QString&)),this,SLOT(ltypechanged(const QString&)));
	connect(oobox,SIGNAL(activated(const QString&)),this,SLOT(oochanged(const QString&)));
	connect(ftbox,SIGNAL(activated(const QString&)),this,SLOT(ftchanged(const QString&)));
	connect(vpbox,SIGNAL(activated(const QString&)),this,SLOT(vpchanged(const QString&)));
	connect(lubox,SIGNAL(activated(const QString&)),this,SLOT(luchanged(const QString&)));
	connect(okbutton,SIGNAL(clicked()),this,SLOT(okpressed()));
	connect(cancelbutton,SIGNAL(clicked()),this,SLOT(cancelpressed()));
	connect(helpbutton,SIGNAL(clicked()),this,SLOT(helppressed()));
	connect(selectallbutton,SIGNAL(clicked()),this,SLOT(selectall()));
	connect(unselectallbutton,SIGNAL(clicked()),this,SLOT(unselectall()));
	connect(onallbutton,SIGNAL(clicked()),this,SLOT(onall()));
	connect(offallbutton,SIGNAL(clicked()),this,SLOT(offall()));
	connect(thawallbutton,SIGNAL(clicked()),this,SLOT(thawall()));
	connect(freezeallbutton,SIGNAL(clicked()),this,SLOT(freezeall()));
	connect(deletelayerbutton,SIGNAL(clicked()),this,SLOT(deletelayer()));
	connect(newlayerbutton,SIGNAL(clicked()),this,SLOT(newlayer()));
}

void LX_LayerDialog::populatelv1()
{
    AD_LTYPE adltp;
    AD_OBJHANDLE layerhandle;
    char on;
    char frozen;
    char vpfrozen;
    char locked;
    QString col0("");
    QString col1("");
    QString col2("");
    QString col3("");
    QString col4("");
    QString col5("");
    QString col6("");
    QString col7("");

    adhd = adHeaderPointer(filehandle);
    adHancpy(layerhandle,adhd->curlayerobjhandle);
    adSeekLayer(filehandle,layerhandle,&currentlayer);	

    lv1->clear();
    layernamebox->clear();	
    layercolorbox->clear();
    selectcolorbutton->setPalette(layerprop->backgroundColor());
    ltypebox->clear();
    oobox->clear();
    ftbox->clear();
    vpbox->clear();
    lubox->clear();

    layerupdaterequired = false;

    layernamebox->setDisabled(true);
    layercolorbox->setDisabled(true);
    selectcolorbutton->setDisabled(true);
    ltypebox->setDisabled(true);
    oobox->setDisabled(true);
    ftbox->setDisabled(true);
    vpbox->setDisabled(true);
    lubox->setDisabled(true);
		
    adStartLayerGet(filehandle);
    for (int i = 0; i < (short)adNumLayers(filehandle);i++)
    {
        adGetLayer(filehandle,&adtb->lay);
        if (!adtb->lay.purgedflag)
        {
            adGetLayerState(filehandle,adtb->lay.objhandle,&on,&frozen,&vpfrozen,&locked);
            adSeekLinetype(filehandle,adtb->lay.linetypeobjhandle,&adltp);
            short coloridx = adtb->lay.color;
            if (coloridx < 0) coloridx *= -1; //	color of off layers is negative, we want the color as if it was on
            QString colornum = QString::number(coloridx);
	
            col0 = QString(adtb->lay.name);
            col1 = blank_string;
            if (col0 == currentlayer.name) col1 = current_string;	
            col2 = QString(tr(" Color %1 ").arg(colornum.rightJustify(4,' ')));
            col3 = QString(adltp.name);
            col4 = on ? QString(on_string):QString(off_string);
            col5 = frozen ? QString(freeze_string):QString(thaw_string);
            col6 = vpfrozen? QString(freeze_string):QString(thaw_string);
            col7 = locked? QString(lock_string):QString(unlock_string);
            lv1item = new QListViewItem(lv1,col0,col1,col2,col3,col4,col5,col6,col7);

            int cellheight = lv1item->height();	
            int w = cellheight + 7;
            int h = cellheight - 6;
            QPixmap col2pm(w,h);
            QPainter p;
            p.begin(&col2pm);
            p.setPen(LX_GetColor(256));	//	black border on rectangle
            p.setBrush(QBrush(LX_GetColor(coloridx),SolidPattern));
            p.drawRect(0,0,w,h);
            p.end();
            lv1item->setPixmap(2,col2pm);
        }
    }
}

void LX_LayerDialog::resetlayerdatabase()
{
    AD_OBJHANDLE layerhandle;
    AD_OBJHANDLE ltypehandle;
    AD_LAY adlay;
    const int base(10);
    const bool cs(false);
    bool ok(false);
    char on;
    char frozen;
    char vpfrozen;
    char locked;
	
    QListViewItemIterator it( lv1 );
    for ( ; it.current(); ++it )
    {
        adFindLayerByName(filehandle,const_cast<char*>((it.current()->text(0)).latin1()),layerhandle);
        adFindLinetypeByName(filehandle,const_cast<char*>((it.current()->text(3)).latin1()),ltypehandle);
        adSeekLayer(filehandle,layerhandle,&adlay);
        adGetLayerState(filehandle,layerhandle,&on,&frozen,&vpfrozen,&locked);
        
        on = it.current()->text(4) == off_string ? '\0' : '1' ;
        frozen = it.current()->text(5) == thaw_string ? '\0' : '1';
        vpfrozen = it.current()->text(6) == thaw_string ?  '\0' : '1';
        locked = it.current()->text(7) == unlock_string ?  '\0' : '1';
        
        QString cnum(it.current()->text(2));
        cnum = cnum.stripWhiteSpace();
        if(cnum.contains(QChar(' '))) cnum.remove(0,cnum.find(QChar(' '),0,cs));
        int temp = cnum.toInt(&ok,base);
        if (ok) adlay.color = temp;
        adHancpy(adlay.linetypeobjhandle,ltypehandle);
        
        if (!(adReplaceLayer(filehandle,&adlay)))
            DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,
                           QMessageBox::Warning,
                           QString(tr("<center>Error Writing Layer Information.  %1").arg(adErrorStr(adError()))));
        adSetLayerState(filehandle,layerhandle,on,frozen,vpfrozen,locked);
    }
}
    
//	Dialog Slots (public)
void LX_LayerDialog::okpressed()
{


    if (layerupdaterequired)
    {
        resetlayerdatabase();
        done(2);
    }
    else done(1);
}

void LX_LayerDialog::cancelpressed(){done(0);}
void LX_LayerDialog::helppressed(){done(-1);}
void LX_LayerDialog::selectltypebuttonpushed(){}
void LX_LayerDialog::deletelayer(){}
void LX_LayerDialog::unselectall(){lv1->clearSelection();}
void LX_LayerDialog::selectall()
{
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
	{
		it.current()->setSelected(true);
		it.current()->repaint();
	}
	lvselectionchanged();
}

void LX_LayerDialog::layernamechanged(const QString& s)
{
    AD_OBJHANDLE layerhandle;
    AD_LAY laybuffer;
    QString temp = s;
    temp = temp.simplifyWhiteSpace();
		const short buffsize(AD_MAX_STRLEN);		//	AD_MAX_STRLEN defined in ad2.h
		short laynamesize(AD_MAX_STRLEN);

	//	if the layername is varies, or if the layer name already exists,
	//	or the layer name would be an empty string then return	
    if (temp == varies_string) return;
    if (adFindLayerByName(filehandle,const_cast<char*>(temp.latin1()),layerhandle)) return;
    if (temp.isEmpty()) return;

	//	This is a new layer name, so change the listview item and update the drawing header layer table
    QListViewItem* curitem = lv1->currentItem();
    {
        adFindLayerByName(filehandle,const_cast<char*>((curitem->text(0)).latin1()),layerhandle);
        adSeekLayer(filehandle,layerhandle,&laybuffer);
				laynamesize = strlen(temp.latin1());
				if (laynamesize < buffsize)
				{
					memcpy(laybuffer.name,temp.latin1(),laynamesize);
					laybuffer.name[laynamesize] = '\0';
					curitem->setText(0,temp);
				}
				else
				{
					DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,
						QMessageBox::Warning,
						QString(tr("<center>Layer name \"%1\" exceeds %2 characters").arg(temp).arg(buffsize-1)));
						return;
				}
        if (!(adReplaceLayer(filehandle,&laybuffer)))
            DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,
                           QMessageBox::Warning,
                           QString(tr("<center>Error Writing Layer Information. %1 ").arg(adErrorStr(adError()))));
    }
}

void LX_LayerDialog::newlayer()
{
    const int base(10);
    const int defaultcolor = 7; // white
    QString defaultname = tr("NEWLAYER-");
    const QString defaultltype = tr("CONTINUOUS");
    AD_OBJHANDLE ltypehandle;
    AD_OBJHANDLE namehandle;
		const short buffsize(AD_MAX_STRLEN);		//	AD_MAX_STRLEN defined in ad2.h
		short laynamesize(AD_MAX_STRLEN);

	//	if linetype CONTINUOUS is not defined present the linetype of the first
	//	item in the listview as the default linetype
    if (!(adFindLinetypeByName(filehandle,const_cast<char*>(defaultltype.latin1()),ltypehandle)))
    {
        QListViewItemIterator it(lv1);
        adFindLayerByName(filehandle,const_cast<char*>((it.current()->text(3)).latin1()),ltypehandle);
    }		

	//	if layer NEWLAYER-1 does not exist, make it the new one, otherwise
	//	keep adding one to the end until we get a name that does not already exist
    int i = 0;
    do
    {
        i++;
    } while (adFindLayerByName(filehandle,const_cast<char*>((defaultname + QString::number(i,base)).latin1()),namehandle) == 1);			

    adSetDefaultLayer(filehandle,&adtb->lay);
		laynamesize = strlen((defaultname + QString::number(i,base)).latin1());
		if (laynamesize < buffsize)
		{
			memcpy(adtb->lay.name,(defaultname + QString::number(i,base)).latin1(),laynamesize);
			adtb->lay.name[laynamesize] = '\0';
		}	//	
		else	//	absolutely should never make it this far
		{
			DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,
				QString(tr("<center>Default layer name exceeds %1 characters").arg(buffsize-1)));
			return;
		}
    adtb->lay.color = defaultcolor;
    adHancpy(adtb->lay.linetypeobjhandle,ltypehandle);
    adGenerateObjhandle(filehandle,adtb->lay.objhandle);
    if (!adAddLayer (filehandle,&adtb->lay))
    {
        DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,
                       QString(tr("<center>There was an error creating this layer. %1 ").arg(adErrorStr(adError()))));
        return;
    }

    populatelv1();
    QListViewItemIterator it( lv1 );
    for ( ; it.current(); ++it )
        if(it.current()->text(0) == defaultname + QString::number(i,base))
        {
            lv1->setSelected(it.current(),true);
            lv1->setCurrentItem(it.current());
            lv1->ensureItemVisible(it.current());
        }
}

void LX_LayerDialog::freezeall()
{
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		if(it.current()->text(0) != currentlayer.name) it.current()->setText(5,freeze_string);
	layerupdaterequired = true;
	lvselectionchanged();
}
void LX_LayerDialog::thawall()
{
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		it.current()->setText(5,thaw_string);
	layerupdaterequired = true;
	lvselectionchanged();
}
void LX_LayerDialog::onall()
{
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		it.current()->setText(4,on_string);
	layerupdaterequired = true;
	lvselectionchanged();
}

void LX_LayerDialog::offall()
{
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		it.current()->setText(4,off_string);
	layerupdaterequired = true;
	lvselectionchanged();
}

void LX_LayerDialog::ltypechanged(const QString& s)
{
	if (s == varies_string) return;
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		if (it.current()->isSelected())it.current()->setText(3,s);
 	layerupdaterequired = true;
}
void LX_LayerDialog::oochanged(const QString& s)
{
	if (s == varies_string) return;
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		if (it.current()->isSelected())it.current()->setText(4,s);
	layerupdaterequired = true;
}
void LX_LayerDialog::ftchanged(const QString& s)
{
	if (s == varies_string) return;
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
	{
		if (it.current()->isSelected())

		{
			if (!(it.current()->text(0) == currentlayer.name && s == freeze_string)) it.current()->setText(5,s);
		}
	}		
	layerupdaterequired = true;
}
void LX_LayerDialog::vpchanged(const QString& s)
{
	if (s == varies_string) return;
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		if (it.current()->isSelected())it.current()->setText(6,s);
	 layerupdaterequired = true;
}
void LX_LayerDialog::luchanged(const QString& s)
{
	if (s == varies_string) return;
	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		if (it.current()->isSelected())it.current()->setText(7,s);
	layerupdaterequired = true;
}

//	slot to control what happens when the text in the color line edit changes
void LX_LayerDialog::colorchanged(const QString& s)
{	
	const bool cs(false);
	const int base = 10;
	bool ok(true);
	QString temp("");
	
	if (s.contains(varies_string,cs)) return;		//	only interested if text is distinct

	temp = s;
	temp = temp.stripWhiteSpace();
	if(temp.contains(QChar(' '))) temp.remove(0,temp.find(QChar(' '),0,cs));
	int colnum = temp.toInt(&ok,base);
	if (ok) selectcolorbutton->setPalette(LX_GetColor(colnum));
	else
	{
		selectcolorbutton->setPalette(layerprop->backgroundColor());
		return;
	}	//	no color we can deal with picked so return

	QListViewItemIterator it( lv1 );
	for ( ; it.current(); ++it )
		{
    	if (it.current()->isSelected() && it.current()->text(2) != s)
			{
				QPainter p;
				QPixmap col2pm(it.current()->pixmap(2)->width(),it.current()->pixmap(2)->height());
				p.begin(&col2pm);
				p.setPen(LX_GetColor(256));	//	black border on rectangle
				p.setBrush(QBrush(LX_GetColor(colnum),SolidPattern));
				p.drawRect(0,0,it.current()->pixmap(2)->width(),it.current()->pixmap(2)->height());
				p.end();

				it.current()->setText(2,layercolorbox->text());
  		  it.current()->setPixmap(2,col2pm);
				layerupdaterequired = true;
			}
		}
}


//	slot to control what happens when the color selection button is pushed
void LX_LayerDialog::selectcolorbuttonpushed()
{
	LX_ColorDialog* colordiag = new LX_ColorDialog(COLOR_DIAG_FOR_LAYER);
  int i = 0;							
  i = colordiag->exec();
  if (i > 0)
  {
		QString colornum = QString::number(i);
    layercolorbox->setText(QString(tr(" Color%1 ").arg(colornum.rightJustify(4,' '))));
   }

	delete colordiag;
}

//	slot to manage the events when the list view selections change.  Most of the code
//	below is to determine if one or multiple lines were selected, and if multiple is the
//	information contained in them the same or does it vary.
void LX_LayerDialog::lvselectionchanged()
{	
	const int base(10);
	bool ok(true);
	bool cs(false);
	bool noselection(true);
	QString layername("");
	QString layercolor("");
	QString ltypetext("");
	QString ootext("");
	QString fttext("");
	QString vptext("");
	QString lutext("");
	QStringList sl = QStringList();
	
	QListViewItemIterator it( lv1 );

	for ( ; it.current(); ++it )
	{
		if (it.current()->isSelected())
		{
			if (layername.isEmpty())
			{
				layername = it.current()->text(0);
				layercolor = it.current()->text(2);
				ltypetext = it.current()->text(3);
				ootext = it.current()->text(4);
				fttext = it.current()->text(5);
				vptext = it.current()->text(6);
				lutext = it.current()->text(7);
			}
		else
			{
				layername = varies_string;
				if (layercolor != it.current()->text(2))	layercolor = varies_string;
				if (ltypetext != it.current()->text(3))		ltypetext = varies_string;
				if (ootext != it.current()->text(4))			ootext = varies_string;
				if (fttext != it.current()->text(5))			fttext = varies_string;
				if (vptext != it.current()->text(6))			vptext = varies_string;
				if (lutext != it.current()->text(7))			lutext = varies_string;
			}	//	else				
		noselection = false;
		}	//	if (it.current()->isSelected...
  }	//	for loop

	layernamebox->setDisabled(layername == varies_string? true : noselection);
	layercolorbox->setDisabled(noselection);
	selectcolorbutton->setDisabled(noselection);
	ltypebox->setDisabled(noselection);
	oobox->setDisabled(noselection);
	ftbox->setDisabled(noselection);
	vpbox->setDisabled(noselection);
	lubox->setDisabled(noselection);
	
	layernamebox->clear();	
	layercolorbox->clear();
	selectcolorbutton->setPalette(layerprop->backgroundColor());

	ltypebox->clear();
	oobox->clear();
	ftbox->clear();
	vpbox->clear();
	lubox->clear();

	if (noselection) return;		//	

	layernamebox->setText(layername);
	layercolorbox->setText(layercolor);	
	layercolor = layercolor.stripWhiteSpace();
	if(layercolor.contains(QChar(' '))) layercolor.remove(0,layercolor.find(QChar(' '),0,cs));
	int colnum = layercolor.toInt(&ok,base);
	if (ok) selectcolorbutton->setPalette(LX_GetColor(colnum));
	
	adStartLinetypeGet(filehandle);
	
	int i;
	for (i = 0; i < (short)adNumLinetypes(filehandle);i++)
	{
		adGetLinetype(filehandle,&adtb->ltype);
		if (!adtb->ltype.purgedflag)
			if(QString(adtb->ltype.name) != "ByLayer" && QString(adtb->ltype.name) != "ByBlock")sl.append(QString(adtb->ltype.name));
	}	
	if (ltypetext == varies_string) sl.append(varies_string);
	sl.sort();
	ltypebox->insertStringList(sl);
	for (i = 0; i < ltypebox->count(); i++)
	{
		if (ltypetext == ltypebox->text(i)) ltypebox->setCurrentItem(i);
	}

	oobox->insertItem(on_string);
	oobox->insertItem(off_string);
	if (ootext == varies_string)
	{
		oobox->insertItem(varies_string);
		oobox->setCurrentItem(2);
	}
	else oobox->setCurrentItem((ootext==on_string)? 0 : 1);

	ftbox->insertItem(freeze_string);
	ftbox->insertItem(thaw_string);
	if (fttext == varies_string)
	{
		ftbox->insertItem(varies_string);
		ftbox->setCurrentItem(2);
	}
	else ftbox->setCurrentItem((fttext==freeze_string)? 0 : 1);
	
	vpbox->insertItem(freeze_string);
	vpbox->insertItem(thaw_string);
	if (vptext == varies_string)
	{
		vpbox->insertItem(varies_string);
		vpbox->setCurrentItem(2);
	}
	else vpbox->setCurrentItem((vptext==freeze_string)? 0 : 1);
	
	lubox->insertItem(lock_string);
	lubox->insertItem(unlock_string);
	if (lutext == varies_string)
	{
		lubox->insertItem(varies_string);
		lubox->setCurrentItem(2);
	}
	else lubox->setCurrentItem((lutext==lock_string)? 0 : 1);
}

//	slot to figure out which part of a listview item was clicked
void LX_LayerDialog::lvitemclicked(int button, QListViewItem* item, const QPoint& pos, int c)
{
	(void)button;
	(void)pos;
	AD_OBJHANDLE layerhandle;

	if (!item) return;		//	did not select on a listview item

	switch (c)
	{
		case 1:		//	change the current layer
			if(item->text(5) == freeze_string)
			{
				DisplayMessage(CANNOT_SET_CURRENT_TO_FROZEN);
				return;
			}
			{
			QListViewItemIterator it( lv1 );
			for ( ; it.current(); ++it )
				it.current()->setText(1,blank_string);
      }
			adFindLayerByName(filehandle,const_cast<char*>((item->text(0)).latin1()),layerhandle);
			adSeekLayer(filehandle,layerhandle,&currentlayer);
			item->setText(1,current_string);

			adhd = adHeaderPointer(filehandle);
			adHancpy(adhd->curlayerobjhandle,currentlayer.objhandle);
			break;

		case 2:
    	selectcolorbuttonpushed();
			break;	
	}
}
		
		

/////////////////////////////////////////////////////////////////////////
//								constructor for the layer bar
//////////////////////////////////////////////////////////////////////////
LX_LayerBar::LX_LayerBar(QWidget* parent, const char* name, WFlags f):QWidget(parent,name,f),pixmapsize(16),gap(5),border(1)
{
  //	graphics primitive sizes
	const int layerwidth = 260;
	const int ltypewidth = 135;
	const int colorwidth = 112;
	const int buttonwidth = 30;
	const int barheight = 25;
	int x = border;
	int y = border;

	setMinimumSize(2*border+3*gap+layerwidth+ltypewidth+colorwidth+buttonwidth,2*border+barheight);
	setMaximumSize(2*border+3*gap+layerwidth+ltypewidth+colorwidth+buttonwidth,2*border+barheight);

	//	widget members
	QIconSet QIS_lprop;
	QIS_lprop = QIconSet(QPixmap(layerprop));
	calllayerdialog = new QToolButton(this,"call layer dialog button");
		calllayerdialog->setIconSet(QIS_lprop);
		calllayerdialog->setAutoRaise(true);
		calllayerdialog->setGeometry(x,y,buttonwidth,barheight);
	
	x += buttonwidth + gap;
	layerinfo = new LX_LayerComboBox(false,this);
		layerinfo->setGeometry(x,y,layerwidth,barheight);
		layerinfo->setFocusPolicy(QWidget::NoFocus);

	x += layerwidth + gap;				
  color = new QComboBox(false,this);
		color->setGeometry(x,y,colorwidth,barheight);
		color->setFocusPolicy(QWidget::NoFocus);

	x += colorwidth + gap;
	ltype = new QComboBox(false,this);
		ltype->setGeometry(x,y,ltypewidth,barheight);	
		ltype->setFocusPolicy(QWidget::NoFocus);
	
//	now connect some signals and slots
	connect(color,SIGNAL(activated(const QString&)),this,SLOT(coloractivated(const QString&)));
	connect(ltype,SIGNAL(activated(const QString&)),this,SLOT(ltypeactivated(const QString&)));
	connect(calllayerdialog,SIGNAL(clicked()),this,SLOT(showdialog()));

	QWhatsThis::add(layerinfo,tr("Displays the layers and their states defined in the drawing"
			"\n\nThe icons from left to right are ON/OFF, FREEZE/THAW, FREEZEALLVP's/THAWALLVP's, and LOCK/UNLOCK "
			"followed by a rectangle indicating the current color assigned to the layer"));
	QWhatsThis::add(color,tr("Displays the colors available for drawing."
			"\n\nCurrently this bar has no function in the program other than to display the colors"));
	QWhatsThis::add(ltype,tr("Displays the linetypes defined with the drawing."
			"\n\nCurrently this bar has no function in the program other than to display loaded linetypes"));
	QWhatsThis::add(calllayerdialog,tr("Opens the full layer dialog box to change the states and settings of "
			"individual layers or for groups of layers.  Also you can add new layers from this dialog."));
	QToolTip::add(calllayerdialog,tr("Opens the layer dialog box"));
}


// layerbar public functions
void LX_LayerBar::clearcombos()
{
	layerinfo->clear();
	color->clear();
	ltype->clear();
}

void LX_LayerBar::initializebar(AD_DB_HANDLE openfile)
{
	layerinfo->filehandle = openfile;
	filehandle = openfile;
	populateltypebox(openfile);
	populatecolorbox();
	layerinfo->populatelayerinfobox();
	layerinfo->layeritemtodisplay();
}

//	layerbar public slots
//	the first 2 don't actually do anything right now.
void LX_LayerBar::coloractivated(const QString& s)
{
	(void)s;
	return;
}
void LX_LayerBar::ltypeactivated(const QString& s)
{
	(void)s;
	return;
}
void LX_LayerBar::showdialog()
{	
	extern DrawingBoard* db;

	db->printMessage(GetResourceString(ID_SETTINGS_LAYERDIAG));

	LX_LayerDialog* layerdiag = new LX_LayerDialog(filehandle, this);
	layerdiag->populatelv1();
	int r = layerdiag->exec();
	delete layerdiag;

	if (r >= 1)
	{
		layerinfo->populatelayerinfobox();
		layerinfo->layeritemtodisplay();
	}
	if (r >= 2)	db->switchview(ID_VIEW_REGEN);	
}

// LayerBar private functions
void LX_LayerBar::populateltypebox(const AD_DB_HANDLE& filehandle)
{
	QStringList sl = QStringList();
	ltype->clear();
	
	adStartLinetypeGet(filehandle);
	for (int i = 0; i < (short)adNumLinetypes(filehandle);i++)
	{
		adGetLinetype(filehandle,&adtb->ltype);
		if (!adtb->ltype.purgedflag) sl.append(QString(adtb->ltype.name));
	}	
	sl.sort();
	ltype->insertStringList(sl);
}

void LX_LayerBar::populatecolorbox()
{
    const int base = 10;
    int w = pixmapsize+4;
    int h = pixmapsize-2; // pictures in layerinfobox are 16 tall less 1 border T&B = 14 tall (actually drawn on)
	
    color->clear();

    QPixmap col1pm(w,h+4);
    col1pm.fill(Qt::white);
    QPainter p;
    p.begin(&col1pm);
    p.setPen(LX_GetColor(256));	//	black border on rectangle
    p.setBrush(QBrush(QColor(0xff,0xff,0xff),SolidPattern));
    p.drawRect(0,2,w,h);
    p.end();
    color->insertItem(col1pm,QString(" ByLayer "));
    color->insertItem(col1pm,QString(" ByBlock "));

    for (int i = 1; i <= 255; i++)
    {		
        col1pm.fill(Qt::white);
        QPainter p;
        p.begin(&col1pm);
        p.setPen(LX_GetColor(256));	//	black border on rectangle
        p.setBrush(QBrush(LX_GetColor(i),SolidPattern));
        p.drawRect(0,2,w,h);
        p.end();
	
        color->insertItem(col1pm,QString(tr(" Color %1").arg(QString::number(i,base))));
    }
}


///////////////////////////////////////////////////////////////////////////////////////////
//										LX_Layer Combo Box
//////////////////////////////////////////////////////////////////////////////////////////
LX_LayerComboBox::LX_LayerComboBox(bool rw,QWidget* parent, const char* name):QComboBox(rw,parent,name)
{
	pixmapsize = 16;
	gap = 5;
	border = 1;
	pmlon = QPixmap(layeron);
	pmloff = QPixmap(layeroff);
	pmlfreeze = QPixmap(layerfreeze);
	pmlthaw = QPixmap(layerthaw);
	pmlfreezevp = QPixmap(layerfreezeallvp);
	pmlthawvp = QPixmap(layerthawallvp);
	pmllock = QPixmap(layerlock);
	pmlunlock = QPixmap(layerunlock);	
	layerstatechanged = false;
	installEventFilter(this);
}

//  After much fiddling it appears that the first button press after the last selection
//	in the open LX_LayerComboBox is in a QListBox object.  I have no idea why this happens,
//	but we can use it to signal the revectorization requried.

//	I also discovered that the listbox popdown is controlled by a mouse release event.  So use
//	this filter to intercept and eat all mouse release events to the ComboBox.  This avoids
//	the flicker in selecting multiple items in the layer bar since we now don't have to keep
//	popup()ing the box.

//	The observed behavior is as follows:
//	This event filter only receives events when the LayerComboBox is active.  The first click to
//	open the box occurs in a LX_LayerComboBox object, all subsequent clicks on the open list occur
//	in QWidget objects,  and finally the last click somewhere else (while the combo box is still open)
//	occurs in a QListBox. 	

///*************************************************************************************************
///			NOTES FOR QT3.x
//	The previous worked for QT2.x but things have changed.  For 3.x we need to check if the
//	mouse release occurs in a QListBox object (not a QWidget object).  We will need to trap
//	all mouse clicks and releases and test for what they are.  We can no longer just look for
//	what QObject the mouse click was in. Eat all mouse release events so we only close the list box
//	when the user clicks outside of it.

bool LX_LayerComboBox::eventFilter(QObject* o, QEvent* e)
{
	extern DrawingBoard* db;
	
	if (o->isA("QListBox") && e->type() == QEvent::MouseButtonRelease) return true;	//	eat all mouse releases
	if (o->isA("QListBox") && e->type() == QEvent::MouseButtonPress)
  {
		QMouseEvent* e1 = (QMouseEvent*)e;

		if (this->listBox()->rect().contains(e1->pos() ) )	//	mouse click inside of the list box
		{
			int i = listBox()->currentItem();
			switch (layeractivated(i))
			{
				case 1:  			//	make selected current, change to that layer and drop the list
					setCurrentItem(i);
					return QComboBox::eventFilter(o,e);
				case 2:				//	display a message box & keep the list up
					DisplayMessage(CANNOT_SET_CURRENT_TO_FROZEN);
					break;
				case 3:				//	make the selected current, change to that layer and keep list up
					setCurrentItem(i);
					break;
				default:			//	keep the list up	
				break;	
			}		
			return true;
		}
	
	//	At this point we have a mouse click outside of the listbox, regen if a layerstate has changed
		if (layerstatechanged)
		{
			layerstatechanged = false;
			db->switchview(ID_VIEW_REGEN);
		}
	}

	//	event was nothing we are interested in, so pass it along
	return QComboBox::eventFilter(o,e);
}
	

//	populate the layerinfobox.  First read the layer data from the drawing
//	into a QStringList.  Then sort the list in alphabetical order.  Finally use a
//	list iterator to run back through the layer names reading the on/frozen/vpf/locked
//	states and create the pixmaps accordingly		
void LX_LayerComboBox::populatelayerinfobox()

{
	QStringList sl = QStringList();	
	QPixmap col1pm(5*pixmapsize+5*gap,pixmapsize+2);
	AD_OBJHANDLE layerhandle;
	AD_LAY lay;
		
	clear();

	adStartLayerGet(filehandle);
	for (int i = 0; i < (short)adNumLayers(filehandle);i++)
	{
		adGetLayer(filehandle,&adtb->lay);
		if (!adtb->lay.purgedflag) sl.append(QString(adtb->lay.name));
	}
	sl.sort();
	for (QStringList::Iterator it = sl.begin(); it!= sl.end(); ++it)
	{
		adFindLayerByName(filehandle,const_cast<char*>((*it).latin1()),layerhandle);
		adGetLayerState(filehandle,layerhandle,&on,&frozen,&vpfrozen,&locked);
		col1pm.fill(Qt::white);
		bitBlt(&col1pm,0*pixmapsize+0*gap,1,on? &pmlon:&pmloff,0,0,pixmapsize,pixmapsize,Qt::CopyROP);
		bitBlt(&col1pm,1*pixmapsize+1*gap,1,frozen? &pmlfreeze:&pmlthaw,0,0,pixmapsize,pixmapsize,Qt::CopyROP);
		bitBlt(&col1pm,2*pixmapsize+2*gap,1,vpfrozen? &pmlfreezevp:&pmlthawvp,0,0,pixmapsize,pixmapsize,Qt::CopyROP);
		bitBlt(&col1pm,3*pixmapsize+3*gap,1,locked? &pmllock:&pmlunlock,0,0,pixmapsize,pixmapsize,Qt::CopyROP);
		adSeekLayer(filehandle,layerhandle,&lay);	//	put the layer STRUCT info into a local variable
		laycolor = lay.color;

		if (laycolor < 0) laycolor *= -1;	//	colors of off layers are negative, we want the color as if it was on
		QPainter p;
		p.begin(&col1pm);
		p.setPen(LX_GetColor(256));	//	black border on rectangle
		p.setBrush(QBrush(LX_GetColor(laycolor),SolidPattern));
		p.drawRect(4*pixmapsize+4*gap,1,20,pixmapsize-2);
		p.end();
		insertItem(col1pm,(*it).latin1());
	}
}



//	public functions
//	return values signify the following:
//	1 - no states changed by click, layer selected is not frozen
//	2 - no states changed by click, layer selected is frozen
//	3 - states changed, click is on a layer not frozen
//	4 - states changed, click is on a frozen layer
//	results of the return values are as follows (most are done in the eventfilter)
//	1 - make selected current, change to that layer and drop the list
//	2 - display a message box & keep the list up
//	3 - make the selected current, change to that layer and keep list up
//	4 - keep the list up
short LX_LayerComboBox::layeractivated(int index)
{
	int n = 0;	//	number of the pixmap number
	QPixmap col1pm;
	QString layername;
	AD_OBJHANDLE layerhandle;
	AD_LAY currentlayer;
	
	col1pm = QPixmap(*pixmap(index));
	layername = text(index);

	adhd = adHeaderPointer(filehandle);
	adHancpy(layerhandle,adhd->curlayerobjhandle);
	adSeekLayer(filehandle,layerhandle,&currentlayer);
	
  adFindLayerByName(filehandle,const_cast<char*>(layername.latin1()),layerhandle);
	adGetLayerState(filehandle,layerhandle,&on,&frozen,&vpfrozen,&locked);
	
	QPoint pt = mapFromGlobal(QCursor::pos());
	n = (pt.x()-border+pixmapsize+gap)/(pixmapsize+gap);
	if (n >  4)
	{
		if (frozen) return 2;
		adHancpy(adhd->curlayerobjhandle,layerhandle);
		return 1;
	}

	switch (n)
	{
		case 1:
			on = on ? '\0':'1';
			bitBlt(&col1pm,0*pixmapsize+0*gap,1,on? &pmlon:&pmloff,0,0,pixmapsize,pixmapsize,Qt::CopyROP);
			layerstatechanged = true;
			break;
		case 2:
	    if (currentlayer.name == layername)
			{
			 	DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,
				QString(tr("<center>Layer %1 is the current layer.<center>It cannot be frozen").arg(layername)));	
			}
			else
			{
				frozen = frozen ? '\0':'1';
				bitBlt(&col1pm,1*pixmapsize+1*gap,1,frozen? &pmlfreeze:&pmlthaw,0,0,pixmapsize,pixmapsize,Qt::CopyROP);
				layerstatechanged = true;
			}
			break;
		case 3:
			vpfrozen = vpfrozen ? '\0':'1';
			bitBlt(&col1pm,2*pixmapsize+2*gap,1,vpfrozen? &pmlfreezevp:&pmlthawvp,0,0,pixmapsize,pixmapsize,Qt::CopyROP);	
			layerstatechanged = true;	
			break;
		case 4:
			locked = locked? '\0':'1';
			bitBlt(&col1pm,3*pixmapsize+3*gap,1,locked? &pmllock:&pmlunlock,0,0,pixmapsize,pixmapsize,Qt::CopyROP);
			layerstatechanged = true;
			break;
	}

	changeItem(col1pm,layername,index);
	adSetLayerState(filehandle,layerhandle,on,frozen,vpfrozen,locked);
	if (frozen) return 4;
	adHancpy(adhd->curlayerobjhandle,layerhandle);
	return 3;	
}
	
void LX_LayerComboBox::layeritemtodisplay()
{
	AD_OBJHANDLE layerhandle;
	AD_LAY layerbuffer;

	adhd = adHeaderPointer(filehandle);
	adHancpy(layerhandle,adhd->curlayerobjhandle);
	adSeekLayer(filehandle,layerhandle,&layerbuffer);
	for (int i = 0; i < count(); i++)
			if (text(i) == layerbuffer.name) setCurrentItem(i);
}		

