/***************************************************************************
                          layermanager.h  -  description
                             -------------------
    begin                : Tue Feb 26 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

/* Stuff to manage (on/off freeze/thaw/ lock/unlock layers) plus set layer line types and colors */

#ifndef LAYERMANAGER_H
#define LAYERMANAGER_H

/* QT includes */
#include <qwidget.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qpainter.h>
#include <qwhatsthis.h>
#include <qevent.h>
#include <qstringlist.h>
#include <qdialog.h>
#include <qlistview.h>
#include <qgroupbox.h>
#include <qtoolbutton.h>
#include <qlabel.h>

/* LX includes */
#include "colorfunctions.h"
#include "utilities.h"
#include "infomessage.h"

/* OD includes */
#define AD_PROTOTYPES
#include "ad2.h"
#include "ad3.h"

/* LayerDialog -- this is the main layer dialog control box it is built on a QDialog and includes a QListView */

class LX_LayerDialog : public QDialog
{
	Q_OBJECT
	
	public:
		LX_LayerDialog(AD_DB_HANDLE, QWidget* parent = 0, const char* name = 0,bool modal = TRUE, WFlags = 0);
	  void populatelv1();
		void resetlayerdatabase();
		AD_DB_HANDLE filehandle;
		AD_LAY currentlayer;

	private:
		/* listview members */
		QListView* lv1;
			QListViewItem* lv1item;
		/* groupbox members */
  	QGroupBox* layerprop;
			QLineEdit* layernamebox;
			QLineEdit* layercolorbox;
			QComboBox* ltypebox;
			QComboBox* oobox;
			QComboBox* ftbox;
			QComboBox* vpbox;
			QComboBox* lubox;
			QLabel* layernamelabel;
			QLabel* layercolorlabel;
			QLabel* ltypelabel;
			QLabel* oolabel;
			QLabel* ftlabel;
			QLabel* vplabel;
			QLabel* lulabel;
			QPushButton* selectcolorbutton;
			QPushButton* selectltypebutton;
		/* buttons at bottom */
		QPushButton* okbutton;
		QPushButton* cancelbutton;
		QPushButton* helpbutton;
		QPushButton* selectallbutton;
		QPushButton* unselectallbutton;
		QPushButton* onallbutton;
		QPushButton* offallbutton;
		QPushButton* thawallbutton;
		QPushButton* freezeallbutton;
		QPushButton* deletelayerbutton;
		QPushButton* newlayerbutton;

		/* other members */
		bool layerupdaterequired;
		const QString varies_string;
		const QString on_string;
		const QString off_string;
		const QString thaw_string;
		const QString freeze_string;
		const QString lock_string;
		const QString unlock_string;
		const QString blank_string;
		const QString current_string;
		
	public slots:
		void okpressed();
		void cancelpressed();
		void helppressed();
		void selectall();
		void unselectall();
		void onall();
		void offall();
		void thawall();
		void freezeall();
		void deletelayer();
		void newlayer();
		void lvselectionchanged();
		void lvitemclicked(int,QListViewItem*,const QPoint&,int);
		void selectcolorbuttonpushed();
		void selectltypebuttonpushed();
		void layernamechanged(const QString&);
		void colorchanged(const QString&);
		void ltypechanged(const QString&);
		void oochanged(const QString&);
		void ftchanged(const QString&);
		void vpchanged(const QString&);
		void luchanged(const QString&);
};

/* custom ComboBox derived from QComboBox, everything can be public since this is a private	*/
/* member of the LayerBar class below	*/
class LX_LayerComboBox : public QComboBox
{
	Q_OBJECT

	public:
		LX_LayerComboBox(bool rw = FALSE,QWidget* parent = 0, const char* name = 0);	/* constructor */
  	void populatelayerinfobox();

		bool eventFilter(QObject*,QEvent*);
	
		/* public data members */
		bool layerstatechanged;
		short pixmapsize;
		short gap;
		short border;
		char on;
		char frozen;
		char vpfrozen;
		char locked;
		short laycolor;
		QPixmap pmlon;
		QPixmap pmloff;
		QPixmap pmlfreeze;
		QPixmap pmlthaw;
		QPixmap pmlfreezevp;
		QPixmap pmlthawvp;
		QPixmap pmllock;
		QPixmap pmlunlock;
		AD_DB_HANDLE filehandle;
	
		short layeractivated(int);
		void layeritemtodisplay();
};	

/* layer bar which is inserted in a tool bar somewhere around the desktop */
class LX_LayerBar : public QWidget
{
 		Q_OBJECT
	
	public:
	LX_LayerBar(QWidget* parent=0, const char* name=0, WFlags f=0);		/* constructor */

	private:				
	const short pixmapsize;
	const short gap;
	const short border;
	AD_DB_HANDLE filehandle;
	
	/* layer bar members */
	LX_LayerComboBox* layerinfo;
	QComboBox* color;
	QComboBox* ltype;
	QToolButton* calllayerdialog;

	/* functions */
	void populateltypebox(const AD_DB_HANDLE&);
	void populatecolorbox();

	public:
	void clearcombos();
	void initializebar(AD_DB_HANDLE);

	public slots:
	void ltypeactivated(const QString&);
	void coloractivated(const QString&);
	void showdialog();
};

#endif
