/***************************************************************************
                          main.cpp
                    -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : buyer242299@aol.com
    source               : http://lx-viewer.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

/*	comment out the following line (the one that reads: #define USE_WINDOWS_STYLE)
		if you receive compile errors about	a missing or bad QWindowsStyle file*/

#define USE_WINDOWS_STYLE

//	QT includes
#include <qfont.h> 	// header for Qt fonts
#include <qfile.h>	// file handling class
#include <qtextstream.h>	// convenience class for text streams
#ifdef USE_WINDOWS_STYLE	// if USE_WIDOWS_STYLE is defined do....
#include <qwindowsstyle.h>	// gives a Microsoft Windows look and feel
#endif

//	LX includes
#include "splashapp.h"	// spalsh screen
#include "drawingboard.h"	// main work area
#include "resource.h"		//
#include "infomessage.h"	// information messages

//	OpenDwg includes
#define AD_PROTOTYPES
#define _(a) a
#include "ad2.h"
#include "ad3.h"
#include "odio.h"

DrawingBoard* db;

bool checkSystemVersionOk()
{
#ifdef _WIN32

	OSVERSIONINFOEX osvi;
	bool bOsVersionInfoEx;

	// Try calling GetVersionEx using the OSVERSIONINFOEX structure,
	// which is supported on Windows 2000.
	//
	// If that fails, try using the OSVERSIONINFO structure.
	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
	{
		// If OSVERSIONINFOEX doesn't work, try OSVERSIONINFO.
		osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
		if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) ) 
			return false;
	}

	switch (osvi.dwPlatformId)
	{
	case VER_PLATFORM_WIN32_NT:
		if ( osvi.dwMajorVersion <= 4 )
			qDebug( "System is Windows NT");
		else if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
			qDebug("System is Windows 2000");
		else if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )
			qDebug("System is Windows XP");
		else
			qDebug("System is Windows 2003 or something");
		return true;

	case VER_PLATFORM_WIN32_WINDOWS:
		if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0)
			qDebug("System is Windows 95");
		else if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 10)
			qDebug("System is Windows 98");
		else if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 90)
			qDebug("System is Windows ME");
		else
			// shouldn't get here
			qDebug("System is some sort of Windows 9x");
		return false;

	case VER_PLATFORM_WIN32s:
		// definitely shouldn't get here
		qDebug("System is Win32s !!");
		return false;

	default:
		// some platform we don't know about
		// might as well give it a go
		return true;
	}

#else
	// Linux version is always OK
	return true;
#endif
}

int main(int argc, char *argv[])
{
	
	QApplication::setDesktopSettingsAware(FALSE);

#ifdef USE_WINDOWS_STYLE
	QApplication::setStyle(new QWindowsStyle);
#endif
	
//	look for command line arguments that should be processed before
//	we call the application.
	if (argc > 1)
	{
		int i = 1;
		const bool cs(false);	//	string matches insensitive to case		
		bool returnnow(false);
//	create a psudo file and a text stream to direct the output to

		QFile console;
		console.open(IO_WriteOnly,stdout);
		QTextStream consoleOut(&console);

		for (i = 1; i <= argc; i++)
		{
			if (QString(argv[i]).contains(QT_TR_NOOP("VERSION"),cs))
			{
				consoleOut << "\n" << QT_TR_NOOP("LX-Viewer Version: ") << LX_VERSION << "\n\n";
				returnnow = true;
			}
			if (QString(argv[i]).contains(QT_TR_NOOP("HELP"),cs))
			{
				consoleOut << "\n" << GetResourceString(COMMAND_LINE_LIST);
				returnnow = true;
			}
  	}
		if (returnnow) return 0;
	}

//	any command line arguments remaining need to operate on an open drawing
  SplashApplication a(argc,argv);

  QLabel* splash = a.showSplash();

  db = new DrawingBoard(0,"Drawing Board");
  db->setCaption(QT_TR_NOOP("LX-Drawing Viewer"));
  a.setMainWidget(db);
  db->showMaximized();

	allocateadptrs();
	initializeODtoolkit();
  initializeODviewkit();
  registerfuns();

	delete splash;

  db->printMessage(QT_TR_NOOP("Drawing Viewer Initialized"),SBAR_TWIN);
	db->lookForSpptFiles();
  if (argc > 1) db->cmdbar->processCommandLineArgs();
  int rtnval = a.exec();
	adCloseAd3();
	adCloseAd2();
  freeadptrs();
	
  return rtnval;
}

