/***************************************************************************
                          plotter.cpp  -  description
                             -------------------
    begin                : Sat Jan 26 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

/*	The plotter class and related helper functions are contained
		in this file
*/

#include "plotter.h"
#include "drawingboard.h"
#include <qlayout.h>
//#define NO_HPGL_SUPPORT

//////////////////////////////////////////////////
//	plot dialog box
/////////////////////////////////////////////////

//	constructor
LX_PlotDialog::LX_PlotDialog(QWidget* parent,const char* name, bool modal, WFlags f):QTabDialog(parent,name,modal,f)
{
	//	data members, the default values are used in case the pcp file being read is defective
    leftsidescale = 0.125;				//	the left side term of a scale	
    rightsidescale = 12.0;  	    //	the right side term of a scale
    resolution = 300;							//	generic resolution value (to seed spinbox etc)
    psresolution = resolution;		//	ps printer resolution in dpi
    hpglresolution = resolution;	//	hpgl printer resolution in dpi
    margin = static_cast<int>(resolution*.625);//	paper margin in dots default 5/8"
    mmperinch = 25.4;							//
		s_varies = QString(tr("Varies"));
		frompoint = QPoint();
		topoint = QPoint();
    plotter = new QPrinter(QPrinter::HighResolution);
    plotter->setResolution(resolution);
    plotter->setPageSize(QPrinter::Letter);
    plotter->setOrientation(QPrinter::Portrait);
    plotter->setFullPage(TRUE);
    plotter->setColorMode(QPrinter::Color);
    plotter->setMinMax(1,1);

    int i;
    for (i = 0; i < 256; i++)			//	initialize the PenMap array, again in case
    {                                   //	default.pcp is defective or missing
        pendef[i].outcolor = i;
        pendef[i].penwidth = 0.010f;
    }

	//	sizes of graphics primitives
    const int border = 7;
    const int gap = 10;
    const int labelheight = 25;
    const int boxwidth = 100;
    const int sboxwidth = boxwidth-15;
    const int boxheight = 32;
    int radiobut = 0;

	
	//	derived sizes
    const int curpcpwidth = border*2 + boxwidth*2 + gap;
    const int curpcpheight = border*2 + boxheight + labelheight + gap;
    const int mappropheight = border*2 + boxheight*3 + labelheight*3;
    const int lv1width = border*2+boxwidth*2+gap+boxwidth;
    const int lv1height = mappropheight + border + curpcpheight;

    int x = 0;
    int y = 0;
	//create page 1
    page1 = new QWidget();

    printarea = new QButtonGroup(page1);
    printarea->setTitle(tr("Print Area"));
    pa1 = new QRadioButton(tr("Current View"),printarea);
		radiobut = gap + (pa1->sizeHint()).height();
	  pa1->move(gap,radiobut);
  	pa1->setChecked(TRUE);
 	  pa2 = new QRadioButton(tr("Saved View"),printarea);
 	  pa2->move(gap,radiobut*2);
    pa2->setDisabled(TRUE);
    pa3 = new QRadioButton(tr("Extents"),printarea);
    pa3->move(gap,radiobut*3);
    pa4 = new QRadioButton(tr("Limits"),printarea);
    pa4->move(gap,radiobut*4);
    pa4->setDisabled(TRUE);
    pa5 = new QRadioButton(tr("Window"),printarea);
    pa5->move(gap,radiobut*5);
    selectwindowbutton = new QPushButton(tr("Pick Area"),printarea,"select window button");
		selectwindowbutton->move(gap + boxwidth,radiobut*5);
		selectwindowbutton->setDisabled(TRUE);
		plotcoords = new QLabel("Plot Area",printarea,"text label to display selected win coords");
		plotcoords->setAlignment(AlignTop | AlignLeft);
		plotcoords->setText(tr("Plot Window: No Window Selected"));
		plotcoords->setGeometry(gap,radiobut*6+gap,lv1width-2*gap,labelheight*3);
		plotcoords->setDisabled(true);
		
		format = new QButtonGroup(tr("Output Format"),page1);
    f1 = new QRadioButton(tr("Postscript"),format);
    f1->setChecked(TRUE);
    f1->move(gap,radiobut);
    f1->setMaximumWidth(boxwidth-2);
    f2 = new QRadioButton(tr("HPGL"),format);
#ifdef NO_HPGL_SUPPORT
    f2->setDisabled(TRUE);
#endif
		f2->move(gap,radiobut*2+gap);
    f2->setMaximumWidth(boxwidth-2);
    marginlabel = new QLabel(tr("Margin"),format);
    marginlabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
    marginlabel->setGeometry(lv1width-3*border-2*sboxwidth-gap,gap,sboxwidth,labelheight);
    margindata = new QSpinBox(0,600,1,format);
    margindata->setValue(margin);
    margindata->setSuffix(tr(" dots"));
    margindata->setGeometry(lv1width-3*border-2*sboxwidth-gap,gap+labelheight,sboxwidth,boxheight);
    margindata->setWrapping(TRUE);
    resolutionlabel = new QLabel(tr("Resolution"),format);
    resolutionlabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
    resolutionlabel->setGeometry(lv1width-3*border-sboxwidth,gap,sboxwidth,labelheight);
    resolutiondata = new QSpinBox(1,1000,50,format);	
    resolutiondata->setValue(resolution);		
    resolutiondata->setSuffix(tr(" dpi"));
    resolutiondata->setGeometry(lv1width-3*border-sboxwidth,gap+labelheight,sboxwidth,boxheight);
    resolutiondata->setWrapping(TRUE);

		setupbutton = new QPushButton(tr("Printer Setup"),page1);
		writeBCFbut = new QPushButton(tr("Write BCF"),page1);
		printprev = new QPushButton(tr("Print Preview"),page1);
    printprev->setDisabled(TRUE);

    entitiestoprint = new QButtonGroup(page1);
    entitiestoprint->setTitle(tr("Entities to Print"));
    e2p1 = new QRadioButton(tr("All entities within Print Area"),entitiestoprint);
    e2p1->setMinimumWidth(curpcpwidth-2*gap);
    e2p1->move(gap,radiobut);
    e2p1->setChecked(TRUE);
    e2p2 = new QRadioButton(tr("Selected entities within Print Area"),entitiestoprint);
    e2p2->setMinimumWidth(curpcpwidth-2*gap);
    e2p2->move(gap,2*radiobut);
    e2p2->setDisabled(TRUE);
    entitiestoprint->insert(e2p1);
    entitiestoprint->insert(e2p2);	

    printscale = new QGroupBox(page1);
    printscale->setTitle(tr("Print Scale"));
    ps1 = new QCheckBox(tr("Fit Print to Size of Paper"),printscale);
    ps1->setMinimumWidth(curpcpwidth-2*gap);
    x = border;
    y = radiobut;
    ps1->move(x,y);
    y += radiobut;
    printedunit = new QLabel(tr("Printed Inches"),printscale);
    printedunit->setGeometry(x,y,boxwidth,labelheight);
    printedunit->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
    x += boxwidth + gap;
    drawingunit = new QLabel(tr("Drawing Units"),printscale);
    drawingunit->setGeometry(x,y,boxwidth,labelheight);
    drawingunit->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
    x = border;
    y += labelheight;	
    printedunittext = new QLineEdit(printscale);
    printedunittext->setGeometry(x,y,boxwidth,boxheight);
    printedunittext->setText(QString::number(leftsidescale,'g',3));
    x += boxwidth;
    equal = new QLabel("=",printscale);
    equal->setGeometry(x,y,gap,boxheight);
    equal->setAlignment(Qt::AlignCenter);
    x += gap;
    drawingunittext = new QLineEdit(printscale);
    drawingunittext->setGeometry(x,y,boxwidth,boxheight);
    drawingunittext->setText(QString::number(rightsidescale,'g',3));
    x = border + boxwidth;
    y += boxheight + gap;
    units = new QButtonGroup(tr("Paper Units"),printscale);
    units->setGeometry(x,y,boxwidth+gap,gap*2+radiobut*3);
    u1 = new QRadioButton(tr("Inches"),units);
    u1->setChecked(TRUE);
    u1->move(gap,radiobut * 1);
    u1->setMaximumWidth(boxwidth-2);
    u2 = new QRadioButton(tr("Millimeters"),units);
    u2->move(gap,radiobut * 2 + gap);
    u2->setMaximumWidth(boxwidth-2);
			
	//create page 2
    page2 = new QWidget();
    lv1 = new QListView(page2);
    x = border;
    y = border;
    lv1->setGeometry(x,y,lv1width,lv1height);
    lv1->setRootIsDecorated(FALSE);
    lv1->addColumn(tr("Screen Color"));
    lv1->addColumn(tr("Output Color"));
    lv1->addColumn(tr("Pen Width"));
    lv1->setShowSortIndicator(TRUE);
    lv1->setAllColumnsShowFocus(TRUE);
    lv1->setSelectionMode(QListView::Extended);
    lv1->setItemMargin(2);
    lv1->setColumnWidth(2,lv1->columnWidth(2)+40);
    mapprop = new QGroupBox(page2);
    x += (lv1width + border);
		y = radiobut;
    mapprop->setTitle(tr("Map Properties"));
    screenlabel = new QLabel(tr("Screen Color"),mapprop);
    screenlabel->setGeometry(border,y,boxwidth,labelheight);
    screenlabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
    y += labelheight;
    screencolortext = new QLineEdit(mapprop);
    screencolortext->setGeometry(border,y,boxwidth,boxheight);
    screencolortext->setDisabled(TRUE);
    screencolor = new QLabel(mapprop);
    screencolor->setGeometry(border+boxwidth+gap,y,boxheight,boxheight);
    screencolor->setFrameStyle(QFrame::Box | QFrame::Plain);
    y += boxheight;
    outlabel = new QLabel(tr("Output Color"),mapprop);
    outlabel->setGeometry(border,y,boxwidth,labelheight);
    outlabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
    y += labelheight;
    outcolortext = new QLineEdit(mapprop);
    outcolortext->setGeometry(border,y,boxwidth,boxheight);
    outcolortext->setDisabled(TRUE);
    outcolor = new QPushButton(mapprop);
    outcolor->setGeometry(border+boxwidth+gap,y,boxheight,boxheight);
    outcolor->setDisabled(TRUE);
    y += boxheight;
    linelabel = new QLabel(tr("LineWeight"),mapprop);
    linelabel->setGeometry(border,y,boxwidth,labelheight);
    linelabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
    y += labelheight;
    lineweight = new QLineEdit(mapprop);
    lineweight->setGeometry(border,y,boxwidth,boxheight);	
    lineweight->setDisabled(TRUE);
    unitlabel = new QLabel(mapprop);
    unitlabel->setGeometry(border+boxwidth+gap,y,boxwidth,boxheight);
    unitlabel->setAlignment(Qt::AlignVCenter | Qt::AlignLeft);
    unitlabel->setText(tr(" Inches"));

    curpcp = new QGroupBox(page2);
    curpcp->setTitle(tr("Printer Control File"));
    pcplabel = new QLabel(curpcp);
    pcplabel->setGeometry(border,radiobut,curpcpwidth-2*border,radiobut);
    pcplabel->setText(tr("Current PCP file: default.pcp"));
    pcplabel->setAlignment (Qt::AlignCenter);
    openpcp = new QPushButton(tr("Open PCP"),curpcp);
    openpcp->setGeometry(border, border+labelheight+radiobut*2,boxwidth,boxheight);
    savepcp = new QPushButton(tr("Save PCP"),curpcp);
    savepcp->setGeometry(border+boxwidth+gap, border+labelheight+radiobut*2,boxwidth,boxheight);
	
//	now define some parameters for the main tab box
    this->insertTab(page1, tr("Scale/View"));
    this->insertTab(page2, tr("Color/Width"));
    setCaption(tr("Plot Options"));
    setApplyButton(tr("Print"));
    QString null;
    setOkButton(null);
    setCancelButton();
    setHelpButton();

//	now insert items into layouts
		QHBoxLayout* hb11 = new QHBoxLayout(page1,border,gap,"main layout for page 1");
		QVBoxLayout* vb11 = new QVBoxLayout(0,0,-1,"left pane of page 1 layout");
		QHBoxLayout* hb12 = new QHBoxLayout(0,0,-1,"buttons at bottom of left pane in page 1");
		QVBoxLayout* vb12 = new QVBoxLayout(0,0,-1,"right pane of page 1 layout");
		hb11->addLayout(vb11);
		hb11->addStretch();
		hb11->addLayout(vb12);
		vb11->addWidget(printarea);
		vb11->addStretch();
		vb11->addWidget(format);
		vb11->addStretch();
		vb11->addLayout(hb12);
		hb12->addWidget(setupbutton);
		hb12->addStretch();
		hb12->addWidget(writeBCFbut);
		hb12->addStretch();
		hb12->addWidget(printprev);
		vb12->addWidget(entitiestoprint);
		vb12->addStretch();
		vb12->addWidget(printscale);

		QHBoxLayout* hb21 = new QHBoxLayout(page2,border,gap,"main layout for page 2");
		QVBoxLayout* vb21 = new QVBoxLayout(0,0,-1,"right pane of page 2 layout");
		hb21->addWidget(lv1);
		hb21->addLayout(vb21);
		vb21->addWidget(mapprop);
		vb21->addStretch();
		vb21->addWidget(curpcp);

//	now read the default pcp file and adjust assumed values as necessary
//	need to explictly set leftsidescale and rightsidescale because signals and
//	slots are not yet connected.  If the file does not exist no message box will
//	be displayed, if the file exists but cannot be opened then a box will appear.
    QFile defaultpcp(QDir::homeDirPath() + LXV_HOME + LXV_PCP + "/default.pcp");
    if(defaultpcp.exists())
    {
        readpcpfile(QDir::homeDirPath() + LXV_HOME + LXV_PCP + "/default.pcp");
        leftsidescale = ArchToDouble(printedunittext->text());
        rightsidescale = ArchToDouble(drawingunittext->text());
    }	

//	pcp file is read, now populate the list box
    for (i = 1; i <= 255; i++)
    {
        lv1item = new QListViewItem(lv1,pentotext(i),pentotext(pendef[i].outcolor),QString::number(pendef[i].penwidth,'f',3));
        int cellheight = lv1item->height();	
        int w = cellheight + 7;
        int h = cellheight - 6;
        QPixmap col1pm(w,h);


        QPixmap col2pm(w,h);
        QPainter p;
        p.begin(&col1pm);
        p.setPen(LX_GetColor(256));	//	black border on rectangle
        p.setBrush(QBrush(LX_GetColor(i),SolidPattern));
        p.drawRect(0,0,w,h);
        p.end();
        p.begin(&col2pm);
        p.setPen(LX_GetColor(256));	//	black border on rectangle
        p.setBrush(QBrush(LX_GetColor(pendef[i].outcolor),SolidPattern));
        p.drawRect(0,0,w,h);
        p.end();

	
        lv1item->setPixmap(0,col1pm);
        lv1item->setPixmap(1,col2pm);
    }	// 	for loop
	
//	finally connect signals and slots
    connect(setupbutton,SIGNAL(clicked()),this,SLOT(plottersetup()));
    connect(savepcp,SIGNAL(clicked()),this,SLOT(pcpwrite()));
    connect(openpcp,SIGNAL(clicked()),this,SLOT(pcpread()));
    connect(units,SIGNAL(clicked(int)),this,SLOT(changeunits(int)));
    connect(printedunittext,SIGNAL(textChanged(const QString&)),this,SLOT(printedunitchanged(const QString&)));
    connect(drawingunittext,SIGNAL(textChanged(const QString&)),this,SLOT(drawingunitchanged(const QString&)));
    connect(lv1,SIGNAL(selectionChanged()),this,SLOT(lvselectionchanged()));
    connect(outcolor,SIGNAL(clicked()),this,SLOT(outcolorbuttonpushed()));
    connect(lineweight,SIGNAL(textChanged(const QString&)),this,SLOT(penwidthtextchanged(const QString&)));
    connect(outcolortext,SIGNAL(textChanged(const QString&)),this,SLOT(outcolortextchanged(const QString&)));
    connect(ps1,SIGNAL(stateChanged(int)),this,SLOT(fitprintchanged(int)));
    connect(margindata,SIGNAL(valueChanged(int)),this,SLOT(marginchanged(int)));
    connect(this,SIGNAL(applyButtonPressed()),this,SLOT(printpressed()));
    connect(this,SIGNAL(cancelButtonPressed()),this,SLOT(cancelpressed()));
    connect(this,SIGNAL(helpButtonPressed()),this,SLOT(helppressed()));
    connect(format,SIGNAL(clicked(int)),this,SLOT(outformatchanged(int)));
    connect(resolutiondata,SIGNAL(valueChanged(int)),this,SLOT(resolutionchanged(int)));
		connect(pa5,SIGNAL(toggled(bool)),this,SLOT(pa5changed(bool)));
		connect(selectwindowbutton,SIGNAL(clicked()),this,SLOT(setPlotWindow()));
		connect(writeBCFbut,SIGNAL(clicked()),this,SLOT(writeBCF()));
}

///////////////////////////////////////////////////////////////////
//						Private Functions
///////////////////////////////////////////////////////////////////
//	function to read a pcp file from disk
int LX_PlotDialog::readpcpfile(QString filetoread)
{
    const bool cs = FALSE;
    const QChar eqchar('=');
    const int base = 10;
    int charpos = 0;
    int colornum = 0;
    int pennum = 0;
    float penw = 0.0;
    bool ok = TRUE;

    QFile existpcp(filetoread);
    if(!existpcp.open (IO_ReadOnly))

    {	
        DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,
                       QString(tr("<center>An error occured opening:  <center>"
                                  "<center>Please check that the file exists and that you have permission to open it.").arg(filetoread)));
        return 0;
    } 		
			
    QTextStream rs (&existpcp);
    QString s;

    while (!rs.eof())
    {
        s = rs.readLine();
        if (s.contains("UNITS = _I",cs))
        {
            u1->setChecked(TRUE);;
            printedunit->setText(tr("Printed Inches"));
            unitlabel->setText(tr(" Inches"));
        }
        if (s.contains("UNITS = _M",cs))
        {
            u2->setChecked(TRUE);
            printedunit->setText(tr("Printed Millimeters"));
            unitlabel->setText(tr(" Millimeters"));
        }
        if (s.contains("SCALE")) scaleparser(s);
        if (s.contains("BEGIN_COLOR",cs))
        {
            charpos = s.find(eqchar,0,cs);
            s.remove(0,charpos+1);
            colornum = s.toInt(&ok,base);
            if (&ok)   //	if colornumber is valid
            {
                while (!s.contains("END_COLOR",cs))
                {
                    s = rs.readLine();
                    if (s.contains("PEN_NUMBER",cs))
                    {
                        charpos = s.find(eqchar,0,cs);
                        s.remove(0,charpos+1);
                        pennum = s.toInt(&ok,base);
                        if (&ok) pendef[colornum].outcolor=pennum;
                    }
                    if (s.contains("PEN_WEIGHT",cs))	
                    {
                        charpos = s.find(eqchar,0,cs);
                        s.remove(0,charpos+1);
                        penw = s.toFloat(&ok);
                        if (&ok) pendef[colornum].penwidth=penw;
                    }
                }	//	while loop
            }	//	if color number is valid
        }	//	if BEGIN_COLOR was true			
    }	//	while rs is not at eof()		

    existpcp.close();
    return 1;
}

//	function to write a pcp file to disk
int LX_PlotDialog::writepcpfile(QString filetowrite)
{
    QString ls;
    QString rs;
    ls = printedunittext->text();
    rs = drawingunittext->text();
    if ((ls.contains(QChar('/'))) && (!ls.contains(QChar(' ')))) ls = QString("0 ") + ls;
    if ((rs.contains(QChar('/'))) && (!rs.contains(QChar(' ')))) rs = QString("0 ") + rs;

    QFile newpcp(filetowrite);
    if(!newpcp.open (IO_WriteOnly)) 		
    {	
        DisplayMessage(CUSTOM_MESSAGE,CAPTION_WARNING,QMessageBox::Warning,
                       QString(tr("<center>An error occured opening:<center>%1"
                                  "<center>Please check that you have permission to write to it.")
                               .arg(filetowrite)));
        return 0;
    } 		
    QTextStream ws (&newpcp);
	
    ws << QString("VERSION = 1.0");								//	this line not used by us
    u1->isOn() ? ws << QString("\nUNITS = _I") :ws << QString("\nUNITS = _M");
    ws << QString("\nORIGIN = 0.00,0.00");		    //	this line not used by us
    ws << QString("\nSIZE = MAX");								//  this line not used by us
    ws << QString("\nROTATE = 0");								//  this line not used by us
    ws << QString("\nHIDE = _N");									// 	this line not used by us
    ws << QString("\nPEN_WIDTH = 0.010000");			//	this line not used by us
    if (ps1->isChecked()) ws << QString("\nSCALE = _F");
    else ws << QString("\nSCALE = ") + ls + " = " + rs;
    ws << QString("\nPLOT_FILE = NONE");					//	this line not used by us
    ws << QString("\nFILL_ADJUST = _N");					//	this line not used by us
    ws << QString("\nOPTIMIZE_LEVEL = 0");				//	this line not used by us


    for (short i = 1; i < 256; i++)
    {
        ws << QString("\n\nBEGIN_COLOR = ");
        ws << i;

        ws << QString("\n  PEN_NUMBER = ");
        ws << pendef[i].outcolor;
		
        ws << QString("\n  HW_LINETYPE = 0");			//	this line not used by us
        ws << QString("\n  PEN_SPEED = 0");				//	this line not used by us
			
        ws << QString("\n  PEN_WEIGHT = ");
        ws << pendef[i].penwidth;

        ws << QString("\nEND_COLOR");
    }

    newpcp.close();
    return 1;
}

//	function to retrun a pen name from the integer pen data
QString LX_PlotDialog::pentotext(const int& pn)
{
  const int base(10);
	QString rtn("");
	
	switch (pn)
	{
		case 1:
			rtn = "   1 Red   ";
			break;
		case 2:
			rtn = "   2 Yellow   ";
			break;

		case 3:
			rtn = "   3 Green   ";
			break;
		case 4:
			rtn = "   4 Cyan   ";
			break;
		case 5:
			rtn = "   5 Blue   ";
			break;
		case 6:
			rtn = "   6 Magenta   ";
			break;
		case 7:
			rtn = "   7 White   ";
			break;
		case 8:
			rtn = "   8 Gray   ";
			break;
		case 256:
			rtn = " 256 Black   ";
			break;
		default:
			rtn = QString::number(pn,base);
			rtn.stripWhiteSpace();
			rtn = QString(" ") + rtn + QString("   ");
			if (pn <= 99) rtn = QString(" ") + rtn;
			if (pn <= 9 ) rtn = QString(" ") + rtn;
	}			

	return rtn;
}

//	function called when the word SCALE is read from the pcp file.  It breaks down the line
//	to pieces of text we can extract the scale data from.  We assume there are 2 equal signs
//	in the string, if not the function ArchToDouble should return an error (-1) which we
//	can test for
void LX_PlotDialog::scaleparser(QString s)
{
	QString ls;
	QString rs;
	const QChar equalsign('=');
	const bool cs = FALSE;
	int index = 0;
	double rtnval;
	
	if (s.contains("_F",cs))
	{
		ps1->setChecked(TRUE);
		return;
	}

//	since we are at this point it would appear that there is some sort of scale
//	lets try to parse it.  If text comes back with an error set scale to 1.0

	ps1->setChecked(FALSE);

	index = s.find(equalsign,0,cs);
	s.remove(0,index+1);
	index = s.find(equalsign,0,cs);
	ls = rs = s;
	ls.remove(index,s.length());
	rs.remove(0,index+1);

	ls = ls.simplifyWhiteSpace();
	if (ls.startsWith(QString("0 "))) ls.remove(0,2);
	rtnval = ArchToDouble(ls);
	if (rtnval <= 0.0) printedunittext->setText("1.00");
		else printedunittext->setText(ls);
		
	rs = rs.simplifyWhiteSpace();
	if (rs.startsWith(QString("0 "))) rs.remove(0,2);
	rtnval = ArchToDouble(rs);
	if (rtnval <= 0.0) drawingunittext->setText("1.00");
		else drawingunittext->setText(rs);
}

//	function to populate the list view.  Only need to deal with columns 2 and 3 since column
//	1 is set in the constructor and is constant
void LX_PlotDialog::populatelistview()
{
	QPixmap col2pm;
	QString col2("");
	QString col3("");
	int i = 1;

	lv1->setSorting(0,TRUE);	//	sort by first column in ascending order
	lv1->sort();	
	QListViewItemIterator it( lv1 );

	for ( ; it.current(); ++it )
	{
		col2 = pentotext(pendef[i].outcolor);
		col3 = QString::number(pendef[i].penwidth,'f',3);
		
		QPainter p;
		QPixmap col2pm(it.current()->pixmap(1)->width(),it.current()->pixmap(1)->height());
		p.begin(&col2pm);

		p.setPen(LX_GetColor(256));	//	black border on rectangle
		p.setBrush(QBrush(LX_GetColor(pendef[i].outcolor),SolidPattern));
		p.drawRect(0,0,it.current()->pixmap(1)->width(),it.current()->pixmap(1)->height());
		p.end();
		
		it.current()->setText(1,col2);
		it.current()->setText(2,col3);
    it.current()->setPixmap(1,col2pm);
	
		i++;
  }	
}
////////////////////////////////////////////////////////////////////////
//						Public slots
////////////////////////////////////////////////////////////////////////
void LX_PlotDialog::plottersetup()
{
	plotter->setup();
}

void LX_PlotDialog::pcpread()
{
    const QChar slash('/');
    int lastslash = 0;

    QString openname = QFileDialog::getOpenFileName(QDir::homeDirPath() + LXV_HOME + LXV_PCP, "Printer Control Files (*.pcp)",this);
    if (!openname.isEmpty())
    {
        if(readpcpfile(openname))
        {
            lastslash = openname.findRev(slash);
            openname = openname.remove(0,lastslash+1);
            pcplabel->setText(QString(tr("Current PCP file: %1")
                                      .arg(openname)));
            populatelistview();		
        }
    }
    return;
}

void LX_PlotDialog::printpressed()
	{
		if (f2->isChecked()) done(2); 	//	hpgl output
		else done(1);                   //	ps output (or anything else if it ever exists)
	}
void LX_PlotDialog::cancelpressed(){done(0);}
void LX_PlotDialog::helppressed(){done(-1);}

void LX_PlotDialog::pcpwrite()
{
    const QChar dot('.');
    const QChar slash('/');
    int lastslash = 0;
    int lastdot = 0;
    QString ftype = "";

    QString savename = QFileDialog::getSaveFileName(QDir::homeDirPath() + LXV_HOME + LXV_PCP,"Printer Control Files (*.pcp)",this );
    if (!savename.isEmpty())
    {
        ftype = savename;
        lastdot=savename.findRev(dot);
        ftype.remove(0,lastdot+1);
        if (!(ftype.lower()=="pcp"))
            savename = savename + ".pcp";	
        if(writepcpfile(savename))
        {
            lastslash = savename.findRev(slash);
            savename = savename.remove(0,lastslash+1);
            pcplabel->setText(QString(tr("Current PCP file: %1")
                                      .arg(savename)));
        }
    }
    return;
}

//	slot to deal with the things which need to be changed when the units
//	are changed
void LX_PlotDialog::changeunits(int buttonpressed)
{
    QString t("");
    double d = 0.0;
    double factor = 1.0;
    bool ok = TRUE;

    switch (buttonpressed)
    {
        case 0:		//	button u1 or inches
            printedunit->setText(tr("Printed Inches"));
            unitlabel->setText(tr(" Inches"));
            factor /= mmperinch;
            break;
        case 1:		//	button u2 or millimeters
            printedunit->setText(tr("Printed Millimeters"));
            unitlabel->setText(tr(" Millimeters"));
            factor = mmperinch;
            break;
    }
	
	//	now recalculate all the pen widths
    int i = 1;
    QListViewItemIterator it( lv1 );
    for ( ; it.current(); ++it )
    {
        t = it.current()->text(2);
        d = t.toDouble(&ok);
        if (ok)
        {
            d *= factor;
            t = QString::number(d,'f',3);
            it.current()->setText(2,t);
            pendef[i].penwidth = d;
        }
        i++;
    }
}

//	Slot to change the left side scale - the drawing inches/millimeters text box
void LX_PlotDialog::printedunitchanged(const QString& s)
{
	double d;

	d = ArchToDouble(s);
	if (d >= 0.0) leftsidescale = d;
}

//	Slot to change the right side scale - the plotted units text box
void LX_PlotDialog::drawingunitchanged(const QString& s)
{
	double d;
		
	d = ArchToDouble(s);
	if (d >= 0.0) rightsidescale = d;
}

//	slot to manage the events when the list view selections change.  Most of the code
//	below is to determine if one or multiple lines were selected, and if multiple is the
//	information contained in them the same or does it vary.
void LX_PlotDialog::lvselectionchanged()
{
	bool ok(TRUE);
	bool cs(FALSE);
	int base = 10;
	int scnumber = 0;
	QString temp("");
	QString sctext("");
	QString octext("");
	QString pwtext("");

	outcolortext->setDisabled(FALSE);
	outcolor->setDisabled(FALSE);
	if (f1->isChecked())lineweight->setDisabled(FALSE);
	
	QListViewItemIterator it( lv1 );

	for ( ; it.current(); ++it )
	{
		if (it.current()->isSelected())
		{
			temp = it.current()->text(0);
			temp = temp.stripWhiteSpace();
			if(temp.contains(QChar(' '))) temp.remove(temp.find(QChar(' '),0,cs), temp.length());
			scnumber = temp.toInt(&ok,base);
			if (!ok) return;
			if (sctext.isEmpty()) sctext = temp;
				else sctext = s_varies;
		
			if (octext.isEmpty()) octext = pentotext(pendef[scnumber].outcolor);
			if (octext != pentotext(pendef[scnumber].outcolor)) octext = s_varies;

			if (pwtext.isEmpty()) pwtext = QString::number(pendef[scnumber].penwidth,'f',3);
			if (pwtext != QString::number(pendef[scnumber].penwidth,'f',3)) pwtext = s_varies;
		}
  }

		if (sctext != s_varies)
		{	screencolortext->setText(pentotext(scnumber));
			screencolor->setPalette(LX_GetColor(scnumber));
		}
		else
		{	screencolortext->setText(s_varies);
			screencolor->setPalette(lv1->backgroundColor());
		}

		if (octext != s_varies)
		{ outcolortext->setText(pentotext(pendef[scnumber].outcolor));
			outcolor->setPalette(LX_GetColor(pendef[scnumber].outcolor));
		}
		else
		{	outcolortext->setText(s_varies);
			outcolor->setPalette(lv1->backgroundColor());
		}

		if (pwtext != s_varies)
			lineweight->setText(QString::number(pendef[scnumber].penwidth,'f',3));
		else
			lineweight->setText(s_varies);	

	return;
}

//	slot to call up the color dialog box
void LX_PlotDialog::outcolorbuttonpushed()
{
	LX_ColorDialog* colordiag = new LX_ColorDialog(COLOR_DIAG_FOR_PLOTTER);
	int i = 0;
							
	i = colordiag->exec();
	if (i >= 0) outcolortext->setText(pentotext(i));

	delete colordiag;
}

//	slot to change the output color selected
void LX_PlotDialog::outcolortextchanged(const QString& s)
{	
	const bool cs(FALSE);
	const int base = 10;
	bool ok(TRUE);
	QString temp("");
	int i = 0;
	int cn = 0;
	QPixmap col2pm;

	if (s.contains(s_varies),cs) return;			//	we are only interested if the text is distinct

	QListViewItemIterator it( lv1 );
		for ( ; it.current(); ++it )
		{
    	if (it.current()->isSelected())
			{	temp = it.current()->text(0);
				temp = temp.stripWhiteSpace();
				if(temp.contains(QChar(' '))) temp.remove(temp.find(QChar(' '),0,cs), temp.length());
				i = temp.toInt(&ok,base);
        if (!ok) return;
			  temp = s;

				temp = temp.stripWhiteSpace();
				if(temp.contains(QChar(' '))) temp.remove(temp.find(QChar(' '),0,cs), temp.length());
				cn = temp.toInt(&ok,base);
				if (!ok) return;
		
				QPainter p;
				QPixmap col2pm(it.current()->pixmap(1)->width(),it.current()->pixmap(1)->height());
				p.begin(&col2pm);
				p.setPen(LX_GetColor(256));	//	black border on rectangle
				p.setBrush(QBrush(LX_GetColor(cn),SolidPattern));
				p.drawRect(0,0,it.current()->pixmap(1)->width(),it.current()->pixmap(1)->height());
				p.end();
		
				it.current()->setText(1,pentotext(cn));
			  pendef[i].outcolor = cn;
  		  it.current()->setPixmap(1,col2pm);
			}
		}
	 outcolor->setPalette(LX_GetColor(cn));	//	change the color of the layer button
}

//	slot to change the pen width selected
void LX_PlotDialog::penwidthtextchanged(const QString& s)
{
	const bool cs(FALSE);
	const int base = 10;
	bool ok(TRUE);
	QString temp("");
	int i = 0;
	double d = 0.0;

	if (s.contains(s_varies,cs)) return;			//	we are only interested if the text is distinct
	QListViewItemIterator it( lv1 );
		for ( ; it.current(); ++it )
		{
    	if (it.current()->isSelected())
			{	temp = it.current()->text(0);
				temp = temp.stripWhiteSpace();
				if(temp.contains(QChar(' '))) temp.remove(temp.find(QChar(' '),0,cs), temp.length());
				i = temp.toInt(&ok,base);
        if (!ok) return;
				d = s.toDouble(&ok);
				if (!ok) return;
			
			  it.current()->setText(2,s);
			  pendef[i].penwidth =d;
			}
		}
}

//	slot to change things when the status of the fit drawing checkbox is changed
//	state = 2 if on
//	state = 1 if no change
//	state = 0 if off
void LX_PlotDialog::fitprintchanged(int state)
{
	static QString savedls("");	
	static QString savedrs("");

	switch(state)
	{
		case 2:      	//	fit to print selected
		{
			savedls = printedunittext->text();
			printedunittext->clear();
			printedunittext->setDisabled(TRUE);
			savedrs = drawingunittext->text();
			drawingunittext->clear();
			drawingunittext->setDisabled(TRUE);
			break;
		}
		default:       //	any other button state
		{
			printedunittext->setText(savedls);
			printedunittext->setDisabled(FALSE);
			drawingunittext->setText(savedrs);
			drawingunittext->setDisabled(FALSE);
		}
	}
}

//	function to change the margin number
void LX_PlotDialog::marginchanged(int newmargin){margin = newmargin;}
		
//	function to change things when the user selects HPGL or Postscript
void LX_PlotDialog::outformatchanged(int newformat)
{
	switch(newformat)
	{
		case 0:					//	PS selected
		{
			hpglresolution = resolution;
			resolutiondata->setValue(psresolution);
			outlabel->setText(tr("Output Color"));
			lineweight->setDisabled(FALSE);
			lv1->setColumnText(1,tr("Output Color"));
			break;
		}
		case 1:					//	HPGL selected
		{
			psresolution = resolution;
			resolutiondata->setValue(hpglresolution);
			outlabel->setText(tr("Pen Number"));
			lineweight->setDisabled(TRUE);
			lv1->setColumnText(1,tr("Pen Number"));		
			break;
		}		//case1
	}		//switch
}		// function

//	function to change the setting of the resolution value
void LX_PlotDialog::resolutionchanged(int res)
{
	resolution = res;
	plotter->setResolution(resolution);
}

//	function to activate or deactivate the select window button
void LX_PlotDialog::pa5changed(bool oo)
{
	selectwindowbutton->setEnabled(oo);
	plotcoords->setEnabled(oo);
}

void LX_PlotDialog::writeBCF()
{
	const QString eq ("  =  ");
	const QString comma(" , ");
	const QString yes("YES");
	const QString no("NO");
	extern DrawingBoard* db;

	QString plot_bcf = QDir::homeDirPath() + LXV_HOME + "/plot.bcf";
	QFileInfo fi = QFileInfo(plot_bcf);
	QFileInfo fd = fi.dirPath(true);
	if ((fi.exists() && !fi.isWritable()) || !fd.isWritable())
	{
		DisplayMessage(NO_WRITE_PERMISSION);
		db->printMessage(tr("\nWarning: %1 was not saved to disk - write permission not granted").arg(plot_bcf));
		return;
	}

	QFile f(plot_bcf);
	if (f.open(IO_WriteOnly))
	{
		QTextStream stream( &f );
		stream << ";   LX-VIEWER BATCH CONTROL FILE - plot.bcf";
		stream << "\n;\n;   This file contains the information needed to do batch plotting.";
		stream << "\n;   It is only used when supplying a directory of drawings to the program on the command line";
		stream << "\n;\n;   We recommend that you do not edit this file manually";
		stream << "\n;\n;";
		stream << "\n;\n; Printer Settings";
		stream << "\nPrinter" << eq << plotter->printerName();
		stream << "\nPrint to File" << eq << (plotter->outputToFile() ? yes : no);
		stream << "\nOrientation" << eq << static_cast<int>(plotter->orientation());
		stream << "\nPage Size" << eq << static_cast<int>(plotter->pageSize());
		stream << "\nColor Mode" << eq << static_cast<int>(plotter->colorMode());				
		stream << "\n;\n; Page 1 Settings";
		stream << "\nPrint Area" << eq << printarea->id(printarea->selected());
    stream << "\nFrom Point X" << eq << frompoint.x();
		stream << "\nFrom Point Y" << eq << frompoint.y();
		stream << "\nTo Point X" << eq << topoint.x();
		stream << "\nTo Point Y" << eq << topoint.y();
		stream << "\nFormat" << eq << format->id(format->selected());
		stream << "\nMargin" << eq << margindata->cleanText();
		stream << "\nResolution" << eq << resolutiondata->cleanText();
		stream << "\nEntities" << eq << entitiestoprint->id(entitiestoprint->selected());
		stream <<	"\nPrint to Fit" << eq << (ps1->isChecked()? yes : no);
		stream << "\nPrinted Unit" << eq << printedunittext->text();
		stream << "\nDrawing Unit" << eq << drawingunittext->text();
		stream << "\nUnits" << eq << units->id(units->selected());
		stream << "\n;\n; Page 2 Pen Mappings";		
		QListViewItemIterator it( lv1 );
		for (int i = 1; i < 255; i++)
    	stream << "\nPen Map" << eq << i << comma << pendef[i].outcolor << comma << pendef[i].penwidth;

  	QMessageBox* mb = new QMessageBox (tr("Need User Input"),
					tr("<center>After batch processing do you wish to exit LX-Viewer?"
						"<center>If you select <b>YES</b> then LX-Viewer will exit to a command prompt after "		
						"the batch processing ends.  If you select <b>NO</b> then LX-Viewer will remain open "
						"and running after the batch process ends."),
					QMessageBox::Information,
					QMessageBox::Yes | QMessageBox::Default,
					QMessageBox::No  | QMessageBox::Escape,
					0,this);
		int rtnval = mb->exec();
		delete mb;
		stream << "\nTerminate" << eq << (rtnval == QMessageBox::No ? "NO" : "YES");	
		f.close();
	}
	db->printMessage(tr("\nBatch Control File %1 saved to disk").arg(plot_bcf));
}

void LX_PlotDialog::setPlotWindow(){done(-2);}	//	return to drawingboard to pick the plot window
	

///////////////////////////////////////////////////////////////////////////
//	                Plotter public functions
///////////////////////////////////////////////////////////////////////////
//	batching functions
void LX_PlotDialog::setOptions(QString dir2process)
{
	bool terminate(false);
	extern DrawingBoard* db;

//	open the batch control file for reading, and set the options accordingly
	QFile plot_bcf(QDir::homeDirPath() + LXV_HOME + "/plot.bcf");
  if(plot_bcf.exists()) 		// there is a BCF so continue processing
	{
		db->printMessage(QFileDialog::tr("Attempting to open %1").arg(plot_bcf.name()),TWIN);
  	if(!plot_bcf.open (IO_ReadOnly))	//	BCF file exists but for some reason we can't open it	
  	{	
			db->printMessage(QFileDialog::tr("Read error opening %1 -- File not read").arg(plot_bcf.name()),TWIN);
	 	}
    else
		{
			//	now read each line of the file
			QTextStream rs (&plot_bcf);
			const QChar eq('=');
			const QChar semi(';');
			const QChar comma(',');
			const bool cs(false);
			bool ok;
			QString key;
			QString val;

			while (!rs.eof())
			{
				key = rs.readLine();
				key = key.simplifyWhiteSpace();
				val = key;
				if (key != semi && key.contains(eq,cs))		//	the line read is not a comment and appears properly formatted
				{
					int split = key.find(eq,0,cs);
		 		  key = key.left(split);
					val = val.remove(0,++split);
					key = key.simplifyWhiteSpace();
					val = val.simplifyWhiteSpace();
				
					if (key == "Pen Map")
					{
						QStringList sl = QStringList::split(comma,val,false);
						QString zero, one, two;
						bool ok0,ok1,ok2;
						zero = sl[0];
						one = sl[1];
						two = sl[2];
						int i = val.toInt(&ok0);
						int c = val.toInt(&ok1);
						double d = val.toDouble(&ok2);
            if (ok0 && ok1 && ok2)
						{
							pendef[i].outcolor = c;
							pendef[i].penwidth = d;
						}
   				}			
					else if (key == "Terminate")
						terminate = ((val == "YES") ? true : false);	
          else if (key == "Printer") plotter->setPrinterName(val);
					else if (key == "Print to File") plotter->setOutputToFile(val == "YES" ? true : false);
					else if (key == "Orientation")
						{
						int b = val.toInt(&ok);
						if (ok) plotter->setOrientation(static_cast<QPrinter::Orientation>(b));
						}
					else if (key == "Page Size")
						{
						int b = val.toInt(&ok);
						if (ok) plotter->setPageSize(static_cast<QPrinter::PageSize>(b));
						}
					else if (key == "Color Mode")
						{
						int b = val.toInt(&ok);
						if (ok) plotter->setColorMode(static_cast<QPrinter::ColorMode>(b));
						}
					else if (key == "Print Area")
					 	{
						int b = val.toInt(&ok);
						if (ok) printarea->setButton(b);
						}
					else if (key == "From Point X")
						{
						int b = val.toInt(&ok);
						if (ok) frompoint.setX(b);
						}
					else if (key == "From Point Y")
						{
						int b = val.toInt(&ok);
						if (ok) frompoint.setY(b);
						}
					else if (key == "To Point X")
						{
						int b = val.toInt(&ok);
						if (ok) topoint.setX(b);
						}
					else if (key == "To Point Y")
						{
						int b = val.toInt(&ok);
						if (ok) topoint.setY(b);
						}
					else if (key == "Format")
					 	{
						int b = val.toInt(&ok);
						if (ok) format->setButton(b);
						}
					else if (key == "Margin")
					 	{
						int b = val.toInt(&ok);
						if (ok) margindata->setValue(b);
						}
					else if (key == "Resolution")
					 	{
						int b = val.toInt(&ok);
						if (ok) resolutiondata->setValue(b);
						}
					else if (key == "Entities")
					 	{
						int b = val.toInt(&ok);
						if (ok) entitiestoprint->setButton(b);
						}
					else if (key == "Print to Fit") ps1->setChecked(val == "YES" ? true : false);
					else if (key == "Printed Unit") printedunittext->setText(val);
					else if (key == "Drawing Unit") drawingunittext->setText(val);
					else if (key == "Units")
					 	{
						int b = val.toInt(&ok);
						if (ok) units->setButton(b);
						}
				}	//	if we can read the line
			}	//	while rs is not at eof()	
			populatelistview();	
			db->printMessage(QFileDialog::tr("Batch Control File %1 processed").arg(plot_bcf.name()),TWIN);
		}	//	else statement		
		plot_bcf.close();
	}
	else
	{
		db->printMessage(QFileDialog::tr("Batch Control File %1 not found").arg(plot_bcf.name()),TWIN);
		return;
	}

//	commandproc.cpp arranges for the directory to process to be the last item in the
//	options list.  By doing this we only need to remove the text up to and including
//	the = sign to get the directory name to process
	short mark = 0;
	mark = dir2process.find('=');
	if (mark  > 0)
	{
		dir2process.remove(0,++mark);
	}
	batchProcess(dir2process);

//	Can't exit normally at this point because we have not started the main
//	event loop a.exec().  This is a cruddy piece of code, but it does allow a clean
//	exit from the program when started from the command line
	if (terminate) exit(0);
}

//	batch process routine
void LX_PlotDialog::batchProcess(QString dir2process)
{
	extern DrawingBoard* db;
	QStringList filebatch;
	QString currentDrawing;
	QString openname;
	AD_OBJHANDLE psBlockHandle;
	AD_DB_HANDLE	ithandle;	

	adSetupDwgRead();
	adSetupDxfRead();
	db->setCancelPending(false);

//	if a drawing is currently open save its name and release all of the memory associated with it
	if (db->getDwgHandle())
	{
		if (db->getOpenFileName()) currentDrawing = db->getOpenFileName();
		db->closefile();
	}	

	if (dir2process.isNull())	//	only call the openfile dialog if no directory was provided
	{	
		QFileDialog* p2fsd = new QFileDialog(this,"Process directory for batch plotting dialog",true);
		p2fsd->setMode(QFileDialog::ExistingFiles);
		p2fsd->setDir(QDir::homeDirPath());
		p2fsd->setFilter("Drawing or DXF Files (*.dwg *.DWG *.dxf *.DXF)");
		p2fsd->setCaption(tr("Batch Plot - Multiple File Select"));
		p2fsd->setViewMode(QFileDialog::List);
		int rtnval = p2fsd->exec();
		filebatch = p2fsd->selectedFiles();
    delete p2fsd;

		if (rtnval == QDialog::Rejected || filebatch.isEmpty())	
		{
			db->printMessage(tr("No files selected"),SBAR_TWIN);
			return;
		}
		//	now provide an opportunity to change plotter settings
		rtnval = this->exec();
		if (rtnval <= 0) return;
	}
	else		//	directory provided so process it
	{
		QDir d = QDir(dir2process);
		d.setNameFilter("*.dwg *.DWG *.dxf *.DXF");
		QStringList namelist = d.entryList();
		QStringList::Iterator it = namelist.begin();
		filebatch.clear();
		while (it != namelist.end())
		{	
			filebatch.append(dir2process + '/' + *it);
			++it;
		}
	}
	db->LX_ClearScreen();	
	QStringList::Iterator it = filebatch.begin();
	db->printMessage(tr("\n\nBatch Plot:"));

	while( it != filebatch.end() )
	{
		openname = *it;
		QFileInfo fi = QFileInfo(openname);
		if (fi.isReadable())
		{
			if ((ithandle=adLoadFile(const_cast<char*>(openname.latin1()),AD_PRELOAD_ALL,1)))
			{
				db->printMessage(QFileDialog::tr("Plotting: %1").arg(openname));	
				if (plotter->outputToFile())
				{
					QFileInfo fi = QFileInfo(openname);
					plotter->setOutputFileName(fi.dirPath(true) + '/' + fi.baseName(true) + ".ps");
				}
				setcontrolvariables(); //	function in vectorize.cpp (for OD-viewkit)
				initializegraphics();  //	function in vectorize.cpp (to set graphics area)
				vectorize(ithandle,psBlockHandle);

				if (f1->isChecked()) //	system or postscript selected
				{
					db->miscpainter.begin(this->plotter);
					plotps(ithandle,psBlockHandle);
					db->miscpainter.end();
				}
				else if (f2->isChecked())  //	hpgl selected
				 	plothpgl(ithandle,psBlockHandle);

				if (ithandle) {adTerminateVectorize(ithandle); adCloseFile(ithandle);}	
			}	//	if we got a drawing handle
			else db->printMessage(QFileDialog::tr("Error opening file %1: %2").arg(openname).arg(QString(adErrorStr(adError()))));			 		
		}	//	if the file was readable
	else db->printMessage(QFileDialog::tr("Warning: %1  was not read - read permission not granted").arg(openname));
	++it;
	}	//	while loop

	// now reload the saved drawing
	if (!currentDrawing.isEmpty()) db->openfile(currentDrawing);
}

//	function to return the printed output width in ACAD real units
double LX_PlotDialog::getplotareawidth()
{
	extern AD_VIEW_PARMS adviewparms;
	extern AD_VIEW_PARMS plotviewparms;
	QPaintDeviceMetrics pdm(plotter);	
	int pw = pdm.width()-2*margin;
	int ph = pdm.height()-2*margin;
	double sw = 0.0;
	double sh = 0.0;
	double rtnval = (pw)*rightsidescale/(leftsidescale*resolution/(u2->isChecked()? mmperinch:1.0));	

	switch (getPrintAreaSetting())
	{
	case 0:	//	plot the current view - adjust viewparm sizes to avoid a regen	
		sw = adviewparms.xsize;
		sh = adviewparms.ysize;
		if (ps1->isChecked())
			if ((sh/sw) > (ph/pw)) rtnval = sh*pw/ph;
			else rtnval = sw;
		break;
  case 2: 	//	plot extents - already have done a regen (actually a zoom extents) so just return the size
		if (ps1->isChecked()) rtnval = plotviewparms.xsize;
		break;
  case 4:	//	plot window - again adjust sizes to avoid a regen
		sw = plotviewparms.xsize;
		sh = plotviewparms.ysize;
		if (ps1->isChecked())
			if ((sh/sw) > (ph/pw)) rtnval = sh*pw/ph;
			else rtnval = sw;
		break;
	}	//	switch statement
	return rtnval;	
}

//	function to return the printed output height in ACAD real units
double LX_PlotDialog::getplotareaheight()
{
	extern AD_VIEW_PARMS adviewparms;
	extern AD_VIEW_PARMS plotviewparms;
	QPaintDeviceMetrics pdm(plotter);	
	int pw = pdm.width()-2*margin;
	int ph = pdm.height()-2*margin;
	double sw = 0.0;
	double sh = 0.0;
	double rtnval = (ph)*rightsidescale/(leftsidescale*resolution/(u2->isChecked()? mmperinch:1.0));
	
	switch (getPrintAreaSetting())
	{
	case 0:	//	plot the current view - adjust viewparm sizes to avoid a regen	
		sw = adviewparms.xsize;
		sh = adviewparms.ysize;
		if (ps1->isChecked())
			if ((sh/sw) > (ph/pw)) rtnval = sh;
			else rtnval = sw*ph/pw;
		break;
	case 2: 	//	plot extents - already have done a regen (actyally a zoom extents) so just return the size
		if (ps1->isChecked()) rtnval = plotviewparms.ysize;
		break;
  case 4:	//	plot window - again adjust sizes to avoid a regen
		sw = plotviewparms.xsize;
		sh = plotviewparms.ysize;
		if (ps1->isChecked())
			if ((sh/sw) > (ph/pw)) rtnval = sh;
			else rtnval = sw*ph/pw;
		break;
	}	//	switch statement
	return rtnval;	
}

//	function to return the printed output width in dots
double LX_PlotDialog::getoutputwidth()
{
	QPaintDeviceMetrics pdm(plotter);
	return pdm.width()-2*margin;
}

//	function to return the printed output height in dots
double LX_PlotDialog::getoutputheight()
{
	QPaintDeviceMetrics pdm(plotter);	
	return pdm.height()-2*margin;
}

//	function to return the pen color
short LX_PlotDialog::getpencolor(const int& ac){return pendef[ac].outcolor;}


//	function will return a pen line width measured in pixels
//	this function can return a pen width of 0 which is legal in
//	QT.  A width of zero draws a  line very fast but not with precision
//	It seems to be a very thin line so keep it.
int LX_PlotDialog::getoutpenwidth(const int& ac)
{
	int div;
  double rem;

	if (u1->isChecked()) rem = pendef[ac].penwidth * resolution;
	else rem = pendef[ac].penwidth / mmperinch * resolution;
	
	div = static_cast<int>(rem);
	rem -= div;
	if (rem >= 0.5) div++;

	return div;
}

//	function to return the margin size in pixels
double LX_PlotDialog::getmargin(){return margin;}

//	function to return the name of the hpgl output device
QString LX_PlotDialog::gethpgloutdevice()
{
	if (plotter->outputToFile())
		 return plotter->outputFileName();
  else return QString("SERIALPORT");
}

//	function to return the selected radio button in PrintArea
int LX_PlotDialog::getPrintAreaSetting(){return printarea->id(printarea->selected());}

//	function to return one of the plot window corners (0 == from point, anything else == topoint)
QPoint LX_PlotDialog::getWindowCorner(short corner){return (corner == 0 ? frompoint : topoint);}

//	function to set the window points
void LX_PlotDialog::setPlotCoordsLabel(const QPoint& p1, const QPoint& p2, QString s)
{
	frompoint = p1;
	topoint = p2;

  if (s.isEmpty())plotcoords->setText(tr("Plot Window: No Window Selected"));
	else plotcoords->setText(tr("Plot Window : %1").arg(s));
}






