/***************************************************************************
                          plotter.h  -  description
                             -------------------
    begin                : Sat Jan 26 2002
    copyright            : (C) 2002 by Andrew Bibb
    source               : http://lx-viewer.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

// The plotter class and related helper functions are contained in this file

#ifndef PLOTTER_H
#define PLOTTER_H

// QT includes
#include <qprinter.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qtabdialog.h>
#include <qlistview.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qpen.h>
#include <qpixmap.h>
#include <qbrush.h>
#include <qpaintdevicemetrics.h>
#include <qpoint.h>

// C++ includes
// #include <math.h>

// LX includes
#include "colorfunctions.h"
#include "utilities.h"
#include "resource.h"
#include "infomessage.h"

// OD includes
#include "ad2.h"
#include "ad3.h"

// structure to contain the pen mapping information
struct PenMap
	{
	short	outcolor;	// short integer to define the output color (0 to 255)	
	float	penwidth;       // double to define the width of the output line	
	};

// plot dialog box
class LX_PlotDialog : public QTabDialog
{
 		Q_OBJECT
	
	public:
		LX_PlotDialog(QWidget* parent=0, const char* name=0, bool modal=TRUE, WFlags f=0);
		QPrinter* plotter;							//  plotter

  private:
		// data members
		int resolution;			// current setting for spinbox (also use to seed the spinbox) 	
		int psresolution;		// resolution for ps printer 					
		int hpglresolution;		// resolution of hpgl printer					
		double leftsidescale;		// left side of drawing scale					
		double rightsidescale;		// right side of drawing scale					
		int margin;			// constant margin around the paper(dots)			
		double mmperinch;		// number of mm in an inch					
		PenMap pendef[256];		// pen map array color and lineweight				
		QString s_varies;
		QPoint frompoint;	//	first corner of window pick (pixel coordinates)
		QPoint topoint;   //	second corner of window pick (pixel coordinates)
		
    // dialog members
		QWidget* page1;
		QWidget* page2;

		// page1 declarations
		QButtonGroup* printarea;
			QRadioButton* pa1;
			QRadioButton* pa2;
			QRadioButton* pa3;
			QRadioButton* pa4;
			QRadioButton* pa5;
			QPushButton* selectwindowbutton;
			QLabel* 		 plotcoords;
      QPushButton* setupbutton;	
			QPushButton* writeBCFbut;					
			QPushButton* printprev;		
			QButtonGroup* format;
				QRadioButton* f1;
				QRadioButton* f2;			
				QLabel* marginlabel;
				QSpinBox* margindata;
				QLabel* resolutionlabel;
		    QSpinBox* resolutiondata;
		QButtonGroup* entitiestoprint;
			QRadioButton* e2p1;
			QRadioButton* e2p2;
		QGroupBox* printscale;
			QCheckBox* ps1;
			QLabel* printedunit;
			QLabel* drawingunit;
			QLabel* equal;
			QLineEdit* printedunittext;
			QLineEdit* drawingunittext;
			QButtonGroup* units;
				QRadioButton* u1;
				QRadioButton* u2;
   	
		// page2 declarations
		QListView* lv1;
			QListViewItem* lv1item;
  	QGroupBox* mapprop;
			QLineEdit* screencolortext;
			QLineEdit* outcolortext;
			QLineEdit* lineweight;
			QLabel* screencolor;
			QPushButton* outcolor;
			QLabel* unitlabel;
			QLabel* screenlabel;
			QLabel* outlabel;
			QLabel* linelabel;
		QGroupBox* curpcp;
			QLabel* pcplabel;
			QPushButton* openpcp;
			QPushButton* savepcp;

		// private functions
		int readpcpfile(QString);		// function to read a pcp file				
		int writepcpfile(QString);		// function to write a pcp file				
		QString pentotext(const int&);		// function that will return the pen info as text	
		void scaleparser(QString);		// function to parse the SCALE line in the pcp file	
		void populatelistview();		// function to fill up the list view			

	public slots:
		void plottersetup();
		void printpressed();
		void cancelpressed();
		void helppressed();
		void pcpread();
		void pcpwrite();
		void changeunits(int);
		void printedunitchanged(const QString&);
		void drawingunitchanged(const QString&);
		void lvselectionchanged();
		void outcolorbuttonpushed();
		void outcolortextchanged(const QString&);
		void penwidthtextchanged(const QString&);
		void fitprintchanged(int);
		void marginchanged(int);
		void outformatchanged(int);
		void resolutionchanged(int);
		void pa5changed(bool);
		void writeBCF();
		void setPlotWindow();
		
		// other public functions
	public:
		void setOptions(QString);		//	read BCF and set options for batching
		void batchProcess(QString files = QString());
		double getplotareawidth();		// function to return the width of the plot area	
		double getplotareaheight();		// function to return the height of the plot area	
		double getoutputwidth();		// function to return the width of the output field	
		double getoutputheight();		// function to return the height of the output field	
		short getpencolor(const int&);		// function that will return the output pen color	
		int getoutpenwidth(const int&);		// function that will return the output pen width	
		double getmargin();			// function to return the margin size in pixels		
		QString gethpgloutdevice();		// function to return the name of the hpgl output file	
		int getPrintAreaSetting();		//	function to return the number of the radio button selected
		QPoint getWindowCorner(short corner = 0);	//	function to return one of the window corners
		void setPlotCoordsLabel(const QPoint&, const QPoint&, QString s = QString());	//	function to set the label for the window points		
};
	

#endif
