/***************************************************************************
                          report.cpp  -  description
                             -------------------
    begin                : Sat Sep 21 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : buyer242299@aol.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//	classes and functions to generate reports about the drawing currently open

#include "report.h"
#include "resource.h"
#include "drawingboard.h"
#include "infomessage.h"
#include "OD_interface.h"
#include <math.h>
#include <qlayout.h>

extern PAD_DWGHDR adhd;
extern PAD_TB adtb;
extern PAD_ENT_HDR adenhd;
extern PAD_ENT aden;

//////////////////////////////////////////////////
//	report generator dialog box
/////////////////////////////////////////////////

//	constructor
LX_ReportDialog::LX_ReportDialog(QWidget* parent,const char* name, bool modal, WFlags f):QTabDialog(parent,name,modal,f),
								bk_string(tr("Blocks")), an_string(tr("Anonymous")), at_string(tr("Attributes")),
								ii_string(tr("Insertion Information")), dt_string(tr("Date and Time")),
								ds_string(tr("Dimension Styles")), xf_string(tr("External References")),
								sf_string(tr("Fonts and Shape Files")),la_string(tr("Layers")),lt_string(tr("Linetypes")),
								ra_string(tr("Registered Applications")), uc_string(tr("User Coordinate Systems")),
								vp_string(tr("View Ports (Model Space)")),vw_string(tr("Views")),
								ih_string(tr("Insert Handle")),nb_string(tr("Nested Block Reference")),
								bg_string(tr("Binding")),ph_string(tr("Xref Path"))
{
	//	sizes of graphics primitives
	const int border = 12;
	const int gap = 7;
	const int radiobut = 20;
	const int shortline = 150;
	const int longline =	250;
	const int buttonwidth = 125;
	const int buttonheight = 35;
	const int previewwidth	=	640;
	const int previewheight =	600;
	
	//	derived sizes
	const int selectwidth = gap + longline;
	const int selectheight = 2*border + 14*radiobut + 13*gap;
	const int formatwidth = gap + shortline;
	const int formatheight = 2*border + 2*radiobut + 1*gap;
	const int delimiterwidth = gap + shortline;
	const int delimiterheight = 2*border + 5*radiobut + 4*gap;
 	const int lv1width = selectwidth + 1*border + formatwidth;
//	const int lv1height = selectheight;
	const int lv1height = 3*border+formatheight+delimiterheight+6*gap+6*buttonheight;


//	create printer and print preview
	preview  = new reportPreview(0,"report preview");
		preview->setGeometry(5*border,5*border,previewwidth,previewheight);
		preview->setReadOnly(true);
		preview->setCaption(tr("Report Preview"));
		preview->hide();
	reportprinter = new QPrinter(QPrinter::ScreenResolution);
	reportprinter->setPageSize(QPrinter::Letter);
	reportprinter->setOrientation(QPrinter::Portrait);
	reportprinter->setColorMode(QPrinter::Color);

//create page 1
	page1 = new QWidget();
  	lv1 = new QListView(page1);
			lv1->setGeometry(border,border,lv1width,lv1height);
			lv1->setRootIsDecorated(true);
			lv1->setShowSortIndicator(TRUE);
			lv1->addColumn(tr(""),-1);
			lv1->addColumn(tr(""),-1);
			lv1->setAllColumnsShowFocus(FALSE);
			lv1->setItemMargin(2);
			lv1->setResizeMode(QListView::LastColumn);

	//create page 2
	page2 = new QWidget();
		select = new QButtonGroup(page2);
			select->setGeometry(border,border,selectwidth,selectheight);
			select->setTitle(tr("Select for Report"));
			select->setDisabled(true);
				blocks = new QCheckBox(bk_string,select);
					blocks->move(gap,border+0*radiobut+0*gap);
					blocks->setMinimumWidth(longline-2*border);
					blocks->setChecked(false);
				anonymous = new QCheckBox(an_string,select);
					anonymous->move(3*gap,border+1*radiobut+1*gap);
					anonymous->setMinimumWidth(longline-3*gap-2*border);
					anonymous->setChecked(false);
					anonymous->setDisabled(true);
				attributes = new QCheckBox(at_string,select);
					attributes->move(3*gap,border+2*radiobut+2*gap);
					attributes->setMinimumWidth(longline-3*gap-2*border);
					attributes->setChecked(false);
					attributes->setDisabled(true);
				insertinfo = new QCheckBox(ii_string,select);
					insertinfo->move(3*gap,border+3*radiobut+3*gap);
					insertinfo->setMinimumWidth(longline-3*gap-2*border);
					insertinfo->setChecked(false);
					insertinfo->setDisabled(true);
				datetime = new QCheckBox(dt_string,select);
					datetime->move(gap,border+4*radiobut+4*gap);
					datetime->setMinimumWidth(longline-2*border);
					datetime->setChecked(false);
				dimstyles = new QCheckBox(ds_string,select);
					dimstyles->move(gap,border+5*radiobut+5*gap);
					dimstyles->setMinimumWidth(longline-2*border);
					dimstyles->setChecked(false);
				xref = new QCheckBox(xf_string,select);
					xref->move(gap,border+6*radiobut+6*gap);
					xref->setMinimumWidth(longline-2*border);
					xref->setChecked(false);
				shapefiles = new QCheckBox(sf_string,select);
					shapefiles->move(gap,border+7*radiobut+7*gap);
					shapefiles->setMinimumWidth(longline-2*border);
					shapefiles->setChecked(false);
        layers = new QCheckBox(la_string,select);
					layers->move(gap,border+8*radiobut+8*gap );
					layers->setMinimumWidth(longline-2*border);
					layers->setChecked(false);
				linetypes = new QCheckBox(lt_string,select);
					linetypes->move(gap,border+9*radiobut+9*gap);
					linetypes->setMinimumWidth(longline-2*border);
					linetypes->setChecked(false);
				regapps = new QCheckBox(ra_string,select);
					regapps->move(gap,border+10*radiobut+10*gap);
					regapps->setMinimumWidth(longline-2*border);
					regapps->setChecked(false);
				ucs = new QCheckBox(uc_string,select);
					ucs->move(gap,border+11*radiobut+11*gap);
					ucs->setMinimumWidth(longline-2*border);
					ucs->setChecked(false);
				vports = new QCheckBox(vp_string,select);
					vports->move(gap,border+12*radiobut+12*gap);
					vports->setMinimumWidth(longline-2*border);
					vports->setChecked(false);
				views = new QCheckBox(vw_string,select);
					views->move(gap,border+13*radiobut+13*gap);
					views->setMinimumWidth(longline-2*border);
					views->setChecked(false);
		format = new QButtonGroup(tr("Output Format"),page2);
			format->setGeometry(border+selectwidth+border,border,formatwidth,formatheight);
			format->setDisabled(true);
				plaintext = new QRadioButton(tr("Plain Text"),format);
						plaintext->move(gap,border+0*radiobut+0*gap);
						plaintext->setChecked(TRUE);
				richtext = new QRadioButton(tr("HTML"),format);
						richtext->move(gap,border+1*radiobut+1*gap);
		delimiter = new QButtonGroup(tr("Select Delimiter"),page2);
			delimiter->setGeometry(border+selectwidth+border,2*border+formatheight,delimiterwidth,delimiterheight);
			delimiter->setDisabled(true);
				tabdelim = new QRadioButton(tr("Tab Delimiter"),delimiter);
					tabdelim->move(gap,border+0*radiobut+0*gap);
					tabdelim->setChecked(TRUE);
				colondelim = new QRadioButton(tr("Colon"),delimiter);
					colondelim->move(gap,border+1*radiobut+1*gap);
				semicolondelim = new QRadioButton(tr("Semi-Colon"),delimiter);
					semicolondelim->move(gap,border+2*radiobut+2*gap);
				commadelim = new QRadioButton(tr("Comma"),delimiter);
					commadelim->move(gap,border+3*radiobut+3*gap);
				spacedelim = new QRadioButton(tr("Five Spaces"),delimiter);
					spacedelim->move(gap,border+4*radiobut+4*gap);
		previewbutton = new LX_LightedButton(page2,"preview button");
			previewbutton->setGeometry(border+selectwidth+border,3*border+formatheight+delimiterheight+0*gap+0*buttonheight,buttonwidth,buttonheight);
			previewbutton->setText(tr("Show Preview"));
			previewbutton->setDisabled(true);
		pickfont = new QPushButton(page2,"pick font button");
			pickfont->setGeometry(border+selectwidth+border,3*border+formatheight+delimiterheight+1*gap+1*buttonheight,buttonwidth,buttonheight);
			pickfont->setText(tr("Select Font"));
			pickfont->setDisabled(true);
		printreport = new QPushButton(page2,"print report button");
			printreport->setGeometry(border+selectwidth+border,3*border+formatheight+delimiterheight+2*gap+2*buttonheight,buttonwidth,buttonheight);
			printreport->setText(tr("Print Report"));
			printreport->setDisabled(true);
		savetodisk = new QPushButton(page2,"save report to disk");
			savetodisk->setGeometry(border+selectwidth+border,3*border+formatheight+delimiterheight+3*gap+3*buttonheight,buttonwidth,buttonheight);
			savetodisk->setText(tr("Save to Disk"));
			savetodisk->setDisabled(true);
		writeBCFbut = new QPushButton(page2,"write a batch control file");
      writeBCFbut->setGeometry(border+selectwidth+border,3*border+formatheight+delimiterheight+4*gap+4*buttonheight,buttonwidth,buttonheight);
			writeBCFbut->setText(tr("Write BCF"));
			writeBCFbut->setDisabled(true);
		batchprocessbut = new QPushButton(page2,"process a directory of drawings");
			batchprocessbut->setGeometry(border+selectwidth+border,3*border+formatheight+delimiterheight+5*gap+5*buttonheight,buttonwidth,buttonheight);
			batchprocessbut->setText(tr("Batch Process"));
		
//	now define some parameters for the main tab box
	this->insertTab(page1, tr("Drawing Entities"));
	this->insertTab(page2, tr("Report Options"));
	page2->setMinimumWidth(lv1width + 2*border);
	page2->setMinimumHeight(lv1height + 2*border);
	resize(page2->width()+2*border, page2->height()+2*border+60);
	setCaption(tr("Drawing Report"));

//	insert items into layouts
	QHBoxLayout* hb1 = new QHBoxLayout(page1,0,-1,"hbox layout for page 1");
	hb1->addWidget(lv1);
	
//	finally connect signals and slots
	connect(previewbutton,SIGNAL(clicked()),this,SLOT(togglePreview()));
	connect(preview,SIGNAL(previewClosed()),this,SLOT(togglePreview()));
	connect(blocks,SIGNAL(stateChanged(int)),this,SLOT(blockChecked(int)));
	connect(select,SIGNAL(clicked(int)),this,SLOT(generateReport(int)));
	connect(printreport,SIGNAL(clicked()),this,SLOT(print()));
	connect(pickfont,SIGNAL(clicked()),this,SLOT(selectFont()));
	connect(format,SIGNAL(clicked(int)),this,SLOT(generateReport(int)));
	connect(savetodisk,SIGNAL(clicked()),this,SLOT(saveToFile()));
	connect(delimiter,SIGNAL(clicked(int)),this,SLOT(delimChecked(int)));
	connect(writeBCFbut,SIGNAL(clicked()),this,SLOT(writeBCF()));
	connect(batchprocessbut,SIGNAL(clicked()),this,SLOT(callBatch()));	
}

/////////////////////////////////////////////////////////////
//				SLOTS
/////////////////////////////////////////////////////////////

//	slot to write the Batch Control File
void LX_ReportDialog::writeBCF()
{
	const QString eq ("  =  ");
	const QString yes("YES");
	const QString no("NO");
	extern DrawingBoard* db;

	QString report_bcf = QDir::homeDirPath() + LXV_HOME + "/report.bcf";
	QFileInfo fi = QFileInfo(report_bcf);
	QFileInfo fd = fi.dirPath(true);
	if ((fi.exists() && !fi.isWritable()) || !fd.isWritable())
	{
		DisplayMessage(NO_WRITE_PERMISSION);
		db->printMessage(tr("\nWarning: %1 was not saved to disk - write permission not granted").arg(report_bcf));
		return;
	}

	QFile f(report_bcf);
	if (f.open(IO_WriteOnly))
	{
		QTextStream stream( &f );
		stream << (";   LX-VIEWER BATCH CONTROL FILE - report.bcf");
		stream << ("\n;\n;   This file contains the information needed to batch process reports.");
		stream << ("\n;   It is only used when supplying a directory of drawings to the program on the command line");
		stream << ("\n;\n;   We recommend that you do not edit this file manually");
		stream << ("\n;\n;");
    stream << "\n" << bk_string << eq << (blocks->isChecked() ? yes : no);
		stream << "\n" << an_string << eq << (anonymous->isChecked() ? yes : no);
		stream << "\n" << at_string << eq << (attributes->isChecked() ? yes : no);
		stream << "\n" << ii_string << eq << (insertinfo->isChecked() ? yes : no);			
		stream << "\n" << dt_string << eq << (datetime->isChecked() ? yes : no);			
		stream << "\n" << ds_string << eq << (dimstyles->isChecked() ? yes : no);		
		stream << "\n" << xf_string << eq << (xref->isChecked() ? yes : no);
		stream << "\n" << sf_string << eq << (shapefiles->isChecked() ? yes : no);
		stream << "\n" << la_string << eq << (layers->isChecked() ? yes : no);
		stream << "\n" << lt_string << eq << (linetypes->isChecked() ? yes : no);
		stream << "\n" << ra_string << eq << (regapps->isChecked() ? yes : no);
		stream << "\n" << uc_string << eq << (ucs->isChecked() ? yes : no);
		stream << "\n" << vp_string << eq << (vports->isChecked() ? yes : no);
		stream << "\n" << vw_string << eq << (views->isChecked() ? yes : no);
		stream << "\n" << "Format" << eq << format->id(format->selected());
		stream << "\n" << "Delimiter" << eq << delimiter->id(delimiter->selected());	
		stream << "\n" << "Font" << eq << (preview->font()).toString(); 		
		f.close();
	}
	db->printMessage(tr("\nBatch Control File %1 saved to disk").arg(report_bcf));
}

//	slot to call the batchProcess (used to match signal to slot arguments on pushbutton)
void LX_ReportDialog::callBatch(){batchProcess();}


//	slot to save the report to a file                 6
void LX_ReportDialog::saveToFile()
{
	extern DrawingBoard* db;
	QString savename;

	QFileDialog* p2fsd = new QFileDialog(this,"Report Save File Dialog",true);
	p2fsd->setDir(QDir::homeDirPath());
	p2fsd->setMode(QFileDialog::AnyFile);
	p2fsd->setCaption(tr("Save File"));
	p2fsd->setViewMode(QFileDialog::List);
	int rtnval = p2fsd->exec();
	savename = p2fsd->selectedFile();
	delete p2fsd;
	
	db->printMessage(tr("\nSave:  %1").arg(savename));
	if (rtnval == QDialog::Rejected || savename.isNull())	
	{
		db->printMessage(tr("No file selected"),SBAR_TWIN);
		return;
	}
	QFileInfo fi = QFileInfo(savename);
	QFileInfo fd = fi.dirPath(true);
	if ((fi.exists() && !fi.isWritable()) || !fd.isWritable())
	{
		DisplayMessage(NO_WRITE_PERMISSION);
		db->printMessage(tr("\nWarning: %1 was not saved to disk - write permission not granted").arg(savename));
		return;
	}

	QFile f(savename);
	if (f.open(IO_WriteOnly))
	{
		QTextStream stream( &f );
		if (plaintext->isChecked()) stream << report_text;
		if (richtext->isChecked())
		{                                //	strip out rich text code unnecessary for html
			QString report_html = report_text;
			report_html.replace(QRegExp("<center>"),"");
			report_html.replace(QRegExp("</center>"),"");
			report_html.replace(QRegExp(" align=center>"),">");	
			report_html.replace(QRegExp("<b>"),"");
			report_html.replace(QRegExp("</b>"),"");
			stream << report_html;
		}
		
		f.close();
	}
}


//	slot to select a font for the report
void LX_ReportDialog::selectFont()
{
	preview->setCurrentFont(QFontDialog::getFont(0,preview->font()));
	preview->setFont(preview->font());
}

//	slot to print the report
void LX_ReportDialog::print()
{
	extern DrawingBoard* db;
	const int Margin = 10;
	int pageNo = 1;

	if (reportprinter->setup(this))
	{
		db->printMessage(tr("\nPrinting Report  "));
		if (plaintext->isChecked())
		{
			db->printMessage(tr("\nPrinting page 1..."),SBAR);
			QPainter p;
			if(!p.begin(reportprinter)) return;         // paint on printer

			p.setFont( preview->font() );
			int yPos = 0;          						          // y-position for each line
			QFontMetrics fm = p.fontMetrics();
			QPaintDeviceMetrics metrics(reportprinter); // need width/height of printer surface
			for( int i = 0 ; i < preview->lines() ; i++ )
			{
				if ( Margin + yPos > metrics.height() - Margin )
				{
					qApp->processEvents();
					QString msg(tr("\nPrinting page %1...").arg(QString::number(++pageNo)));
					db->printMessage(msg,SBAR);
					reportprinter->newPage();             // no more room on this page
					yPos = 0;   		                    // back to top of page
				}
				p.drawText( Margin, Margin + yPos,
				metrics.width(), fm.lineSpacing(),
				ExpandTabs | DontClip,
				preview->text( i ) );
				yPos = yPos + fm.lineSpacing();
			}
			p.end();                                // send job to printer
		} //	if plaintext was checked
		if (richtext->isChecked())
		{
			db->printMessage(tr("\nPrinting page 1..."),SBAR);
			reportprinter->setFullPage(TRUE);
			QPainter p(reportprinter);
			QPaintDeviceMetrics metrics(p.device());
			int dpix = metrics.logicalDpiX();
			int dpiy = metrics.logicalDpiY();
			const int margin = 72; // pt
			QRect body(	margin*dpix/72,
									margin*dpiy/72,

									metrics.width()-margin*dpix/72*2,
									metrics.height()-margin*dpiy/72*2);
			QFont font = preview->font();
			QSimpleRichText richText( preview->text(), font,
																preview->context(),
																preview->styleSheet(),
																preview->mimeSourceFactory(),
																body.height() );
																richText.setWidth( &p, body.width() );
			QRect view( body );
			pageNo = 1;
			do
		 {
				richText.draw( &p, body.left(), body.top(), view, colorGroup() );
				view.moveBy( 0, body.height() );
				p.translate( 0 , -body.height() );
				p.setFont( font );
				p.drawText( view.right() - p.fontMetrics().width( QString::number(pageNo) ),
										view.bottom() + p.fontMetrics().ascent() + 5, QString::number(pageNo) );
				if (view.top()  >= body.top() + richText.height())
					break;
				reportprinter->newPage();
				pageNo++;
				qApp->processEvents();
				QString msg(tr("\nPrinting page %1...").arg(QString::number(pageNo)));
				db->printMessage(msg,SBAR);
			} while (true);
		} //	if richtext is checked
	}	//	if setup dialog returns true		
}	//	function	



//	slot to toggle the preview dialog up and down
void LX_ReportDialog::togglePreview()
{
	if (preview->isVisible())
	{
		preview->hide();
	  previewbutton->setOn(false);
	}
	else
	{
		preview->show();
		generateReport(0);
		previewbutton->setOn(true);
	}
}

//	slot to generate the preview of the report
void LX_ReportDialog::generateReport(int i)
{
	(void)i;
	QString nm,hd,bi,ne,pa;
	bool closetable = false;
	QListViewItem*	parentitem;
	QListViewItem*	heading;

	preview->clear();
	report_text = QString();
	parentitem = lv1->firstChild();

	newline = "\n";
	newpara = "\n\n";
	tab = "\t";
	if (!dlm) dlm = "\t";

	if (plaintext->isChecked())
	{
		preview->setTextFormat(Qt::PlainText);
		delimiter->setDisabled(false);
	}

	if (richtext->isChecked())
	{
	 	preview->setTextFormat(Qt::RichText);
		delimiter->setDisabled(true);
		report_text = QString("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"");
		report_text.append(" \"http://www.w3.org/2002/08/xhtml/xhtml1-strict.dtd\">");
		report_text.append(newline + "<html>" + newline + "<head>");
		report_text.append(newline + "     <meta http-equiv=\"content-type\" content=\"text/html;charset=ISO-8859-1\"/>");
		report_text.append(newline + "     <title>Contents Report</title>");
		report_text.append(newline + "<style type = \"text/css\">");
		report_text.append(newline + "<!--");
		report_text.append(newline + "     table {border:0;}");
		report_text.append(newline + "-->");
		report_text.append(newline + "</style>" + newline + "</head>" + newline + newline + "<body>");
	}
	
	while(parentitem)
	{
    if (plaintext->isChecked())
		{
			if (report_text) report_text.append(newpara);
 			report_text.append(tr("Report of the Contents of drawing:   %1").arg(parentitem->text(0)));
		}
		if (richtext->isChecked()) report_text.append(newline + "<h1><center>" + tr("Report of the Contents of Drawing:   %1").arg(parentitem->text(0)) + "</center></h1>");
 	
		if (blocks->isChecked())
		{
			const QString bname(tr("Block Name"));
			const QString bhdl(tr("Insert Handle"));
			const QString atag(tr("Attribute"));
			const QString aval(tr("Value"));
			const QString cln(": ");
			
			heading = parentitem->firstChild();
			while (heading->text(0) != bk_string) {heading = heading->nextSibling();}
			QListViewItem* child = heading->firstChild();
			if (child)
			{
				if (plaintext->isChecked())	report_text.append(newpara + bk_string);
				if (richtext->isChecked())	report_text.append(newline + "<h2>" + bk_string + "</h2>");
				closetable = true;
 	   }
			while (child)
			{
				nm = child->text(0);
				QListViewItem* grandchild = child->firstChild();
				while (grandchild && child->text(0) != an_string)	//	
				{					
					if (grandchild->text(0) == ih_string)
					{
						hd = grandchild->text(1);
						if (plaintext->isChecked()) report_text.append(newline + bname + cln + nm + newline + tab + bhdl + cln + hd);	
						if (richtext->isChecked()) report_text.append(QString(newline + "<h3>" + nm +"</h3><table><tr><td>" + bhdl + cln + "</td><td>" + hd + "</td></tr>"));
						QListViewItem* ggc = grandchild->firstChild();
						while (ggc)
						{
							if (ggc->text(0) == ii_string && insertinfo->isChecked())
							{
								QListViewItem* gggc = ggc->firstChild();
								while(gggc)
								{
									if (plaintext->isChecked()) report_text.append(newline + tab + gggc->text(0) + dlm + gggc->text(1));
									if (richtext->isChecked()) report_text.append(newline + "<tr><td>" + gggc->text(0) + "</td><td>" + gggc->text(1) + "</td></tr>");
									gggc = gggc->nextSibling();
								}	//	while gggc
							}	//	if insertinfo is checked and present
							if (ggc->text(0) == at_string && attributes->isChecked())
							{
								QListViewItem* gggc = ggc->firstChild();
								while(gggc)
								{
									if (plaintext->isChecked()) report_text.append(newline + tab + atag + cln + gggc->text(0) + dlm + gggc->text(1));
									if (richtext->isChecked()) report_text.append(newline + "<tr><td>" + atag + cln + "</td><td>" + gggc->text(0) + "</td><td>" + aval + cln + "</td><td>" + gggc->text(1) + "</td></tr>");
									gggc = gggc->nextSibling();
								}	//	while gggc
							}	//	if attributes are checked and present
							ggc=ggc->nextSibling();
						}	//	while ggc
					if (richtext->isChecked()) report_text.append(newline + "</table>");
					}	//	if grandchild == insert handle string
					grandchild = grandchild->nextSibling();			
				}	//	while grandchild
				child = child->nextSibling();
			}	//	child while loop		
		}	//	if blocks are checked
 	
		if (datetime->isChecked())		report_text.append(basicReportParagraph(parentitem,dt_string));
		if (dimstyles->isChecked())		report_text.append(basicReportParagraph(parentitem,ds_string));
		if (xref->isChecked())
		{
			closetable = false;

	   	heading = parentitem->firstChild();
			while (heading->text(0) != xf_string) {heading = heading->nextSibling();}
			QListViewItem* child = heading->firstChild();
			if (child)
			{
				if(plaintext->isChecked()) report_text.append(newpara + xf_string);
				if(richtext->isChecked())
				{
					report_text.append(newline + "<h2>" + xf_string + "</h2>");
					report_text.append(newline + "<table><tr><th align=center><b>" + tr("Xref Name") +
														"</b></th><th align=center><b>" + ih_string + "</b></th><th align=center><b>" + bg_string +
														"</b></th><th align=center><b>" + nb_string + "</b></th><th align=center><b>" + ph_string + "</b></th></tr>");
					closetable = true;
				}
			}
			while (child)
			{
				nm = child->text(0);
				ne = tr("No Nested Block References");
				QListViewItem* grandchild = child->firstChild();
				while (grandchild)	//	first cut get the block info
					{
						if (grandchild->text(0) == bg_string) bi = grandchild->text(1);
						if (grandchild->text(0) == nb_string) ne = tr("Contains Nested Block Ref") ;
						if (grandchild->text(0) == ph_string) pa = grandchild->text(1);				
						grandchild = grandchild->nextSibling();
					}	//	grandchild while loop
 	
				grandchild = child->firstChild();	//	second cut look for inserts of this block
				while (grandchild)	
				{
					if(grandchild->text(0) == ih_string)
					{
						hd = grandchild->text(1);	
						if (plaintext->isChecked()) report_text.append(newline + nm + dlm + hd + dlm + bi + dlm + ne + dlm + pa);
						if (richtext->isChecked()) report_text.append(newline + "<tr><td align=center>" + nm + "</td><td align=center>" + hd + "</td><td align=center>" + bi +"</td><td align=center>" + ne +"</td><td align=center>" + pa +"</td></tr>");
					}						
					grandchild = grandchild->nextSibling();
				}	//	grandchild while loop
				child = child->nextSibling();
			}	//	child while loop	
			if (richtext->isChecked() && closetable) report_text.append(newline + "</table>");
		}	//	is xref checked
 	
		if (shapefiles->isChecked())	report_text.append(basicReportParagraph(parentitem,sf_string));
		if (layers->isChecked())			report_text.append(basicReportParagraph(parentitem,la_string,tr("Layer Name"),true));
		if (linetypes->isChecked())		report_text.append(basicReportParagraph(parentitem,lt_string));
		if (regapps->isChecked())			report_text.append(basicReportParagraph(parentitem,ra_string));
		if (ucs->isChecked())					report_text.append(basicReportParagraph(parentitem,uc_string));
		if (vports->isChecked())			report_text.append(basicReportParagraph(parentitem,vp_string,tr("Viewport Name"),true));
		if (views->isChecked())				report_text.append(basicReportParagraph(parentitem,vw_string));
		parentitem = parentitem->nextSibling();
	}	//	while loop

	if (richtext->isChecked()) report_text.append(newline + "</body></html>");
	preview->setText(report_text);
}	

//	slot to activate and unactivate boxes that only matter when block is checked
void LX_ReportDialog::blockChecked(int i)
{
//	anonymous->setDisabled (i == 2 ? false : true);
	attributes->setDisabled(i == 2 ? false : true);
	insertinfo->setDisabled(i == 2 ? false : true);	
}

//	slot to pick the delimiter to be used
void LX_ReportDialog::delimChecked(int i)
{
	switch (i)
	{
		case 0:
			dlm = "\t";
			break;
		case 1:
			dlm = " : ";
			break;
		case 2:
			dlm = " ; ";
			break;
		case 3:
			dlm = " , ";
			break;
		case 4:
			dlm = "     ";
			break;
		default:
			dlm = "\t";
	}
	generateReport(i);
}

///////////////////////////////////////////////////////////////
//				FUNCTIONS
///////////////////////////////////////////////////////////////
//	function to process an entire directory of files
void LX_ReportDialog::batchProcess(QString dir2process)
{
	extern DrawingBoard* db;
	QStringList filebatch;
	QString currentDrawing;
	AD_DB_HANDLE	ithandle;

	if (db->getOpenFileName()) currentDrawing = db->getOpenFileName();
	else adSetupDwgRead();

	if (dir2process.isNull())	//	only call the openfile dialog is no directory was provided
	{	
		QFileDialog* p2pdd = new QFileDialog(this,"Process directory dialog",true);
			p2pdd->setMode(QFileDialog::ExistingFiles);
			p2pdd->setDir(QDir::homeDirPath());
			p2pdd->setFilter("Standard Drawing Files (*.dwg *.DWG)");
			p2pdd->setCaption(tr("Batch Report - Multiple File Select"));
			p2pdd->setViewMode(QFileDialog::List);
		int rtnval = p2pdd->exec();
		filebatch = p2pdd->selectedFiles();
		delete p2pdd;

		if (rtnval == QDialog::Rejected || filebatch.isEmpty())	
		{
			db->printMessage(tr("No files selected"),SBAR_TWIN);
			return;
		}
	}
	else
	{
		QDir d = QDir(dir2process);
		d.setNameFilter("*.dwg *.DWG");
		QStringList namelist = d.entryList();
		QStringList::Iterator it = namelist.begin();
		filebatch.clear();
		while (it != namelist.end())
		{	
			filebatch.append(dir2process + '/' + *it);
			++it;
		}
	}
	
	db->LX_ClearScreen();
	initializeListView();
	QString fn;
	QStringList::Iterator it = filebatch.begin();
	db->printMessage(tr("\n\nBatch Report:"));
	while( it != filebatch.end() )
	{
		fn = *it;
		QFileInfo fi = QFileInfo(fn);
		if (fi.isReadable())
		{
			if ((ithandle=adLoadFile(const_cast<char*>(fn.latin1()),AD_PRELOAD_ALL,1)))
			{
				db->printMessage(tr("Opening %1 for reporting").arg(fn));
			  populateListView(ithandle,fn);	
				if (ithandle) adCloseFile(ithandle);
			}
			else db->printMessage(tr("Error opening file %1: %2").arg(fn).arg(QString(adErrorStr(adError()))));
		}	
		else db->printMessage(tr("Warning: %1  was not read - read permission not granted").arg(fn));
		++it;
	}
	
// now reload the saved drawing
	if (currentDrawing) db->openfile(currentDrawing);
}

//	function to create a typical report paragraph.  Used only for sections with column 0
//	and column 1 data
//	s1 is one of the defined constant strings at the top of this class
//	s2 is an optional title string for the report,  if grandkids exist s2 is a subheading
QString LX_ReportDialog::basicReportParagraph(QListViewItem* drawing, const QString& s1, QString s2, bool gk)
{
	QString para = QString();
	bool firsttime = true;
	QString header = QString();
	QString line = QString();

	QListViewItem* heading;
	heading = drawing->firstChild();
	while (heading->text(0) != s1) {heading = heading->nextSibling();}
	QListViewItem* child = heading->firstChild();
	if (plaintext->isChecked()) para.append(newpara + s1);
	if (richtext->isChecked())  para.append(newline + "<h2>" + s1 + "</h2>");
	
	while (child)
	{
		if (!gk)
		{
			if (plaintext->isChecked()) para.append(newline + child->text(0) + dlm + child->text(1));
			if (richtext->isChecked())
			{
				if (firsttime) para.append(newline + "<table>");
				para.append(newline + "<tr><td>" + child->text(0) + "</td><td>" + child->text(1) + "</td></tr>");
				firsttime = false;
			}
		}

		else	//	(else is gk = true)
		{
			QListViewItem* grandchild = child->firstChild();
			if (plaintext->isChecked())
			{
				para.append(newline + s2 + dlm + child->text(0));
        while (grandchild)
				{
					para.append(newline + tab + grandchild->text(0) + dlm + grandchild->text(1));
					grandchild = grandchild->nextSibling();
				}
			}	//	if there are grandchildren and plaintext is checked
			if (richtext->isChecked())
			{
				if (firsttime) header.append(newline + "<table><tr><th align=center><b>" + s2 + "</b></th>");	
				line = QString(newline + "<tr><td align=center>" + child->text(0) + "</td>");
				
				while (grandchild)
				{					
					if (firsttime) header.append(newline + "<th align=center><b>" + grandchild->text(0) + "</b></th>");
					line.append(newline + "<td align=center>" + grandchild->text(1) + "</td>");
					grandchild = grandchild->nextSibling();	
				}
				if (firsttime)
				{
					firsttime = false;
					para.append(header + "</tr>");
				}
				para.append(line + "</tr>");
			}	//	if there are grandchildren and richtext is checked		
		}	//	if(gk)
		child = child->nextSibling();
	}	//	while(child) loop

	if (richtext->isChecked() && !firsttime) para.append(newline + "</table>");
	
	return para;
}

//	function to burrow recursively down a block chain to the
//	end to root out nested blocks.
void LX_ReportDialog::blockMole(AD_DB_HANDLE dwghandle,QListViewItem* parent,AD_VMADDR entlist)
{
	AD_BLKH blkhdr;
	QListViewItem* lvptr1 = NULL;
	QListViewItem* lvptr2 = NULL;
	short etype;
	short rtnval;
	const QString nestblk = nb_string + QString(":  ");
	
	adStartEntityGet(entlist);
	do
	{
		rtnval = adGetEntity(entlist,adenhd,aden);
	  etype = adenhd->enttype;
		if (etype == AD_ENT_INSERT)
		{	
			if(adSeekBlockheader(dwghandle,aden->insert.blockheaderobjhandle,&blkhdr))
			{
				lvptr1 = new QListViewItem(parent,nestblk,blkhdr.name);
				
			//	now lets summarize the Attribute entities if there are any
				if (aden->insert.attribsfollow == 1)
				{
					if (adGetEntity(entlist,adenhd,aden))
					{
						lvptr2 = new QListViewItem(lvptr1,at_string);
						while (adenhd->enttype != AD_ENT_SEQEND)
						{
							item = new QListViewItem(lvptr2,QString(aden->attrib.tag),QString(aden->attrib.attval));	
							adGetEntity(entlist,adenhd,aden);
						}	//	while loop
					}	//	adgetentity if statement	
				}	//	attrribsfollow if statement						
			}	//	if adseekblockheader
		blockMole(dwghandle,lvptr1,blkhdr.entitylist);	//	this is the recursive call
		}	//	if this was an insert entity
	} while(rtnval && etype != AD_ENT_ENDBLK);	//	while we are not at an end block
}	//	function
 		
//	function to initialize the list view
void LX_ReportDialog::initializeListView(){lv1->clear();}

//	function to initialize the list view

void LX_ReportDialog::populateListView(AD_DB_HANDLE dwghandle,QString dwgfile)
{
  if (!dwghandle) return;
	
	QString s1 = QString();
	QString s2 = QString();
	QString s3 = QString();
	QListViewItem* lvptr1 = NULL;
	QListViewItem* lvptr2 = NULL;
	char storebuf[17];

	//	there must be a drawing open, so activate the buttons
	select->setDisabled(false);
	format->setDisabled(false);
	delimiter->setDisabled(false);
	previewbutton->setDisabled(false);
	pickfont->setDisabled(false);
	printreport->setDisabled(false);
	savetodisk->setDisabled(false);
	writeBCFbut->setDisabled(false);
	
  //	clear any existing data and set the root of the list box to be this drawing
	cdi = new QListViewItem(lv1,(dwgfile ? dwgfile : tr("Current Drawing")));
	
	//	now create the main children items inside the drawing
	dti = new QListViewItem(cdi,dt_string);
	lai = new QListViewItem(cdi,la_string);
	lti = new QListViewItem(cdi,lt_string);
	shi = new QListViewItem(cdi,sf_string);
	dmi = new QListViewItem(cdi,ds_string);
	vpi = new QListViewItem(cdi,vp_string);
	vwi = new QListViewItem(cdi,vw_string);

	csi = new QListViewItem(cdi,uc_string);
	rai = new QListViewItem(cdi,ra_string);
	bki	= new QListViewItem(cdi,bk_string);
	xfi = new QListViewItem(cdi,xf_string);
	
//	start with the time and date data
	short month;
	short day;
	short year;
	short hour;
	short min;
	short sec;
	adhd = adHeaderPointer(dwghandle);
	
	adDecodeAcadDate(adhd->tdcreate,&day,&month,&year,&hour,&min,&sec);
	QDateTime dt = QDateTime(QDate(year,month,day),QTime(hour,min,sec));
	item = new QListViewItem(dti,tr("Creation Date"),dt.toString(Qt::TextDate));
	
	adDecodeAcadDate(adhd->tdupdate,&day,&month,&year,&hour,&min,&sec);
	dt = QDateTime(QDate(year,month,day),QTime(hour,min,sec));
	item = new QListViewItem(dti,tr("Last Updated"),dt.toString(Qt::TextDate));
	
	adDecodeAcadTime(adhd->tdindwg,&day,&hour,&min,&sec);
	item = new QListViewItem(dti,tr("Total Time in Drawing"),
																	QString::number(day) + (day == 1 ? tr(" day ") : tr(" days ")) +
																	QString::number(hour) + (hour == 1 ? tr(" hour ") : tr(" hours ")) +
																	QString::number(min) + tr(" min ") +
																	QString::number(sec) + tr(" sec"));

	adDecodeAcadTime(adhd->tdusrtimer,&day,&hour,&min,&sec);
	item = new QListViewItem(dti,tr("Elapsed User Timer"),
																	QString::number(day) + (day == 1 ? tr(" day ") : tr(" days ")) +
																	QString::number(hour) + (hour == 1 ? tr(" hour ") : tr(" hours ")) +
																	QString::number(min) + tr(" min ") +
																	QString::number(sec) + tr(" sec"));	

//	summarize the layer information
	char on;
	char frozen;
	char vpfrozen;
	char locked;

	adStartLayerGet(dwghandle);
	int i;
	for (i = 0; i < adNumLayers(dwghandle); i++)
	{
	 	adGetLayer(dwghandle,&adtb->lay);
		s1 = QString(adtb->lay.name);
		if (!adtb->lay.purgedflag)
		{
			lvptr1 = new QListViewItem(lai,s1);
			s2 = QString::number(adtb->lay.color * (adtb->lay.color < 0 ? -1 : 1));
			adGetLayerState(dwghandle,adtb->lay.objhandle,&on,&frozen,&vpfrozen,&locked);
			item = new QListViewItem(lvptr1,tr("Color"),s2);
			item = new QListViewItem(lvptr1,tr("Visibility"),on ? tr("ON") : tr("OFF"));
			item = new QListViewItem(lvptr1,tr("Edit Status"),frozen ? tr("FROZEN") : tr("THAWED"));
			item = new QListViewItem(lvptr1,tr("VPort Edit Status"),vpfrozen ? tr("FROZEN") : tr("THAWED"));
			item = new QListViewItem(lvptr1,tr("Protection"),locked ? tr("LOCKED") : tr("UNLOCKED"));
			item = new QListViewItem(lvptr1,tr("Plottable"),(adtb->lay.plotflag == 0 ? tr("NO") : tr("YES")));
		}	//	if
	}	//	for loop
			
//	linetype stuff
	adStartLinetypeGet(dwghandle);
	for (i = 0; i < adNumLinetypes(dwghandle); i++)
	{
		adGetLinetype(dwghandle,&adtb->ltype);
		s1 = QString(adtb->ltype.name);
		s2 = QString(adtb->ltype.text);
		if (!adtb->ltype.purgedflag && s1 != "ByBlock" && s1 != "ByLayer")				
			item = new QListViewItem(lti,s1,s2);
	}

//	shape files
	adStartShapefileGet(dwghandle);
	for (i = 0; i < adNumShapefiles(dwghandle); i++)
	{
		adGetShapefile(dwghandle,&adtb->shptb);
		s1 = QString(adtb->shptb.name);
		s2 = QString(adtb->shptb.file);
		if (!adtb->shptb.purgedflag)
			item = new QListViewItem(shi,s1,s2);
	}

//	dimension styles
	adStartDimstyleGet(dwghandle);
	for (i = 0; i < adNumDimstyles(dwghandle); i++)
	{
		adGetDimstyle(dwghandle,&adtb->dimst);
		s1 = QString(adtb->dimst.name);
		if (!adtb->dimst.purgedflag)
			item = new QListViewItem(dmi,s1);
	}

//	viewports
	const char f = 'f';	//	double

	const int prec = 3;	//	show 3 decimal places
	const QString comma = " , ";

	adStartVportGet(dwghandle);
	for (i = 0; i < adNumVports(dwghandle); i++)
	{
		adGetVport(dwghandle,&adtb->vport);
		s1 =	QString(adtb->vport.name);
		s2 =	QString("(") + QString::number(adtb->vport.target[0],f,prec) + comma +
					QString::number(adtb->vport.target[1],f,prec) + comma +
					QString::number(adtb->vport.target[2],f,prec) + ")";
		s3 =	QString("(") + QString::number(adtb->vport.viewdir[0],f,prec) + comma +
					QString::number(adtb->vport.viewdir[1],f,prec) + comma +
					QString::number(adtb->vport.viewdir[2],f,prec) + ")";
		if (!adtb->vport.purgedflag)
		{
			lvptr1 = new QListViewItem(vpi,s1);
				item = new QListViewItem(lvptr1,tr("Target"),s2);
				item = new QListViewItem(lvptr1,tr("View Direction"),s3);
		}
	}

//	views
	adStartViewGet(dwghandle);
	for (i = 0; i < adNumViews(dwghandle); i++)
	{
		adGetView(dwghandle,&adtb->view);
		s1 = QString(adtb->view.name);
		if (!adtb->view.purgedflag)
			item = new QListViewItem(vwi,s1);
	}

//	User Coordinate Systems
	adStartUcsGet(dwghandle);
	for (i = 0; i < adNumUcs(dwghandle); i++)
	{
		adGetUcs(dwghandle,&adtb->ucs);
		s1 = QString(adtb->ucs.name);
		if (!adtb->ucs.purgedflag)
			item = new QListViewItem(csi,s1);
	}

//	Registered Applications
	adStartRegappGet(dwghandle);
	for (i = 0; i < adNumRegapps(dwghandle); i++)
	{
		adGetRegapp(dwghandle,&adtb->app);
		s1 = QString(adtb->app.name);
		if (!adtb->app.purgedflag)
			item = new QListViewItem(rai,s1);
	}

// Blocks and Xref's -- these are the actual block definitions
	AD_VMADDR entlist;
	AD_OBJHANDLE saveplace;
	bool no_anonymous_blocks(true);
	int rtnval = 0;
	const QString sinsert = tr(" PS Insert");
	const QString pinsert = tr(" PS Inserts");
	const QString sep = tr(" MS and ");

	adStartBlockheaderGet(dwghandle);
	for (i = 0; i < adNumBlockheaders(dwghandle); i++)	
	{
		adGetBlockheader(dwghandle,&adtb->blkh);
		s1 = QString(adtb->blkh.name);
		s2 = QString::number(0) + sep + QString::number(0) + pinsert;
		adBinaryObjhandleToAscii(adtb->blkh.objhandle,storebuf);
		s3 = QString(storebuf);	//	stored but not displayed
	
		if (!adtb->blkh.purgedflag && !s1.contains("*model_space",false) && !s1.contains("*paper_space",false))
		{
			if (adtb->blkh.flag & AD_BLOCK_ANONYMOUS)
			{
				if(no_anonymous_blocks)
				{
					lvptr2 = new QListViewItem(bki,tr("Anonymous Blocks"));
					no_anonymous_blocks = false;
				}
				item = new QListViewItem(lvptr2,s1,s2,s3);
			}
	
			else
			{
				if (adtb->blkh.flag & AD_BLOCK_XREF)
				{	
					lvptr1 = new QListViewItem(xfi,s1,s2,s3);
					item = new QListViewItem(lvptr1,bg_string,adtb->blkh.xrefblkisoverlaid ? tr("OVERLAID") : tr("ATTACHED"));
					entlist = adtb->blkh.entitylist;
					adStartEntityGet(entlist);				
					do
					{
						rtnval = adGetEntity(entlist,adenhd,aden);
						if (adenhd->enttype == AD_ENT_BLOCK)
							item = new QListViewItem(lvptr1,ph_string,QString(aden->block.xrefpath));
					} while (rtnval && adenhd->enttype != AD_ENT_ENDBLK);
				}
				else lvptr1 = new QListViewItem(bki,s1,s2,s3);

			}	//	else

			adHancpy(saveplace,adtb->blkh.objhandle);
			blockMole(dwghandle,lvptr1,adtb->blkh.entitylist);
			adSeekBlockheader(dwghandle,saveplace,&adtb->blkh);

		}	//	if !purgedflag
	}	//	for loop

//	now cycle through the entity list looking for inserts, sort and count them accordingly
//	AD_VMADDR	entlist;
	AD_OBJHANDLE mspsblockobjhandle;
	
	for (i = 1; i <= 2; i++)
	{
		if (adGetBlockHandle(dwghandle,mspsblockobjhandle,(i==1? AD_MODELSPACE_HANDLE : AD_PAPERSPACE_HANDLE)))
		{
			entlist = adEntityList(dwghandle,mspsblockobjhandle);
			adStartEntityGet(entlist);

			while (adGetEntity(entlist,adenhd,aden))
			{
				if (adenhd->enttype == AD_ENT_INSERT)
				{	
					adBinaryObjhandleToAscii(aden->insert.blockheaderobjhandle,storebuf);
					bool foundit(false);
					QListViewItemIterator it (lv1);
					while (!foundit)
					{
						s3 = it.current()->text(2);
						if(s3)
							if(s3 == QString(storebuf)) foundit = true;
							else ++it;
						else ++it;
					}
					QString s = it.current()->text(1);
					int c1 = s.find(sep,0,false);
					int c2 = s.find(sinsert,0,false);
					QString msinserts = s.left(c1);
					QString psinserts = s.mid((c1 + sep.length()),c2-c1-sep.length());
					int mscount = msinserts.toInt();
					int pscount = psinserts.toInt();
				  i == 1 ? mscount++ : pscount++;
					it.current()->setText(1,QString::number(mscount) + sep + QString::number(pscount) + ((mscount+pscount)==1 ? sinsert:pinsert));

					//	now lets summarize insertion information for the insert entity				
					adBinaryObjhandleToAscii(adenhd->enthandle,storebuf);
					lvptr1 = new QListViewItem(it.current(),ih_string,QString(storebuf));
					lvptr2 = new QListViewItem(lvptr1,(ii_string));
						item = new QListViewItem(lvptr2,tr("Insertion Point:"),
																						QString("(") + QString::number(aden->insert.pt0[0],f,prec) + comma +
																						QString::number(aden->insert.pt0[1],f,prec) + comma +	
																						QString::number(aden->insert.pt0[2],f,prec) + ")");
						item = new QListViewItem(lvptr2,tr("X Scale Factor:"), QString::number(aden->insert.xscale,f,prec));
						item = new QListViewItem(lvptr2,tr("Y Scale Factor:"), QString::number(aden->insert.yscale,f,prec));
						item = new QListViewItem(lvptr2,tr("Z Scale Factor:"), QString::number(aden->insert.zscale,f,prec));
						item = new QListViewItem(lvptr2,tr("Rotation Angle:"), QString::number((180.0/M_PI)*aden->insert.rotang) + " Degrees");
						item = new QListViewItem(lvptr2,tr("Insertion is in:"),(i == 1 ? tr("Model Space"):tr("Paper Space")));

					//	now lets summarize the Attribute entities if there are any
					if (aden->insert.attribsfollow == 1)
					{
						if (adGetEntity(entlist,adenhd,aden))
						{
							lvptr2 = new QListViewItem(lvptr1,at_string);
							while (adenhd->enttype != AD_ENT_SEQEND)
							{
								item = new QListViewItem(lvptr2,QString(aden->attrib.tag),QString(aden->attrib.attval));	
								adGetEntity(entlist,adenhd,aden);
							}	//	while loop
						}	//	adgetentity if statement
					}	//	attrribsfollow if statement						
				}	//	if this was an insert entity					
			}	//	while there are entities to get
		}	//	if the adGetBlockHandle
	}	// modelspace/paperspace for loop

//	clear the preview window
	preview->clear();

//	Expand the tree to the first level
	lv1->setOpen(cdi,true);
}

//	function to set the options checkboxes
void LX_ReportDialog::setOptions(QString options)
{
	extern DrawingBoard* db;

//	open the batch control file for reading, and set the options accordingly
	QFile report_bcf(QDir::homeDirPath() + LXV_HOME + "/report.bcf");
  if(report_bcf.exists()) 		// there is an BCF so continue processing
	{
		db->printMessage(tr("Attempting to open %1").arg(report_bcf.name()),TWIN);
  	if(!report_bcf.open (IO_ReadOnly))	//	BCF file exists but for some reason we can't open it	
  	{	
			db->printMessage(tr("Read error opening %1 -- File not read").arg(report_bcf.name()),TWIN);
	 	}
    else
		{
			//	now read each line of the file
			QTextStream rs (&report_bcf);
			const QChar eq('=');
			const QChar semi(';');
			const QString yes("YES");
			const bool cs(false);
			QString key;
			QString val;

			while (!rs.eof())
			{
				key = rs.readLine();
				key = key.simplifyWhiteSpace();
				val = key;
				if (key != semi && key.contains(eq,cs))		//	the line read is not a comment and appears properly formatted
				{
					int split = key.find(eq,0,cs);
		 		  key = key.left(split);
					val = val.remove(0,++split);
					key = key.simplifyWhiteSpace();
					val = val.simplifyWhiteSpace();
					if (key == bk_string) blocks->setChecked(val == yes ? true : false);
						else if (key == an_string) anonymous->setChecked(val == yes ? true : false);
							else if	(key == at_string) attributes->setChecked(val == yes ? true : false);
								else if (key == ii_string) insertinfo->setChecked(val == yes ? true : false);			
									else if (key == dt_string) datetime->setChecked(val == yes ? true : false);			
										else if (key == ds_string) dimstyles->setChecked(val == yes ? true : false);		
											else if (key == xf_string) xref->setChecked(val == yes ? true : false);
												else if (key == sf_string) shapefiles->setChecked(val == yes ? true : false);
													else if (key == la_string) layers->setChecked(val == yes ? true : false);
														else if (key == lt_string) linetypes->setChecked(val == yes ? true : false);
															else if (key == ra_string) regapps->setChecked(val == yes ? true : false);
																else if (key == uc_string) ucs->setChecked(val == yes ? true : false);
																	else if (key == vp_string) vports->setChecked(val == yes ? true : false);
																		else if (key == vw_string) views->setChecked(val == yes ? true : false);
																			else if (key == "Format")
																			{
																				bool ok;
																				int b = val.toInt(&ok);
																				if (ok) format->setButton(b);
																			}
																				else if (key == "Delimiter")
																				{
																				bool ok;
																				int b = val.toInt(&ok);
																				if (ok) delimiter->setButton(b);
																				}
																					else if (key == "Font")
																					{
																						QFont qf;

																						qf.fromString(val);
		                                        preview->setCurrentFont(qf);
																						preview->setFont(qf);
																					}
				}
			}	//	while rs is not at eof()		
			db->printMessage(tr("Batch Control File %1 processed").arg(report_bcf.name()),TWIN);
		}	//	else statement		
		report_bcf.close();
	}
	else db->printMessage(tr("Batch Control File %1 not found").arg(report_bcf.name()),TWIN);

//	commandproc.cpp arranges for the directory to process to be the last item in the
//	options list.  By doing this we only need to remove the text up to and including
//	the = sign to get the directory name to process
	short mark = 0;
	mark = options.find('=');
	if (mark  > 0)
	{
		options.remove(0,++mark);
		batchProcess(options);
	}
	else populateListView(db->getDwgHandle(),db->getOpenFileName());
	
	if (!options.isNull() && !preview->isVisible() && db->getDwgHandle()) togglePreview();
}


//**********************************************************************************************
//					reportPreview
//**********************************************************************************************
//	reportPreview constructor
reportPreview::reportPreview(QWidget* parent, const char* name):QTextEdit(parent,name)
{
}	

//	report preview close event
void reportPreview::closeEvent(QCloseEvent* e)
{
	e->accept();
	emit previewClosed();
}


	
			
