/***************************************************************************
                          report.h  -  description
                             -------------------
    begin                : Sat Sep 21 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : buyer242299@aol.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//	classes and functions to generate reports about the drawing currently open

#ifndef REPORT_H
#define REPORT_H

//	QT includes
#include <qtabdialog.h>
#include <qlistview.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qfontmetrics.h>
#include <qpaintdevicemetrics.h>
#include <qdatetime.h>
#include <qfontdialog.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qsimplerichtext.h>
#include <qregexp.h>
#include <qdir.h>
#include <qstringlist.h>

//	LX includes
#include "drawingenvironment.h"

//	OD includes
#include "ad2.h"

//	class to define the report preview, really just a QTextEdit object with a
//	custom close event to link window closes (hides) to the toggle button
class reportPreview : public QTextEdit
{
	Q_OBJECT
	public:
		reportPreview(QWidget* parent=0,const char*name = 0);
	protected:
		void closeEvent(QCloseEvent*);
  signals:
		void previewClosed(void);
};

//	report dialog box
class LX_ReportDialog : public QTabDialog
{
 		Q_OBJECT
	
	public:
		LX_ReportDialog(QWidget* parent=0, const char* name=0, bool modal=TRUE, WFlags f=0);
		

	private:
		//	data members
		reportPreview*	preview;
		QPrinter*		reportprinter;
		QString			report_text;

		const QString bk_string;
		const QString an_string;
		const QString at_string;
		const QString ii_string;
		const QString dt_string;
		const QString ds_string;
		const QString xf_string;
		const QString sf_string;
		const QString la_string;
		const QString lt_string;
		const QString ra_string;
		const QString uc_string;
		const QString vp_string;
		const QString vw_string;
		const QString ih_string;
		const QString nb_string;
		const QString bg_string;
		const QString ph_string;

		QString newline;
		QString newpara;
		QString dlm;
		QString tab;

    // dialog members
		QWidget* page1;
		QWidget* page2;

		//page1 declarations
		QListView* lv1;
			QListViewItem* cdi;
			QListViewItem* dti;
			QListViewItem* lai;
			QListViewItem* lti;
			QListViewItem* shi;
			QListViewItem* dmi;
			QListViewItem* vpi;
			QListViewItem* vwi;
			QListViewItem* csi;
			QListViewItem* rai;
			QListViewItem* bki;
			QListViewItem* xfi;
			QListViewItem* item;	

		//page2 declarations
	QButtonGroup* select;
		QCheckBox*	datetime;
		QCheckBox*	layers;
		QCheckBox*	linetypes;
		QCheckBox*	shapefiles;
		QCheckBox*	dimstyles;
		QCheckBox*	vports;
		QCheckBox*	views;
		QCheckBox*	ucs;
		QCheckBox*	regapps;
		QCheckBox*	blocks;
		QCheckBox*	anonymous;
		QCheckBox*	attributes;
		QCheckBox*	insertinfo;		
		QCheckBox*	xref;			
	QButtonGroup* format;
		QRadioButton*	plaintext;
		QRadioButton*	richtext;
	QButtonGroup*	delimiter;
		QRadioButton*	tabdelim;
		QRadioButton*	colondelim;
		QRadioButton*	semicolondelim;
		QRadioButton*	commadelim;
		QRadioButton*	spacedelim;
	LX_LightedButton*	previewbutton;
	QPushButton*	pickfont;
	QPushButton*	printreport;
	QPushButton*	savetodisk;
	QPushButton*	writeBCFbut;
	QPushButton* 	batchprocessbut;
		
//	public slots
	public slots:
		void writeBCF();
		void callBatch();
		void saveToFile();
		void selectFont();
		void print();
		void togglePreview();	
		void generateReport(int);
		void blockChecked(int);
		void delimChecked(int);
		

//	other public functions
	public:
		void batchProcess(QString dir2process = QString());
		QString basicReportParagraph(QListViewItem*, const QString&, QString s2 = QString(), bool grandkids = false);
		void blockMole(AD_DB_HANDLE,QListViewItem*,AD_VMADDR);
		void initializeListView();
		void populateListView(AD_DB_HANDLE,QString dwgfile = QString());	
		void setOptions(QString);
};


#endif

