/***************************************************************************
                          utilities.cpp  -  description
                             -------------------
    begin                : Sat Feb 9 2002
    copyright            : (C) 2002 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//			Utility functions, mostly conversions from one type to another

#include "utilities.h"

//	function that will return the absolute path to the executable file.  Used to
//	get into our file direcory structure
QString PathToExec()
{
	const bool abspath(true);
	const bool cs(true);
	const char pathsep(':');
	int index(0);
  static QString pathtoexec;
	QString temppath;
	QString envpath;
	QString filename;

	if(pathtoexec) return (pathtoexec);		//	if we already know the path just return it

	QFileInfo fi = QString(qApp->argv()[0]);
	filename = fi.fileName();

	if (fi.isSymLink())	//	if argv[0] is a symlink get it's source
	{
		temppath = fi.readLink();
		fi.setFile(temppath);
	}

	pathtoexec = fi.dirPath(abspath);		//	we have a file name either from a symlink or argv[0]
	if (fi.exists()) return pathtoexec;	//	see if the file exists and if so return abs path to i

	envpath = QString(getenv("PATH"));   //	look for file somewhere in the path and return
	envpath.append(pathsep);             //	the abs. path to the first name that matches
	while (envpath.contains(pathsep,cs))
	{
		index = envpath.find(pathsep,0,cs);
		temppath = envpath;
		temppath.remove(index,temppath.length());
		envpath.remove(0,index+1);

		fi.setFile(temppath + "/" + filename);
		if (fi.isSymLink())               		//	if argv[0] is a symlink in the path
		{                                     //	return the actual location of the
			temppath =fi.readLink();            //	executable
			fi.setFile(temppath);
		}
		pathtoexec = fi.dirPath(abspath);
		if (fi.exists()) return (pathtoexec);
	}
			
	pathtoexec = QString();
	return pathtoexec;					//	no luck anywhere, return a null string
}

//	Function that will return a real (double) of a text representation of an architectural
//	string.  The following conversions are examples:
//	10'-1 1/4"		returns 121.25
//	0 1/8" 				returns 0.125
//	10'1.25"			returns 121.25
//	10.125				returns 10.125
//	if any of the conversions fail return -1.0
double ArchToDouble(const QString& s)
{
	const bool cs(FALSE);
	const QChar space(' ');
	const QChar dash('-');
	const QChar slash('/');
	const QChar inunit('\"');
	const QChar ftunit('\'');
	bool ok (TRUE);
	int index = 0;
	double feet = 0;
	double inch = 0;
	double frac = 0;
	QString left("");
	QString right("");
	QString num("");
	QString den("");

	left = right = s.simplifyWhiteSpace();

	if (right.contains(slash,cs))			//	bust out the fractional pieces if they exist
	{	
		if (right.contains(space,cs)) right.remove(0,right.find(space,0,cs)+1);
		index = right.find(slash,0,cs);
		num = den = right;
		num = num.remove(index,right.length());
		den = den.remove(0,index+1);
    if (den.contains(inunit,cs)) den = den.remove(den.find(inunit,0,cs),1);
		frac = num.toDouble(&ok);
		if (!ok) return -1.0;
		frac = frac / den.toDouble(&ok);
		if (!ok) return -1.0;
		if (left.contains(space,cs))
			left.remove(left.find(space,0,cs),left.length());
		else left = "0.0";
	}

	right = left;



	if (left.contains(ftunit,cs))		//	foot part if it exists
	{
		index = left.find(ftunit,0,cs);
		left.remove(index,left.length());
		right.remove(0,index+1);
		feet = left.toDouble(&ok);
		if (!ok) return -1.0;
	}

	if (right.contains(dash,cs)) right.remove(0,right.find(dash,0,cs)+1);
	if (right.contains(inunit,cs)) right = right.remove(right.find(inunit,0,cs),1);
	inch = right.toDouble(&ok);
	if (!ok) return -1;

	return 12.0*feet + inch + frac;
}

//	Function that will return a QString representing an architectural dimension
//	passed a double.  Precision is the power of 2 the fractional part is to take
//	such as 1/2 , 1/4, 1/8, 1/16, etc.
QString DoubleToArch(const double& d, const int& prec)
{
	(void)d;
	(void)prec;
	return QString("To be written");
}

//	Function that will test if a specific bit is set in an unsigned character
bool isBitSet(unsigned char data, short bitplace)
{
	if (bitplace < 1 || bitplace > 8 ) return false;

	unsigned char bitmask[8] = {1,2,4,8,16,32,64,128};
	return ((data & bitmask[bitplace-1]) > 0 ? true : false);
}


