/***************************************************************************
                          vectorize.cpp
                             -------------------
    begin                : Sun Apr 22 2001
    copyright            : (C) 2001 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, version 2 of the License, or any later  * 
 *   version.                                                              *
 *                                                                         *
 ***************************************************************************/

//	Routines to vectorize and display the drawing

//	LX includes
#include "drawingboard.h"
#include "vectorize.h"
#include "3Dview.h"
#include "utilities.h"
#include "serialport.h"
#include "infomessage.h"
#include "resource.h"

#ifndef _WIN32
#include <unistd.h>
#else
#include <io.h>
#endif

#include <qfile.h>

//	global variables
AD_VIEW_PARMS adviewparms;
AD_VIEW_PARMS plotviewparms;
rectlong globwincoords;
rectlong plottercoords;
rectlong dlrect;
bool extentscalculated = FALSE;
int rubberbandtype = NORUBBERBAND;
typedef QPtrStack<adviewparmstru> ADVStack;
ADVStack viewhistory;



//	function to initialize the graphics area, determine the pixel space, etc
void initializegraphics(void)								
{
	extern DrawingBoard* db;

	globwincoords.Xmin=0L;
	globwincoords.Ymin=0L;
	globwincoords.Xmax=db->XY(0)-1;
	globwincoords.Ymax=db->XY(1)-1;		
}

//	function to initialize the plotting area - determine how much we can draw on
void initializeplotarea(void)
{
	extern DrawingBoard* db;

	plottercoords.Xmin=0L;
	plottercoords.Ymin=0L;
	plottercoords.Xmax=static_cast<long>(db->getoutputwidth() - 1);
	plottercoords.Ymax=static_cast<long>(db->getoutputheight() - 1);
}
//	function to set the ad3 control variables, these variables are data members
//	of the structure ad3parm.  Map Autocad colors to OD colors 0=0, 1=1, etc.
//	deal with the color mapping our own way in "colorfunctions.cpp"
void setcontrolvariables(void)
{	
	ad3parm.vecfunclimit=25;
	ad3parm.keepvectorizing=1;
	ad3parm.accuracy=1.0;
	ad3parm.maxcirclesegs=0;
	ad3parm.aspectratio=1.0;
	ad3parm.dolinetypes=1;

	for (int i = 0;i < 256;i++)
		ad3parm.colormap[i]=i;	
}

//	function to vectorize the drawing
void vectorize(AD_DB_HANDLE dwghandle,AD_OBJHANDLE psBlockHandle)
{
	extern DrawingBoard* db;

	adSetDlpMode(DLPON);

	if (!adInitVectorize(dwghandle,&adviewparms,&globwincoords,&dlrect,psBlockHandle))
	{
		DisplayMessage(VECTORIZE_ERROR,CAPTION_INITIALIZE,QMessageBox::Critical);
		freeadptrs();
		exit(1);
	}
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	db->LX_ClearScreen();	
	adDrawDrawing(dwghandle);		
	extentscalculated=TRUE;
}

void resizewindow(AD_DB_HANDLE dwghandle,AD_OBJHANDLE psBlockHandle,short key)
{
	//	key == 0 only if we enter this routine as a return from an an iconization
	//	otherwise regenerate the drawing based on the new window size
	extern DrawingBoard* db;
	adHanclear(psBlockHandle);	

	initializegraphics();
	
	switch (key)
  {
		case 0:
			db->LX_ClearScreen();
			adDrawDrawing(dwghandle);
			break;
		default:
			vectorize(dwghandle,psBlockHandle);
	}
}

//	function to zoom while keeping the proper aspect ratio
void newsizes(QPoint pt1,QPoint pt2)
{
	double oldaspect;
	double newxsize;
	double newysize;
	
	oldaspect=adviewparms.ysize/adviewparms.xsize;
	newxsize=(pt1.x()-pt2.x())*adviewparms.xsize/globwincoords.Xmax;
	newysize=(pt1.y()-pt2.y())*adviewparms.ysize/globwincoords.Ymax;
	if (newxsize<0.0) newxsize*=(-1.0);
	if (newysize<0.0) newysize*=(-1.0);
	
	if (newysize/newxsize < oldaspect) newysize=newxsize*oldaspect;
	else newxsize=newysize/oldaspect;

	adviewparms.xsize=newxsize;
	adviewparms.ysize=newysize;
}

void plotps(AD_DB_HANDLE dwghandle, AD_OBJHANDLE layoutHandle)
{
	extern DrawingBoard* db;
		
	setplotfuns();
	initializeplotarea();
	adSetDlpMode(DLPOFF);
	adInitVectorize(dwghandle,&plotviewparms,&plottercoords,&dlrect,layoutHandle);
	db->setPlotParms();
	db->printMessage(QT_TR_NOOP("Plotting Drawing using PostScript"));
	adForceRegen(dwghandle);	
	adMakeViewTransform(dwghandle,&plotviewparms,&plottercoords,&dlrect);
	adDrawDrawing(dwghandle);		
	adSetDlpMode(DLPON);	
	
	registerfuns();
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);		
}

void plothpgl(AD_DB_HANDLE dwghandle, AD_OBJHANDLE layoutHandle)
{
	extern DrawingBoard* db;
	extern int hpglport;
	QString hpgldevice = db->gethpgloutdevice();
	
	if (hpgldevice == QString("SERIALPORT"))hpglport=openPort();
	else
	{
		QFile hp(hpgldevice);		//	Create the file using QT
		if (hp.exists()) hp.remove();
		hp.open(IO_ReadWrite);
		hp.close();
		hpglport = open(hpgldevice.latin1(),O_WRONLY);
	}

	writeToPort(hpglport,";IN;");		//	initialize the plotter
	sethpglfuns();
	initializeplotarea();
	adSetDlpMode(DLPOFF);
	adInitVectorize(dwghandle,&plotviewparms,&plottercoords,&dlrect,layoutHandle);
	for (short i = 0; i < 4; i++)
	{
		plotviewparms.target[i] = adviewparms.target[i];
		plotviewparms.viewdir[i] = adviewparms.viewdir[i];
		plotviewparms.up[i] = adviewparms.up[i];
	}
	plotviewparms.xsize=db->getplotareawidth();
	plotviewparms.ysize=db->getplotareaheight();
	db->printMessage(QT_TR_NOOP("Plotting Drawing using HPGL"));
	adForceRegen(dwghandle);	
	adMakeViewTransform(dwghandle,&plotviewparms,&plottercoords,&dlrect);
	adDrawDrawing(dwghandle);		
	adSetDlpMode(DLPON);	
	writeToPort(hpglport,"SP0;");		//	put the last pen away
	if (hpgldevice == QString("SERIALPORT"))closePort(hpglport);
	else hpglport = close(hpglport);
		
	registerfuns();
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);		
}

void LX_Redraw(AD_DB_HANDLE dwghandle)
{
	extern DrawingBoard* db;
	db->LX_ClearScreen();
	adDrawDrawing(dwghandle);
}

void LX_Regen(AD_DB_HANDLE dwghandle)
{
	extern DrawingBoard* db;
	db->LX_ClearScreen();
	adForceRegen(dwghandle);
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	adDrawDrawing(dwghandle);	
}

void LX_Pan(AD_DB_HANDLE dwghandle, QPoint anchorpoint)
{
	extern DrawingBoard* db;
	QPoint endpoint;
	double deltax;
	double deltay;
		
	if (!dwghandle) return;
	db->LX_SetCursor(QCursor(Qt::PointingHandCursor));
	db->cmdbar->clearCmdBar();
	db->setCancelPending(false);
	if (anchorpoint.x() < 0)	//	we don't provide an anchor point so go get one
	{		
		db->printMessage(QMainWindow::tr("Base point:  "));
		db->LX_InitializePoints();
		do
			{
				if (db->getCancelPending()){db->LX_SetCursor(QCursor(Qt::ArrowCursor)); return;}	
				anchorpoint=db->getselectedpoint();
				qApp->processOneEvent();
			} while (anchorpoint.x()<0);
	}
	
	rubberbandtype=RUBBERLINE;
	db->LX_InitializePoints();
	db->printMessage(QMainWindow::tr("Displacement point: "));
	do
		{
			if (db->getCancelPending())
			{
				QPointArray qpa = QPointArray(2);
				qpa.setPoint(0,anchorpoint);
				qpa.setPoint(1,db->p2dwgarea->currentpoint);
				db->LX_SetRasterOp(XOR);
				db->LX_PaintPline(qpa);	
				db->LX_SetRasterOp(COPY);
				rubberbandtype=NORUBBERBAND;
				db->LX_SetCursor(QCursor(Qt::ArrowCursor));
				return;
			}	
			endpoint=db->getselectedpoint();
			qApp->processEvents();
		} while (endpoint.x()<0);	

	rubberbandtype=NORUBBERBAND;
	db->LX_SetCursor(QCursor(Qt::ArrowCursor));
	deltax=(anchorpoint.x()-endpoint.x())*adviewparms.xsize/globwincoords.Xmax;
	deltay=(anchorpoint.y()-endpoint.y())*adviewparms.ysize/globwincoords.Ymax;
	deltay*=(-1.0);	// y pixels are positive downwards so we need to multiply by -1
	viewhistory.push(new AD_VIEW_PARMS (adviewparms));
	adPanTarget(&adviewparms,deltax,deltay);
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	db->LX_ClearScreen();
	adDrawDrawing(dwghandle);		
}

void LX_PanOrtho(AD_DB_HANDLE dwghandle, float xfact, float yfact)
{
	extern DrawingBoard* db;
	viewhistory.push(new AD_VIEW_PARMS (adviewparms));
	adPanTarget(&adviewparms,adviewparms.xsize * xfact,adviewparms.ysize * yfact);
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);

	db->LX_ClearScreen();
	adDrawDrawing(dwghandle);	
}

void LX_ZoomIn(AD_DB_HANDLE dwghandle, float zoomfactor)
{
	extern DrawingBoard* db;
	db->LX_ClearScreen();

	viewhistory.push(new AD_VIEW_PARMS (adviewparms));
	adviewparms.xsize *= zoomfactor;
	adviewparms.ysize *= zoomfactor;
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	adDrawDrawing(dwghandle);	
}

void LX_ZoomOut(AD_DB_HANDLE dwghandle, float zoomfactor)
{
	extern DrawingBoard* db;
	db->LX_ClearScreen();


	viewhistory.push(new AD_VIEW_PARMS (adviewparms));
	adviewparms.xsize /= zoomfactor;
	adviewparms.ysize /= zoomfactor;
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	adDrawDrawing(dwghandle);	
}

void LX_ZoomWindow(AD_DB_HANDLE dwghandle)
{
	extern DrawingBoard* db;
	QPoint anchorpoint;
	QPoint endpoint;
	double deltax;
	double deltay;
		
	db->LX_SetCursor(QCursor(Qt::CrossCursor));
	db->printMessage(QT_TR_NOOP("Window base point"));
	db->LX_InitializePoints();
	db->setCancelPending(false);
	do
		{
			if (db->getCancelPending()){db->LX_SetCursor(QCursor(Qt::ArrowCursor)); return;}	
			anchorpoint=db->getselectedpoint();
			qApp->processOneEvent();
		} while (anchorpoint.x()<0);

	rubberbandtype=RUBBERBOX;
	db->LX_InitializePoints();
	db->printMessage(QT_TR_NOOP("Other Corner"));
	do
		{
			if (db->getCancelPending())
			{
				db->LX_SetRasterOp(XOR);
				db->LX_PaintPline(QPointArray(QRect(db->p2dwgarea->currentpoint,anchorpoint),TRUE));
				db->LX_SetRasterOp(COPY);
				rubberbandtype=NORUBBERBAND;
				db->LX_SetCursor(QCursor(Qt::ArrowCursor));
				return;
			}	
			endpoint=db->getselectedpoint();
			qApp->processEvents();
		} while (endpoint.x()<0);	

	rubberbandtype=NORUBBERBAND;
	db->LX_SetCursor(QCursor(Qt::ArrowCursor));
	deltax=((anchorpoint.x()+endpoint.x())/2.0-globwincoords.Xmax/2.0)*adviewparms.xsize/globwincoords.Xmax;
	deltay=((anchorpoint.y()+endpoint.y())/2.0-globwincoords.Ymax/2.0)*adviewparms.ysize/globwincoords.Ymax;
	deltay*=(-1.0);	// y pixels are positive downwards so we need to multiply by -1
	viewhistory.push(new AD_VIEW_PARMS (adviewparms));
	newsizes(anchorpoint,endpoint);
	adPanTarget(&adviewparms,deltax,deltay);
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	db->LX_ClearScreen();
	adDrawDrawing(dwghandle);	
}

void	LX_ZoomExtents(AD_DB_HANDLE dwghandle)
{
	extern DrawingBoard* db;
	db->LX_ClearScreen();
	viewhistory.push(new AD_VIEW_PARMS (adviewparms));
	adZoomExtents(dwghandle,&adviewparms);
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	adDrawDrawing(dwghandle);

	updateViewStructure(dwghandle);
}	

void	LX_ZoomPrevious(AD_DB_HANDLE dwghandle)
{
	PAD_VIEW_PARMS tempview;
	extern DrawingBoard* db;
	db->LX_ClearScreen();

	viewhistory.setAutoDelete(TRUE);
	tempview=viewhistory.pop();
	if (!tempview) return;
//	now copy the stored view parameters to the current master view parameters
	for (short i = 0; i < 4; i++)
		{
			adviewparms.target[i] = tempview->target[i];
			adviewparms.viewdir[i] = tempview->viewdir[i];
			adviewparms.up[i] = tempview->up[i];
		}
		adviewparms.xsize = tempview->xsize;
		adviewparms.ysize = tempview->ysize;
//	and revectorize with the stored view
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);	
	adDrawDrawing(dwghandle);
}


void	LX_ZoomViewmax(AD_DB_HANDLE dwghandle)
{
	extern DrawingBoard* db;
	db->LX_ClearScreen();
	viewhistory.push(new AD_VIEW_PARMS (adviewparms));	
	adZoomVsMax(dwghandle,&adviewparms);
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	adDrawDrawing(dwghandle);	
}

void	LX_ZoomCenter(AD_DB_HANDLE dwghandle)
{
	extern DrawingBoard* db;
	QPoint anchorpoint;
	QPoint endpoint;
	double deltax;
	double deltay;
	double viewheight;
	
	db->printMessage(QT_TR_NOOP("Select the center point"));
	db->LX_InitializePoints();
	do
		{
			anchorpoint=db->getselectedpoint();
			qApp->processOneEvent();
		} while (anchorpoint.x()<0);


	rubberbandtype=RUBBERLINE;
	db->LX_InitializePoints();
	db->printMessage(QT_TR_NOOP("Zoom Height"));
	do
		{
			endpoint=db->getselectedpoint();
			qApp->processEvents();
		} while (endpoint.x()<0);	
	rubberbandtype=NORUBBERBAND;
	deltax=(anchorpoint.x()-globwincoords.Xmax/2.0)*adviewparms.xsize/globwincoords.Xmax;
	deltay=(anchorpoint.y()-globwincoords.Ymax/2.0)*adviewparms.ysize/globwincoords.Ymax;
	deltay*=(-1.0);	// y pixels are positive downwards so we need to multiply by -1
	viewhistory.push(new AD_VIEW_PARMS (adviewparms));
	adPanTarget(&adviewparms,deltax,deltay);
	viewheight=sqrt((anchorpoint.x()-endpoint.x())*(anchorpoint.x()-endpoint.x())+
									(anchorpoint.y()-endpoint.y())*(anchorpoint.y()-endpoint.y()));
	adviewparms.xsize /= (viewheight*adviewparms.ysize/globwincoords.Ymax);
	adviewparms.ysize /= (viewheight*adviewparms.ysize/globwincoords.Ymax);
	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	db->LX_ClearScreen();
	adDrawDrawing(dwghandle);		
}


void LX_3DView(AD_DB_HANDLE dwghandle)
{	
	extern DrawingBoard* db;

	if (!dwghandle) return;

	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	db->LX_ClearScreen();
	adDrawDrawing(dwghandle);		
}

void LX_PredefinedView(AD_DB_HANDLE dwghandle,short key)
{
	extern DrawingBoard* db;
	const double root3 = sqrt(3.0);		//	used to normalize the view vectors - don't need
	const double root6 = sqrt(6.0);   // 	to calc vector norm for predefined views
	
	if (!dwghandle) return;
	switch (key)
	{
		case 1:		//	Aerial Left Front View
			adviewparms.viewdir[0] = -1.0/root3;
			adviewparms.viewdir[1] = -1.0/root3;
			adviewparms.viewdir[2] = 1.0/root3;		
			adviewparms.up[0] = 1.0/root6;
			adviewparms.up[1] = 1.0/root6;
			adviewparms.up[2] = 2.0/root6;
			break;
			
		case 2:		//	Front View
			adviewparms.viewdir[0] = 0.0;
			adviewparms.viewdir[1] = -1.0;
			adviewparms.viewdir[2] = 0.0;		
			adviewparms.up[0] = 0.0;
			adviewparms.up[1] = 0.0;
			adviewparms.up[2] = 1.0;
			break;

		case 3:		//	Aerial Right Front View
			adviewparms.viewdir[0] = 1.0/root3;
			adviewparms.viewdir[1] = -1.0/root3;
			adviewparms.viewdir[2] = 1.0/root3;		
			adviewparms.up[0] = -1.0/root6;
			adviewparms.up[1] = 1.0/root6;
			adviewparms.up[2] = 2.0/root6;
			break;

		case 4:		//	Left side view
			adviewparms.viewdir[0] = -1.0;
			adviewparms.viewdir[1] = 0.0;
			adviewparms.viewdir[2] = 0.0;		
			adviewparms.up[0] = 0.0;
			adviewparms.up[1] = 0.0;
			adviewparms.up[2] = 1.0;
			break;

		case 5:		//	WCS Plan View or Top View
			adviewparms.viewdir[0] = 0.0;
			adviewparms.viewdir[1] = 0.0;
			adviewparms.viewdir[2] = 1.0;		
			adviewparms.up[0] = 0.0;
			adviewparms.up[1] = 1.0;
			adviewparms.up[2] = 0.0;
			break;
	
		case 6:		//	Right side View
			adviewparms.viewdir[0] = 1.0;
			adviewparms.viewdir[1] = 0.0;
			adviewparms.viewdir[2] = 0.0;		
			adviewparms.up[0] = 0.0;
			adviewparms.up[1] = 0.0;
			adviewparms.up[2] = 1.0;
			break;

		case 7:		//	Aerial Left Rear View
			adviewparms.viewdir[0] = -1.0/root3;
			adviewparms.viewdir[1] = 1.0/root3;
			adviewparms.viewdir[2] = 1.0/root3;		
			adviewparms.up[0] = 1.0/root6;
			adviewparms.up[1] = -1.0/root6;
			adviewparms.up[2] = 2.0/root6;
			break;

		case 8:		//	Back View
			adviewparms.viewdir[0] = 0.0;
			adviewparms.viewdir[1] = 1.0;
			adviewparms.viewdir[2] = 0.0;		
			adviewparms.up[0] = 0.0;
			adviewparms.up[1] = 0.0;
			adviewparms.up[2] = 1.0;
			break;

		case 9:		//	Aerial Right Rear View
			adviewparms.viewdir[0] = 1.0/root3;
			adviewparms.viewdir[1] = 1.0/root3;
			adviewparms.viewdir[2] = 1.0/root3;		
			adviewparms.up[0] = -1.0/root6;
			adviewparms.up[1] = -1.0/root6;
			adviewparms.up[2] = 2.0/root6;
			break;

		case 11:		//	Bottom View
			adviewparms.viewdir[0] = 0.0;
			adviewparms.viewdir[1] = 0.0;
			adviewparms.viewdir[2] = -1.0;		
			adviewparms.up[0] = 0.0;
			adviewparms.up[1] = 1.0;
			adviewparms.up[2] = 0.0;
			break;

		case 12:		//	Bottom Right Front View
			adviewparms.viewdir[0] = 1.0/root3;
			adviewparms.viewdir[1] = -1.0/root3;
			adviewparms.viewdir[2] = -1.0/root3;		
			adviewparms.up[0] = 1.0/root6;
			adviewparms.up[1] = -1.0/root6;
			adviewparms.up[2] = 2.0/root6;
			break;	

		case 13:		//	Bottom Left Front View
			adviewparms.viewdir[0] = -1.0/root3;
			adviewparms.viewdir[1] = -1.0/root3;
			adviewparms.viewdir[2] = -1.0/root3;		
			adviewparms.up[0] = -1.0/root6;
			adviewparms.up[1] = -1.0/root6;
			adviewparms.up[2] = 2.0/root6;
			break;			

		case 14:		//	Bottom Right Rear View
			adviewparms.viewdir[0] = 1.0/root3;
			adviewparms.viewdir[1] = 1.0/root3;
			adviewparms.viewdir[2] = -1.0/root3;		
			adviewparms.up[0] = 1.0/root6;
			adviewparms.up[1] = 1.0/root6;
			adviewparms.up[2] = 2.0/root6;
			break;		

		case 15:		//	Bottom Left Rear View
			adviewparms.viewdir[0] = -1.0/root3;

			adviewparms.viewdir[1] = 1.0/root3;
			adviewparms.viewdir[2] = -1.0/root3;		
			adviewparms.up[0] = -1.0/root6;
			adviewparms.up[1] = 1.0/root6;
			adviewparms.up[2] = 2.0/root6;
			break;	
  }

	adviewparms.viewdir[3] = 1.0;
	adviewparms.up[3] = 1.0;

	adMakeViewTransform(dwghandle,&adviewparms,&globwincoords,&dlrect);
	db->LX_ClearScreen();
	adDrawDrawing(dwghandle);
}

//	function to keep the drawing header in sync with the view structure
//	needs work -- need to update the remainder of the viewdata
//	see the article printed from OpenDWG
void updateViewStructure(AD_DB_HANDLE dh)
{
	extern PAD_DWGHDR adhd;
		
	adhd = adHeaderPointer(dh);

	for (short i = 0; i < 3; i++)
	{
		adhd->viewctr[i] = adviewparms.target[i];
		adhd->vpointhdr[i] = adviewparms.viewdir[i];
	}
}
	

